/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef OMA_DM_STATUS_DB_H_
#define OMA_DM_STATUS_DB_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "common/dm-status/oma_dm_status_db_common.h"

/*
 * =============================================================================================================================
 * Database Open & Close & Create table
 * =============================================================================================================================
 *
 */

/**
 * @par Description: API to dm open agent db handler
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[out]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_open_agent(DB_HANDLER ** db_handler);

/**
 * @par Description: API to dm close agent db handler
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_close_agent(DB_HANDLER * db_handler);

/**
 * @par Description: API to create dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_create_table(DB_HANDLER * db_handler);

/**
 * @par Description: API to delete dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_delete_table(DB_HANDLER * db_handler);

/**
 * @par Description: API to reset dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_reset_table(DB_HANDLER * db_handler);

/*
 * =============================================================================================================================
 * Control Transaction
 * =============================================================================================================================
 */

/**
 * @par Description: API to begin transaction dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_begin_transaction(DB_HANDLER * db_handler);

/**
 * @par Description: API to end transaction dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	DB_TRANSACTION (commit, rollback)
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_end_transaction(DB_HANDLER * db_handler, DB_TRANSACTION transaction);

/*
 * =============================================================================================================================
 * engine_status table
 * =============================================================================================================================
 */

/**
 * @par Description: API to init dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_init_engine_status(DB_HANDLER * db_handler);

/**
 * @par Description: API to add engine status structure in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	engine status structure
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_add_engine_status(DB_HANDLER * db_handler, engine_status * status);

/**
 * @par Description: API to delete engine status structure in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	engine status id
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_delete_engine_status(DB_HANDLER * db_handler, int engine_id);

/**
 * @par Description: API to update engine status structure in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	engine status structure
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_update_engine_status(DB_HANDLER * db_handler, engine_status * status);

/**
 * @par Description: API to update engine status column in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	engine status key
 * @param[in]	engine status value
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_update_engine_status_column(DB_HANDLER * db_handler, int engine_id, ENGINE_STATUS_VALUE engine_status_value, void *value);

/**
 * @par Description: API to get engine status structure in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	engine status id
 * @param[out]	engine status structure
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_get_engine_status(DB_HANDLER * db_handler, int engine_id, engine_status ** status);

/**
 * @par Description: API to get all engine status structure in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[out]	engine status structures
 * @param[out]	engine status count
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_get_aII_engine_status(DB_HANDLER * db_handler, engine_status ** status_list, int *count);

/**
 * @par Description: API to check exist  engine status in dm service db table
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	db handler
 * @param[in]	engine status id
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int dm_isExist_engine_id(DB_HANDLER * db_handler, int engine_id);

#endif				/* OMA_DM_STATUS_DB_H_ */
