/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef OMA_DM_STATUS_DB_COMMON_H_
#define OMA_DM_STATUS_DB_COMMON_H_

/*lib*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <glib.h>
#include <time.h>
#include <sqlite3.h>
#include <pthread.h>

#define DB_HANDLER								sqlite3

#define GET_THREAD_ID							pthread_self()

#define RETRY_COUNT							10

#define MAX_QUERY_LENGTH					1024

#define STRDUP(src)								(src == NULL) ?  NULL : strdup(src);

#define	ATOI(src)									(src == NULL) ? 0 : atoi(src);

#define MEMORY_FREE(src) \
	if (src != NULL) {										\
		free(src);												\
		src = NULL;											\
	}																\

typedef sqlite3_stmt(*db_stmt);

/* define enum & structure */
typedef enum {
	ENGINE_STATUS_TBL,

	DB_TABLE_NAME_MAX
} DB_TABLE_NAME;

typedef enum {
	TRANSACTION_COMMIT_ = 0,
	TRANSACTION_ROLLBACK_
} DB_TRANSACTION;

typedef enum {
	VALUE_ENGINE_ID,
	VALUE_ENGINE_STATUS,
	VALUE_SERVER_ID,
	VALUE_SERVER_URL,
	VALUE_CORRELATOR,
	VALUE_MO_PATH,
	VALUE_RESULT_STATUS,
	VALUE_UI_MODE,
	VALUE_TASK_ID,
	VALUE_UI_NOTI_TYPE,
	VALUE_DOWNLOAD_CLICK,
	VALUE_UPDATE_TIME
} ENGINE_STATUS_VALUE;

typedef struct {
	GHashTable *db_handlerTable;	/* db handler hash table */
	pthread_mutex_t *db_handlerTable_mutex;	/* mutex for db handler hash table */
} db_handler_mgr;

typedef struct {
	int engine_id;
	int engine_status;
	char *server_id;
	char *server_url;
	char *correlator;
	char *mo_path;
	int result_status;
	char *ui_mode;
	int task_id;
	int ui_noti_type;
	int download_click;
	time_t update_time;

} engine_status;

#endif				/* OMA_DM_STATUS_DB_COMMON_H_ */
