/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 *   @DM_Error.h
 *   @version									0.1
 *   @brief										This file is the header file of defined common error
 */

#ifndef OMA_DM_ERROR_H_
#define OMA_DM_ERROR_H_

typedef enum {
	DM_ERR_UNKNOWN = 0,
	DM_OK = 200,

	/*dm spec */
	DM_ERR_ACCEPTED_FOR_PROCESSING = 202,
	DM_ERR_AUTH_ACCEPTED = 212,
	DM_ERR_CHUNKED_ITEM_ACCEPTED = 213,
	DM_ERR_OPERATION_CANCELLED = 214,
	DM_ERR_NOT_EXECUTED = 215,
	DM_ERR_ATOMIC_ROLL_BACK_OK = 216,

	DM_ERR_NOT_MODIFIED = 304,

	DM_ERR_UNAUTHORIZED = 401,
	DM_ERR_FORBIDDEN = 403,
	DM_ERR_NOT_FOUND = 404,
	DM_ERR_COMMAND_NOT_ALLOWED = 405,
	DM_ERR_OPTIONAL_FEATURE_NOT_SUPPORTED = 406,
	DM_ERR_AUTHENTICATION_REQUIRED = 407,
	DM_ERR_REQUEST_TIMEOUT = 408,
	DM_ERR_SIZE_REQUIRED = 411,
	DM_ERR_INCOMPLETE_COMMAND = 412,
	DM_ERR_REQUEST_ENTITY_TOO_LARGE = 413,
	DM_ERR_URI_TOO_LONG = 414,
	DM_ERR_UNSUPPORTED_MEDIA_TYPE_OR_FORMAT = 415,
	DM_ERR_REQUESTED_SIZE_TOO_BIG = 416,
	DM_ERR_ALEADY_EXISTS = 418,
	DM_ERR_DEVICE_FULL = 420,
	DM_ERR_SIZE_MISMATCH = 424,
	DM_ERR_PERMISSION_DENIED = 425,
	DM_ERR_NOT_EXEC_FOTA_FLAG = 426,

	/*dm command */
	DM_ERR_COMMAND_FAILED = 500,
	DM_ERR_ATOMIC_FAILED = 507,
	DM_ERR_REQUIRE_REFRESH = 508,
	DM_ERR_SERVER_FAILURE = 509,
	DM_ERR_ATOMIC_ROLL_BACK_FAILED = 516,
	DM_ERR_ATOMIC_RESPONSE_TOO_LARGE_TO_FIT = 517,

	/*common error */
	COMMON_ERR_CAN_NOT_BOOTSTRAP = 549,
	COMMON_ERR_BOOTSTRAP = 550,
	COMMON_ERR_INTERNAL_NO_MEMORY = 551,
	COMMON_ERR_INTERNAL_NOT_DEFINED = 552,
	COMMON_ERR_INTERNAL_BINDER_ERROR = 553,
	COMMON_ERR_INTERNAL_CONNECTION_ERROR = 554,
	COMMON_ERR_GENERIC = 555,
	COMMON_ERR_MO_NOT_FOUND_IN_DB = 556,
	COMMON_ERR_GET_TYPE_NOT_FOUND = 557,
	COMMON_ERR_DELETE = 558,
	COMMON_ERR_NOT_FOUND = 559,
	COMMON_ERR_IS_NULL = 560,
	COMMON_ERR_IS_NOT_NULL = 561,
	COMMON_ERR_NET_CANCEL = 562,
	COMMON_ERR_IPC = 563,
	COMMON_ERR_BUFFER_OVERFLOW = 564,
	COMMON_ERR_ALLOC = 565,
	COMMON_ERR_FW_CONFIG = 566,

	/*ddf error */
	DM_ERR_DDF_PROCCESS = 601,
	DM_ERR_TNDS = 602,

	/*fumo error */
	DM_ERR_FUMO_CAN_NOT_PROCCESS = 701,
	DM_ERR_FUMO_CAN_NOT_FOUND_SERVICE = 702,
	DM_ERR_FUMO_INTERNAL_SCHEDULER_ERROR = 703,
	DM_ERR_FUMO_SCHEDULER_ERROR = 704,

	/* data download err */
	DM_DD_SUCCESS = 900,
	DM_ERR_INSUFFICIENT_MEMORY = 901,
	DM_ERR_USER_CANDELLED = 902,
	DM_ERR_LOSS_OF_SERVICE = 903,
	DM_ERR_ATTR_MISMATCH = 905,
	DM_ERR_INVALID_DESCRIPTOR = 906,
	DM_ERR_INVALID_DDVERSION = 951,
	DM_ERR_DEVICE_ABORTED = 952,
	DM_ERR_NON_ACCEPTABLE_CONTENT = 953,
	DM_ERR_REMINDER_INTERVAL = 954,
	DM_ERR_LOADER_ERROR = 955,
	DM_ERR_DOWNLOAD_INFO = 956,
	DM_DOWNLOAD_POPUP = 957,
	DM_INSTALL_POPUP = 958,
	DM_RESUME_IDLE = 959,

	/*bootstrap err */
	DM_ERR_BOOTSTRAP = 1001,

	/* database err */
	DM_ERR_MORE_DATA = 1101,

	/*ui inteval setting */
	DM_DOWNLOAD_LATER_SETTING = 1201,
	DM_DOWNLOAD_LATER_CANCEL = 1201,
	DM_INSTALL_LATER_SETTING = 1202,
	DM_INSTALL_LATER_CANCEL = 1203,

	/*ETC */
	DM_GET_BATTERY_ERROR = 1301,
	DM_LOW_BATTERY_ERROR = 1302,
	/*telephony */
	DM_TELEPHONY_ERROR = 1311,
	/*wifi */
	DM_WIFI_ONLY_ERROR = 1321,
	/*sd card */
	DM_SD_CARD_ERROR = 1331,
	DM_MEMORY_ERROR = 1332,
	DM_OVER_MEMORY_ERROR = 1333,
	/*roaming */
	DM_ROAMING_ERROR = 1334,
	/*account */
	DM_ERR_FOTA_ACCOUNT_REGISTRATION = 1335,
	DM_ERR_LAWMO_ACCOUNT_REGISTRATION = 1336,

	/*fota polling */
	DM_ERR_FOTA_POLL = 1350,

	/*cp */
	DM_ERR_EVENT_DS = 1400,

	/*ui launch */
	DM_FOTA_UI_LAUNCH_ERROR = 1501,
	DM_NOTI_UI_LAUNCH_ERROR = 1502,
	DM_ALERT_UI_LAUNCH_ERROR = 1503,
	DM_CP_UI_LAUNCH_ERROR = 1504,

	/*task */
	DM_ERR_ASYNC_TASK = 1601,
	DM_ERR_SYNC_TASK = 1602
} DM_ERROR;

#endif				/* OMA_DM_ERROR_H_ */
