/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UTIL_H_
#define UTIL_H_

/*lib*/
#include <stdio.h>
#include <assert.h>
#include <errno.h>
#include <string.h>
#include <stdbool.h>

/*dm-agent*/
#include "common/dm_common.h"

#define errno_abort(text) do {\
	fprintf(stderr, "%s at \"%s\":%d: %s\n", text, __FILE__, __LINE__, strerror(errno));\
	abort();\
	} while(0)

#define assert_condition(condition) do{\
	if (!(condition)) {\
		fprintf(stderr, "assert at \"%s\":%d: %s\n", __FILE__, __LINE__, strerror(errno));\
		abort();\
	}\
	} while(0)

/**
 * @par Description: API to free string
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] string
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void str_free(char **str);

/**
 * @par Description: API to exchange char to int
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] string
 * @param[out] int
 *
 * @return			1 success
 * 					0 error
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int chartoint(char *data, int *value);

/**
 * @par Description: API to launch oma dm fumo ui
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				1 success
 * 						0 error
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int launch_oma_dm_fumo_ui();

/**
 * @par Description: API to launch oma dm fumo noti ui
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] noti type ( user interaction, imformative, background )
 * @param[in] session id
 * @param[in] server id
 * @param[in] triger type ( push , polling )
 *
 * @return				1 success
 * 						0 error
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int launch_om_dm_fumo_noti_ui(int noti_data, char *session_id, char *server_id, int triger_type);

/**
 * @par Description: API to launch oma dm alert ui
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				1 success
 * 						0 error
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int launch_om_dm_fumo_alert_ui();

/**
 * @par Description: API to launch oma dm cp ui
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] pin type
 * @param[in] ext id
 *
 * @return				1 success
 * 						0 error
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int launch_oma_dm_cp_ui(int sec_type, int ext_id);

/**
 * @par Description: API to terminate oma dm ui
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] return result
 * @param[in] service engine id
 *
 * @return
 *
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void terminate_oma_dm_ui(int ret, ENGINE_ID service_engine_id);

/**
 * @par Description: API to terminate ui status
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] noty type
 * @param[in] ui mode
 * @param[in] return result
 *
 * @return
 *
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void terminate_ui_status(int noti_type, char *ui_mode, int ret);

/**
 * @par Description: API to terminate ui status
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				1 success
 * 						0 error
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int check_csc();

/**
 * @par Description: API to get wifi state
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				1 success
 *
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
int get_wifi_state();

/**
 * @par Description: API to nonce decode
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in] nextNonce
 * @param[out] nextNonceDecode
 * @param[out] out_len
 *
 * @return				DM_OK success
 *
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR nonce_decode(char *nextNonce, unsigned char **nextNonceDecode, unsigned int *out_len);

/* hex2char*/
/*bool hex2char(char const* szHex, unsigned char* rch);*/

#endif				/* UTIL_H_ */
