/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 *   @DM_Error.h
 *   @version									0.1
 *   @brief										This file is the header file of defined factory bootstrap
 */

#ifndef FACTORY_BOOTSTRAP_H_
#define FACTORY_BOOTSTRAP_H_

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_error.h"
#include "common/dm_common_def.h"

#define DDF_FILES "/usr/share/oma-dm-cfg/ddf/slp_ddfs.txt"
#define DDF_MSCFUMO_DMACC_FILE "/usr/share/oma-dm-cfg/ddf/slp_msctestserver_dmacc_ddf.txt"
#define DDF_MSCLAWMO_DMACC_FILE "/usr/share/oma-dm-cfg/ddf/slp_mscserver_dmacc_ddf.txt"
#define DDF_GCF_DMACC_FILE "/usr/share/oma-dm-cfg/ddf/slp_gcf_dmacc_ddf.txt"

#define DDF_SPEC_COUNT 		10
#define MAX_FILE_PATH	 			100
#define MAX_SERVICE_NAME 		50

typedef struct {
	sync_agent_dm_mo_type_e mo_type;							/**< mo_type : 	SYNC_AGENT_DM_MO_TYPE_DMACC, SYNC_AGENT_DM_MO_TYPE_DEVINFO, SYNC_AGENT_DM_MO_TYPE_DEVDETAIL,	SYNC_AGENT_DM_MO_TYPE_FUMO, SYNC_AGENT_DM_MO_TYPE_LAWMO, SYNC_AGENT_DM_MO_TYPE_SCOMO, SYNC_AGENT_DM_MO_TYPE_NO_TYPE */
	char schema_file[MAX_FILE_PATH];		/**< schema_file : schema_file paht*/
	int service_ddf;									/**< service_ddf : 1,2,3,4,5,6,7*/
	int server_type;									/**< service_type : 4, 8, 16*/
} ddf_spec;

/**
 * @par Description: API to start factory bootstrap
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		file size
 *
 * @return			DM_OK on success
 *						DM_ERROR, DM_ERR_DDF_PROCCESS on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR factory_bootstrap(SERVICE_SERVER_TYPE server_type);

#endif				/* FACTORY_BOOTSTRAP_H_ */
