/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DM_CP_PROCESSOR_H_
#define DM_CP_PROCESSOR_H_

/*dm-agent*/
#include "dm_cp_error.h"
#include "dm-engine/cp/dm_cp_struct.h"

typedef enum {
	ADD_MESSAGE,
	REMOVE_MESSAGE,
} SENDMSG_TYPE;

/**
 * @par Description: API to recieve cp wbxml
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		msg body
 * @param[in]		msg size
 * @param[in]		ext id
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Recieved_CP_WBXML(const char *msgBody, unsigned int msgSize, int ext_id);

/**
 * @par Description: API to recieve cp xml
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		msg body
 * @param[in]		msg size
 * @param[in]		ext id
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Recieved_CP_XML(const char *msgBody, unsigned int msgSize, int ext_id);

/**
 * @par Description: API to check pincode
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		initial or not (0,1)
 * @param[in]		user data
 * @param[out]		correct (0,1)
 * @param[in]		ext id
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Check_Pincode(int is_initial, char *user_data, int *is_correct, int ext_id);

/**
 * @par Description: API to invalid pincode
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		ext id
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Invalid_Pincode(int ext_id);	/* invalid PIN code ==> send delete message to MF and delete CP data */

/**
 * @par Description: API to setting cp
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		ext id
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Setting_CP(int ext_id);

/**
 * @par Description: API to delete all cp
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Delete_All_CP_Data();	/* delete all CP data */

/**
 * @par Description: API to delete cp data
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		ext id
 *
 * @return		CP_ERROR 1 on success
 * 								2 on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CP_ERROR process_Delete_CP_Data(int ext_id);	/* delete 1 CP data about specific extId */

/**
 * @par Description: API to set msgid
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		msg id
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void process_Set_msgId(int msgId);

/**
 * @par Description: API to free application struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		app
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_application(DM_CP_Application * app);
/**
 * @par Description: API to free application addr struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		app_addr appaddr
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_application_app_addr(DM_CP_Application_AppAddr * app_addr);

/**
 * @par Description: API to free application resource struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		resource
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_application_resource(DM_CP_Application_Resource * resource);

/**
 * @par Description: API to free application auth struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		appauth
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_application_app_auth(DM_CP_Application_AppAuth * appAuth);

/**
 * @par Description: API to free port struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		port
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_port(DM_CP_Port * port);

/**
 * @par Description: API to free string struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		service
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_string_node(String_Node * service);

/**
 * @par Description: API to free napdef struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		napdef
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_napdef(DM_CP_Napdef * napdef);

/**
 * @par Description: API to free auth info struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		auth_info
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_napdef_authinfo(DM_CP_Napdef_AuthInfo * auth_info);

/**
 * @par Description: API to free validity struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		validity
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_napdef_validity(DM_CP_Napdef_validity * validity);

/**
 * @par Description: API to free bootstraps struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		bootstraps
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_bootstrap(DM_CP_Bootstrap * bootstraps);

/**
 * @par Description: API to free pxLogical struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		pxLogical
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_pxlogical(DM_CP_PxLogical * pxLogical);

/**
 * @par Description: API to free pxAuthInfo struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		pxAuthInfo
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_pxlogical_pxauthinfo(DM_CP_PxLogical_PxAuthInfo * pxAuthInfo);

/**
 * @par Description: API to free pxPhysical struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]		pxPhysical
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_cp_pxLogical_pxphsical(DM_CP_PxLogical_PxPhysical * pxPhysical);

#endif				/* DM_CP_PROCESSOR_H_ */
