/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DM_CP_STRUCT_H_
#define DM_CP_STRUCT_H_

typedef enum {
	DM_CP_APP_OMADS = 1
} DM_CP_APP_ID;

typedef struct string_node {
	char *data;
	struct string_node *next;
} String_Node;

typedef struct dm_cp_port {
	char *portnbr;
	String_Node *service;	/* * */
	struct dm_cp_port *next;
} DM_CP_Port;

/************************ PXLOGICAL *****************************/

typedef struct dm_cp_pxlogical_pxauthinfo {
	char *pxAuth_type;
	char *pxAuth_id;	/* ? */
	char *pxAuth_pw;	/* ? */
	struct dm_cp_pxlogical_pxauthinfo *next;
} DM_CP_PxLogical_PxAuthInfo;

typedef struct dm_cp_pxlogical_pxphysical {
	char *physical_proxy_id;
	String_Node *domain;	/* * */
	char *pxAddr;
	char *pxAddrType;	/* ? */
	char *pxAddr_FQDN;	/* ? */
	char *wsp_version;	/* ? */
	char *push_enabled;	/* ? */
	char *pull_enabled;	/* ? */
	String_Node *to_napid;	/* + */
	DM_CP_Port *port;	/* * */
	struct dm_cp_pxlogical_pxphysical *next;
} DM_CP_PxLogical_PxPhysical;

typedef struct dm_cp_pxlogical {
	char *proxy_id;
	char *proxy_pw;		/* ? */
	char *ppg_auth_type;	/* ? */
	char *proxy_provider_id;	/* ? */
	char *name;
	String_Node *domain;	/* * */
	char *trust;		/* ? */
	char *master;		/* ? */
	char *start_page;	/* ? */
	char *bas_auth_id;	/* ? */
	char *bas_auth_pw;	/* ? */
	char *wsp_version;	/* ? */
	char *push_enabled;	/* ? */
	char *pull_enabled;	/* ? */
	DM_CP_PxLogical_PxAuthInfo *pxAuthInfo;	/* * */
	DM_CP_Port *port;	/* * */
	DM_CP_PxLogical_PxPhysical *pxPhysical;	/* + */
	struct dm_cp_pxlogical *next;
} DM_CP_PxLogical;

/************************ BOOTSTRAP *****************************/

typedef struct dm_cp_bootstrap {
	char *name;
	String_Node *network;
	char *country;
	String_Node *proxy_id;
	char *provurl;
	char *context_allow;
	struct dm_cp_bootstrap *next;
} DM_CP_Bootstrap;

/************************ APPLICATION *****************************/

typedef struct dm_cp_application_appaddr {
	char *addr;
	char *addr_type;	/* ? */
	DM_CP_Port *port;	/* * */
	struct dm_cp_application_appaddr *next;
} DM_CP_Application_AppAddr;

typedef struct dm_cp_application_appauth {
	char *auth_level;	/* ? */
	char *auth_type;	/* ? */
	char *auth_name;	/* ? */
	char *auth_secret;	/* ? */
	char *auth_data;	/* ? */
	struct dm_cp_application_appauth *next;
} DM_CP_Application_AppAuth;

typedef struct dm_cp_application_resource {
	char *uri;
	char *name;		/* ? */
	char *accept;		/* ? */
	char *auth_type;	/* ? */
	char *auth_name;	/* ? */
	char *auth_secret;	/* ? */
	char *auth_data;	/* ? */
	char *start_page;	/* ? */
	struct dm_cp_application_resource *next;
} DM_CP_Application_Resource;

typedef struct dm_cp_application {
	char *app_ID;
	char *provider_ID;	/* ? */
	char *name;		/* ? */
	char *accept;		/* ? */
	char *protocol;		/* ? */
	String_Node *to_proxy;	/* * */
	String_Node *to_napid;	/* * */
	String_Node *addr;	/* * */
	DM_CP_Application_AppAddr *appAddr;	/* * */
	DM_CP_Application_AppAuth *appAuth;	/* * */
	DM_CP_Application_Resource *resource;	/* * */
	struct dm_cp_application *next;
} DM_CP_Application;

/************************ NAPDEF *****************************/

typedef struct dm_cp_napdef_napauthinfo {
	char *auth_type;
	char *auth_name;	/* ? */
	char *auth_secret;	/* ? */
	String_Node *auth_entity;	/* * */
	char *spi;		/* ? */
	struct dm_cp_napdef_napauthinfo *next;
} DM_CP_Napdef_AuthInfo;

typedef struct dm_cp_napdef_validity {
	char *country;		/* ? */
	char *network;		/* ? */
	char *sid;		/* ? */
	char *soc;		/* ? */
	char *validuntil;	/* ? */
	struct dm_cp_napdef_validity *next;
} DM_CP_Napdef_validity;

typedef struct dm_cp_napdef {
	char *nap_ID;
	String_Node *bearer;	/* * */
	char *name;
	char *internet;		/* ? */
	char *nap_address;
	char *nap_addr_type;	/* ? */
	String_Node *dns_addr;	/* * */
	char *call_type;	/* ? */
	char *local_addr;	/* ? */
	char *local_addr_type;	/* ? */
	char *link_speed;	/* ? */
	char *dn_link_speed;	/* ? */
	char *linger;		/* ? */
	char *delivery_err_sdu;	/* ? */
	char *delivery_order;	/* ? */
	char *traffic_class;	/* ? */
	char *max_sdu_size;	/* ? */
	char *max_bitrate_uplink;	/* ? */
	char *max_bitrate_dnlink;	/* ? */
	char *residual_ber;	/* ? */
	char *sdu_error_ratio;	/* ? */
	char *traffic_handl_prio;	/* ? */
	char *transfer_delay;	/* ? */
	char *guaranteed_bitrate_uplink;	/* ? */
	char *guaranteed_bitrate_dnlink;	/* ? */
	char *max_num_retry;	/* ? */
	char *first_retry_timeout;	/* ? */
	char *rereg_threshold;	/* ? */
	char *t_bit;		/* ? */
	DM_CP_Napdef_AuthInfo *auth_info;
	DM_CP_Napdef_validity *validity;
	struct dm_cp_napdef *next;
} DM_CP_Napdef;

/************************ CP *****************************/

typedef struct {
	DM_CP_Bootstrap *bootstraps;
	DM_CP_Napdef *napdefs;
	DM_CP_Application *applications;
	DM_CP_PxLogical *pxLogical;
} DM_CP;

#endif				/* DM_CP_STRUCT_H_ */
