/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FW_DOWNLOADER_H_
#define FW_DOWNLOADER_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "dm-engine/dl-manager/dd_object.h"

typedef enum {
	EXIST_PARTIAL_FILE = 1,
	EXIST_COMPLETED_FILE,
	NOT_EXIST_FILE
} DOWNLOAD_FILE_STATUS;

typedef struct {
	int current_data_size;
	int total_data_size;
	char *file_path;
} Data_Resume_Infomation;

/**
 * @par Description: API to get object information from dd_server_uri
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]	dd_server_uri					dd server uri
 * @param[out]	download_descriptor		download_descriptor
 * @param[out]	dd_object_information		dd object information
 *
 * @return				DM_OK
 * 						DM_ERROR
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_object_information(char *dd_server_uri, int *file_size, Download_Descriptor ** download_descriptor, DM_ERROR * download_status);

/**
 * @par Description: API to check file resume
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]	checked_download_folder	folder path
 * @param[in]	download_descriptor			download_descriptor
 * @param[out]	file_status							file status
 *
 * @return				DM_OK
 * 						DM_ERROR
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR check_file_resume(const char *checked_download_folder, DOWNLOAD_FILE_STATUS * file_status, Download_Descriptor * download_descriptor, Data_Resume_Infomation ** data_resume_info);

/**
 * @par Description: API to download object for object path
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]	download_folder					downloaded folder path
 * @parma[in]	config								wifi config
 * @param[in]	download_descriptor			download_descriptor
 * @param[out]	object_downloaded_path		downloaded file path
 *
 *
 * @return				DM_OK
 * 						DM_ERROR
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR download_object(const char *download_folder, char **object_downloaded_path, Data_Resume_Infomation * data_resume_info, int config, Download_Descriptor * download_descriptor, DM_ERROR * downoad_status);

/**
 * @par Description: API to send download status to server
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]	download_status		DM_ERROR
 * @param[in]	download_descriptor	download_descriptor
 *
 * @return				DM_OK
 * 						DM_ERROR
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR send_donwload_status(DM_ERROR download_status, Download_Descriptor * download_descriptor);

/**
 * @par Description: API to free download descriptor
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par par Method of function operation:
 * @par Important notes:
 * @param[in]	download_descriptor	download_descriptor
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_Download_Descriptor(Download_Descriptor ** download_descriptor);

#endif				/* FW_DOWNLOADER_H_ */
