/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DM_COMMON_ENGINE_H_
#define DM_COMMON_ENGINE_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "common/dm_common_def.h"
#include "common/dm_common.h"
#include "framework/san-parser/pm_sanparser.h"
#include "framework/platform-event-handler/dm_platform_event_handler_internal.h"
#include "framework/platform-event-handler/dm_platform_event_handler.h"

//#include "serviceadapter/sa_define.h"
#include "serviceadapter/sa_elements_internal.h"

#define CHECK_OPERATION 			 1
#define NON_CHECK_OPERATION	 0

/**
 * @par Description: API to get service engine id
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[out]	engine_id						engine id
 * @param[in]	ui mode							ui mode
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void get_service_engine_id(ENGINE_ID * service_engine_id, char *ui_mode);

/**
 * @par Description: API to init dm engine
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[out]	engine_id						engine id
 * @param[in]	ui mode							ui mode
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void init_Dm_Engine();

/**
 * @par Description: API to engine Start
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	server id						server id
 * @param[in]	full_path 						full path
 * @param[in]	correlator						correlator
 * @param[in]	resultCode						resultCode
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR engine_start(char *server_id, char *full_path, char *correlator, ENGINE_ID * service_engine_id, int *resultCode);

/**
 * @par Description: API to engine Start about MO replace command.
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	server id						server id
 * @param[in]	full_path 						full path
 * @param[in]	resultCode						resultCode
 * @param[in]	item								item
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR engine_replace_start(char *server_id, char *full_path, ENGINE_ID * service_engine_id, int *resultCode, Item * item);

/**
 * @par Description: API to service_engine_Start
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	service engine id			service engine id
 * @param[out]	service_ret						service_ret
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR service_engine_start(ENGINE_ID service_engine_id, DM_ERROR * service_ret);

/**
 * @par Description: API to set_engine_status_idle
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	service engine id			service engine id
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_engine_status_idle(ENGINE_ID service_engine_id);

/**
 * @par Description: API to set_end_engine_status_idle
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	service engine id			service engine id
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_end_engine_status_idle();

/**
 * @par Description: API to dm_common_start
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	Event_Contents				Event_Contents
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_common_start(Event_Contents * pEvent_data);

/**
 * @par Description: API to dm common operation (called by task)
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	Event_Contents				Event_Contents
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_common_operation(Event_Contents * pEvent_data);

/**
 * @par Description: API to genericAelrt_operation
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *	@param[in] 	session							session
 * @param[in] 	isFinish							isFinish
 * @param[in] 	service engine id			service engine id
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR genericalert_operation(void **session, int *isFinish, ENGINE_ID service_engine_id);

#endif				/* DM_COMMON_ENGINE_H_ */
