/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FUMO_ENGINE_H_
#define FUMO_ENGINE_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "dm-engine/fumo/fumo_engine_internal.h"

/**
 * @par Description: API to execute fumo operation
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[in]	value							value
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR fumo_exec(char *full_path, char *value, FUMO_Error * fumo_return);

/**
 * @par Description: API to execute fumo operation about fumo MO replace cmd.
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[in]	value							value
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR fumo_replace(char *full_path, char *value, FUMO_Error * fumo_return);

/**
 * @par Description: API to operate firmware download
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR firmware_download_opeartion(char *full_path, FUMO_Error * fumo_return);

/**
 * @par Description: API to operate firmware update
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR firmware_update_operation(char *full_path, FUMO_Error * fumo_return);

/**
 * @par Description: API to operate firmware download&update
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR firmware_downloadupdate_operation(char *full_path, FUMO_Error * fumo_return);

/**
 * @par Description: API to operate firmware EXT
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR vendor_dependency_firmware_operation(char *full_path, FUMO_Error * fumo_return);

/**
 * @par Description: API to fumo engine operation (engine state flow)
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[in]	down_server				down server url
 * @param[in]	fumo_state				fumo state
 * @param[out]	fumo_return				fumo return value
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR fumo_engine_state_operation(char *full_path, char *down_server, FUMO_Error * fumo_return);

/**
 * @par Description: API to get current fumo state
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path						mo full path
 * @param[out]	fumo_state_data			fumo state data
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_current_fumo_state(char *full_path, char **fumo_state_data);

/**
 * @par Description: API to set current fumo state
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path						mo full path
 * @param[in]	fumo_state_data			fumo state data
 *
 * @return		DM_OK							success
 *					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_current_fumo_state(char *full_path, FUMO_State fumo_status);

#endif				/* FUMO_ENGINE_H_ */
