/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FUMO_ENGINE_INTERNAL_H_
#define FUMO_ENGINE_INTERNAL_H_

 /*
  *   1. DevInfo
  *  - DevID                    : device id
  *  - Man                      : manufacture
  *  - Mod                      : model id or model number
  *  - DmV                      : OMA DM client version
  *  - Lang                     : current setting language
  *  - Bearer           : bearer
  *
  *  2. DevDetail
  *  - KernelV          : kernel version
  *  - SwV                      : software version
  *  - HwV                      : hardware version
  *  - FwV                      : firmware version
  *  - ModemV           : modem version
  *  - PDAV                     : PDA version
  *  - OEM                      : original equipment manufacturer
  *  - DevType          : device type
  *  - Bearer           : bearer
  *  - FOTADownDir: firmware download directory
  *  - FOTASaveDir:     firmware save directory
  */
/*
sync_agent_dev_return_e sync_agent_get_devinfo(const char *platform, char *info_name, char **value);

sync_agent_dev_return_e sync_agent_execute_dev_function(2, "fota_flag", 1, memory_type);
sync_agent_dev_return_e sync_agent_execute_dev_function(2, "fota_result", 1, (int *)error);
*/

#define MAX_DELTA_FILE_SIZE						"/usr/share/oma-dm-cfg/delta_size/max_size.txt"

typedef enum {
	UNKNOWN = 0,
	IDLE_START = 10,	/*No pending operation */
	DOWNLOAD_FAILED = 20,	/*Download failed */
	DOWNLOAD_PROGRESSING = 30,	/*Download has stared */
	DOWNLOAD_COMPLETE = 40,	/*Download has been completed successfully */
	READY_TO_UPDATE = 50,	/*Have data and awaiting command to start update */
	UPDATE_PROGRESSING = 60,	/*Update has stared */
	UPDATE_FAILED_HAVE_DATA = 70,	/*Update failed but have update package */
	UPDATE_FAILED_NO_DATA = 80,	/*Update failed and no update package available */
	UPDATE_SUCCESSFUL_HAVE_DATA = 90,	/*Update complete and data still available */
	UPDATE_SUCCESSFUL_NO_DATA = 100	/*Data deleted or removed after a successful Update */
} FUMO_State;

typedef enum {
	FUMO_SUCCESS = 200,
	/* 250~299 VENDOR */
	FUMO_MANAGEMENT_CLIENT_ERROR = 400,
	FUMO_USER_CANCELLED = 401,
	FUMO_CORRUPTED_FIRMWARE_UPDATE = 402,
	DEVICE_MISMATCH = 403,
	FUMO_FAILED_VALIDATION = 404,
	FUMO_NOT_ACCEPTABLE = 405,
	FUMO_ALTERNATED_DOWNLOAD_AUTHENTICATION_FAILED = 406,
	FUMO_ALTERANTED_DOWNLOAD_REQUEST_TIME_OUT = 407,
	FUMO_NOT_IMPLEMENTED = 408,
	FUMO_UNDEFINED_ERROR = 409,
	FUMO_UPDATE_FAILED = 410,
	FUMO_MALFORMED_OR_BAD_URL = 411,
	FUMO_ALTERNATED_DOWNLOAD_SERVER_UNABAILABLE = 412,
	/* 450~499 VENDOR */
	FUMO_ALTERNATE_DOWNLOAD_SERVER_ERROR = 500,
	FUMO_DOWNLOAD_FAILS_DUE_TO_DEVICE_OUT_OF_MEMORY = 501,
	FUMO_UPDATE_FAILS_DUE_TO_DEVICE_OUT_OF_MEMORY = 502,
	FUMO_DOWNLOAD_FAILS_DUE_TO_NETWORK_ISSUES = 503,
	/*550~599 */

} FUMO_Error;

#endif				/* FUMO_ENGINE_INTERNAL_H_ */
