/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef LAWMO_ENGINE_H_
#define LAWMO_ENGINE_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "dm-engine/lawmo/lawmo_engine_internal.h"

/**
 * @par Description: API to execute lawmo operation
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[in]	value							value
 * @param[in]	check operation			check operation
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR lawmo_exec(char *full_path, char *correlator, LAWMO_Result_code * lawmo_return, int check_operation);

/**
 * @par Description: API to execute lawmo operation for lawmo MO replace cmd.
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[in]	value							value
 * @param[in]	check operation			check operation
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR lawmo_replace(char *full_path, char *value, LAWMO_Result_code * lawmo_return, int check_operation);

/**
 * @par Description: API to execute lawmo operation
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[out]	lawmo state data	lawmo state data
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_current_lawmo_state(char *full_path, char **lawmo_state_data);

/**
 * @par Description: API to execute lawmo operation
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	full_path					mo full path
 * @param[in]	lawmo state				lawmo state
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_current_lawmo_state(char *full_path, LAWMO_State lawmo_state);

#endif				/* LAWMO_ENGINE_H_ */
