/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 *   @PM_SanParser.h
 *   @version									0.1
 *   @brief										This file is the header file of interface and defined structure for San Parser
 */

#ifndef PM_SANPARSER_H_
#define PM_SANPARSER_H_

typedef enum {
	SAN_UIMODE_UNSPECIFIED = 0,
	SAN_UIMODE_BACKGROUND = 1,
	SAN_UIMODE_INFORMATIVE = 2,
	SAN_UIMODE_USER = 3
} SanUIMode;

typedef enum {
	SAN_INITIATOR_USER = 0,
	SAN_INITIATOR_SERVER = 1
} SanInitiator;

typedef struct SanCred SanCred;
struct SanCred {
	char *credFormat;
	char *credAuth;
	char *credData;
};

typedef struct SanPackage SanPackage;
struct SanPackage {
	char *msgBodyWithoutDigest;
	unsigned int msgBodyWithoutDigestLength;
	char *digest;
	unsigned int version;
	SanUIMode uiMode;
	SanInitiator initiator;
	unsigned char *sessionID;
	SanCred *cred;
	char *serverID;
};

/**
 * @par Description: API to sanpackage_noti_parser
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	msgBody					msgBody
 * @param[in]	msgSize					msgSize
 *
 * @return			SanPackage structure
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
SanPackage *sanpackage_noti_parser(const char *msgBody, unsigned int msgSize);

/**
 * @par Description: API to sanpackage_xml_parser
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	msgBody					msgBody
 * @param[in]	msgSize					msgSize
 *
 * @return			SanPackage structure
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
SanPackage *sanpackage_xml_parser(const char *msgBody, unsigned int msgSize);

/**
 * @par Description: API to free_sanpackageparser
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	SanPackage
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_sanpackageparser(SanPackage * san);

/**
 * @par Description: API to san_printmsg
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	SanPackage
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void san_printmsg(SanPackage * san);

#endif				/* PM_SANPARSER_H_ */
