/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef OMA_DM_TASK_ENABLE_H_
#define OMA_DM_TASK_ENABLE_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "common/dm_common.h"
#include "framework/platform-event-handler/dm_platform_event_handler.h"
#include "dm-engine/fumo/fumo_engine.h"
#include "dm-engine/lawmo/lawmo_engine.h"

/**
 * @par Description: API to init dm task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_init_task_request();

/**
 * @par Description: API to reset dm task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_reset_task_request();

/**
 * @par Description: API to bootstrap task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR bootstrap_task_request();

/**
 * @par Description: API to dm common task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	event_data						Event_data
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_fumo_common_task_request(Event_Contents * pEvent_data);

/**
 * @par Description: API to dm common task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	event_data						Event_data
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_fumo_ui_task_request(Event_Contents * pEvent_data);

/**
 * @par Description: API to dm common task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	event_data						Event_data
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR dm_lawmo_common_task_request(Event_Contents * pEvent_data);

/**
 * @par Description: API to gemericalert task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR genericAlert_task_request();

/**
 * @par Description: API to fumo serviceengine task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR fumo_serviceEngine_task_request();

/**
 * @par Description: API to lawmo serviceengine task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR lawmo_serviceEngine_task_request();

/**
 * @par Description: API to service engine task request
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	Reminder_Interval			Reminder Interval
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR fumo_schedule_task_request(Reminder_Interval remider_interval_type);

/**
 * @par Description: API to cancel task
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	task id
 *
 * @return				DM_OK			success
 *							DM_ERROR	fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void cancel_task(int task_id);

#endif				/* OMA_DM_TASK_ENABLE_H_ */
