/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef OMA_DM_TASK_SPEC_H_
#define OMA_DM_TASK_SPEC_H_

/*lib*/
#include <stdio.h>

/*sync-agent*/
#include <sync_agent.h>

#define EC_MSG_TYPE_DM_INIT_CONTAINER_TASK_REQUEST							0
#define EC_MSG_TYPE_NAME_DM_INIT_CONTAINER_TASK_REQUEST				"dm_init_contatiner"

#define EC_MSG_TYPE_DM_INIT_TASK_REQUEST												1
#define EC_MSG_TYPE_NAME_DM_INIT_TASK_REQUEST									"dm_init"

#define EC_MSG_TYPE_DM_RESET_TASK_REQUEST											2
#define	EC_MSG_TYPE_NAME_DM_RESET_TASK_REQUEST								"dm_reset"

#define EC_MSG_TYPE_BOOTSTRAP_TASK_REQUEST 										3
#define EC_MSG_TYPE_NAME_BOOTSTRAP_TASK_REQUEST								"dm_bootstrap"

#define EC_MSG_TYPE_DM_FUMO_COMMON_TASK_REQUEST 								4
#define EC_MSG_TYPE_NAME_DM_FUMO_COMMON_TASK_REQUEST						"dm_fumo_common"

#define EC_MSG_TYPE_DM_FUMO_UI_TASK_REQUEST 										5
#define EC_MSG_TYPE_NAME_DM_FUMO_UI_TASK_REQUEST							"dm_fumo_ui"

#define EC_MSG_TYPE_DM_LAWMO_COMMON_TASK_REQUEST 						6
#define EC_MSG_TYPE_NAME_DM_LAWMO_COMMON_TASK_REQUEST				"dm_lawmo_common"

#define EC_MSG_TYPE_GENERICALERT_TASK_REQUEST									7
#define EC_MSG_TYPE_NAME_GENERICALERT_TASK_REQUEST							"dm_genericalert"

#define EC_MSG_TYPE_FUMO_SERVICE_ENGINE_TASK_REQUEST						8
#define EC_MSG_TYPE_NAME_FUMO_SERVICE_ENGINE_TASK_REQUEST			"dm_fumo_service_engine"

#define EC_MSG_TYPE_LAWMO_SERVICE_ENGINE_TASK_REQUEST					9
#define EC_MSG_TYPE_NAME_LAWMO_SERVICE_ENGINE_TASK_REQUEST			"dm_lawmo_service_engine"

#define EC_MSG_TYPE_FUMO_SCHEDULE_TASK_REQUEST								10
#define EC_MSG_TYPE_NAME_FUMO_SCHEDULE_TASK_REQUEST					"dm_fumo_schedule"

/**
 * @par Description: API to make_dm_init_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_dm_init_task_spec();

/**
 * @par Description: API to start_dm_init_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_dm_init_process(unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_dm_reset_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_dm_reset_task_spec();

/**
 * @par Description: API to start_dm_reset_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_dm_reset_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_bootstrap_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_bootstrap_task_spec();

/**
 * @par Description: API to start_bootstrap_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_bootstrap_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_dm_fumo_common_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_dm_fumo_common_process_task_spec();

/**
 * @par Description: API to start_dm_fumo_common_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_dm_fumo_common_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_dm_fumo_ui_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_dm_fumo_ui_process_task_spec();

/**
 * @par Description: API to start_dm_fumo_ui_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_dm_fumo_ui_process(unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_dm_lawmo_common_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_dm_lawmo_common_process_task_spec();

/**
 * @par Description: API to start_dm_lawmo_common_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_dm_lawmo_common_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_genericAlert_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_genericAlert_process_task_spec();

/**
 * @par Description: API to start_genericAlert_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_genericAlert_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_fumo_serviceEngine_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_fumo_serviceEngine_process_task_spec();

/**
 * @par Description: API to start_fumo_serviceEngine_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_fumo_serviceEngine_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_lawmo_serviceEngine_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_lawmo_serviceEngine_process_task_spec();

/**
 * @par Description: API to start_lawmo_serviceEngine_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_lawmo_serviceEngine_process( unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

/**
 * @par Description: API to make_fumo_schedule_process_task_spec
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return				sync_agent_ec_task_spec_s
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
sync_agent_ec_task_spec_s *make_fumo_schedule_process_task_spec();

/**
 * @par Description: API to start_fumo_schedule_process
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	param count		count
 * @param[out]	sync_agent_ec_param_param_s				param array
 *
 * @return				sync_agent_ec_task_error_e
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
//sync_agent_ec_task_error_e start_fumo_schedule_process(unsigned int param_cnt, sync_agent_ec_param_param_s** param_array);

#endif				/* DM_TASK_SPEC_H_ */
