/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IPC_COMMON_H_
#define IPC_COMMON_H_

typedef enum {
	EVENT_UNKNOWN = 0,
	/* Fumo, Noti UI Event */
	EVENT_SOFTWARE_UPDATE = 1,	// Type 1, sync
	EVENT_CANCEL,		// Type 1, sync
	EVENT_DOWNLOAD,		// Type 1, sync
	EVENT_INSTALL,		// Type 1, sync
	EVENT_SET_INTERVAL,	// Type 1, sync
	EVENT_ACCOUNT_REGISTRATION,	// Type 1, sync

	EVENT_GET_FUMO_CONFIG = 7,	// Type 1, sync
	EVENT_SET_FUMO_CONFIG,	// Type 1, sync

	/* Admin UI Event */
	EVENT_GET_PROFILE = 9,	// Type 1, sync
	EVENT_SET_PROFILE,	// Type 1, sync
	EVENT_SET_RESET_PROFILE,	// Type 1, sync

	/*ALERT UI */
	EVENT_UIC_ALERT = 12,	// Type 3, response of noti

	/* CP UI Event */
	EVENT_CHECK_PIN_CODE,	// Type 1

	EVENT_ADD_DMACC,
	EVENT_REMOVE_DMACC,

	EVENT_NUM
} EVENT_ID;

typedef enum {
	NOTI_UNKNOWN = 0,
	NOTI_DOWNLOAD = 1,   				// Type 0
	NOTI_INSTALL,                      // Type 0
	NOTI_DOWNLOAD_INFO,                // Type 0

	NOTI_ENGINE_START = 4,				// Type 0
	NOTI_ENGINE_FAIL,						// Type 0
	NOTI_WIFI_ONLY_DOWNLOAD_FAIL,		// Type 0
	NOTI_MEMORY_FULL,						// Type 0
	NOTI_OVER_BIG_SIZE,					// Type 0
	NOTI_LOW_BATTERY,						// Type 0

	NOTI_ALERT_DISPLAY = 10,				// Type 0, response event
	NOTI_ALERT_CONFIRMATION,				// Type 0, response event
	NOTI_ALERT_INPUTTEXT,				// Type 0, response event
	NOTI_ALERT_SINGLE_CHOICE,			// Type 0, response event
	NOTI_ALERT_MULTIPLE_CHOICE,			// Type 0, response event

	NOTI_POLL_INTERACTION = 15,
	NOTI_UPDATE_RESULT,					// Type 0
	NOTI_NUM
} NOTI_ID;

typedef enum {
	PUSH_TYPE_NONE = -1,
	PUSH_NOT_SPECIFIED = 0,
	PUSH_BACKGROUND,
	PUSH_INFORMATIVE,
	PUSH_INTERACTION,
	POLL_INTERACTION
} PUSH_TYPE;

typedef enum {
	IPC_EVENT_TYPE_0 = 0,
	IPC_EVENT_TYPE_1,
	IPC_EVENT_TYPE_2,
	IPC_EVENT_TYPE_3,

} IPC_EVENT_TYPE;

typedef enum {
	IPC_NOTI_TYPE_0 = 0,
	IPC_NOTI_TYPE_1,
	IPC_NOTI_TYPE_2,

} IPC_NOTI_TYPE;

typedef enum {
	PARAM_TYPE_NULL,
	PARAM_TYPE_INT,
	PARAM_TYPE_STR,
} PARAM_TYPE;

typedef enum {
	CONNECTION_STATUS_FAILURE,
	CONNECTION_STATUS_SUCCESS,
	CONNECTION_STATUS_UPTODATA,
	CONNECTION_STATUS_FAILURE_AUTHENTICATION
} CONNECTION_STATUS;

typedef enum {
	RESPONSE_TYPE_NONE = 0,
	RESPONSE_TYPE_OK,
	RESPONSE_TYPE_CANCEL,
	RESPONSE_TYPE_TIMEOVER,
	//Todo: add the response type if U need

	//RESPONSE_TYPE_REMINDER_INTERVAL_NONE,
	RESPONSE_TYPE_REMINDER_INTERVAL_1_HOUR,
	RESPONSE_TYPE_REMINDER_INTERVAL_3_HOURS,
	RESPONSE_TYPE_REMINDER_INTERVAL_6_HOURS,
	RESPONSE_TYPE_REMINDER_INTERVAL_12_HOURS,
	RESPONSE_TYPE_REMINDER_INTERVAL_1_DAY,
	//RESPONSE_TYPE_REMINDER_INTERVAL_7_DAYS,

	RESPONSE_TYPE_NUM
} RESPONSE_TYPE;

typedef enum {

	/*for EVENT_CONNECT_TO_SERVER */

	NO_REPLY_SUCCESS = 0,	// success in the case of no reply
	NO_REPLY_FAILURE,	// failusre in the case of no reply

	REPLY_FAIL_TO_CONNECT,	// failure to connect

	REPLY_SUCCESS_TO_CONNECT,	// success to connect

	REPLY_NEW_UPDATE_EXISTS,	// success to connect
	REPLY_NO_UPDATE_EXISTS,	// success to connect

	REPLY_POSTPONE_STATUS,	// success to connect
	REPLY_NO_POSTPONE_STATUS,	// success to connect

	REPLY_ALL_MEMORYCARD_FULL,	// success to connect
	REPLY_NOT_ALL_MEMORYCARD_FULL,	// success to connect

	//Todo Other replies...

} REPLY_TYPE;

typedef enum {
	FOTA_COMMON_UI,
	FOTA_NOTI_UI,
	FOTA_BACKGROUND_UI,
	ADMIN_UI,
	ALERT_UI,
	CP_UI
} UI_TYPE;

//2012.11.20
typedef enum {
	EVENT_RESPONSE_FAIL = 0,
	EVENT_RESPONSE_OK = 1
} EVENT_RESPONSE;

//2012.11.29
typedef enum _uic_type {
	UIC_NONE_TYPE,
	UIC_DISPLAY_TYPE,
	UIC_CONFIRMATION_TYPE,
	UIC_INPUT_TEXT_TYPE,
	UIC_SINGLE_CHOICE_TYPE,
	UIC_MULTIPLE_CHOICE_TYPE
} uic_type;

#endif				/* IPC_COMMON_H_ */
