/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_USERINTERACTION_H_
#define SA_USERINTERACTION_H_

/*lib*/
#include <glib.h>

/*dm-agent*/
#include "common/dm_error.h"
#include "serviceadapter/sa_define.h"
#include "ipc_common.h"

typedef struct _uic_option {
	char *min_t;
	char *max_t;
	char *default_data;
	char *maxlen;
	char *input_type;
	char *echo_type;
} uic_option;

typedef struct _uic_res_data {
	uic_type type;
	int status;
	char *input_text;
	GList *res_multi_data;
} uic_res_data;

typedef struct _uic_data {
	uic_option *pUic_option;
	char *display;
	GList *multi_data;
	uic_res_data *res_data;
} uic_data;

/**
 * @par Description: API to free uic data
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	uic_data
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_uic_value(uic_data ** uic_value);

/**
 * @par Description: API to free uic res data
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	res_data
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_res_data(uic_res_data * res_data);
/**
 * @par Description: API to execute user interaction option
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	data
 * @param[in]	uic_option
 *
 * @return			DM_OK				success
 * 					DM_ERROR		fail
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR exec_userinteration_option(char *data, uic_option ** uic_value);

/**
 * @par Description: API to display uic
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	uic_data
 *
 * @return			DM_OK				success
 * 					DM_ERROR		fail
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR ui_display(uic_data * uic_value);

/**
 * @par Description: API to confirm uic
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	uic_data
 *
 * @return			DM_OK				success
 * 					DM_ERROR		fail
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR ui_confirm_or_reject(uic_data * uic_value);

/**
 * @par Description: API to input uic
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	uic_data
 *
 * @return			DM_OK				success
 * 					DM_ERROR		fail
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR ui_text_input(uic_data * uic_value, char **result_data);

/**
 * @par Description: API to single choice uic
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	uic_data
 * @param[in]	response item
 *
 * @return			DM_OK				success
 * 					DM_ERROR		fail
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR ui_single_choice(uic_data * uic_value, GList ** responseItems);

/**
 * @par Description: API to multiple choice uic
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	uic_data
 * @param[in]	response item
 *
 * @return			DM_OK				success
 * 					DM_ERROR		fail
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR ui_multiple_choice(uic_data * uic_value, GList ** responseItems);

#endif				/* SA_USERINTERACTION_H_ */
