/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DM_MO_GENERATER_H_
#define DM_MO_GENERATER_H_

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_error.h"
#include "serviceadapter/sa_define.h"
#include "dm_mo_common.h"

/**
 * @par Description: API to clean mo db for reset or delete dm
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR clean_dm_mo();

/**
 * @par Description: API to add item to mo with add cmd
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	item
 * @param[in] server_id
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR add_itemtomo(Item * item, char *server_id);

/**
 * @par Description: API to replace item to mo with replace cmd (update)
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	item
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR replace_itemtomo(Item * item);

/**
 * @par Description: API to replace mo data
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		mo_full_path					mo full path
 * param[in]		value								value
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR replace_mo_data(char *mo_full_path, char *value);

/**
 * @par Description: API to replace mo acl property
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		mo_full_path					mo full path
 * param[in]		value								value
 * param[in]		acl								acl
 * param[in]		contenttype					contenttype
 * param[in]		format							format
 * param[in]		prop_type						prop_type
 *
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR replace_mo_property(char *mo_full_path, char *value, char *acl, char *contenttype, char *format, GetType prop_type);

/**
 * @par Description: API to add mo
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		server type					server type
 * param[in]		mo_full_path					mo full path
 * param[in]		value								value
 * param[in]		node_type						node type
 * param[in]		contenttype					contenttype
 * param[in]		format							format
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR add_mo(int server_type, char *mo_full_path, char *value, sync_agent_dm_mo_node_type_e node_type, char *contenttype, char *format);

/**
 * @par Description: API to delete item to mo
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		item
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR delete_itemtomo(Item * item);

/**
 * @par Description: API to replace mo (update)
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		mo_full_path					mo full path
 *  param[in]		value								value
 *  param[in]		prop_value						prop_value
 *  param[in]		contenttype					contenttype
 *  param[in]		format							format
 *  parma[in]		prop_type						prop_type
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR replace_mo(char *mo_full_path, char *value, char *acl, char *contenttype, char *format, GetType prop_type);

/**
 * @par Description: API to delete mo
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		mo_full_path					mo full path
 *  param[in]		value								value
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR delete_mo(char *mo_full_path);

/**
 * @par Description: API to get mo data
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * param[in]		mo_full_path					mo full path
 *  param[in]		data								data
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_mo_data(char *full_path, char **data);

/**
 * @par Description: API to get mo child list
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		mo_full_path					mo full path
 * @param[out		pItems							child mo liste
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_child_mo_list(char *mo_full_path, GList ** pItems);

/**
 * @par Description: API to get mo data
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		mo_full_path					mo full path
 * @param[in]		get_type						get type
 * @param[out]		real_full_path					real full path
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_type_mo_tree(char *mo_full_path, GetType * get_type, char **real_full_path);

/**
 * @par Description: API to get Struct
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		mo_full_path					mo full path
 * @param[out		pItems							child mo list with get type Struct
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_struct(char *mo_full_path, GList ** pItems);

/**
 * @par Description: API to get StructData
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		mo_full_path					mo full path
 * @param[out]		pItems							child mo list with get type StructData
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_structdata(char *mo_full_path, GList ** pItems);

/**
 * @par Description: API to get TNDS
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		 contenttype					contenttype
 * @param[in]		mo_full_path					mo full path
 * @param[out]	stream_size					stream_size
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_tnds(char *contenttype, char *mo_full_path, char **tnds, unsigned int *stream_size);

/**
 * @par Description: API to set_client_nextnonce
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 *  @param[in]		server id
 * @param[in]		next nonce
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_client_nextnonce(char *pServer_id, char *next_nonce);

/**
 * @par Description: API to set_client_authtype
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		server id
 * @param[in]		AuthType
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_client_authtype(char *pServer_id, AuthType auth_type);

/**
 * @par Description: API to check_mo_scope
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		full_path
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR check_mo_scope(char *full_path);

/**
 * @par Description: API to copy_itemtomo
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		item
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR copy_itemtomo(Item * item);

/**
 * @par Description: API to copy_itemtomo
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		target url
 * @param[in]		source url
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR copy_mo_data(char *target_url, char *source_url);

/**
 * @par Description: API to copy_mo
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		source url
 * @param[in]		target url
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR copy_mo(char *target_url, char *source_url);

/**
 * @par Description: API to get Operation Type
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		mo_full_path					mo full path
 * @param[out]		result_oper_type				result oper type
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_mo_operationtype(char *full_path, char **result_oper_type);

/**
 * @par Description: API to get_mo_node_format_contenttype
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	mo_full_path				mo full path
 * @param[in]	format						format
 * @param[in]	contenttype				contenttype
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_mo_node_format_contenttype(const char *mo_path, char **format, char **contenttype);

/**
 * @par Description: API to get_mo_node_property
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	mo_full_path				mo full path
 * @param[in]	pr_type			 			pr_type
 * @param[out]	property						property
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_mo_node_property(const char *mo_path, propoerty_type pr_type, char **property);

/*=================================================================
 *								FOR TEST
 * =================================================================
 */
/**
 * @par Description: API to this code is temp code for test
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	mo_full_path				mo full path
 * @param[in]	acl							acl
 *
 * @return			DM_OK							success
 * 					DM_ERRORS					fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR pre_define_mo();

#endif				/* DM_MO_GENERATER_H_ */
