/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
   @file	SyncSA_plugin_interface.h
   @brief	common interface for Sync Service Plug-in's

	Define common interface that all sync service plugin have to define\n

 */
#ifndef SA_COMMON_INTERFACE_H_
#define SA_COMMON_INTERFACE_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "serviceadapter/sa_command_internal.h"

/**
 * @par Description: API to Pkg1 and Pkg2 in OMA DM
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	transport Type			(obex, http ...)
 * @param[in]	session						session structure
 * @param[in]	pServer_id					server id
 * @param[in]	pSession_id 				session id
 * @param[in]	pSession_type			pSession type
 * @param[in]	Engine id					service engine id
 * @param[in]	isFinish 						is Finish
 * @param[in]	isgeneicAlert 				is generic alert
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR setup_phase(int transportType, Session ** session, char *pServer_id, char *pSession_id, int session_type, ENGINE_ID * service_engine_id, int *isFinish, int isgenericAlert);

/**
 * @par Description: API to Pkg3 and Pkg4 in OMA DM
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	transport Type			(obex, http ...)
 * @param[in]	session						session structure
 * @param[in]	pSession_id 				session id
 * @param[in]	Engine id					engine id
 * @param[in]	isFinish 						is Finish
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR management_phase(int transportType, Session ** session, char *pSession_id, ENGINE_ID * service_engine_id, int *isFinish);

 /**
  * @par Description: API to generic alert
  *
  *
  * @par Purpose:
  * @par Typical use case:
  * @par Method of function operation:
  * @par Important notes:
  * @param[in]	transport Type			(obex, http ...)
  * @param[in]	session						session structure
  * @param[in]	pServer id					server id
  * @param[in]	pSession_id 				session id
  * @param[in]	isFinish 						is Finish
  * @param[in]	isgeneticAlert				isgeneticAlert
  * @param[in]	Engine id					engine id
  * @param[in]	result 						is result
  *
  *
  * @return				DM_OK	on success
  *							DM_ERR_UNKNOWN or Other DM_ERROR
  *
  *
  * @par Errors:
  *
  * @pre None.
  * @post
  * @see
  * @remarks None.
  *
  * @par Sample Code:
  * @code
  * @endcode
  */
DM_ERROR generic_alert(int transportType, Session ** session, char *pServer_id, char *session_id, int *isFinish, int isgeneticAlert, ENGINE_ID service_engine_status, int result);

 /**
  * @par Description: API to send msg oma sepc, syncml sepc
  *
  *
  * @par Purpose:
  * @par Typical use case:
  * @par Method of function operation:
  * @par Important notes:
  * @param[in]	accType						(obex, http ...)
  * @param[in]	transportType 			transportType
  * @param[in]	session						session structure
  * @param[in]	header_info 				header_info
  * @param[in]	sendMsg					sendMsg
  * @param[in]	sendMsg_length			sendMsg_length
  * @param[in]	recvMsg					recvMsg
  * @param[in]	recvMsg_length	 		recvMsg_length
  *
  *
  * @return				DM_OK	on success
  *							DM_ERR_UNKNOWN or Other DM_ERROR
  *
  *
  * @par Errors:
  *
  * @pre None.
  * @post
  * @see
  * @remarks None.
  *
  * @par Sample Code:
  * @code
  * @endcode
  */
DM_ERROR send_msg(char *accType, int transportType, Session ** session, GList * header_info, char *sendMsg, unsigned int sendMsg_length, GList ** recv_header, char **recvMsg, unsigned int *recvMsg_length);

/**
 * @par Description: API to receive msg form server
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]	session					session structure
 * @param[in]	recvMsg				recvMsg
 * @param[in]	recvMsg_length	 	recvMsg_length
 * @param[in]	syncReturnObj 		syncReturnObj
 * @param[in]	isFinish					isFinish
 * @param[in]	isgeneticAlert			isgeneticAlert
 *
 *
 * @return				DM_OK	on success
 *							DM_ERR_UNKNOWN or Other DM_ERROR
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR recv_msg(Session ** session, char *recvMsg, unsigned int recvMsg_length, ENGINE_ID * service_engine_id, int *isFinish, int isgeneticAlert);

#endif				/* SA_COMMON_INTERFACE_H_ */
