/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_COMMAND_H_
#define SA_COMMAND_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "serviceadapter/sa_define.h"

/**
 * @par Description: API to create add command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[in]		Location structure(about source info)
 * @param[in]		string type, content type
 * @param[in]		string type, data value
 * @param[in]		int type, size
 * @param[in]		int type, moreData
 * @param[out]		Command structure
 *
 * @return			ERROR_INTERNAL_OK on success
 *						ERROR_INTERNAL_NOT_DEFINED, ERROR_INTERNAL_NO_MEMORY on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_add_cmd(Session * session, char *luid, const char *contenttype, char *data, unsigned int size, int moreData, Command ** pCommand);

/**
 * @par Description: API to create delete command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[in]		Location structure(about source info)
 * @param[in]		string type, content type
 * @param[out]		Command structure
 *
 * @return			ERROR_INTERNAL_OK on success
 *						ERROR_INTERNAL_NOT_DEFINED, ERROR_INTERNAL_NO_MEMORY on error
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_delete_cmd(Session * session, char *luid, const char *contenttype, Command ** pCommand);

/**
 * @par Description: API to create alert command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[in]		enum type, alert type
 * @param[out]		Command structure
 *
 * @return			DM_OK on success
 *						DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_alert_cmd(Session * session, AlertType syncType, Command ** pCommand);

/**
 * @par Description: API to create replace command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[out]		Command structure
 *
 * @return			DM_OK on success
 *						DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_replace_cmd(Session * session, Command ** pCommand);

/**
 * @par Description: API to create result command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[out]		Command structure
 *
 * @return			DM_OK on success
 *						DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_results_cmd(Session * session, Command ** pCommand);

/**
 * @par Description: API to create duplicate command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		source
 * @param[out]	target
 *
 * @return			DM_OK on success
 *						DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR duplicate_command(Command * source, Command ** target);

/**
 * @par Description: API to free commands
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		commnads
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_commands(GList * commands);

/**
 * @par Description: API to free command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Command structure
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_command(Command * pCommand);

/**
 * @par Description: API to free command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Command structure
 * @param[in]		string type, correlator
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_correlator(Command ** pAlertCommand, char *pCorrelator);

/**
 * @par Description: API to set result command msg ref
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Command structure
 * @param[in]		msg ref
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_resultscommand_msgref(Command * pCommand, unsigned int msgRef);

/**
 * @par Description: API to set result command cmd ref
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Command structure
 * @param[in]		msg ref
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_resultscommand_cmdref(Command * pCommand, unsigned int cmdRef);

/**
 * @par Description: API to set result command cmd ref
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Command structure
 * @param[in]		location target uri
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_resultscommand_targetref(Command * pCommand, Location * pLocation);

/**
 * @par Description: API to create new status location
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		session structure
 * @param[in]		enum type, dm error
 * @param[in]		command structure
 * @param[in]		string type, loaction source uri
 * @param[in]		string type, location target uri
 * @param[in]		enum type, command type
 * @param[out]		status structure
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_newstatuslocation(Session * session, DM_ERROR data, Command * command, Location * sourceref, Location * targetref, CommandType type, Status ** pStatus);

/**
 * @par Description: API to create new status
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		session structure
 * @param[in]		enum type, dm error
 * @param[in]		command structure
 * @param[in]		string type, loaction source uri
 * @param[in]		string type, location target uri
 * @param[in]		enum type, command type
 * @param[in]		status structure
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_newstatus(Session * session, DM_ERROR data, Command * command, Location * sourceref, Location * targetref, CommandType type, Status ** pStatus);

/**
 * @par Description: API to create new status location
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		enum type, dm error
 * @param[in]		int type, cmd id
 * @param[in]		int type, msg ref
 * @param[in]		int type, cmd ref
 * @param[in]		string type, location source ref
 * @param[in]		string type, location target ref
 * @param[in]		chalenge structure
 * @param[in]		enum type, commadn type
 * @param[out]		status structure
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_status(DM_ERROR data, unsigned int cmdID, unsigned int msgref, unsigned int cmdref, Location * sourceref, Location * targetref, Chal * pChal, CommandType type, Status ** pStatus);

/**
 * @par Description: API to create new status location
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		session structure
 * @param[in]		command structure
 * @param[in]		item structure
 * @param[in]		enum type, dm error
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_status_data(Session * session, Command * pCommand, Item * item, DM_ERROR data);

/**
 * @par Description: API to free status list
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		status list
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_statuses(GList * status);

/**
 * @par Description: API to free status
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		status
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_status(Status * status);

/**
 * @par Description: API to get status code
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[out]		status
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR get_statuscode(Status * status);

/**
 * @par Description: API to convert int type to change type
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		type
 *
 * @return				enum type, ChangeType
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
ChangeType converttochangetype(unsigned int type);

/**
 * @par Description: API to convert change type to int type
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		type
 *
 * @return				int type, int
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
unsigned int convertfromchangetype(ChangeType changeType);

/**
 * @par Description: API to convert string type to command type
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		string type, type
 *
 * @return				CommandType
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
CommandType converttocommandtype(char *type);

/**
 * @par Description: API to convert command type to string
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes: *
 * @param[in]		string type, type
 *
 * @return				string type, string
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
char *convertfromcommandtype(CommandType type);

#endif				/* SA_COMMAND_H_ */
