/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_COMMAND_INTERNAL_H_
#define SA_COMMAND_INTERNAL_H_

/*dm-agent*/
#include "serviceadapter/sa_define.h"

typedef struct CommandAlertPrivate {
	AlertType type;
	unsigned int maxObjSize;
	/*Item *item; */
	GList *items;
	char *Correlator;
} CommandAlertPrivate;

typedef struct CommandAccessPrivate {
	char *type;
	GList *items;
} CommandAccessPrivate;

typedef struct CommandChangePrivate {
	GList *items;
	ChangeType type;
} CommandChangePrivate;

typedef struct CommandResultsPrivate {
	unsigned int msgRef;
	unsigned int cmdRef;
	Location *targetRef;
	char *type;
	GList *items;
} CommandResultsPrivate;

typedef struct CommandExecPrivate {
	char *correlator;
	Item *item;
} CommandExecPrivate;

typedef struct CommandSequence_AtomicPrivate {
	GList *commands;
} CommandSequence_AtomicPrivate;

typedef union CommandPrivate {
	CommandAlertPrivate alert;
	CommandAccessPrivate access;	/*get */
	CommandChangePrivate change;	/*add, delete, replace */
	CommandResultsPrivate results;
	CommandExecPrivate exec;
	CommandSequence_AtomicPrivate sequence_atomic;

} CommandPrivate;

struct Command {
	int refCount;
	CommandType type;
	unsigned int cmdID;
	unsigned int msgID;
	CommandPrivate private;
	Location *source;
	Location *target;
	int noResp;
};

struct Status {
	unsigned int cmdID;
	unsigned int msgRef;
	unsigned int cmdRef;
	CommandType type;
	char *data;
	Location *sourceRef;
	Location *targetRef;
	Cred *cred;
	Chal *chal;
	GList *items;
	//Item *item;
};

#endif				/* SA_COMMAND_INTERNAL_H_ */
