/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_DEFINE_H_
#define SA_DEFINE_H_

/*lib*/
#include <malloc.h>
#include <glib.h>
#include <assert.h>
#include <string.h>
#include <stdbool.h>

#define ELEMENT_WBXML "wbxml"
#define ELEMENT_XML "xml"

#define ELEMENT_DM_WBXML "application/vnd.syncml.dm+wbxml"
#define ELEMENT_DM_XML "application/vnd.syncml.dm+xml"

#define ELEMENT_DMTNDS_WBXML "application/vnd.syncml.dmtnds+wbxml"
#define ELEMENT_DMTNDS_XML "application/vnd.syncml.dmtnds+xml"

/*
#define ELEMENT_AUTH_BASIC "syncml:auth-basic"
#define ELEMENT_AUTH_MD5 "syncml:auth-md5"
#define ELEMENT_FORMAT_BASE64 "b64"
*/

#define ELEMENT_HTTP_BASIC "HTTP_BASIC"
#define ELEMENT_HTTP_DIGEST "HTTP_DIGEST"
#define ELEMENT_BASIC "BASIC"
#define ELEMENT_DIGEST	"DIGEST"
#define ELEMENT_HMAC "HMAC"
#define ELEMENT_X509 "X509"
#define ELEMENT_SECURID "SECURID"
#define ELEMENT_SAFEWORD "SAFEWORD"
#define ELEMENT_DIGIPASS "DIGIPASS"
#define ELEMENT_TRANSPORT "TRANSPORT"

#define ELEMENT_DEFAULT_CLIENT_NONCE "MTIzNDU="

#define	ELEMENT_STRUCT "?list=Struct"
#define	ELEMENT_STRUCTDATA "?list=StructData"
#define	ELEMENT_TNDS "?list=TNDS"
#define ELEMENT_PROPERTY "?prop"

#define	ELEMENT_PROP_ACL "?prop=ACL"
#define 	ELEMENT_PROP_SIZE "?prop=Size"
#define 	ELEMENT_PROP_TYPE "?prop=Type"
#define 	ELEMENT_PROP_NAME "?prop=Name"
#define 	ELEMENT_PROP_FORMAT "?prop=Format"

#define ELEMENT_TEXTPLAIN "text/plain"

#define DEFAULT_MAX_MSG_SIZE 1024*12
#define DEFAULT_MAX_OBJ_SIZE 1024*1024

#define DEFAULT_REST_SIZE 300
#define DEFAULT_MORE_REST_SIZE 500

typedef struct Item Item;
typedef struct Status Status;
typedef struct Command Command;
typedef struct Location Location;
typedef struct Cred Cred;
typedef struct SyncML SyncML;
typedef struct SyncHdr SyncHdr;
typedef struct Chal Chal;
typedef struct Hmac Hmac;
typedef struct Session Session;
typedef struct PendingStatus PendingStatus;

typedef enum {
	VERSION_UNKNOWN = 0,
	VERSION_10 = 1,
	VERSION_11 = 2,
	VERSION_12 = 3
} ProtocolVersion;

typedef enum {
	PROTOCOL_TYPE_UNKNOWN = 0,
	PROTOCOL_TYPE_DS = 1,
	PROTOCOL_TYPE_DM = 2
} ProtocolType;

typedef enum {
	COMMAND_TYPE_UNKNOWN = 0,
	COMMAND_TYPE_ALERT = 1,
	COMMAND_TYPE_HEADER = 2,
	COMMAND_TYPE_ADD = 3,
	COMMAND_TYPE_REPLACE = 4,
	COMMAND_TYPE_DELETE = 5,
	COMMAND_TYPE_GET = 6,
	COMMAND_TYPE_RESULTS = 7,
	COMMAND_TYPE_EXEC = 8,
	COMMAND_TYPE_SEQUENCE = 9,
	COMMAND_TYPE_ATOMIC = 10,
	COMMAND_TYPE_COPY = 11
} CommandType;

typedef enum {
	GET_TYPE_NOMAL,
	GET_TYPE_STRUCT,
	GET_TYPE_STRUCTDATA,
	GET_TYPE_TNDS,
	GET_PROPERTY,
	GET_TYPE_ACL_PROP,
	GET_TYPE_SIZE_PROP,
	GET_TYPE_TYPE_PROP,
	GET_TYPE_FORMAT_PROP,
	GET_TYPE_NAME_PROP
} GetType;

typedef enum {
	PROP_TYPE_ACL,
	PROP_TYPE_SIZE,
	PROP_TYPE_TYPE,
	PROP_TYPE_FORMAT,
	PROP_TYPE_NAME
} propoerty_type;

typedef enum {
	AUTH_TYPE_UNKNOWN = 0,
	AUTH_TYPE_HTTP_BASIC,
	AUTH_TYPE_HTTP_DIGEST,
	AUTH_TYPE_BASIC,
	AUTH_TYPE_MD5,
	AUTH_TYPE_HMAC,
	AUTH_TYPE_X509,
	AUTH_TYPE_SECURID,
	AUTH_TYPE_SAFEWORD,
	AUTH_TYPE_DIGIPASS,
	AUTH_TYPE_TRANSPORT
} AuthType;

#if defined (_OMA_DM_V12_)

typedef enum {
	AUTH_LEVEL_NONE,
	AUTH_LEVEL_CLCRED,
	AUTH_LEVEL_SRVCRED,
	AUTH_LEVEL_OBEX,
	AUTH_LEVEL_HTTP
} AAUTHLEVEL;

#endif

typedef enum {
	FORMAT_TYPE_UNKNOWN = 0,
	FORMAT_TYPE_BASE64 = 1
} FormatType;

typedef enum {
	SYNCML_PKG_1 = 1,
	SYNCML_PKG_2 = 2,
	SYNCML_PKG_3 = 3,
	SYNCML_PKG_4 = 4
} PkgStatus;

typedef enum {
	ITEM_UNKNOWN = 0,
	ITEM_DATA = 1
} ItemDataType;

typedef enum {
	ALERT_UNKNOWN = 0,

	/*User interaction alert codes */
	DM_ALERT_DISPLAY = 1100,
	DM_ALERT_CONFIRM_OR_REJECT = 1101,
	DM_ALERT_TEXT_INPUT = 1102,
	DM_ALERT_SINGLE_CHOICE = 1103,
	DM_ALERT_MULTIPLE_CHOICE = 1104,
	/*reversed syncml 1105~1199 */

	/*Device management session alert codes */
	DM_ALERT_SERVER_INITIATED_MGMT = 1200,
	DM_ALERT_CLIENT_INITIATED_MGMT = 1201,
	/*reversed syncml 1202~1220 */

	/*Special device management alert codes */
	DM_ALERT_NEXT_MESSAGE = 1222,
	DM_ALERT_SESSION_ABORT = 1223,
	DM_ALERT_CLIENT_EVENT = 1224,
	DM_ALERT_NO_END_OF_DATA = 1225,
	DM_ALERT_GENERIC_ALERT = 1226
	    /*reversed syncml 1227~1299 */
} AlertType;

typedef enum {
	CHANGE_UNKNOWN = 0,
	CHANGE_ADD = 1,
	CHANGE_REPLACE = 2,
	CHANGE_DELETE = 3,
	CHANGE_COPY = 4
} ChangeType;

#endif				/* SA_DEFINE_H_ */
