/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_ELEMENTS_INTERNAL_H_
#define SA_ELEMENTS_INTERNAL_H_

/*dm-agent*/
#include "serviceadapter/sa_define.h"

typedef union ItemData {
	char *data;
} ItemData;

struct Hmac {
	char *authtype;
	char *username;
	char *mac;
};

struct Chal {
	FormatType format;
	AuthType type;
	char *nonce_plain;
	unsigned int nonce_length;
	char *nonce_b64;
};

struct Item {
	ItemDataType dataType;
	ItemData private;
	Location *source;
	Location *target;

	unsigned int size;	//-> for data
	char *contenttype;	//-> for data
	char *format;
	int moreData;		//-> for moredata
};

struct Location {
	char *locURI;
	char *locName;
};

struct Cred {
	FormatType format;
	AuthType type;
	char *data;
	char *username;
	char *password;
};

struct SyncHdr {
	char *sessionID;
	ProtocolVersion version;
	ProtocolType protocol;
	Location *source;
	Location *target;
	Cred *cred;
	char *responseURI;
	unsigned int messageID;

	//in meta element
	unsigned int maxmsgsize;
	unsigned int maxobjsize;
	//char *emi;(?)         // Experimental Meta Information
};

struct SyncML {
	SyncHdr *hdr;
	GList *status;			/**< containing  Status struct */
	GList *commands;		/**< containing  Command struct */
	int final;
};

#endif				/* SA_ELEMENTS_INTERNAL_H_ */
