/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_SESSION_H_
#define SA_SESSION_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "serviceadapter/sa_define.h"

/**
 * @par Description: API to create Session structure
 * This API can be used to create Session structure
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		enum type, protocol version(VERSION_UNKNOWN, VERSION_10, VERSION_11, VERSION_12);
 * @param[in]		enum type, protocol type(PROTOCOL_TYPE_UNKNOWN , PROTOCOL_TYPE_DM)
 * @param[in]		string type, server id
 * @param[in]		string type, client id
 * @param[in]		string type, session id
 * @param[in]		string type, source uri(server ip)
 * @param[in]		string type, target uri(client identifer)
 * @param[out]		Session structure
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_syncml_session(ProtocolVersion protocolVersion, ProtocolType protocolType, char *server_id, char *client_id, char *pSession_id, char *sourceUrl, char *targetUrl, Session ** pSession);

/**
 * @par Description: API to free Session structure
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void free_session(Session * session);

/**
 * @par Description: API to receivce header
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[in]		syncml header structure
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR receive_header(Session * session, SyncHdr * header);

/**
 * @par Description: API to receivce header
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[in]		receive status list
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR receive_statuses(Session * session, GList * receiveStatus);

/**
 * @par Description: API to receivce command
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		Session structure
 * @param[in]		receive command list
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR receive_cmd(Session * session, GList * receiveCommand);

/**
 * @par Description: API to create prepending status
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		int type, msg id
 * @param[in]		int type, command id
 * @param[out]		pending status structure
 *
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_prependingstatus(unsigned int msgID, unsigned int cmdID, PendingStatus ** pPendingStatus);

/**
 * @par Description: API to set auth cred
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		session structure
 * @param[in]		cred structure
 *
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR set_session_authcred(Session * session, Cred * cred);

/**
 * @par Description: API to reset command id
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		session structure
 *
 *
 * @return
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void reset_cmdid_session(Session * session);

/**
 * @par Description: API to add response hmac info
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		session structure
 * @param[in]		string type, temp hmac
 *
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR add_response_hmacinfo(Session ** session, char *temp_hmac);
/*DM_ERROR get_session_clientauthtype(Session * session, AuthType * authType);
DM_ERROR get_session_serverauthtype(Session * session, AuthType * authType);*/

/**
 * @par Description: API to get session authtype
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		string type, auth data
 *
 *
 * @return				Auth type
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
AuthType __get_Session_AuthType_enum(char *data);

/**
 * @par Description: API to get session authname
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		enum type, Auth type
 *	@param[out]		string type, auth name
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void __set_Session_AuthType_name(AuthType authType, char **authname);

#endif				/* SE_SESSION_H_ */
