/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SA_UTIL_H_
#define SA_UTIL_H_

/*dm-agent*/
#include "common/dm_error.h"
#include "serviceadapter/sa_define.h"

/**
 * @par Description: API to create auth credstring
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		command list
 * @param[in]		command last list value
 * @param[in]		command structure
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void putCmdIntoList(GList ** commands, GList ** commands_last, void *pCommand);

/**
 * @par Description: API to create auth credstring
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		enum type, auth type
 * @param[in]		string type, user name
 * @param[in]		string type, password
 * @param[in]		string type, nonce
 * @param[in]		int type, nonce size
 * @param[out]		string type, cred string
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR create_auth_credString(AuthType type, const char *username, const char *password, const unsigned char *nonce, const unsigned int nonce_size, char **pCred);

/**
 * @par Description: API to generate hmac
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		string type, user name
 * @param[in]		string type, password
 * @param[in]		string type, nonce
 * @param[in]		string type, msg
 * @param[in]		int type, msg size
 * @param[out]		string type, hmac
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR gemerate_hmac(char *username, char *password, unsigned char *nonce, unsigned int nonce_size, char *msg, unsigned int msg_size, char **mac);

/**
 * @par Description: API to find string value
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		string type, original string
 * @param[in]		string type, find param
 * @param[in]		string type, find seperator
 * @param[out]		string type, find value
 *
 * @return				DM_OK on success
 *							DM_ERROR on fail
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
DM_ERROR findStgringValue(char *original, char *findParam, char *findSep, char **findValue);

/**
 * @par Description: API to set xml to file
 *
 *
 * @par Purpose:
 * @par Typical use case:
 * @par Method of function operation:
 * @par Important notes:
 * @param[in]		string type, xml
 * @param[in]		string type, path
 *
 * @return
 *
 *
 * @par Errors:
 *
 * @pre None.
 * @post
 * @see
 * @remarks None.
 *
 * @par Sample Code:
 * @code
 * @endcode
 */
void set_xml_to_file(char *xml, const char *path);

#endif				/* SA_UTIL_H_ */
