/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <unistd.h>

/*lib*/
#include <vconf.h>
#include <vconf-keys.h>

/*sync-agent*/
#include <sync_agent.h>
#include <plugin/plugin_slp_sysnoti_alarm.h>

/*dm-agent*/
#include "framework/platform-event-handler/dm_platform_event_handler.h"
#include "framework/platform-event-handler/dm_platform_event_handler_internal.h"
#include "framework/task/oma_dm_task_register.h"
#include "framework/task/oma_dm_task_request.h"
#include "common/dm_common.h"
#include "common/util/util.h"
#include "mo-handler/dm_mo_common.h"
#include "mo-handler/dm_mo_handler.h"
#include "dm-engine/bootstrap/factory_bootstrap.h"
#include "dm-engine/fumo/fumo_account.h"
#include "dm-engine/lawmo/lawmo_account.h"
#include "dm-engine/cp/dm_cp_security.h"
#include "ipc_common.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"DM_COMMON"
#endif

#define MAX_TRY		3

static DM_ERROR _create_engine_status_db();

char *get_new_uri(char *old_object_uri)
{
	_EXTERN_FUNC_ENTER;

	char *new_object_uri = NULL;
	retvm_if(old_object_uri == NULL, NULL, "old_object_uri is NULL!!");
	_DEBUG_INFO(" old_object_uri = %s ", old_object_uri);
	new_object_uri = g_strdup_printf("%s", old_object_uri);

	_EXTERN_FUNC_EXIT;
	return new_object_uri;
}

static DM_ERROR _create_engine_status_db()
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	ret = Alloc_DB_Handler_Mgr();
	_DEBUG_TRACE(" Alloc DB Handler Mgr : %d\n", ret);

	ret = DB_Open();
	_DEBUG_TRACE(" Done Open DB : %d\n", ret);

	ret = DB_Create_Table();
	_DEBUG_TRACE(" Done Create_Table : %d\n", ret);

	_INNER_FUNC_EXIT;
	return ret;

}

DM_ERROR init_dm()
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	ret = _create_engine_status_db();
	_DEBUG_INFO(" Done Create_engine_status_db : %d\n", ret);

	ret = task_register();
	if (ret != DM_OK) {
		goto error;
	}
	/* will delete */
	ret = register_scheduler();
	if (ret == -1) {
		_DEBUG_INFO("fail register alarm : %d ", ret);
	} else {
		_DEBUG_INFO("success register alarm : %d ", ret);
	}

	ret = register_network();
	if (ret == -1) {
		_DEBUG_INFO("fail register network : %d ", ret);
	} else {
		_DEBUG_INFO("success register network : %d ", ret);
	}

	ret = register_wap_push();
	if (ret == -1) {
		_DEBUG_INFO("fail register wap push : %d ", ret);
	} else {
		_DEBUG_INFO("success register wap push : %d ", ret);
	}

/*
	ret = register_fumo_ip_push();
	if (ret == -1) {
		_DEBUG_INFO("fail register fumo ip push : %d ", ret);
	} else {
		_DEBUG_INFO("success register fumo ip push : %d ", ret);

		//todo temp code
		//  will integrate notification UI
		//ret = register_fota_account();
		//if (ret == -1) {
		//	_DEBUG_INFO("fail register fota account : %d ", ret);
		//} else {
		//	_DEBUG_INFO("success register fota account : %d ", ret);
		//}
	}
*/
/*
	ret = register_lawmo_samsung_account();
	if (ret == -1) {
		_DEBUG_INFO("fail register fmm server : %d ", ret);
	} else {
		_DEBUG_INFO("success register fmm server : %d ", ret);
	}
*/
	ret = register_telephony();
	if (ret == -1) {
		_DEBUG_INFO("fail register telephony : %d ", ret);
	} else {
		_DEBUG_INFO("success register telephony : %d ", ret);
	}

	init_default_fumo_config();

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_EXTERN_FUNC_EXIT;
	_DEBUG_INFO(" end error : %d \n", ret);
	return ret;
}

DM_ERROR end_dm()
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR reset_dm()
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	_DEBUG_INFO("--------------------------------------------------------------------------------------------");
	/* ddf mo db reset */
	ret = clean_dm_mo();
	if (ret != DM_OK)
		goto error;

	/* engine status db reset */
	//ret = DB_Delete_Table();
	ret = DB_Reset_Table();
	if (ret != DM_OK)
		goto error;

	/* omadm db */
	sync_agent_empty_agent_default_table(SYNC_AGENT_DA_TABLE_NAME_CONFIG);
	//sync_agent_empty_all_agent_table()

	/*fumo, lawmo data in alarm db */
	_DEBUG_INFO("--------------------------------------------------------------------------------------------");

	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_EXTERN_FUNC_EXIT;
	_DEBUG_INFO(" end error : %d \n", ret);
	return ret;
}

DM_ERROR auto_operate_service_engine()
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	engine_status *status = NULL;

	if (IsExist_Engine_id(FUMO_SERVICE_ENGINE) != 0) {
		printf("engine id VALUE : %d ", FUMO_SERVICE_ENGINE);
		_DEBUG_INFO("engine id VALUE : %d ", FUMO_SERVICE_ENGINE);

		ret = Get_Engine_Status(FUMO_SERVICE_ENGINE, &status);
		if (ret != DM_OK || status == NULL)
			goto error;

		if (status->engine_status == 0) {
			ret = Delete_Engine_Status(FUMO_SERVICE_ENGINE);
			if (ret != DM_OK)
				goto error;
		} else {
			ret = fumo_serviceEngine_task_request();
			_DEBUG_INFO("fumo server engine task request : %d", ret);
		}
		if (status != NULL) {
			Free_Memory_Engine_Status(&status, 1);
		}
	}

	if (IsExist_Engine_id(LAWMO_SERVICE_ENGINE) != 0) {
		printf("engine id VALUE : %d ", LAWMO_SERVICE_ENGINE);
		_DEBUG_INFO("engine id VALUE : %d ", LAWMO_SERVICE_ENGINE);

		ret = Get_Engine_Status(LAWMO_SERVICE_ENGINE, &status);
		if (ret != DM_OK || status == NULL)
			goto error;

		if (status->engine_status == 0) {
			ret = Delete_Engine_Status(LAWMO_SERVICE_ENGINE);
			if (ret != DM_OK)
				goto error;
		} else {
			ret = lawmo_serviceEngine_task_request();
		}
		if (status != NULL)
			Free_Memory_Engine_Status(&status, 1);
	}

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	if (status != NULL)
		Free_Memory_Engine_Status(&status, 1);

	_EXTERN_FUNC_EXIT;
	return ret;
}

int register_wap_push()
{
	_EXTERN_FUNC_ENTER;

	int ret = 1;
	int i = 0;

	for (i = 0; i < MAX_TRY; ++i) {
		sync_agent_pm_return_e err_wappush = sync_agnet_register_user_callback(6, NULL, 2, dm_wap_push_incomming_callback, dm_wap_push_operation_callback);

		if (err_wappush != SYNC_AGENT_PM_SUCCESS) {
			_DEBUG_INFO("failed set_WAP-PUSH_Callback()");
			ret = -1;
			sleep(1);
		} else {
			_DEBUG_INFO("WAP-PUSH SUCCESS");
			break;
		}
	}
	_EXTERN_FUNC_EXIT;

	return ret;
}

int register_fumo_ip_push()
{
	_EXTERN_FUNC_ENTER;

	int ret = 1;
	int i = 0;

	for (i = 0; i < MAX_TRY; ++i) {
		sync_agent_pm_return_e err_ippush = sync_agnet_register_user_callback(2, NULL, 1, dm_ip_push_callback);
		if (err_ippush != SYNC_AGENT_PM_SUCCESS) {
//                      if (err_ippush == PMCI_CONNECTION_FAIL) {
//                              _DEBUG_INFO("failed set_IP-PUSH_Callback() : connection failed !!");
//
//                      } else if (err_ippush == PMCI_REGISTRATION_FAIL) {
//                              _DEBUG_INFO("failed set_IP-PUSH_Callback() : registration failed !!");
//                      } else {
			_DEBUG_INFO("failed set_IP-PUSH_Callback()");
//                      }

			sleep(1);
			ret = -1;
		} else {
			_DEBUG_INFO("IP-PUSH SUCCESS");
			ret = 1;
			break;
		}
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}

int register_lawmo_samsung_account()
{
	_EXTERN_FUNC_ENTER;

	int ret = 1;
	int i = 0;

	for (i = 0; i < MAX_TRY; ++i) {
		sync_agent_pm_return_e err_samsung_account = sync_agnet_register_user_callback(4, NULL, 2, register_lawmo_account, deregister_lawmo_account);
		if (err_samsung_account != SYNC_AGENT_PM_SUCCESS) {
//                      if (err_samsung_account == PMCI_CONNECTION_FAIL) {
//                              _DEBUG_INFO("failed set_SAMSUNG_ACCOUNT_Callback() : connection failed !!");
//
//                      } else if (err_samsung_account == PMCI_REGISTRATION_FAIL) {
//                              _DEBUG_INFO("failed set_SAMSUNG_ACCOUNT_Callback() : registration failed !!");
//                      } else {
			_DEBUG_INFO("failed set_SAMSUNG_ACCOUNT_Callback()");
//                      }

			sleep(1);
			ret = -1;
		} else {
			_DEBUG_INFO("SAMSUNG_ACCOUNT SUCCESS");
			ret = 1;
			break;
		}
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}

int register_telephony()
{
	_EXTERN_FUNC_ENTER;

	int ret = 1;
	int i = 0;

	for (i = 0; i < MAX_TRY; ++i) {
		sync_agent_pm_return_e err_telephony = sync_agnet_register_user_callback(5, NULL, 2, dm_telephony_incomming_callback, dm_telephony_end_callback);
		if (err_telephony != SYNC_AGENT_PM_SUCCESS) {
			_DEBUG_INFO("failed set_TELEPHONY_Callback()");

			ret = -1;
			sleep(1);
		} else {
			_DEBUG_INFO("TELEPHONY_SUCCESS");
			ret = 1;
			break;
		}
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}

int register_network()
{
	_EXTERN_FUNC_ENTER;

	int ret = 1;
	int i = 0;

	for (i = 0; i < MAX_TRY; ++i) {
		sync_agent_pm_return_e err_network = sync_agnet_register_user_callback(3, NULL, 2, network_on_callback, network_off_callback, network_change_callback);
		if (err_network != SYNC_AGENT_PM_SUCCESS) {
			_DEBUG_INFO("failed set_network_Callback()");
			ret = -1;
			sleep(1);
		} else {
			_DEBUG_INFO("NETWORK_SUCCESS");
			ret = 1;
			break;
		}
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}

int register_scheduler()
{
	_EXTERN_FUNC_ENTER;

	int ret = 0;
	int i = 0;

	for (i = 0; i < MAX_TRY; ++i) {
		sync_agent_pm_return_e err_alarm = sync_agnet_register_user_callback(1, NULL, 1, alarm_callback);
		if (err_alarm != SYNC_AGENT_PM_SUCCESS) {
			_DEBUG_INFO("Fail PMCI_Init_Specific_Type\n");
			ret = -1;
			sleep(1);
		} else {
			_DEBUG_INFO("Success PMCI_Init_Specific_Type\n");
			ret = 1;
			break;
		}
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}

void network_cancel_callback(void *user_data)
{
	_EXTERN_FUNC_ENTER;

	int *net_session_id = (int *)(user_data);
	sync_agent_na_result_e error;
	error = sync_agent_cancel_msg(1, (*net_session_id));
	_DEBUG_INFO("naci cancel msg status : %d\n", error);

	_EXTERN_FUNC_EXIT;
}

void network_connection_delay()
{
	_EXTERN_FUNC_ENTER;

	int network_status = 0;
	int interval = 1;	//(seconds)
	int retry_cnt = 3;

	char file_name[256] = { 0, };
	char *net_status;
	int snd_size;
	FILE *send_fp = NULL;
	_DEBUG_INFO("start");
	sleep(2);

	network_status = sync_agent_check_network_status(interval, retry_cnt);

	net_status = g_strdup_printf("%d", network_status);
	snprintf(file_name, 256, "%s", "/opt/data/network_connect_status.txt");
	send_fp = fopen(file_name, "w");
	if (send_fp != NULL) {
		snd_size = fwrite(net_status, strlen(net_status), 1, send_fp);
		fclose(send_fp);
	} else {
		_DEBUG_INFO("fopen - network_connect_status - fail");
	}

	if (network_status != 1) {
		_DEBUG_INFO("------------------------------------------NETOWKR CONNECT FAIL------------------------------------------");
	}

	_EXTERN_FUNC_EXIT;
}

DM_ERROR set_account_registration_alarm(char *alarm_str, CONFIG_TYPE alarm_type)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	sync_agent_da_return_e result = SYNC_AGENT_DA_SUCCESS;

	int count = 0;
	GList *list = NULL;

	result = sync_agent_get_config_list(alarm_type, &list);
	if (result != SYNC_AGENT_DA_SUCCESS) {
		_EXTERN_FUNC_EXIT;
		return COMMON_ERR_FW_CONFIG;
	}

	count = g_list_length(list);

	_DEBUG_INFO("alarm count: %d\n", count);
	if (count > 0 && list != NULL) {
		_EXTERN_FUNC_EXIT;
		return ret;
	} else {
		ret = add_alarm_item(INTERVAL_1DAY, alarm_str, alarm_type);
		if (ret != DM_OK)
			goto error;
	}

	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_EXTERN_FUNC_EXIT;
	return ret;
}

void delete_account_registration_alarm(char *alarm_str, CONFIG_TYPE alarm_type)
{
	_EXTERN_FUNC_ENTER;

	delete_alarm_item(alarm_str, alarm_type);

	_EXTERN_FUNC_EXIT;
}


void delete_polling_alarm_item(char *alarm_type, CONFIG_TYPE conf_type)
{
	_EXTERN_FUNC_ENTER;

	delete_alarm_item(alarm_type, conf_type);
	_EXTERN_FUNC_EXIT;

}

DM_ERROR add_alarm_item(Reminder_Interval interval, char *alarm_type, CONFIG_TYPE conf_type)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	//int week[] = {DAY_SUN, DAY_MON, DAY_TUE, DAY_WED, DAY_THU, DAY_FRI, DAY_SAT};

	time_t current_time;
	struct tm *struct_time;
	time(&current_time);
	struct_time = localtime(&current_time);

	pmci_alarm_s *alarm_info = (pmci_alarm_s *) calloc(1, sizeof(pmci_alarm_s));
	if (alarm_info == NULL) {
		_DEBUG_INFO("alloc fail");
		_EXTERN_FUNC_EXIT;
		return COMMON_ERR_ALLOC;
	}

	alarm_info->start_alarm_time = g_strdup_printf("%02d-%02d-%02dT%02d:%02d:%02dZ", struct_time->tm_year + 1900, struct_time->tm_mon + 1, struct_time->tm_mday, struct_time->tm_hour, struct_time->tm_min, struct_time->tm_sec);

	//REALE
	alarm_info->is_disposable = 0;

	// TEST
	//alarm_info->is_disposable = 1;

	switch (interval) {
		/*
		   case INTERVAL_NONE :
		   {
		   // 1minute
		   alarm_info->repeat_type = REPEAT ;
		   //alarm_info->repeat_value = ((1*60)*60);
		   alarm_info->repeat_value = ((1*60)*1);
		   }
		   break;
		 */
	case INTERVAL_1HOUR:
		{
			/* 1 hour */
			alarm_info->repeat_type = REPEAT;
			alarm_info->repeat_value = ((1 * 60) * 60);
			//alarm_info->repeat_value = ((1*60)*1);
		}
		break;
	case INTERVAL_3HOURS:
		{
			/* 4 hour */
			alarm_info->repeat_type = REPEAT;
			alarm_info->repeat_value = ((3 * 60) * 60);
		}
		break;
	case INTERVAL_6HOURS:
		{
			/* 6 hour */
			alarm_info->repeat_type = REPEAT;
			alarm_info->repeat_value = ((6 * 60) * 60);

		}
		break;
	case INTERVAL_12HOURS:
		{
			/* 12 hour */
			alarm_info->repeat_type = REPEAT;
			alarm_info->repeat_value = ((12 * 60) * 60);

		}
		break;
	case INTERVAL_1DAY:
		{
			/* 1 day */
			alarm_info->repeat_type = REPEAT;
			alarm_info->repeat_value = ((24 * 60) * 60);
		}
		break;
		/*
		   case INTERVAL_7DAYS:
		   {
		   // 1 week
		   alarm_info->repeat_type = REPEAT_WEEKLY ;
		   alarm_info->repeat_value = week[struct_time->tm_wday];
		   }
		   break;
		 */
	case INTERVAL_1MONTH:
		{
			alarm_info->repeat_type = REPEAT_MONTHLY;
			alarm_info->repeat_value = 0;
		}
	case INTERVAL_CANCEL:
		goto return_part;
		break;
	default:
		break;
	}

	_DEBUG_INFO("######### Alarm Info #####################\n");
	_DEBUG_INFO("time = %s\n", alarm_info->start_alarm_time);
	_DEBUG_INFO("repeat_mode = %d, repeat_value = %d\n", alarm_info->repeat_type, alarm_info->repeat_value);
	_DEBUG_INFO("is_disposable = %d\n", alarm_info->is_disposable);

/*	 if(conf_type == FUMO_INTERVAL_TYPE)
		 delete_alarm_item(FUMO_INTERVAL, FUMO_INTERVAL_TYPE);*/

	int alarm_id = 0;
	int alarm_ret = 0;
	sync_agent_pm_return_e err = SYNC_AGENT_PM_SUCCESS;

	/*add alarm service */
	err = sync_agent_add_service_data(1, alarm_info, &alarm_id);
	if (err != SYNC_AGENT_PM_SUCCESS) {
		_DEBUG_INFO("Fail PMCI_Add_Item_To_Specific_Type\n");
		ret = DM_ERR_FUMO_INTERNAL_SCHEDULER_ERROR;
	} else {
		/*add framework config db */
		alarm_ret = set_alarm_config_int(conf_type, alarm_type, alarm_id, alarm_type);
		if (alarm_ret == 0) {
			ret = DM_ERR_FUMO_INTERNAL_SCHEDULER_ERROR;
		}
	}

	if (alarm_info != NULL) {
		str_free(&(alarm_info->start_alarm_time));
		free(alarm_info);
		alarm_info = NULL;
	}

	_EXTERN_FUNC_EXIT;
	return ret;

 return_part:

	if (alarm_info != NULL) {
		str_free(&(alarm_info->start_alarm_time));
		free(alarm_info);
		alarm_info = NULL;
	}

	_EXTERN_FUNC_EXIT;
	_DEBUG_INFO("end error : %d ", ret);
	return ret;
}

void delete_alarm_item(char *alarm_type, CONFIG_TYPE conf_type)
{
	_EXTERN_FUNC_ENTER;

	int count = 0;
	int before_alarm_id;
	int ret = 0;
	sync_agent_da_return_e result = SYNC_AGENT_DA_SUCCESS;
	sync_agent_pm_return_e pm_result = SYNC_AGENT_PM_SUCCESS;

	sync_agent_da_config_s *config = NULL;

	GList *list = NULL;

	result = sync_agent_get_config_list(conf_type, &list);
	count = g_list_length(list);

	_DEBUG_INFO("alarm count: %d\n", count);
	if (count > 0 && list != 0) {
		config = (sync_agent_da_config_s *) g_list_nth_data(list, 0);
		_DEBUG_INFO("------------------------------------------alarm id : %s----------------------------------------------\n", config->value);
		ret = chartoint(config->value, &before_alarm_id);
		if (ret == 1) {
			pm_result = sync_agent_remove_service_data(1, before_alarm_id);
			_DEBUG_INFO("delete alarm %d", result);

			if (pm_result == SYNC_AGENT_PM_SUCCESS) {
				result = sync_agent_delete_config(conf_type, alarm_type);
				_DEBUG_INFO("delete alarm config %d", result);
			}
		} else {
			_DEBUG_INFO("fail get alarm id");
		}
	}

	sync_agent_free_config_list(list);
	_EXTERN_FUNC_EXIT;

}

SERVICE_SERVER_TYPE get_service_engine_type(const char *server_id)
{
	_EXTERN_FUNC_ENTER;

	retvm_if(server_id == NULL, NO_ENGINE_TYPE, "server_id is NULL!!");

	_DEBUG_INFO("get- service engine type by server id : %s", server_id);
	_EXTERN_FUNC_EXIT;
	return get_engine_type_by_serverid(server_id);
}

DM_ERROR get_server_info(Event_Contents * pEvent_data, char **server_id, char **session_id, int *session_type)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((pEvent_data) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "pEvent_data is NULL!!");
	retvm_if((pEvent_data->server_id) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "server_id is NULL!!");

	(*server_id) = strdup(pEvent_data->server_id);
	_DEBUG_INFO(" server id : %s\n", *server_id);

	if (pEvent_data->session_id != NULL) {
		(*session_id) = strdup(pEvent_data->session_id);
		_DEBUG_INFO(" session_id : %s\n", *session_id);
	}

	(*session_type) = pEvent_data->type;
	_DEBUG_INFO("session type  : %d\n", *session_type);

	_EXTERN_FUNC_EXIT;
	return ret;
}

void get_service_engine_id(ENGINE_ID * service_engine_id, char *ui_mode)
{
	_EXTERN_FUNC_ENTER;

	if (ui_mode == NULL)
		return;

	if (!strcmp(ui_mode, OMADM_FUMO_UI_TYPE) || !strcmp(ui_mode, OMADM_FUMO_NOTI_UI_TYPE)) {
		*service_engine_id = FUMO_SERVICE_ENGINE;
		_DEBUG_INFO("fumo service");
	} else {
		_DEBUG_INFO("other service");
	}

	_EXTERN_FUNC_EXIT;
}

DM_ERROR fumo_service_start(Event_Contents * pEvent_data)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((pEvent_data) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "pEvent_data is NULL!!");

	switch (pEvent_data->noti_type) {
	case NOTI_TYPE_NOT_SPECIFIED:
		_DEBUG_INFO("NOTI_TYPE_NOT_SPECIFIED");
		//temp version for gcf test
		pEvent_data->ui_mode = strdup(OMADM_FUMO_BACKGROUND_UI_TYPE);
		pEvent_data->noti_type = NOTI_TYPE_BACKGRUOUND;
		ret = dm_fumo_common_task_request(pEvent_data);
		_DEBUG_INFO("NOTI NOTI_TYPE_BACKGRUOUND : %d", ret);
		if (ret != DM_OK) {
			goto error;
		}
		break;
	case NOTI_TYPE_BACKGRUOUND:
		pEvent_data->ui_mode = strdup(OMADM_FUMO_BACKGROUND_UI_TYPE);
		ret = dm_fumo_common_task_request(pEvent_data);
		_DEBUG_INFO("NOTI NOTI_TYPE_BACKGRUOUND : %d", ret);
		if (ret != DM_OK) {
			goto error;
		}
		break;
	case NOTI_TYPE_INFOMATIVE:
		pEvent_data->ui_mode = strdup(OMADM_FUMO_BACKGROUND_UI_TYPE);
		ret = dm_fumo_common_task_request(pEvent_data);
		_DEBUG_INFO("NOTI NOTI_TYPE_INFOMATIVE : %d", ret);
		if (ret != DM_OK) {
			goto error;
		}
		break;
	case NOTI_TYPE_USERINTERACTION:
		pEvent_data->ui_mode = strdup(OMADM_FUMO_NOTI_UI_TYPE);
		ret = dm_fumo_ui_task_request(pEvent_data);
		_DEBUG_INFO("NOTI NOTI_TYPE_USERINTERACTION : %d ", ret);
		if (ret != DM_OK) {
			goto error;
		}
		break;
	default:
		break;
	}

	_EXTERN_FUNC_EXIT;
	return DM_OK;
 error:
	event_data_free((void *)pEvent_data);
	_EXTERN_FUNC_EXIT;
	return DM_OK;

}

DM_ERROR get_fumo_ui_mode(char **ui_mode)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	engine_status *status = NULL;
	ret = Get_Engine_Status(FUMO_SERVICE_ENGINE, &status);
	if (ret != DM_OK) {
		if(status != NULL)
			Free_Memory_Engine_Status(&status, 1);

		_EXTERN_FUNC_EXIT;
		return ret;
	}

	if (status != NULL) {

		if (status->ui_mode != NULL) {
			(*ui_mode) = strdup(status->ui_mode);

			_DEBUG_INFO("==================Fumo ui : %s==================\n", *ui_mode);
		} else {
			_DEBUG_INFO("ui mode null");
		}

		Free_Memory_Engine_Status(&status, 1);

	} else {
		_DEBUG_INFO("status is null");
	}

	_DEBUG_INFO("end");
	_EXTERN_FUNC_EXIT;
	return ret;

}

int check_existed_fumo_reminder_interval()
{
	_EXTERN_FUNC_ENTER;

	int exist = 0;
	sync_agent_da_return_e result = SYNC_AGENT_DA_SUCCESS;
	result = sync_agent_is_exist_config(FUMO_INTERVAL_TYPE, FUMO_INTERVAL, &exist);
	_DEBUG_INFO("agent is exist result : %d", result);

	_EXTERN_FUNC_EXIT;
	return exist;
}

int init_default_fumo_config()
{
	_EXTERN_FUNC_ENTER;

	_EXTERN_FUNC_EXIT;
	return set_config_int(FUMO_WIFI_ONLY_TYPE, FUMO_WIFI_ONLY_CONFIG, 0, FUMO, 1);
}

int set_config_int(CONFIG_TYPE config_type, char *key, int value, char *accessName, int isFirst)
{
	_EXTERN_FUNC_ENTER;
	int ret = 1;
	sync_agent_da_return_e result;
	sync_agent_da_config_s config;
	int exist = 0;

	config.access_name = accessName;
	config.config_id = config_type;
	config.key = key;
	config.value = g_strdup_printf("%d", value);
	config.type = "int";

	result = sync_agent_is_exist_config(config_type, key, &exist);
	if (result != SYNC_AGENT_DA_SUCCESS) {
		_DEBUG_INFO("exist config fail : %d", result);
		ret = -1;
	} else {
		if (exist == 0 && isFirst == 1) {
			_DEBUG_INFO("default add config : %s\n", key);
			result = sync_agent_add_config(&config);
			if (result != SYNC_AGENT_DA_SUCCESS) {
				_DEBUG_INFO("fail add : %d", result);
				ret = -1;
			} else {
				_DEBUG_INFO("Success add config : %s, : %d\n", key, value);
			}
		} else {
			result = sync_agent_update_config(&config);
			//free config
			if (result != SYNC_AGENT_DA_SUCCESS) {
				_DEBUG_INFO("fail update : %d", result);
				ret = -1;
			} else {
				_DEBUG_INFO("Success update config : %s, : %d\n", key, value);
			}
		}
	}

	str_free(&(config.value));
	_EXTERN_FUNC_EXIT;

	return ret;
}

int get_config_int(CONFIG_TYPE config_id, char *key)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_da_return_e result = SYNC_AGENT_DA_SUCCESS;
	sync_agent_da_config_s *config = NULL;
	int value = 1;
	int ret = -1;

	result = sync_agent_create_config(&config);
	if (result != SYNC_AGENT_DA_SUCCESS) {
		_EXTERN_FUNC_EXIT;
		return -1;
	}

	result = sync_agent_get_config(config_id, key, &config);
	_DEBUG_INFO("---------------------------result : %d-------------------------\n", result);
	if (result == 1) {
		_DEBUG_INFO("Success get config value");

		ret = chartoint(config->value, &value);
		_DEBUG_INFO("result : %d , key : %s value : %d\n", result, key, value);

		if (ret == 0) {
			_DEBUG_INFO("fail get config value");
			ret = -1;
			goto return_part;
		} else {
			_DEBUG_INFO("success get config value");
			ret = value;
		}
	} else {
		_DEBUG_INFO("fail get config value");
	}

 return_part:

	sync_agent_free_config(config);
	_EXTERN_FUNC_EXIT;
	return ret;
}

int set_alarm_config_int(CONFIG_TYPE config_type, char *key, int value, char *accessName)
{
	_EXTERN_FUNC_ENTER;

	int ret = 0;
	sync_agent_da_return_e result;
	sync_agent_da_config_s config;
	int exist = 0;

	config.access_name = accessName;
	config.config_id = config_type;
	config.key = key;
	config.value = g_strdup_printf("%d", value);
	config.type = "int";

	result = sync_agent_is_exist_config(config_type, key, &exist);
	if (result != SYNC_AGENT_DA_SUCCESS)
		ret = 0;

	if (exist == 0) {
		_DEBUG_INFO("default add config : %s\n", key);
		result = sync_agent_add_config(&config);

		if (result != SYNC_AGENT_DA_SUCCESS)
			ret = 0;
	} else {
		result = sync_agent_update_config(&config);
		_DEBUG_INFO("update alarm config db ");
	}

	ret = 1;
	str_free(&(config.value));
	_DEBUG_INFO("end : %d", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

int get_wifi_only_config()
{
	_EXTERN_FUNC_ENTER;

	_EXTERN_FUNC_EXIT;
	return get_config_int(FUMO_WIFI_ONLY_TYPE, FUMO_WIFI_ONLY_CONFIG);
}

int set_wifi_only_config(int value)
{
	_EXTERN_FUNC_ENTER;

	_EXTERN_FUNC_EXIT;
	return set_config_int(FUMO_WIFI_ONLY_TYPE, FUMO_WIFI_ONLY_CONFIG, value, FUMO, 0);
}

DM_ERROR get_battery_state(char **battery_level)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	sync_agent_dev_return_e err = SYNC_AGENT_DEV_RETURN_SUCCESS;

	err = sync_agent_get_devinfo(2, "Battery", battery_level);
	if (err != SYNC_AGENT_DEV_RETURN_SUCCESS) {
		goto error;
	}
	if ((*battery_level) == NULL) {
		goto error;
	}

	_DEBUG_INFO("\n========battery ======== %s\n", *battery_level);
	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_EXTERN_FUNC_EXIT;
	return DM_GET_BATTERY_ERROR;
}

DM_ERROR get_telephony_state(int *now_calling)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

/*
 * vconf( framework )
 */
	int tel_value;
	if (vconf_get_int(VCONFKEY_CALL_STATE, &tel_value) != 0)
		goto error;

	switch (tel_value) {
	case VCONFKEY_CALL_VOICE_CONNECTING:
	case VCONFKEY_CALL_VOICE_ACTIVE:
	case VCONFKEY_CALL_VIDEO_CONNECTING:
	case VCONFKEY_CALL_VIDEO_ACTIVE:
		(*now_calling) = 1;
		break;
	case VCONFKEY_CALL_OFF:
		(*now_calling) = 0;
		break;
	default:
		break;
	}

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_EXTERN_FUNC_EXIT;
	return DM_TELEPHONY_ERROR;

}

void network_state_callback()
{
	_EXTERN_FUNC_ENTER;

	/*typedef enum {
	   NETWORK_NONE,                                                                         initial network type
	   NETWORK_3G_ON,                                                                network UMTS on
	   NETWORK_3G_OFF,                                                               network UMTS off
	   NETWORK_WIFI_ON,                                                              network WIFI on
	   NETWORK_WIFI_ON_NOT_CONNECTED,        network WIFI on but not connected
	   NETWORK_WIFI_OFF,                                                             network WIFI off
	   NETWORK_WIFI_3G_ON,                                                   network UMTS and WIFI on
	   } FW_NETWORK_TYPE;
	   #endif
	 */

	/*sync_agent_na_result_e err;
	   int net_type;
	   err = NACI_Get_Network_Type(&net_type);
	   if(err != SYNC_AGENT_NA_SUCCESS)
	   goto error;

	   switch(net_type)
	   {
	   case 1:
	   case 3:

	   * search db
	   * generic alert
	   * fumo (not alarm)
	   * lawmo( not alarm)
	   * deregister vconf callback

	   break;
	   case 0:
	   case 2:
	   case 5:
	   case 6:
	   break;
	   default:
	   break;
	   }

	   error: */
	_EXTERN_FUNC_EXIT;
	return;
}

DM_ERROR existed_sd_card_memory(int *existed_sd)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
/*
 * vconf( framework )
 * //only one sd card
 */
	int mmc_value = 0;
	int err = 0;

	err = get_mmc_state(&mmc_value);
	if (err == 0) {
		(*existed_sd) = 0;
		goto error;
	} else {
		if (mmc_value == 1) {
			(*existed_sd) = 1;

			/*for make empty folder */
			char *downlaod_folder = NULL;
			char *save_folder = NULL;
			ret = get_fota_download_dir(MEMORY_SD_CARD, &downlaod_folder);
			if (ret != DM_OK) {
				goto error;
			}
			ret = get_fota_save_dir(MEMORY_SD_CARD, &save_folder);
			if (ret != DM_OK) {
				goto error;
			}

			str_free(&downlaod_folder);
			str_free(&save_folder);
		} else {
			(*existed_sd) = 0;
		}
	}

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	_EXTERN_FUNC_EXIT;
	return DM_SD_CARD_ERROR;
}

void get_roaming_state(int *roaming_state)
{
	_EXTERN_FUNC_ENTER;

	int roaming_config = 0;

	if (vconf_get_int(VCONFKEY_TELEPHONY_SVC_ROAM, roaming_state) != 0) {
		*roaming_state = -1;
		_DEBUG_INFO("end , roaming state : %d", *roaming_state);
	} else {
		if (*roaming_state == 0) {
			/*not roaming svc area */
			_DEBUG_INFO("end , roaming state : %d", *roaming_state);
			_EXTERN_FUNC_EXIT;
			return;
		} else {
			/*roaming svc area */
			if (vconf_get_bool(VCONFKEY_SETAPPL_STATE_DATA_ROAMING_BOOL, &roaming_config) != 0) {
				*roaming_state = -1;
				_DEBUG_INFO("end , roaming state : %d", *roaming_state);
			} else {
				if (roaming_config == 1) {
					/*
					 * do service roaming service
					 *      check wifi
					 */
					_DEBUG_INFO("end , roaming state : %d", *roaming_state);
					return;
				} else {
					/*don not service at roaming svc */
					*roaming_state = -1;
					_DEBUG_INFO("end , roaming state : %d", *roaming_state);
				}
			}
		}
	}
	_EXTERN_FUNC_EXIT;
}

void get_wifi_connected_state(int *wifi_state)
{
	_EXTERN_FUNC_ENTER;

	int wifi_connection = VCONFKEY_WIFI_OFF;
	int nRes_wifistatus = vconf_get_int(VCONFKEY_WIFI_STATE, &wifi_connection);
	*wifi_state = -1;

	if (nRes_wifistatus != 0) {
		_DEBUG_INFO("vconf_get_int get wificonnection fail\n");
		*wifi_state = -1;
		_EXTERN_FUNC_EXIT;
		return;
	}

	if(wifi_connection >= VCONFKEY_WIFI_CONNECTED) {
		_DEBUG_INFO("vconf_get_int get wificonnection success\n");
		*wifi_state = 0;
		_EXTERN_FUNC_EXIT;
		return;
	}

	_EXTERN_FUNC_EXIT;
}

void get_call_sate(int *call_state)
{
	_EXTERN_FUNC_ENTER;
	/*0 : do service
	 * 1 : dont service
	 * 2 : ?
	 */
	if (vconf_get_int(VCONFKEY_TELEPHONY_CALL_STATE, call_state) != 0) {
		*call_state = -1;
		_DEBUG_INFO("end, call state : %d", *call_state);
		_EXTERN_FUNC_EXIT;
		return;
	} else {
		if (*call_state == 0) {
			_DEBUG_INFO("end, call state : %d", *call_state);
			_EXTERN_FUNC_EXIT;
			return;
		} else {
			*call_state = -1;
			_DEBUG_INFO("end, call state : %d", *call_state);
			_EXTERN_FUNC_EXIT;
			return;
		}
	}
}

int get_mmc_state(int *mmc_state)
{
	_EXTERN_FUNC_ENTER;

	if (vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, mmc_state) != 0)
		goto error;

	_EXTERN_FUNC_EXIT;
	return 1;
 error:
	_EXTERN_FUNC_EXIT;
	_DEBUG_INFO("end error");
	return 0;
}

DM_ERROR compare_memory_space(char *memory_type, long double file_size)
{
	_EXTERN_FUNC_ENTER;

	int err;
	long double sizeFree;
	long double sizeTotal;

	retvm_if((memory_type) == NULL, COMMON_ERR_IS_NULL, "memory_type is NULL!!");

	err = sync_agent_get_fs_mem_size(memory_type, &sizeFree, &sizeTotal);
	if (err == 0) {
		_EXTERN_FUNC_EXIT;
		return DM_MEMORY_ERROR;
	}

	_DEBUG_INFO("need free size [%f], memory free size [%f], total size [%f] ", file_size, sizeFree, sizeTotal);
	if (file_size > sizeFree) {
		_EXTERN_FUNC_EXIT;
		return DM_OVER_MEMORY_ERROR;
	}

	//_DEBUG_INFO("below return value is test code"); // for memory not enough - testing
	//return DM_OVER_MEMORY_ERROR; // for memory not enough - testing
	_EXTERN_FUNC_EXIT;

	return DM_OK;
}

DM_ERROR get_fota_download_dir(MEMORY_TYPE type, char **download_folder)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_dev_return_e dci_down_ret = SYNC_AGENT_DEV_RETURN_SUCCESS;

	switch (type) {
	case MEMORY_INTERNAL:
		_DEBUG_INFO("internal memory");
		dci_down_ret = sync_agent_get_devinfo(2, "FOTADownDir", download_folder);
		break;
	case MEMORY_SD_CARD:
		_DEBUG_INFO("sd card memory");
		dci_down_ret = sync_agent_get_devinfo(2, "FOTA_SD_DownDir", download_folder);
		break;
	default:
		_DEBUG_INFO("ERROR NOT USE MEMORY");
		break;
	}

	if (dci_down_ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
		_DEBUG_INFO(" download : %s", *download_folder);

		_EXTERN_FUNC_EXIT;
		return DM_OK;
	}
	_DEBUG_INFO("end error : %d ", DM_MEMORY_ERROR);
	_EXTERN_FUNC_EXIT;
	return DM_MEMORY_ERROR;
}

DM_ERROR get_fota_save_dir(MEMORY_TYPE type, char **save_folder)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_dev_return_e dci_down_ret = SYNC_AGENT_DEV_RETURN_SUCCESS;

	switch (type) {
	case MEMORY_INTERNAL:
		dci_down_ret = sync_agent_get_devinfo(2, "FOTASaveDir", save_folder);
		break;
	case MEMORY_SD_CARD:
		dci_down_ret = sync_agent_get_devinfo(2, "FOTA_SD_SaveDir", save_folder);
		break;
	default:
		_DEBUG_INFO("ERROR NOT USE MEMORY");
		break;
	}

	if (dci_down_ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
		_DEBUG_INFO(" save_folder : %s", *save_folder);

		_EXTERN_FUNC_EXIT;
		return DM_OK;
	}
	_DEBUG_INFO("end error : %d ", DM_MEMORY_ERROR);
	_EXTERN_FUNC_EXIT;
	return DM_MEMORY_ERROR;
}

void delete_fumo_contents(MEMORY_TYPE memory_type)
{
	_EXTERN_FUNC_ENTER;

	_DEBUG_INFO("----------------------------------------------------------firmware update fail delete files-----------------------------------------------------------");
	char *download_folder = NULL;
	char *save_folder = NULL;

	//FOTASaveDir //FOTADownDir
	//dci_save_ret = sync_agent_get_devinfo(2 , "FOTADownDir" , &download_folder);
	get_fota_download_dir(memory_type, &download_folder);
	sync_agent_empty_directory_contents(download_folder);
	//dci_save_ret = sync_agent_get_devinfo(2 , "FOTASaveDir" , &save_folder);
	get_fota_save_dir(memory_type, &save_folder);

	sync_agent_empty_directory_contents(save_folder);
	_DEBUG_INFO("delete save folder ");

	str_free(&save_folder);
	str_free(&download_folder);

	_EXTERN_FUNC_EXIT;

}

void first_server_bootstrap(char *server_id)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	if (server_id == NULL) {
		_DEBUG_INFO("server id null");
		return;
	}

	Event_Contents *ev_data = (Event_Contents *) calloc(1, sizeof(Event_Contents) + 1);
	if (ev_data == NULL) {
		_DEBUG_INFO("alloc fail");
		_EXTERN_FUNC_EXIT;
		return;
	}
	ev_data->server_id = strdup(server_id);
	ev_data->session_id = NULL;
	ev_data->type = SERVER_BOOTSTRAP_EVENT;
	ev_data->ui_mode = NULL;

	_DEBUG_INFO("event type server bootstrap : %d", ev_data->type);
	_DEBUG_INFO("session id : %s", ev_data->session_id);
	_DEBUG_INFO("server id : %s", ev_data->server_id);

	/* UI CONFIRM */
	SERVICE_SERVER_TYPE engine_type;
	engine_type = get_service_engine_type(ev_data->server_id);

	switch (engine_type) {
	case SAMSUNG_FUMO_TYPE:
	case GCF_TYPE:
		{
			//when using telephony
			//DM_ERROR get_telephony_state(int *now_calling);
			_DEBUG_INFO("engine type : %d", engine_type);
			ev_data->noti_type = NOTI_TYPE_BACKGRUOUND;
			ev_data->ui_mode = strdup(OMADM_FUMO_BACKGROUND_UI_TYPE);

			_DEBUG_INFO("noti type : %d", ev_data->noti_type);
			_DEBUG_INFO("ui mode : %s", ev_data->ui_mode);

			ret = dm_fumo_common_task_request(ev_data);
			_DEBUG_INFO("dm fumo conmmon task : %d ", ret);
			if (ret != DM_OK) {
				goto error;
			}
		}
		break;
	case SAMSUNG_FMM_TYPE:

		_DEBUG_INFO("engine type : %d", engine_type);
		ret = dm_lawmo_common_task_request(ev_data);
		_DEBUG_INFO("dm lawmo conmmon task : %d ", ret);
		if (ret != DM_OK) {
			goto error;
		}
		break;
	default:
		_DEBUG_INFO("dm non engine ", ret);
		break;
	}

	_EXTERN_FUNC_EXIT;
	return;

 error:
	event_data_free((void *)ev_data);
	_EXTERN_FUNC_EXIT;
}

int get_default_noti_type(char *ui_mode, NotI_Type noti_type)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((ui_mode) == NULL, COMMON_ERR_IS_NULL, "ui_mode is NULL!!");

	if (strcmp(OMADM_FUMO_UI_TYPE, ui_mode) == 0) {
		_EXTERN_FUNC_EXIT;
		return NOTI_UNKNOWN;	//none type
	} else if (strcmp(OMADM_FUMO_NOTI_UI_TYPE, ui_mode) == 0) {
		switch (noti_type) {
		case NOTI_TYPE_NOT_SPECIFIED:
		case NOTI_TYPE_BACKGRUOUND:
		case NOTI_TYPE_USERINTERACTION:
			_EXTERN_FUNC_EXIT;
			return NOTI_DOWNLOAD;	//none type
			break;
		case NOTI_TYPE_INFOMATIVE:
			_EXTERN_FUNC_EXIT;
			return NOTI_DOWNLOAD;	//connect to server
			break;
		case NOTI_TYPE_POLLINTERACTION:
			_EXTERN_FUNC_EXIT;
			return NOTI_POLL_INTERACTION;	//connect to server
			break;
		default:
			return NOTI_UNKNOWN;	//none type
			break;
		}
	} else if (strcmp(OMADM_FUMO_BACKGROUND_UI_TYPE, ui_mode) == 0) {
		_EXTERN_FUNC_EXIT;
		return NOTI_UNKNOWN;	//none type
	}

	_EXTERN_FUNC_EXIT;
	return NOTI_UNKNOWN;
}

int wifi_only_config(char *config)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((config) == NULL, COMMON_ERR_IS_NULL, "config is NULL!!");

	if (config != NULL) {
		if (strcmp(config, "1") == 0) {
			_DEBUG_INFO("wifi on config");
			_EXTERN_FUNC_EXIT;
			return 1;
		} else if (strcmp(config, "0") == 0) {
			_DEBUG_INFO("wifi off config");
			_EXTERN_FUNC_EXIT;
			return 0;
		} else {
			_DEBUG_INFO("wifi config error");
		}
	} else {
		_DEBUG_INFO("wifi config error");
	}

	_EXTERN_FUNC_EXIT;

	return 0;
}
