/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*lib*/
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <unistd.h>
#include <glib.h>
#include <glib/gprintf.h>

/*sync-agent*/
#include <sync_agent.h>
#include <plugin/plugin_slp_sysnoti_wap_push.h>

/*dm-agent*/
#include "common/dm_common_def.h"
#include "common/util/util.h"
#include "mo-handler/dm_mo_common_internal.h"
#include "mo-handler/dm_mo_common.h"
#include "dm-engine/cp/dm_cp_parser.h"
#include "dm-engine/cp/dm_cp_security.h"
#include "dm-engine/cp/dm_cp_processor.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"OMA_DM_CP"
#endif

#define IMSI_VALUE	"310260487447094"

#define MSG_SIZE	300

#define W2	"Browser"
#define W4	"MMS"
#define W5	"SyncML DS"
#define W7	"SyncML DM"
#define WA	"IM & PS"

#define DELIMITER			" : "
#define L_BRACKET		" ["
#define R_BRACKET		"] "
#define LB						"\n"
#define APPLICATION		"Application : "
#define URL					"URL : "
#define PROXY				"Proxy : "
#define PORT					"Port : "
#define BOOKMARK		"Bookmark : "	/* not used */
#define NO_DATA			"No data "
#define DEFAULT_NAP_TYPE	"APN : "

static int g_msgId;

typedef struct {
	int ext_id;
	DM_CP *dm_cp;
} CP_Data;

static GList *dm_cp_list = NULL;

static CP_ERROR _process_receieved_cp_stream(CP_TYPE cp_type, const char *msgBody, unsigned int msgSize, int ext_id);
static CP_ERROR __send_msg2mf(SENDMSG_TYPE send_type, void *send_msg);
static CP_ERROR _process_setting_oma_ds(DM_CP * dm_cp);
static CP_ERROR _process_setting_oma_dm(DM_CP * dm_cp);

static char __char2hex(char ch);
static char *_get_netpin_key(int *key_len);
static int __make_view_message(char **msgData, DM_CP * dm_cp);

CP_ERROR process_Recieved_CP_WBXML(const char *msgBody, unsigned int msgSize, int ext_id)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((msgBody) == NULL, CP_ERROR_FAIL, "msgBody is NULL!!");

	CP_ERROR ret = CP_ERROR_SUCCESS;

	unsigned int wsp_header_size = (unsigned char)msgBody[2];
	wsp_header_size += 3;
	_DEBUG_INFO("wsp_header_size : %u", wsp_header_size);

	unsigned int wbxml_size = msgSize - wsp_header_size;
	_DEBUG_INFO("msgSize : %d, wbxml_size : %u", msgSize, wbxml_size);

	/*
	 * Register PIN CODE to File
	 */
	char file_buffer[50] = { 0, };
	snprintf(file_buffer, sizeof(file_buffer), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_SEC_FILE_NAME);
	_DEBUG_INFO("file name : %s", file_buffer);

//      int file_ret = sync_agent_write_file(CP_SEC_REPOSITORY_PATH, msgBody, wsp_header_size, 1, 1);
//      int file_ret = sync_agent_write_whole_file(CP_SEC_REPOSITORY_PATH, msgBody, wsp_header_size, 1);
	int file_ret = sync_agent_write_whole_file(file_buffer, msgBody, wsp_header_size, 1);
	if (file_ret != 1) {
		_DEBUG_INFO("Failed to repository CP PIN Info");
	}

	/*
	 * Division CP WBXML to WSP Header
	 */
	char *wbxml_body = (char *)calloc(wbxml_size, sizeof(char));
	if (wbxml_body == NULL) {
		_DEBUG_INFO("alloc fail");
		return 0;
	}
	memcpy(wbxml_body, msgBody + wsp_header_size, wbxml_size);

	ret = _process_receieved_cp_stream(CP_TYPE_WBXML, wbxml_body, wbxml_size, ext_id);
	_DEBUG_INFO("precess receive result : %d", ret);

	free(wbxml_body);
	wbxml_body = NULL;

	return ret;
}

CP_ERROR process_Recieved_CP_XML(const char *msgBody, unsigned int msgSize, int ext_id)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((msgBody) == NULL, CP_ERROR_FAIL, "msgBody is NULL!!");

	_EXTERN_FUNC_EXIT;
	return _process_receieved_cp_stream(CP_TYPE_XML, msgBody, msgSize, ext_id);
}

CP_ERROR process_Check_Pincode(int is_initial, char *user_data, int *is_correct, int ext_id)
{
	_EXTERN_FUNC_ENTER;

	CP_ERROR ret = CP_ERROR_SUCCESS;

	/*
	 * 1. get CP header info in repository(file)
	 */
	/*unsigned long required_length = 50;
	   bool isFinal = 0; */
	unsigned long size_return = 0;
	char *cp_pin_info = 0;
	char *wbxml_body = 0;
	unsigned long wbxml_size = 0;
	char *key = 0;
	int key_len = 0;
	int i = 0;

	char sec_file_name[50] = { 0, };
	snprintf(sec_file_name, sizeof(sec_file_name), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_SEC_FILE_NAME);
	_DEBUG_INFO("sec file name : %s", sec_file_name);

//      int read_ret = sync_agent_read_file(CP_SEC_REPOSITORY_PATH, &cp_pin_info, required_length, &size_return, &isFinal);
//      int read_ret = sync_agent_read_whole_file(CP_SEC_REPOSITORY_PATH, &cp_pin_info, &size_return);
	int read_ret = sync_agent_read_whole_file(sec_file_name, &cp_pin_info, &size_return);
	if (read_ret == 0) {
		_DEBUG_INFO("Failed to sync_agent_read_file( CP_SEC_REPOSITORY_PATH ) !!");
		return CP_ERROR_FAIL;
	}

	char wbxml_file_name[50] = { 0, };
	snprintf(wbxml_file_name, sizeof(wbxml_file_name), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_WBXML_FILE_NAME);
	_DEBUG_INFO("wbxml file name : %s", wbxml_file_name);
//      read_ret = sync_agent_read_whole_file(CP_WBXML_REPOSITORY_PATH, &wbxml_body, &wbxml_size);
	read_ret = sync_agent_read_whole_file(wbxml_file_name, &wbxml_body, &wbxml_size);
	if (read_ret == 0) {
		_DEBUG_INFO("Failed to sync_agent_read_file ( CP_WBXML_REPOSITORY_PATH ) !!");
		return CP_ERROR_FAIL;
	}

	/*
	 * Check SEC TYPE
	 */
	char sec = cp_pin_info[6];
	char sec_type = 0;
	if (sec == CP_SEC) {
		sec_type = cp_pin_info[7];
	}
	_DEBUG_INFO("SEC : %u, %x, SEC_TYPE : %u, %x, %c", sec, sec, sec_type, sec_type, sec_type);

	/*
	 * Check Is Exist MAC in wbxml stream
	 */
	int is_exist_mac = 0;
	if (cp_pin_info[8] == CP_IS_EXIST_MAC) {
		is_exist_mac = 1;
	}

	char mac[CP_MAC_LEN] = { 0, };
	if (is_exist_mac != 0) {
		i = 0;
		for (; i < CP_MAC_LEN; i++) {
			mac[i] = cp_pin_info[9 + i];
		}
	}

	switch ((unsigned char)sec_type) {
	case CP_SEC_TYPE_NETWPIN:
		{
			_DEBUG_INFO("SEC TYPE : NETWPIN");

			char *temp = 0;
			temp = _get_netpin_key(&key_len);
			if (temp == NULL) {
				_DEBUG_INFO("__get_NetPIN_key() failed !!");
				ret = CP_ERROR_FAIL;
				goto return_part;
			}

			key = (char *)calloc(key_len, sizeof(char));
			if (key == NULL) {
				_DEBUG_INFO("calloc failed !!");

				if (temp != NULL)
					free(temp);
				ret = CP_ERROR_FAIL;
				goto return_part;
			}

			memcpy(key, temp, key_len);

			for (i = 0; i < key_len; i++) {
				_DEBUG_INFO("key[%d] : 0x%02X", i, key[i]);
			}

			if (temp != NULL)
				free(temp);
		}
		break;
	case CP_SEC_TYPE_USERPIN:
		{
			_DEBUG_INFO("SEC TYPE : USERPIN");

			int res = 0;
			if (is_initial != 0) {
				res = launch_oma_dm_cp_ui(1, ext_id);
				if (res < 0) {
					_DEBUG_INFO("launch_oma_dm_cp_ui() fail !!");
					ret = CP_ERROR_FAIL;
				} else {
					_DEBUG_INFO("launch_oma_dm_cp_ui() success !!");
				}

				goto return_part;
			} else {
				if (user_data != NULL) {
					key_len = strlen(user_data);

					key = (char *)calloc(key_len, sizeof(char));
					if (key == NULL) {
						_DEBUG_INFO("calloc failed !!");
						ret = CP_ERROR_FAIL;
						goto return_part;
					}

					strncpy(key, user_data, key_len);
					_DEBUG_INFO("key : %s ( %d )", user_data, key_len);
				} else {
					_DEBUG_INFO("launch_oma_dm_cp_ui() fail !!");
					ret = CP_ERROR_FAIL;
					goto return_part;
				}
			}
		}
		break;
	case CP_SEC_TYPE_USERNETWPIN:
		{
			_DEBUG_INFO("SEC TYPE : USERNETWPIN");

			int res = 0;
			if (is_initial != 0) {
				res = launch_oma_dm_cp_ui(2, ext_id);
				if (res < 0) {
					_DEBUG_INFO("launch_oma_dm_cp_ui() fail !!");
					ret = CP_ERROR_FAIL;
				} else {
					_DEBUG_INFO("launch_oma_dm_cp_ui() success !!");
				}

				goto return_part;
			} else {
				/* USERNETWPIN checking logic */
				int net_temp_len = 0;
				char *net_temp = 0;
				net_temp = _get_netpin_key(&net_temp_len);
				if (net_temp == NULL) {
					ret = CP_ERROR_FAIL;
					_DEBUG_INFO("__get_NetPIN_key() failed !!");
					goto return_part;
				}

				for (i = 0; i < net_temp_len; i++) {
					_DEBUG_INFO("net_temp[%d] : 0x%02X", i, net_temp[i]);
				}

				int user_data_len = strlen(user_data);
				char *temp_user_data = (char *)calloc(user_data_len, sizeof(char));
				if (temp_user_data == NULL) {
					_DEBUG_INFO("calloc failed !!");

					if (net_temp != NULL)
						free(net_temp);
					ret = CP_ERROR_FAIL;
					goto return_part;
				}

				strncpy(temp_user_data, user_data, user_data_len);
				_DEBUG_INFO("temp user data : %s", temp_user_data);

				key_len = net_temp_len + user_data_len;
				_DEBUG_INFO("key length : %d", key_len);

				key = (char *)calloc(key_len, sizeof(char));
				if (key == NULL) {
					_DEBUG_INFO("calloc failed !!");
					ret = CP_ERROR_FAIL;
					goto return_part;
				}
				memcpy(key, net_temp, net_temp_len);
				memcpy(key + net_temp_len, temp_user_data, user_data_len);

				for (i = 0; i < key_len; i++) {
					_DEBUG_INFO("key[%d] : 0x%02X", i, key[i]);
				}

				if (net_temp != NULL)
					free(net_temp);

				if (temp_user_data != NULL)
					free(temp_user_data);
			}
		}
		break;
	default:
		break;
	}

	_DEBUG_INFO("start PIN checking logic !!");

	unsigned char *hmac = sync_agent_encrypt_cryptograhic_hash_with_flags(SYNC_AGENT_SA_CRYPTOGRAHIC_HASH_FUNTION_SHA1,
									      wbxml_body, wbxml_size,
									      key, key_len,
									      SYNC_AGENT_SA_CRYPTOGRAHIC_HASH_FUN_HMAC);

	*is_correct = 1;

	i = 0;
	for (; i < 20; i++) {
		char compare[3] = { 0, };
		char *mac_cursor = mac + (i * 2);
		strncpy(compare, mac_cursor, 2);
		_DEBUG_INFO("hmac[%d] : %02X \t  mac[%d, %d] : %s", i, hmac[i], i * 2, i * 2 + 1, compare);
	}

	i = 0;
	for (; i < 20; i++) {
		char data[3] = { 0, };
		snprintf(data, 3, "%02X", hmac[i]);

		char compare[3] = { 0, };
		char *mac_cursor = mac + (i * 2);
		strncpy(compare, mac_cursor, 2);

		_DEBUG_INFO("data : %s, compare : %s ( cnt : %d )", data, compare, i);

		if (strcasecmp(data, compare) != 0) {
			_DEBUG_INFO("failed to compare data !!");
			*is_correct = 0;
			break;
		}
	}

	_DEBUG_INFO("end PIN checking logic !!");

 return_part:
	if (key != NULL)
		free(key);

	sync_agent_free_file(&cp_pin_info);

	if (wbxml_body != NULL)
		sync_agent_free_file(&wbxml_body);

	_DEBUG_INFO("End !!");

	return ret;
}

CP_ERROR process_Invalid_Pincode(int ext_id)
{
	_EXTERN_FUNC_ENTER;

	CP_ERROR ret = CP_ERROR_SUCCESS;

	/*
	 *      1. delete CP data
	 */
	ret = process_Delete_CP_Data(ext_id);
	if (ret != CP_ERROR_SUCCESS) {
		_DEBUG_INFO("process_Delete_CP_Data() fail !!");
		/* todo : exception handling */
	} else {
		_DEBUG_INFO("process_Delete_CP_Data() success !!");
	}

	/*
	 *      2. send remove message to MF
	 */
	if (g_msgId != -1) {
		ret = __send_msg2mf(REMOVE_MESSAGE, NULL);
	}

	_DEBUG_INFO("End !!");

	return ret;
}

CP_ERROR process_Setting_CP(int ext_id)
{
	_DEBUG_INFO("Start !!");

	CP_ERROR ret = CP_ERROR_SUCCESS;

	unsigned long size_return = 0;

	CP_TYPE cp_type = CP_TYPE_WBXML;

	char type_file_name[50] = { 0, };
	snprintf(type_file_name, sizeof(type_file_name), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_TYPE_FILE_NAME);
	_DEBUG_INFO("type file name : %s", type_file_name);

	char *type = 0;
//      int read_ret = sync_agent_read_whole_file(CP_TYPE_REPOSITORY_PATH, &type, &size_return);
	int read_ret = sync_agent_read_whole_file(type_file_name, &type, &size_return);
	if (read_ret == 0) {
		_DEBUG_INFO("Failed to sync_agent_read_file() : %s", type_file_name);
		return CP_ERROR_FAIL;
	}

	if (strcmp("1", type) == 0) {
		cp_type = CP_TYPE_XML;
	} else if (strcmp("2", type) == 0) {
		cp_type = CP_TYPE_WBXML;
	} else {
		cp_type = CP_TYPE_NONE;
	}

	char wbxml_file_name[50] = { 0, };
	snprintf(wbxml_file_name, sizeof(wbxml_file_name), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_WBXML_FILE_NAME);
	_DEBUG_INFO("wbxml file name : %s", wbxml_file_name);

	char *cp_body = 0;
//      read_ret = sync_agent_read_whole_file(CP_WBXML_REPOSITORY_PATH, &cp_body, &size_return);
	read_ret = sync_agent_read_whole_file(wbxml_file_name, &cp_body, &size_return);
	if (read_ret == 0) {
		_DEBUG_INFO("Failed to sync_agent_read_file() : %s", wbxml_file_name);
		return CP_ERROR_FAIL;
	}
//      /*
//       * Parse recieved CP to sturct DM_CP
//       */
//      dm_cp = parsing_cp_stream(cp_type, cp_body, size_return);

	GList *iter = NULL;
	DM_CP *dm_cp;
	for (iter = dm_cp_list; iter != 0; iter = g_list_next(iter)) {
		dm_cp = 0;
		if (iter->data != 0) {
			if (((CP_Data *) (iter->data))->ext_id == ext_id) {
				dm_cp = ((CP_Data *) (iter->data))->dm_cp;
				break;
			}
		}
	}

	if (dm_cp != 0) {
		/* checking app id for multiple app's */
		if (dm_cp->applications->app_ID != NULL) {
			if (strcmp(dm_cp->applications->app_ID, "w2") == 0) {
				_DEBUG_INFO("app id : %s", W2);
				/*
				 *      todo
				 *              ret = __process_Setting_Browser(dm_cp);
				 */
			} else if (strcmp(dm_cp->applications->app_ID, "w4") == 0) {
				_DEBUG_INFO("app id : %s", W4);
				/*
				 *      todo
				 *              ret = __process_Setting_MMS(dm_cp);
				 */
			} else if (strcmp(dm_cp->applications->app_ID, "w5") == 0) {
				_DEBUG_INFO("app id : %s", W5);
				ret = _process_setting_oma_ds(dm_cp);
				if (ret != CP_ERROR_SUCCESS) {
					_DEBUG_INFO("__process_Setting_OMA_DS() failed !!");
					return CP_ERROR_FAIL;
				} else {
					_DEBUG_INFO("__process_Setting_OMA_DS() success !!");
				}
			} else if (strcmp(dm_cp->applications->app_ID, "w7") == 0) {
				_DEBUG_INFO("app id : %s", W7);
				ret = _process_setting_oma_dm(dm_cp);
				if (ret != CP_ERROR_SUCCESS) {
					_DEBUG_INFO("__process_Setting_OMA_DM() failed !!");
					return CP_ERROR_FAIL;
				} else {
					_DEBUG_INFO("__process_Setting_OMA_DM() success !!");
				}
			} else if (strcmp(dm_cp->applications->app_ID, "wA") == 0) {
				_DEBUG_INFO("app id : %s", WA);
				/*
				 *      todo
				 *              ret = __process_Setting_IM_N_PS(dm_cp);
				 */
			} else {
				_DEBUG_INFO("app id : undefined !!");
			}
		}
	} else {
		_DEBUG_INFO("not exist 'CP_Data' for ext_id : %d", ext_id);
		return CP_ERROR_FAIL;
	}

	_DEBUG_INFO("End !!");

	return ret;
}

CP_ERROR process_Delete_All_CP_Data()
{
	_DEBUG_INFO("Start !!");

	CP_ERROR ret = CP_ERROR_SUCCESS;

	/* delete registered all CP data */
	GList *iter = NULL;
	for (iter = dm_cp_list; iter != 0;) {
		if (iter->data != 0) {
			CP_Data *temp = (CP_Data *) (iter->data);
			char folder_name[50] = { 0, };
			snprintf(folder_name, sizeof(folder_name), "%s/%d", CP_DATA_REPOSITORY_PATH, ((CP_Data *) (iter->data))->ext_id);
			_DEBUG_INFO("folder name : %s", folder_name);

			int file_ret = sync_agent_delete_directory(folder_name, TRUE);
			if (file_ret != 1) {
				_DEBUG_INFO("sync_agent_delete_directory( folder : %s ) failed !!", folder_name);
				return CP_ERROR_FAIL;
			}

			dm_cp_list = g_list_remove(dm_cp_list, iter->data);
			iter = g_list_next(iter);

			if (temp->dm_cp != 0)
				free(temp->dm_cp);
			free(temp);
		}
	}

	if (g_list_length(dm_cp_list) == 0) {
		_DEBUG_INFO("dm_cp_list length : 0 !!");
		g_list_free(dm_cp_list);
		dm_cp_list = NULL;
	}

	_DEBUG_INFO("End !!");

	return ret;
}

CP_ERROR process_Delete_CP_Data(int ext_id)
{
	_DEBUG_INFO("Start !!");

	CP_ERROR ret = CP_ERROR_SUCCESS;

	GList *iter = NULL;
	CP_Data *temp = 0;
	for (iter = dm_cp_list; iter != 0; iter = g_list_next(iter)) {
		if (iter->data != 0) {
			if (((CP_Data *) (iter->data))->ext_id == ext_id) {
				temp = (CP_Data *) (iter->data);
				dm_cp_list = g_list_remove(dm_cp_list, iter->data);
				if (temp->dm_cp != 0)
					free(temp->dm_cp);
				free(temp);
				break;
			}
		}
	}

	if (g_list_length(dm_cp_list) == 0) {
		g_list_free(dm_cp_list);
		dm_cp_list = NULL;
	}

	char folder_name[50] = { 0, };
	snprintf(folder_name, sizeof(folder_name), "%s/%d", CP_DATA_REPOSITORY_PATH, ext_id);
	_DEBUG_INFO("folder name : %s", folder_name);

	int file_ret = sync_agent_delete_directory(folder_name, TRUE);
	if (file_ret != 1) {
		_DEBUG_INFO("sync_agent_delete_directory( folder : %s ) failed !!", folder_name);
		return CP_ERROR_FAIL;
	}

	_DEBUG_INFO("End !!");

	return ret;
}

void process_Set_msgId(int msgId)
{
	_DEBUG_INFO("Start !!");

	g_msgId = msgId;
	_DEBUG_INFO("g_msgId : %d", g_msgId);

	_DEBUG_INFO("End !!");
}

/**************************** Impl static function *************************/

static CP_ERROR _process_receieved_cp_stream(CP_TYPE cp_type, const char *msgBody, unsigned int msgSize, int ext_id)
{
	_INNER_FUNC_ENTER;
	CP_ERROR ret = CP_ERROR_SUCCESS;

	retvm_if((msgBody) == NULL, CP_ERROR_FAIL, "msgBody is NULL!!");

	char type[1];
	if (cp_type == CP_TYPE_XML) {
		type[0] = '1';
	} else if (cp_type == CP_TYPE_WBXML) {
		type[0] = '2';
	}

	char type_file_name[50] = { 0, };
	snprintf(type_file_name, sizeof(type_file_name), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_TYPE_FILE_NAME);
	_DEBUG_TRACE("type file name : %s", type_file_name);

//      int file_ret = sync_agent_write_file(CP_TYPE_REPOSITORY_PATH, type, 1, 1, 1);
//      int file_ret = sync_agent_write_whole_file(CP_TYPE_REPOSITORY_PATH, type, 1, 1);
	int file_ret = sync_agent_write_whole_file(type_file_name, type, 1, 1);
	if (file_ret != 1) {
		_DEBUG_TRACE("Failed to repository CP Type");
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	/*
	 * 3. Registry (DM_CP) to storage
	 */
	char wbxml_file_name[50] = { 0, };
	snprintf(wbxml_file_name, sizeof(wbxml_file_name), "%s/%d/%s", CP_DATA_REPOSITORY_PATH, ext_id, CP_WBXML_FILE_NAME);
	_DEBUG_TRACE("wbxml file name : %s", wbxml_file_name);
//      file_ret = sync_agent_write_file(CP_WBXML_REPOSITORY_PATH, msgBody, msgSize, 1, 1);
//      file_ret = sync_agent_write_whole_file(CP_WBXML_REPOSITORY_PATH, msgBody, msgSize, 1);
	file_ret = sync_agent_write_whole_file(wbxml_file_name, msgBody, msgSize, 1);
	if (file_ret != 1) {
		_DEBUG_TRACE("Failed to repository CP Body");
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	/*
	 * 3. Handling Overwrite
	 */
	CP_Data *dm_cp_data = (CP_Data *) calloc(1, sizeof(CP_Data));
	if (dm_cp_data == NULL) {
		_DEBUG_TRACE("CP_Data calloc failed !!");
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	dm_cp_data->ext_id = ext_id;
	/*
	 * Parse recieved CP to sturct DM_CP
	 */
//      dm_cp = parsing_cp_stream(cp_type, msgBody, msgSize);
//      if (dm_cp == 0) {
//              _DEBUG_TRACE("parsing_cp_stream() failed !!");
//              /* todo : exception handling */
//              return CP_ERROR_FAIL;
//      }

	dm_cp_data->dm_cp = parsing_cp_stream(cp_type, msgBody, msgSize);
	if (dm_cp_data->dm_cp == 0) {
		_DEBUG_TRACE("parsing_cp_stream() failed !!");
		/* todo : exception handling */
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	dm_cp_list = g_list_append(dm_cp_list, dm_cp_data);

	/*
	 * 4. Interact Msg F/W
	 */
	// create message
	pmci_san_message_s *send_msg = (pmci_san_message_s *) calloc(1, sizeof(pmci_san_message_s));
	if (send_msg == NULL) {
		_DEBUG_TRACE("calloc fail !!");
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	send_msg->ext_id = ext_id;
//      send_msg->ext_id = 11;                                  // ?? : temp setting value? unique management (id_provider available?) todo??
//      send_msg->pin_code = pin_code;          // PIN code
//      char *msgData = "SYNCML";
	char *msgData = NULL;
	msgData = (char *)calloc(MSG_SIZE, sizeof(char));

	if (msgData == NULL) {
		_DEBUG_TRACE("calloc failed !!");
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	int err = __make_view_message(&msgData, dm_cp_data->dm_cp);
	if (err != 1) {
		_DEBUG_TRACE("__make_view_message() failed !!");

		if (msgData != NULL)
			free(msgData);

		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	} else {

		_DEBUG_TRACE("__make_view_message() success !!");
		_DEBUG_TRACE("msgData : %s", msgData);
	}

//      char *msgData = "Configuration message \n Application : Browser \n Proxy : wap_proxy \n Port : 9201 \n Received date : 17/04/2012";
	if (msgData != NULL)
		send_msg->msg_data = strdup(msgData);	// message that will be sent to MF

	ret = __send_msg2mf(ADD_MESSAGE, (void *)send_msg);
	if (ret != CP_ERROR_SUCCESS) {
		_DEBUG_TRACE("__sendMsg2MF() failed !!");
	} else {
		_DEBUG_TRACE("__sendMsg2MF() success !!");
	}

	_INNER_FUNC_EXIT;
	return ret;
}

static CP_ERROR __send_msg2mf(SENDMSG_TYPE send_type, void *send_msg)
{
	_INNER_FUNC_ENTER;

	CP_ERROR res = CP_ERROR_SUCCESS;
	sync_agent_pm_return_e err_sendMsg = SYNC_AGENT_PM_SUCCESS;

	switch (send_type) {
	case ADD_MESSAGE:
		{
			int out_add_item_id = 0;	// meaningless value

			/* send message (add) */
			err_sendMsg = sync_agent_add_service_data(6, (void *)send_msg, &out_add_item_id);
			if (err_sendMsg != SYNC_AGENT_PM_SUCCESS) {
				_DEBUG_VERBOSE("send msg ( ADD_MESSAGE ) to MF fail !!");
				res = CP_ERROR_FAIL;
				goto return_part;
				/* todo : exception handling */
			} else {
				_DEBUG_VERBOSE("send msg ( ADD_MESSAGE ) to MF success !!");
			}
			_DEBUG_VERBOSE("---------------------------------------");
			_DEBUG_VERBOSE("OUT ADD ITEM %d", out_add_item_id);
			_DEBUG_VERBOSE("---------------------------------------");
		}
		break;
	case REMOVE_MESSAGE:
		{
			/* send message (remove) */
			err_sendMsg = sync_agent_remove_service_data(6, g_msgId);
			if (err_sendMsg != SYNC_AGENT_PM_SUCCESS) {
				_DEBUG_VERBOSE("send msg ( REMOVE_MESSAGE ) to MF fail !!");
				res = CP_ERROR_FAIL;
				goto return_part;
				/* todo : exception handling */
			} else {
				_DEBUG_VERBOSE("send msg ( REMOVE_MESSAGE ) to MF success !!");
			}
		}
		break;
	default:
		break;
	}

 return_part:
	if (send_msg != NULL) {
		if (((pmci_san_message_s *) send_msg)->msg_data != NULL)
			free(((pmci_san_message_s *) send_msg)->msg_data);
		free(send_msg);
	}

	_INNER_FUNC_EXIT;
	return res;
}

static CP_ERROR _process_setting_oma_ds(DM_CP * dm_cp)
{
	_INNER_FUNC_ENTER;
	CP_ERROR ret = CP_ERROR_SUCCESS;

	retvm_if((dm_cp) == NULL, CP_ERROR_FAIL, "dm_cp is NULL!!");

	int event_type = 8;
	/*sync_agent_event_error_e error; */

	sync_agent_event_data_s *event = sync_agent_create_event(event_type);
	if (event == NULL) {
		_DEBUG_TRACE("event is NULL");
		_INNER_FUNC_EXIT;
		return 0;
	}

	DM_CP_Application *cp_application = dm_cp->applications;

	_DEBUG_TRACE("cp_application->name : %s", cp_application->name);
	sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_application->name);	/* profileName */

	String_Node *addr = cp_application->addr;
	if (addr != NULL) {
		_DEBUG_TRACE("cp_application->addr : %s", addr->data);
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, addr->data);	/* addr */
	} else {
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, "");	/* addr */
	}

	DM_CP_Application_AppAuth *cp_auth = cp_application->appAuth;
	if (cp_auth != NULL) {
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_auth->auth_name);	/* id */
		_DEBUG_TRACE("cp_application->auth_name : %s", cp_auth->auth_name);
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_auth->auth_secret);	/* password */
		_DEBUG_TRACE("cp_application->auth_secret : %s", cp_auth->auth_secret);
	} else {
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, "");	/* id */
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, "");	/* password */
	}

	int categoryCount = 0;
	DM_CP_Application_Resource *cp_resource = cp_application->resource;
	while (cp_resource != 0) {
		categoryCount++;
		cp_resource = cp_resource->next;
	}
	sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_INTEGER, &categoryCount);	/* categoryCount */

	cp_resource = cp_application->resource;

	while (cp_resource != 0) {
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_resource->name);	/* name */
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_resource->accept);	/* accept */
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_resource->auth_name);	/* id */
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_resource->auth_secret);	/* password */
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_resource->auth_type);	/* auth_type */
		sync_agent_append_event_data_param(event, SYNC_AGENT_EVENT_PARAM_TYPE_STRING, cp_resource->auth_data);	/* auth_data */

		cp_resource = cp_resource->next;
	}

	/*
	 * IPC ( send msg to OMA DS Agent )
	 * Below rutine is made by Hard Coding?! If that, So Terrible.. I should consider this situation. - RJW
	 */

	int client_len;
	int client_sockfd;
	struct sockaddr_un clientaddr;

	client_sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
	if (client_sockfd == -1) {
		_DEBUG_TRACE("socket create Error!!");
		sync_agent_free_event(event);
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	bzero(&clientaddr, sizeof(clientaddr));
	clientaddr.sun_family = AF_UNIX;

	int str_len = 0;

	str_len = g_strlcpy(clientaddr.sun_path, IPC_DS, (int)sizeof(clientaddr.sun_path));
	if (str_len >= sizeof(clientaddr.sun_path)) {
		_DEBUG_TRACE("buffer overflow");
		sync_agent_free_event(event);
		close(client_sockfd);
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}
	client_len = sizeof(clientaddr);

	if (connect(client_sockfd, (struct sockaddr *)&clientaddr, client_len) < 0) {
		_DEBUG_TRACE("socket connect error");
		sync_agent_free_event(event);
		close(client_sockfd);
		_INNER_FUNC_EXIT;
		return CP_ERROR_FAIL;
	}

	int send_data_size = (event->data - event->size) + 1;
	int write_ret = write(client_sockfd, (void *)(event->size), send_data_size);
	_DEBUG_TRACE("write ret : %d", write_ret);
	close(client_sockfd);
	sync_agent_free_event(event);

	_INNER_FUNC_EXIT;

	return ret;
}

static CP_ERROR _process_setting_oma_dm(DM_CP * dm_cp)
{
	_INNER_FUNC_ENTER;

	CP_ERROR ret = CP_ERROR_SUCCESS;
	DM_ERROR dm_ret = DM_OK;

	retvm_if((dm_cp) == NULL, CP_ERROR_FAIL, "dm_cp is NULL!!");

	DM_CP_Application *applications = dm_cp->applications;
	dm_acc *dmaccount = NULL;
	if (applications != NULL) {
		if (applications->provider_ID != NULL) {
			dmaccount = (dm_acc *) calloc(1, sizeof(dm_acc));
			if (dmaccount == NULL) {
				_DEBUG_TRACE("alloc fail");
				_INNER_FUNC_EXIT;
				return 0;
			}
			char *server_id = NULL;
			server_id = strdup(applications->provider_ID);
			/*get_Serverid(SAMSUNG_FUMO_TYPE , &server_id); */
			_DEBUG_TRACE("server id : %s", server_id);

			dmaccount->server_id = strdup(applications->provider_ID);
			_DEBUG_TRACE("server id : %s", dmaccount->server_id);

			if (applications->addr != NULL) {
				dmaccount->server_url = strdup(applications->addr->data);
				_DEBUG_TRACE("server url : %s", dmaccount->server_url);
			}

			DM_CP_Application_AppAuth *appAuth = applications->appAuth;

			if (appAuth != NULL) {
				while (1) {
					if (appAuth->auth_level != NULL) {
						if (strcmp("APPSRV", appAuth->auth_level) == 0) {
							if (appAuth->auth_secret != NULL) {
								dmaccount->server_pw = strdup(appAuth->auth_secret);
								_DEBUG_TRACE("server_pw : %s", dmaccount->server_pw);
							}
							if (appAuth->auth_type != NULL) {
								if (strstr(appAuth->auth_type, "DIGEST") != NULL) {
									dmaccount->server_auth_type = strdup("DIGEST");
									_DEBUG_TRACE("server_auth_type : %s", dmaccount->server_auth_type);
								} else if (strstr(appAuth->auth_type, "HMAC") != NULL) {
									dmaccount->server_auth_type = strdup("HMAC");
									_DEBUG_TRACE("server_auth_type : %s", dmaccount->server_auth_type);
								} else if (strstr(appAuth->auth_type, "BASIC") != NULL) {
									dmaccount->server_auth_type = strdup("BASIC");
									_DEBUG_TRACE("server_auth_type : %s", dmaccount->server_auth_type);
								} else {
									_DEBUG_TRACE("server_auth_type: %s", appAuth->auth_type);
									dmaccount->server_auth_type = strdup("DIGEST");
									_DEBUG_TRACE("server_nextNonce : %s", dmaccount->server_auth_type);
								}
							} else if (appAuth->auth_type == NULL && dmaccount->server_auth_type == NULL) {
								dmaccount->server_auth_type = strdup("DIGEST");
								_DEBUG_TRACE("user_auth_type : %s", dmaccount->server_auth_type);
							}

							if (appAuth->auth_data != NULL) {
								dmaccount->server_nextNonce = strdup(appAuth->auth_data);
								_DEBUG_TRACE("server_nextNonce : %s", dmaccount->server_nextNonce);
							}
						} else if (!strcmp("CLIENT", appAuth->auth_level)) {
							if (appAuth->auth_secret != NULL) {
								dmaccount->user_pw = strdup(appAuth->auth_secret);
								_DEBUG_TRACE("user_pw : %s", dmaccount->user_pw);
							}

							if (appAuth->auth_type != NULL) {
								if (strstr(appAuth->auth_type, "DIGEST") != NULL) {
									dmaccount->user_auth_type = strdup("DIGEST");
									_DEBUG_TRACE("user_auth_type : %s", dmaccount->user_auth_type);
								} else if (strstr(appAuth->auth_type, "HMAC") != NULL) {
									dmaccount->user_auth_type = strdup("HMAC");
									_DEBUG_TRACE("user_auth_type : %s", dmaccount->user_auth_type);
								} else if (strstr(appAuth->auth_type, "BASIC") != NULL) {
									dmaccount->user_auth_type = strdup("BASIC");
									_DEBUG_TRACE("user_auth_type : %s", dmaccount->user_auth_type);
								} else {
									_DEBUG_TRACE("user_auth_type : %s", appAuth->auth_type);
									dmaccount->user_auth_type = strdup("DIGEST");
									_DEBUG_TRACE("user_auth_type : %s", dmaccount->user_auth_type);
								}
							} else if (appAuth->auth_type == NULL && dmaccount->user_auth_type == NULL) {
								dmaccount->user_auth_type = strdup("DIGEST");
								_DEBUG_TRACE("user_auth_type : %s", dmaccount->user_auth_type);
							}

							if (appAuth->auth_name != NULL) {
								dmaccount->user_id = strdup(appAuth->auth_name);
								_DEBUG_TRACE("user_id : %s", dmaccount->user_id);
							}
							if (appAuth->auth_data != NULL) {
								dmaccount->user_nextNonce = strdup(appAuth->auth_data);
								_DEBUG_TRACE("user_auth_type: %s", dmaccount->user_nextNonce);
							}
						}
					} else {
						ret = CP_ERROR_FAIL;
						free_dm_acc(dmaccount);
						goto error;
					}

					if (appAuth->next != NULL) {
						appAuth = appAuth->next;
					} else {
						break;
					}
				}
			}

			if (server_id != NULL) {
				int is_exist = 0;
				SERVICE_SERVER_TYPE dmacc_type = NO_ENGINE_TYPE;

				dmacc_type = get_engine_type_by_serverid(server_id);
				_DEBUG_TRACE(" dm acc type : %d", dmacc_type);

				if (dmacc_type != NO_ENGINE_TYPE) {
					dm_ret = is_exist_dmacc(dmacc_type, &is_exist);
					if (dm_ret != DM_OK) {
						_DEBUG_TRACE("error is exist dm acc ");
						ret = CP_ERROR_FAIL;
						goto error;
					}

					if (is_exist == 1) {
						_DEBUG_TRACE("exist dm acc ");
						ret = update_dmacc(server_id, dmaccount);
						_DEBUG_TRACE("update dm acc %d", ret);
						if (ret == 1) {
							first_server_bootstrap(server_id);
							ret = CP_ERROR_SUCCESS;
						} else {
							ret = CP_ERROR_FAIL;
							goto error;
						}
					} else {
						_DEBUG_TRACE("not exist dm acc ");
						ret = CP_ERROR_FAIL;
					}
				} else {
					_DEBUG_TRACE("dmacc type is none");
					ret = CP_ERROR_FAIL;
				}
			} else {
				_DEBUG_TRACE("server id is null");
				ret = CP_ERROR_FAIL;
			}

			_DEBUG_TRACE("update dm acc : %d", ret);
			free_dm_acc(dmaccount);
		}
	} else {
		ret = CP_ERROR_FAIL;
		goto error;
	}

	_INNER_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_TRACE("error : %d", ret);
	_INNER_FUNC_EXIT;
	return ret;

}

static char __char2hex(char ch)
{
	_INNER_FUNC_ENTER;

	char ret = 0x00;
	switch (ch) {
	case '0':
		ret = 0x00;
		break;
	case '1':
		ret = 0x01;
		break;
	case '2':
		ret = 0x02;
		break;
	case '3':
		ret = 0x03;
		break;
	case '4':
		ret = 0x04;
		break;
	case '5':
		ret = 0x05;
		break;
	case '6':
		ret = 0x06;
		break;
	case '7':
		ret = 0x07;
		break;
	case '8':
		ret = 0x08;
		break;
	case '9':
		ret = 0x09;
		break;
	default:
		break;
	}

	_DEBUG_VERBOSE("ch : %c ( 0x%02X )", ch, ret);
	_INNER_FUNC_EXIT;
	return ret;
}

static char *_get_netpin_key(int *key_len)
{
	_INNER_FUNC_ENTER;

	sync_agent_dev_return_e err = SYNC_AGENT_DEV_RETURN_SUCCESS;
	char *imsi_value = 0;
	char *trans_imsi_value = 0;
	int imsi_len = 0;

//      char key[8] = {0, };
	char *key = NULL;
	char temp1, temp2;
	int i = 0;

	int flag = 0;

	err = sync_agent_get_devinfo(2, "IMSI", &imsi_value);
	if (err != SYNC_AGENT_DEV_RETURN_SUCCESS) {
		_DEBUG_TRACE("imsi value is error !!");
		goto return_part;
	} else {
		flag = 1;
		_DEBUG_TRACE("sync_agent_get_devinfo( IMSI ) success !!");
	}

	_DEBUG_TRACE("IMSI value : %s", imsi_value);
	int len = 0;

	imsi_len = strlen(imsi_value);
	if ((imsi_len % 2) == 1) {
		imsi_len += 2;
		trans_imsi_value = (char *)calloc(imsi_len, sizeof(char));

		if (trans_imsi_value == NULL) {
			_DEBUG_TRACE("calloc failed !!");
			goto return_part;
		}

		len = g_strlcat(trans_imsi_value, "9", imsi_len);
		len = g_strlcat(trans_imsi_value, imsi_value, imsi_len);

		if (len >= imsi_len) {
			_DEBUG_TRACE(" buffer overflow !!");
			/* todo : exception handling */
		}

	} else {
		imsi_len += 3;
		trans_imsi_value = (char *)calloc(imsi_len, sizeof(char));

		if (trans_imsi_value == NULL) {
			_DEBUG_TRACE("calloc failed !!");
			goto return_part;
		}

		len = g_strlcat(trans_imsi_value, "1", imsi_len);
		len = g_strlcat(trans_imsi_value, imsi_value, imsi_len);
		len = g_strlcat(trans_imsi_value, "F", imsi_len);

		if (len >= imsi_len) {
			_DEBUG_TRACE(" buffer overflow !!");
			/* todo : exception handling */
		}

	}

	*key_len = imsi_len / 2;
	_DEBUG_TRACE("imsi value len : %d", *key_len);

	key = (char *)calloc(*key_len, sizeof(char));
	if (key == NULL) {
		_DEBUG_TRACE("calloc failed !!");
		goto return_part;
	}

	for (; i < *key_len; i++) {
		temp1 = __char2hex(trans_imsi_value[i * 2 + 1]);
		temp2 = __char2hex(trans_imsi_value[i * 2]);
		key[i] = (temp1 << 4) | temp2;
		temp1 = 0;
		temp2 = 0;
		_DEBUG_TRACE("key[%d] : 0x%02X", i, key[i]);
	}

 return_part:
	if ((imsi_value != NULL) && (flag == 1))
		free(imsi_value);

	if (trans_imsi_value != NULL)
		free(trans_imsi_value);

	_INNER_FUNC_EXIT;
	return key;
}

static int __make_view_message(char **msgData, DM_CP * dm_cp)
{
	_INNER_FUNC_ENTER;

	int res = 1;
	int len = 0;

	/* Application */
	retvm_if((dm_cp) == NULL, 0, "dm_cp is NULL!!");

	len = g_strlcat(*msgData, APPLICATION, MSG_SIZE);
	if (dm_cp->applications->app_ID != NULL) {
		if (dm_cp->applications->name != NULL) {
			_DEBUG_VERBOSE("name : %s", dm_cp->applications->name);

			len = g_strlcat(*msgData, dm_cp->applications->name, MSG_SIZE);
		}

		len = g_strlcat(*msgData, L_BRACKET, MSG_SIZE);
		if (strcmp(dm_cp->applications->app_ID, "w2") == 0) {
			_DEBUG_VERBOSE("app id : %s", W2);

			len = g_strlcat(*msgData, W2, MSG_SIZE);
		} else if (strcmp(dm_cp->applications->app_ID, "w4") == 0) {
			_DEBUG_VERBOSE("app id : %s", W4);

			len = g_strlcat(*msgData, W4, MSG_SIZE);
		} else if (strcmp(dm_cp->applications->app_ID, "w5") == 0) {
			_DEBUG_VERBOSE("app id : %s", W5);

			len = g_strlcat(*msgData, W5, MSG_SIZE);
		} else if (strcmp(dm_cp->applications->app_ID, "w7") == 0) {
			_DEBUG_VERBOSE("app id : %s", W7);

			len = g_strlcat(*msgData, W7, MSG_SIZE);
		} else if (strcmp(dm_cp->applications->app_ID, "wA") == 0) {
			_DEBUG_VERBOSE("app id : %s", WA);

			len = g_strlcat(*msgData, WA, MSG_SIZE);
		} else {
			_DEBUG_VERBOSE("app id : undefined !!");
			len = g_strlcat(*msgData, "Undefined ", MSG_SIZE);
		}

		len = g_strlcat(*msgData, R_BRACKET, MSG_SIZE);
	} else {
		_DEBUG_VERBOSE("Application ID is not existed - Invalid format !!");
		_INNER_FUNC_EXIT;
		return 0;
	}

	len = g_strlcat(*msgData, LB, MSG_SIZE);
//      _DEBUG_VERBOSE("msg data : %s", *msgData);

	/* URL */

	len = g_strlcat(*msgData, URL, MSG_SIZE);
	if (dm_cp->applications->addr != NULL) {
		if (dm_cp->applications->addr->data != NULL) {
			_DEBUG_VERBOSE("url : %s", dm_cp->applications->addr->data);

			len = g_strlcat(*msgData, dm_cp->applications->addr->data, MSG_SIZE);
		} else {

			len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
		}
	} else {

		len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);

	}

	len = g_strlcat(*msgData, LB, MSG_SIZE);
//      _DEBUG_VERBOSE("msg data : %s", *msgData);

	/* NAP Addr type */
	if (dm_cp->napdefs != NULL) {
		if (dm_cp->napdefs->nap_addr_type != NULL) {
			_DEBUG_VERBOSE("nap addr type : %s", dm_cp->napdefs->nap_addr_type);

			len = g_strlcat(*msgData, dm_cp->napdefs->nap_addr_type, MSG_SIZE);
			len = g_strlcat(*msgData, DELIMITER, MSG_SIZE);
			if (dm_cp->napdefs->nap_address != NULL) {
				_DEBUG_VERBOSE("nap addr : %s", dm_cp->napdefs->nap_address);
				len = g_strlcat(*msgData, dm_cp->napdefs->nap_address, MSG_SIZE);

			}
		} else {

			len = g_strlcat(*msgData, DEFAULT_NAP_TYPE, MSG_SIZE);
			len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
		}
	} else {

		len = g_strlcat(*msgData, DEFAULT_NAP_TYPE, MSG_SIZE);
		len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
	}

	len = g_strlcat(*msgData, LB, MSG_SIZE);

//      _DEBUG_VERBOSE("msg data : %s", *msgData);

	/* Proxy */

	len = g_strlcat(*msgData, PROXY, MSG_SIZE);

	if (dm_cp->applications->to_proxy != NULL) {
		if (dm_cp->applications->to_proxy->data != NULL) {
			_DEBUG_VERBOSE("proxy : %s", dm_cp->applications->to_proxy->data);

			len = g_strlcat(*msgData, dm_cp->applications->to_proxy->data, MSG_SIZE);
		} else {

			len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);

		}
	} else {

		len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
	}

	len = g_strlcat(*msgData, LB, MSG_SIZE);
//      _DEBUG_VERBOSE("msg data : %s", *msgData);

	/* Port */

	len = g_strlcat(*msgData, PORT, MSG_SIZE);
	if (dm_cp->pxLogical != NULL) {
		if (dm_cp->pxLogical->port != NULL) {
			if (dm_cp->pxLogical->port->portnbr != NULL) {
				_DEBUG_VERBOSE("port : %s", dm_cp->pxLogical->port->portnbr);

				len = g_strlcat(*msgData, dm_cp->pxLogical->port->portnbr, MSG_SIZE);
			} else {

				len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
			}
		} else {

			len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
		}
	} else {

		len = g_strlcat(*msgData, NO_DATA, MSG_SIZE);
	}

	len = g_strlcat(*msgData, LB, MSG_SIZE);
//      _DEBUG_VERBOSE("msg data : %s", *msgData);

	if (len >= MSG_SIZE) {
		_DEBUG_VERBOSE("over flow error");
		res = -1;
	}

	_INNER_FUNC_EXIT;
	return res;
}

void free_cp_application(DM_CP_Application * app)
{
	retm_if((app) == NULL, "dm_cp application is NULL!!");

	if (app->accept != NULL) {
		str_free(&(app->accept));
		app->accept = NULL;
	}
	if (app->addr != NULL) {
		free_string_node(app->addr);
		app->addr = NULL;
	}
	if (app->appAddr != NULL) {
		free_cp_application_app_addr(app->appAddr);
		app->appAddr = NULL;
	}
	if (app->appAuth != NULL) {
		free_cp_application_app_auth(app->appAuth);
		app->appAuth = NULL;
	}
	if (app->app_ID != NULL) {
		str_free(&(app->app_ID));
		app->app_ID = NULL;
	}
	if (app->name != NULL) {
		str_free(&(app->name));
		app->name = NULL;
	}
	if (app->protocol != NULL) {
		str_free(&(app->protocol));
		app->protocol = NULL;
	}
	if (app->provider_ID != NULL) {
		str_free(&(app->provider_ID));
		app->provider_ID = NULL;
	}
	if (app->resource != NULL) {
		free_cp_application_resource(app->resource);
		app->resource = NULL;
	}
	if (app->to_napid != NULL) {
		free_string_node(app->to_napid);
		app->to_napid = NULL;
	}
	if (app->to_proxy != NULL) {
		free_string_node(app->to_proxy);
		app->to_proxy = NULL;
	}
	if (app->next != NULL) {
		free_cp_application(app->next);
		app->next = NULL;
	}
	free(app);
	app = NULL;
}

void free_cp_application_app_addr(DM_CP_Application_AppAddr * app_addr)
{
	retm_if((app_addr) == NULL, "dm_cp is NULL!!");

	if (app_addr->addr != NULL) {
		str_free(&(app_addr->addr));
		app_addr->addr = NULL;
	}

	if (app_addr->addr_type != NULL) {
		str_free(&(app_addr->addr_type));
		app_addr->addr_type = NULL;
	}

	if (app_addr->port != NULL) {
		free_cp_port(app_addr->port);
		app_addr->port = NULL;
	}

	if (app_addr->next != NULL) {
		free_cp_application_app_addr(app_addr->next);
		app_addr->next = NULL;
	}

	free(app_addr);
	app_addr = NULL;
}

void free_cp_application_app_auth(DM_CP_Application_AppAuth * appAuth)
{
	retm_if((appAuth) == NULL, "dm_cp appAuth is NULL!!");

	if (appAuth->auth_data != NULL) {
		str_free(&(appAuth->auth_data));
		appAuth->auth_data = NULL;
	}
	if (appAuth->auth_level != NULL) {
		str_free(&(appAuth->auth_level));
		appAuth->auth_level = NULL;
	}
	if (appAuth->auth_name != NULL) {
		str_free(&(appAuth->auth_name));
		appAuth->auth_name = NULL;
	}
	if (appAuth->auth_secret != NULL) {
		str_free(&(appAuth->auth_secret));
		appAuth->auth_secret = NULL;
	}

	if (appAuth->auth_type != NULL) {
		str_free(&(appAuth->auth_type));
		appAuth->auth_type = NULL;
	}
	if (appAuth->next != NULL) {
		free_cp_application_app_auth(appAuth->next);
		appAuth->next = NULL;
	}

	free(appAuth);
	appAuth = NULL;
}

void free_cp_application_resource(DM_CP_Application_Resource * resource)
{
	retm_if((resource) == NULL, "dm_cp port is NULL!!");

	if (resource->accept != NULL) {
		str_free(&(resource->accept));
		resource->accept = NULL;
	}
	if (resource->auth_data != NULL) {
		str_free(&(resource->auth_data));
		resource->auth_data = NULL;
	}
	if (resource->auth_name != NULL) {
		str_free(&(resource->auth_name));
		resource->auth_name = NULL;
	}
	if (resource->auth_secret != NULL) {
		str_free(&(resource->auth_secret));
		resource->auth_secret = NULL;
	}
	if (resource->auth_type != NULL) {
		str_free(&(resource->auth_type));
		resource->auth_type = NULL;
	}
	if (resource->name != NULL) {
		str_free(&(resource->name));
		resource->name = NULL;
	}
	if (resource->start_page != NULL) {
		str_free(&(resource->start_page));
		resource->start_page = NULL;
	}
	if (resource->uri != NULL) {
		str_free(&(resource->uri));
		resource->uri = NULL;
	}
	if (resource->next != NULL) {
		free_cp_application_resource(resource->next);
		resource->next = NULL;
	}

	free(resource);
	resource = NULL;
}

void free_cp_port(DM_CP_Port * port)
{
	retm_if((port) == NULL, "dm_cp port is NULL!!");

	if (port->portnbr != NULL) {
		str_free(&(port->portnbr));
		port->portnbr = NULL;
	}
	if (port->service != NULL) {
		free_string_node(port->service);
		port->service = NULL;
	}
	if (port->next != NULL) {
		free_cp_port(port->next);
		port->next = NULL;
	}

	free(port);
	port = NULL;
}

void free_string_node(String_Node * service)
{
	retm_if((service) == NULL, "dm_cp string node is NULL!!");

	if (service->data != NULL) {
		str_free(&(service->data));
		service->data = NULL;
	}
	if (service->next != NULL) {
		free_string_node(service->next);
		service->next = NULL;
	}

	free(service);
	service = NULL;
}

void free_cp_napdef(DM_CP_Napdef * napdef)
{
	retm_if((napdef) == NULL, "dm_cp napdef is NULL!!");

	if (napdef->auth_info != NULL) {
		free_cp_napdef_authinfo(napdef->auth_info);
		napdef->auth_info = NULL;
	}
	if (napdef->bearer != NULL) {
		free_string_node(napdef->bearer);
		napdef->bearer = NULL;
	}
	if (napdef->call_type != NULL) {
		str_free(&(napdef->call_type));
		napdef->call_type = NULL;
	}
	if (napdef->delivery_err_sdu != NULL) {
		str_free(&(napdef->delivery_err_sdu));
		napdef->delivery_err_sdu = NULL;
	}
	if (napdef->delivery_order != NULL) {
		str_free(&(napdef->delivery_order));
		napdef->delivery_order = NULL;
	}
	if (napdef->dn_link_speed != NULL) {
		str_free(&(napdef->dn_link_speed));
		napdef->dn_link_speed = NULL;
	}
	if (napdef->dns_addr != NULL) {
		free_string_node(napdef->dns_addr);
		napdef->dns_addr = NULL;
	}
	if (napdef->first_retry_timeout != NULL) {
		str_free(&(napdef->first_retry_timeout));
		napdef->first_retry_timeout = NULL;
	}
	if (napdef->guaranteed_bitrate_dnlink != NULL) {
		str_free(&(napdef->guaranteed_bitrate_dnlink));
		napdef->guaranteed_bitrate_dnlink = NULL;
	}
	if (napdef->guaranteed_bitrate_uplink != NULL) {
		str_free(&(napdef->guaranteed_bitrate_uplink));
		napdef->guaranteed_bitrate_uplink = NULL;
	}
	if (napdef->internet != NULL) {
		str_free(&(napdef->internet));
		napdef->internet = NULL;
	}
	if (napdef->linger != NULL) {
		str_free(&(napdef->linger));
		napdef->linger = NULL;
	}
	if (napdef->link_speed != NULL) {
		str_free(&(napdef->link_speed));
		napdef->link_speed = NULL;
	}
	if (napdef->local_addr != NULL) {
		str_free(&(napdef->local_addr));
		napdef->local_addr = NULL;
	}
	if (napdef->local_addr_type != NULL) {
		str_free(&(napdef->local_addr_type));
		napdef->local_addr_type = NULL;
	}
	if (napdef->max_bitrate_dnlink != NULL) {
		str_free(&(napdef->max_bitrate_dnlink));
		napdef->max_bitrate_dnlink = NULL;
	}
	if (napdef->max_bitrate_uplink != NULL) {
		str_free(&(napdef->max_bitrate_uplink));
		napdef->max_bitrate_uplink = NULL;
	}
	if (napdef->max_num_retry != NULL) {
		str_free(&(napdef->max_num_retry));
		napdef->max_num_retry = NULL;
	}
	if (napdef->max_sdu_size != NULL) {
		str_free(&(napdef->max_sdu_size));
		napdef->max_sdu_size = NULL;
	}
	if (napdef->name != NULL) {
		str_free(&(napdef->name));
		napdef->name = NULL;
	}
	if (napdef->nap_ID != NULL) {
		str_free(&(napdef->nap_ID));
		napdef->nap_ID = NULL;
	}
	if (napdef->nap_addr_type != NULL) {
		str_free(&(napdef->nap_addr_type));
		napdef->nap_addr_type = NULL;
	}
	if (napdef->nap_address != NULL) {
		str_free(&(napdef->nap_address));
		napdef->nap_address = NULL;
	}
	if (napdef->rereg_threshold != NULL) {
		str_free(&(napdef->rereg_threshold));
		napdef->rereg_threshold = NULL;
	}
	if (napdef->residual_ber != NULL) {
		str_free(&(napdef->residual_ber));
		napdef->residual_ber = NULL;
	}
	if (napdef->sdu_error_ratio != NULL) {
		str_free(&(napdef->sdu_error_ratio));
		napdef->sdu_error_ratio = NULL;
	}
	if (napdef->t_bit != NULL) {
		str_free(&(napdef->t_bit));
		napdef->t_bit = NULL;
	}
	if (napdef->traffic_class != NULL) {
		str_free(&(napdef->traffic_class));
		napdef->traffic_class = NULL;
	}
	if (napdef->traffic_handl_prio != NULL) {
		str_free(&(napdef->traffic_handl_prio));
		napdef->traffic_handl_prio = NULL;
	}
	if (napdef->transfer_delay != NULL) {
		str_free(&(napdef->transfer_delay));
		napdef->transfer_delay = NULL;
	}
	if (napdef->validity != NULL) {
		free_cp_napdef_validity(napdef->validity);
		napdef->validity = NULL;
	}
	if (napdef->next != NULL) {
		free_cp_napdef(napdef->next);
		napdef->next = NULL;
	}

	free(napdef);
	napdef = NULL;

}

void free_cp_napdef_authinfo(DM_CP_Napdef_AuthInfo * auth_info)
{
	retm_if((auth_info) == NULL, "dm_cp napdef auth info is NULL!!");

	if (auth_info->auth_entity != NULL) {
		free_string_node(auth_info->auth_entity);
		auth_info->auth_entity = NULL;
	}
	if (auth_info->auth_name != NULL) {
		str_free(&(auth_info->auth_name));
		auth_info->auth_name = NULL;
	}
	if (auth_info->auth_secret != NULL) {
		str_free(&(auth_info->auth_secret));
		auth_info->auth_secret = NULL;
	}
	if (auth_info->auth_type != NULL) {
		str_free(&(auth_info->auth_type));
		auth_info->auth_type = NULL;
	}
	if (auth_info->spi != NULL) {
		str_free(&(auth_info->spi));
		auth_info->spi = NULL;
	}
	if (auth_info->next != NULL) {
		free_cp_napdef_authinfo(auth_info->next);
		auth_info->next = NULL;
	}

	free(auth_info);
	auth_info = NULL;
}

void free_cp_napdef_validity(DM_CP_Napdef_validity * validity)
{
	retm_if((validity) == NULL, "dm_cp napdef validity is NULL!!");

	if (validity->country != NULL) {
		str_free(&(validity->country));
		validity->country = NULL;
	}
	if (validity->network != NULL) {
		str_free(&(validity->network));
		validity->network = NULL;
	}
	if (validity->sid != NULL) {
		str_free(&(validity->sid));
		validity->sid = NULL;
	}
	if (validity->soc != NULL) {
		str_free(&(validity->soc));
		validity->soc = NULL;
	}
	if (validity->validuntil != NULL) {
		str_free(&(validity->validuntil));
		validity->validuntil = NULL;
	}
	if (validity->next != NULL) {
		free_cp_napdef_validity(validity->next);
		validity->next = NULL;
	}

	free(validity);
	validity = NULL;
}

void free_cp_bootstrap(DM_CP_Bootstrap * bootstraps)
{
	retm_if((bootstraps) == NULL, "dm_cp bootstraps is NULL!!");

	if (bootstraps->context_allow != NULL) {
		str_free(&(bootstraps->context_allow));
		bootstraps->context_allow = NULL;
	}
	if (bootstraps->country != NULL) {
		str_free(&(bootstraps->country));
		bootstraps->country = NULL;
	}
	if (bootstraps->name != NULL) {
		str_free(&(bootstraps->name));
		bootstraps->name = NULL;
	}
	if (bootstraps->provurl != NULL) {
		str_free(&(bootstraps->provurl));
		bootstraps->provurl = NULL;
	}
	if (bootstraps->network != NULL) {
		free_string_node(bootstraps->network);
		bootstraps->network = NULL;
	}
	if (bootstraps->proxy_id != NULL) {
		free_string_node(bootstraps->proxy_id);
		bootstraps->proxy_id = NULL;
	}
	if (bootstraps->next != NULL) {
		free_cp_bootstrap(bootstraps->next);
		bootstraps->next = NULL;
	}

	free(bootstraps);
	bootstraps = NULL;
}

void free_cp_pxlogical(DM_CP_PxLogical * pxLogical)
{
	retm_if((pxLogical) == NULL, "dm_cp pxlogical is NULL!!");

	if (pxLogical->bas_auth_id != NULL) {
		str_free(&(pxLogical->bas_auth_id));
		pxLogical->bas_auth_id = NULL;
	}
	if (pxLogical->bas_auth_pw != NULL) {
		str_free(&(pxLogical->bas_auth_pw));
		pxLogical->bas_auth_pw = NULL;
	}
	if (pxLogical->domain != NULL) {
		free_string_node(pxLogical->domain);
		pxLogical->domain = NULL;
	}
	if (pxLogical->master != NULL) {
		str_free(&(pxLogical->master));
		pxLogical->master = NULL;
	}
	if (pxLogical->name != NULL) {
		str_free(&(pxLogical->name));
		pxLogical->name = NULL;
	}
	if (pxLogical->port != NULL) {
		free_cp_port(pxLogical->port);
		pxLogical->port = NULL;
	}
	if (pxLogical->ppg_auth_type != NULL) {
		str_free(&(pxLogical->ppg_auth_type));
		pxLogical->ppg_auth_type = NULL;
	}
	if (pxLogical->proxy_id != NULL) {
		str_free(&(pxLogical->proxy_id));
		pxLogical->proxy_id = NULL;
	}
	if (pxLogical->proxy_provider_id != NULL) {
		str_free(&(pxLogical->proxy_provider_id));
		pxLogical->proxy_provider_id = NULL;
	}
	if (pxLogical->proxy_pw != NULL) {
		str_free(&(pxLogical->proxy_pw));
		pxLogical->proxy_pw = NULL;
	}
	if (pxLogical->pull_enabled != NULL) {
		str_free(&(pxLogical->pull_enabled));
		pxLogical->pull_enabled = NULL;
	}
	if (pxLogical->pxAuthInfo != NULL) {
		free_cp_pxlogical_pxauthinfo(pxLogical->pxAuthInfo);
		pxLogical->pxAuthInfo = NULL;
	}
	if (pxLogical->pxPhysical != NULL) {
		free_cp_pxLogical_pxphsical(pxLogical->pxPhysical);
		pxLogical->pxPhysical = NULL;
	}
	if (pxLogical->start_page != NULL) {
		str_free(&(pxLogical->start_page));
		pxLogical->start_page = NULL;
	}
	if (pxLogical->trust != NULL) {
		str_free(&(pxLogical->trust));
		pxLogical->trust = NULL;
	}
	if (pxLogical->wsp_version != NULL) {
		str_free(&(pxLogical->wsp_version));
		pxLogical->wsp_version = NULL;
	}
	if (pxLogical->next != NULL) {
		free_cp_pxlogical(pxLogical->next);
		pxLogical->next = NULL;
	}

	free(pxLogical);
	pxLogical = NULL;
}

void free_cp_pxlogical_pxauthinfo(DM_CP_PxLogical_PxAuthInfo * pxAuthInfo)
{
	retm_if((pxAuthInfo) == NULL, "dm_cp pxAuthInfo is NULL!!");

	if (pxAuthInfo->pxAuth_id != NULL) {
		str_free(&(pxAuthInfo->pxAuth_id));
		pxAuthInfo->pxAuth_id = NULL;
	}
	if (pxAuthInfo->pxAuth_pw != NULL) {
		str_free(&(pxAuthInfo->pxAuth_pw));
		pxAuthInfo->pxAuth_pw = NULL;
	}
	if (pxAuthInfo->pxAuth_type != NULL) {
		str_free(&(pxAuthInfo->pxAuth_type));
		pxAuthInfo->pxAuth_type = NULL;
	}
	if (pxAuthInfo->next != NULL) {
		free_cp_pxlogical_pxauthinfo(pxAuthInfo->next);
		pxAuthInfo->next = NULL;
	}

	free(pxAuthInfo);
	pxAuthInfo = NULL;

}

void free_cp_pxLogical_pxphsical(DM_CP_PxLogical_PxPhysical * pxPhysical)
{
	retm_if((pxPhysical) == NULL, "dm_cp pxPhysical is NULL!!");

	if (pxPhysical->domain != NULL) {
		free_string_node(pxPhysical->domain);
		pxPhysical->domain = NULL;
	}
	if (pxPhysical->physical_proxy_id != NULL) {
		str_free(&(pxPhysical->physical_proxy_id));
		pxPhysical->physical_proxy_id = NULL;
	}
	if (pxPhysical->port != NULL) {
		free_cp_port(pxPhysical->port);
		pxPhysical->port = NULL;
	}
	if (pxPhysical->pull_enabled != NULL) {
		str_free(&(pxPhysical->pull_enabled));
		pxPhysical->pull_enabled = NULL;
	}
	if (pxPhysical->push_enabled != NULL) {
		str_free(&(pxPhysical->push_enabled));
		pxPhysical->push_enabled = NULL;
	}

	if (pxPhysical->pxAddr != NULL) {
		str_free(&(pxPhysical->pxAddr));
		pxPhysical->pxAddr = NULL;
	}
	if (pxPhysical->pxAddrType != NULL) {
		str_free(&(pxPhysical->pxAddrType));
		pxPhysical->pxAddrType = NULL;
	}
	if (pxPhysical->pxAddr_FQDN != NULL) {
		str_free(&(pxPhysical->pxAddr_FQDN));
		pxPhysical->pxAddr_FQDN = NULL;
	}
	if (pxPhysical->to_napid != NULL) {
		free_string_node(pxPhysical->to_napid);
		pxPhysical->to_napid = NULL;
	}

	if (pxPhysical->wsp_version != NULL) {
		str_free(&(pxPhysical->wsp_version));
		pxPhysical->wsp_version = NULL;
	}
	if (pxPhysical->next != NULL) {
		free_cp_pxLogical_pxphsical(pxPhysical->next);
		pxPhysical->next = NULL;
	}

	free(pxPhysical);
	pxPhysical = NULL;

}
