/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_common.h"
#include "common/util/util.h"
#include "dm-engine/dl-manager/sa_fw_downloader.h"
#include "dm-engine/dl-manager/na_fw_dl_binder.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"OMA_DM_DL"
#endif

//#define DOWNLOAD_BODY_SIZE 	66304
////#define DOWNLOAD_BODY_SIZE    3592624

//#define DOWNLOAD_BODY_SIZE 	     983040
#define DOWNLOAD_BODY_SIZE 	     491520
//#define DOWNLOAD_BODY_SIZE 	     245760

#define MAX_COUNT						3

static sync_agent_na_result_e _download_data(GList * header_info, int na_plugIn_id, unsigned char *download_folder, unsigned int download_start_size,
					     unsigned int download_body_size, unsigned char **out_download_path, sync_agent_na_send_type_e send_type, unsigned int session_id, int config);
static sync_agent_na_result_e __get_data_info(void *http_req_info, void *http_info, int *total_doanload_size, char **current_download_range, char **download_file_name);
static sync_agent_na_result_e __set_data_info(void *http_info, char *current_download_range);

void free_header_info(GList * header_info)
{
	_EXTERN_FUNC_ENTER;

	if (header_info != NULL) {
		GList *iter = NULL;
		sync_agent_na_common_header_info_s *iter_data;
		for (iter = header_info; iter != NULL;) {
			iter_data = NULL;
			iter_data = ((sync_agent_na_common_header_info_s *) (iter->data));

			iter = g_list_next(iter);
			header_info = g_list_remove(header_info, iter_data);

			if (iter_data->key != NULL)
				free(iter_data->key);
			if (iter_data->value != NULL)
				free(iter_data->value);
			free(iter_data);
		}
		g_list_free(header_info);
	}

	_EXTERN_FUNC_EXIT;
}

char *get_file_name(GList * recv_header, char *url_file_name)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((recv_header) == NULL, NULL, "recv_header is NULL!!");
	retvm_if((url_file_name) == NULL, NULL, "url_file_name is NULL!!");

	char *file_name = NULL;
	file_name = get_file_name_from_http_header(recv_header);

	/*gcf server not response Content-Disposition element */
	if (file_name == NULL) {
		char *temp_ptr = NULL;
		_DEBUG_INFO("http content_disposition is null");

		temp_ptr = strrchr(url_file_name, '/');
		_DEBUG_INFO("result_file name : %s", temp_ptr);

		if (temp_ptr != NULL) {
			++temp_ptr;
			_DEBUG_INFO("remove '/' result_file name : %s", temp_ptr);
			file_name = strdup(temp_ptr);

		} else {
			_DEBUG_INFO("dd object uri file name is null");
		}
	} else {
		_DEBUG_INFO(" file name : %s", file_name);
	}

	_EXTERN_FUNC_EXIT;
	return file_name;
}

char *get_file_name_from_http_header(GList * header_info)
{
	_EXTERN_FUNC_ENTER;
	/* pasing file name */
	char *file_name = 0;
	char *value = 0;

	retvm_if((header_info) == NULL, NULL, "header_info is NULL!!");

	int ret = sync_agent_get_header_info(1, header_info, "Content-Disposition", &value);
	if (ret != 1)
		return 0;
	if (value != NULL) {
		_DEBUG_INFO("Key = %s, Value = %s\n", "Content-Disposition", value);
		char *ptr1 = strstr(value, "attachment;");
		ptr1 = ptr1+strlen("attachment;");
		if (ptr1 != NULL) {
			ptr1 = strstr(ptr1, "fileName=\"");
			ptr1 = ptr1+strlen("fileName=\"");
			if (ptr1 != NULL) {

				char *temp = strdup(ptr1);

				char *del = "\"";
				char *temp_ptr = strtok(temp, del);
				//temp_ptr = strtok(NULL, del);

				file_name = strdup(temp_ptr);
				_DEBUG_INFO("*download_file_name = %s\n", file_name);

				if (temp != NULL)
					free(temp);
			}
		}
	} else {

	}
	_EXTERN_FUNC_EXIT;
	return file_name;
}

DM_ERROR send_download_msg(DL_SEND_MSG_TYPE send_dl_type, char *local_uri, char *content_type, GList * header_info, char *sendMsg, unsigned int sendMsg_length, GList ** recv_header, char **recvMsg, unsigned int *recvMsg_length)
{
	_EXTERN_FUNC_ENTER;

	int transportType = 1;

	sync_agent_na_result_e res = SYNC_AGENT_NA_SUCCESS;
	DM_ERROR ret = DM_OK;
	int net_session_id;

	//retvm_if((sendMsg) == NULL, COMMON_ERR_IS_NULL, "sendMsg is NULL!!");

	//timeout : 30 seconds
	res = sync_agent_open_connection(transportType, 30, (unsigned int *)(&(net_session_id)));

	_DEBUG_INFO("res =%d\n", res);

	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto error;
	}

	/* header binding */
	switch (send_dl_type) {
	case GET_FILE_NAME:
		{
			add_header_info(&header_info, "Range", g_strdup_printf("bytes=0-%d", DOWNLOAD_BODY_SIZE - 1));
			/* intentionally not used "break" */
		}
	case DD_DOWNLOAD:
		{
			add_header_info(&header_info, "method", "get");
			add_header_info(&header_info, "uri", local_uri);
			add_header_info(&header_info, "Connection", "Keep-Alive");
			add_header_info(&header_info, "User-Agent", "Samsung electronics GT-I8800 SyncML_DM Client");
			add_header_info(&header_info, "Accept", content_type);
			add_header_info(&header_info, "Accept-Charset", "UTF-8");
			add_header_info(&header_info, "Content-Type", content_type);
		}
		break;
	case SEND_DOWNLOAD_STATUS:
		{
			add_header_info(&header_info, "method", "post");
			add_header_info(&header_info, "uri", local_uri);
			add_header_info(&header_info, "Content-Length", g_strdup_printf("%d", sendMsg_length));
		}
		break;
	default:
		break;
	}

	/* send msg */
	switch (send_dl_type) {
	case DD_DOWNLOAD:
	case GET_FILE_NAME:
		{
			res = sync_agent_send_msg(	/*acc_info (id, pw, uri, ...), */
							 header_info, transportType, NULL, sendMsg_length, recv_header, (unsigned char **)recvMsg, recvMsg_length, SYNC_AGENT_NA_SEND_TYPE_SEND_N_RECEIVE, net_session_id);
		}
		break;
	case SEND_DOWNLOAD_STATUS:
		{
			retvm_if((sendMsg) == NULL, COMMON_ERR_IS_NULL, "sendMsg is NULL!!");
			res = sync_agent_send_msg(	/*acc_info (id, pw, uri, ...), */
							 header_info, transportType, sendMsg, sendMsg_length, recv_header, (unsigned char **)recvMsg, recvMsg_length, SYNC_AGENT_NA_SEND_TYPE_SEND_N_RECEIVE, net_session_id);
		}
		break;
	default:
		break;
	}

	_DEBUG_INFO("res =%d\n", res);

	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto returnerror;
	}

	res = sync_agent_close_connection(transportType, net_session_id);

	_DEBUG_INFO("res =%d\n", res);
	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto error;
	}

	if (sendMsg != NULL)
		free(sendMsg);

	_EXTERN_FUNC_EXIT;
	return ret;

 returnerror:
	_DEBUG_INFO("returnerror = %d\n", ret);

	res = sync_agent_close_connection(transportType, net_session_id);

	_DEBUG_INFO("res =%d\n", res);
	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto error;
	}

	if (sendMsg != NULL)
		free(sendMsg);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO("error = %d\n", ret);

	if (sendMsg != NULL)
		free(sendMsg);

	_EXTERN_FUNC_EXIT;
	return ret;

}

DM_ERROR download_data(char *local_uri, char *content_type, GList * header_info, int current_file_download_size, char *download_folder, int config, char **download_path)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_na_result_e res = SYNC_AGENT_NA_SUCCESS;
	DM_ERROR ret = DM_OK;
	int transportType = 1;
	bool cancel_flag;
	int net_session_id = 0;
	int down_count = 0;

	retvm_if((local_uri) == NULL, COMMON_ERR_IS_NULL, "local_uri is NULL!!");
	//retvm_if((header_info) == NULL, COMMON_ERR_IS_NULL, "header_info is NULL!!");
	warn_if((header_info) == NULL, "header_info is NULL!! But this is normal case !!");
	retvm_if((download_folder) == NULL, COMMON_ERR_IS_NULL, "download_folder is NULL!!");

	//timeout : 30 seconds
	res = sync_agent_open_connection(transportType, 30, (unsigned int *)(&(net_session_id)));

	_DEBUG_INFO("res =%d\n", res);
	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto error;
	}

	add_header_info(&header_info, "method", "get");
	add_header_info(&header_info, "uri", local_uri);
	add_header_info(&header_info, "Connection", "Keep-Alive");
//      add_header_info(&header_info, "User-Agent", "Samsung electronics GT-I8800 SyncML_DM Client");
	add_header_info(&header_info, "Accept", content_type);
	add_header_info(&header_info, "Accept-Charset", "UTF-8");
	add_header_info(&header_info, "Content-Type", content_type);
	add_header_info(&header_info, "Range", g_strdup_printf("bytes=%d-%d", current_file_download_size, current_file_download_size + DOWNLOAD_BODY_SIZE - 1));

	sync_agent_register_cancel_callback(network_cancel_callback, &(net_session_id));

	//check cancel flag (note that this can be called every where inside processing logic)
	cancel_flag = sync_agent_check_cancel_flag();
	if (cancel_flag != 0) {
		ret = DM_ERR_USER_CANDELLED;
		goto returnerror;
	}

	while (down_count < MAX_COUNT) {

		res = _download_data(	/*acc_info (id, pw, uri, ...), */
					    header_info, transportType, (unsigned char *)download_folder, current_file_download_size, DOWNLOAD_BODY_SIZE, (unsigned char **)download_path, SYNC_AGENT_NA_SEND_TYPE_SEND_N_RECEIVE, net_session_id, config);

		down_count++;
		_DEBUG_INFO("response download  =%d\n", res);

		if (res != SYNC_AGENT_NA_SUCCESS) {
			if (res == SYNC_AGENT_NA_NETWORK_UNAVAILABLE) {
				sleep(1);
				if (down_count == MAX_COUNT) {
					_DEBUG_INFO("download data wifi only error");
					ret = DM_WIFI_ONLY_ERROR;
					goto returnerror;
				} else {
					_DEBUG_INFO("download data network retry : %d", down_count);
					continue;
				}
			}
			_DEBUG_INFO("download data error");
			ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
			/*f(res == SYNC_AGENT_NA_SEND_MSG_CANCEL) {
			   } */
			goto returnerror;
		} else {
			_DEBUG_INFO("----------------------------------------download success-------------------------------------------\n");
			down_count = MAX_COUNT;
			break;
		}
	}

 returnerror:
	sync_agent_unregister_cancel_callback(network_cancel_callback, &(net_session_id));

	cancel_flag = sync_agent_check_cancel_flag();
	if (cancel_flag != 0) {
		_DEBUG_INFO("----------------------------------------task cancel flag is : %d-------------------------------------------\n", cancel_flag);
		_DEBUG_INFO("----------------------------------------download cancel-------------------------------------------\n");
		if (cancel_flag != 0) {
			_DEBUG_INFO("download cancel flag");
			ret = DM_ERR_USER_CANDELLED;
		}
	}

	_DEBUG_INFO("returnerror = %d\n", ret);
	res = sync_agent_close_connection(transportType, net_session_id);

	_DEBUG_INFO("res =%d\n", res);
	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_INFO("error = %d\n", ret);
	return ret;
}

static sync_agent_na_result_e _download_data(GList * header_info, int na_plugIn_id, unsigned char *download_folder, unsigned int download_start_size,
					     unsigned int download_body_size, unsigned char **out_download_path, sync_agent_na_send_type_e send_type, unsigned int session_id, int config)
{
	_INNER_FUNC_ENTER;

	retvm_if((header_info) == NULL, COMMON_ERR_IS_NULL, "header_info is NULL!!");
	retvm_if((download_folder) == NULL, COMMON_ERR_IS_NULL, "download_folder is NULL!!");

	GList *recv_header_info = 0;
	/* char *recvMsg = 0; */
	unsigned char *recvMsg = NULL;
	unsigned int recvMsg_size = 0;

	unsigned int download_current_size = download_start_size;
	/* unsigned int download_total_size = 0; */
	int download_total_size = 0;
	char *current_download_range = NULL;
	char *download_file_name = NULL;

	int count = 0;		/* for test log */
	sync_agent_na_result_e ret = SYNC_AGENT_NA_SUCCESS;
	GList *free_list = NULL;
	sync_agent_na_common_header_info_s *free_item = NULL;
	GList *backup_send_head_info_list = NULL;
	GList *iter = NULL;

	while (download_total_size == 0 || download_current_size < download_total_size - 1) {

		_DEBUG_TRACE("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
		_DEBUG_TRACE("############# %d th download", ++count);
		_DEBUG_TRACE("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");

		/* back up the header list TODO */
		/* GSList *backup_send_head_info_list = 0; */
		backup_send_head_info_list = NULL;
		iter = NULL;
		for (iter = header_info; iter != NULL; iter = g_list_next(iter)) {
			sync_agent_na_common_header_info_s *header_item_info = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
			if (header_item_info == NULL) {
				_DEBUG_TRACE("calloc failed !!");
				ret = SYNC_AGENT_NA_DOWNLOAD_DATA_FAIL;
				goto return_part;
			}

			if (((sync_agent_na_common_header_info_s *) (iter->data))->key != NULL) {
				header_item_info->key = strdup(((sync_agent_na_common_header_info_s *) (iter->data))->key);

				if (((sync_agent_na_common_header_info_s *) (iter->data))->value != NULL) {
					header_item_info->value = strdup(((sync_agent_na_common_header_info_s *) (iter->data))->value);
				} else {
					_DEBUG_TRACE("key : %s, value is NULL !!", header_item_info->key);
					header_item_info->value = NULL;
				}

				backup_send_head_info_list = g_list_append(backup_send_head_info_list, header_item_info);

				if (strcmp(((sync_agent_na_common_header_info_s *) (iter->data))->key, "Range") == 0) {
					_DEBUG_TRACE("[Before] key : %s, value : %s", ((sync_agent_na_common_header_info_s *) (iter->data))->key, ((sync_agent_na_common_header_info_s *) (iter->data))->value);
				}

			} else {
				_DEBUG_TRACE("key is NULL !!");
				header_item_info->key = NULL;
				header_item_info->value = NULL;
				free(header_item_info);
				header_item_info = NULL;
			}
		}

		if (config == 0) {
			_DEBUG_TRACE("wifi only off mode download");
		} else if (config == 1) {
			ret = get_wifi_state();
			_DEBUG_TRACE("wifi only mode : %d", ret);
			if (ret != SYNC_AGENT_NA_SUCCESS) {
				goto free_and_return_part;
			}
		}

		/* send_msg */ // sync_agent_send_msg function include backup_send_head_info_list resource free.
		ret = sync_agent_send_msg(backup_send_head_info_list, na_plugIn_id, 0, 0, &recv_header_info, &recvMsg, &recvMsg_size, send_type, session_id);
		if (ret != SYNC_AGENT_NA_SUCCESS) {
			_DEBUG_TRACE("sync_agent_send_msg() fail");
			ret = SYNC_AGENT_NA_DOWNLOAD_DATA_FAIL;
			goto return_part;
		}

		_DEBUG_TRACE("RecvMSg_Size = %d", recvMsg_size);
		_DEBUG_TRACE("############# sync_agent_send_msg() success !!");

		/* get download total size, download current range */
		ret = __get_data_info(header_info, recv_header_info, &download_total_size, &current_download_range, &download_file_name);
		if (ret != SYNC_AGENT_NA_SUCCESS) {
			_DEBUG_TRACE("Get_Data_Down_Info() fail !!");
			ret = SYNC_AGENT_NA_DOWNLOAD_DATA_FAIL;
			goto return_part;
		}

		_DEBUG_TRACE("############# Get_Data_Down_Info() success !!");
		if (current_download_range == NULL || download_total_size == 0 || download_file_name == NULL) {
			_DEBUG_TRACE("current_download_range == NULL || download_total_size == 0 || download_file_name == NULL");
			ret = SYNC_AGENT_NA_DOWNLOAD_DATA_FAIL;
			goto return_part;
		}

		_DEBUG_TRACE("download_total_size = %d", download_total_size);
		_DEBUG_TRACE("current_download_range = %s", current_download_range);
		_DEBUG_TRACE("download_file_name = %s", download_file_name);

		/* download path setting */
		if (*out_download_path == NULL) {
			*out_download_path = (unsigned char *)g_strdup_printf("%s/%s", download_folder, download_file_name);
			_DEBUG_TRACE("*out_download_path = %s", *out_download_path);
		}

		/* increment download count */
		char *del = "-";
		/* int down_size = 0; */
		char *ptr = strtok(current_download_range, del);
		if (ptr != NULL) {

			int start_size = 0;
			int end_size = 0;

			_DEBUG_TRACE("start_size = %s", ptr);
			start_size = atoi(ptr);

			ptr = strtok(NULL, del);
			_DEBUG_TRACE("end_size = %s", ptr);
			end_size = atoi(ptr);

			_DEBUG_TRACE("prev download_current_size = %d", download_current_size);
			_DEBUG_TRACE("Gap = %d", end_size - start_size);

			download_current_size += (end_size - start_size + 1);
			_DEBUG_TRACE("after download_current_size = %d", download_current_size);
		}

		bool isFinal = false;
		if (download_current_size >= download_total_size - 1)
			isFinal = true;

		/* set file */
		_DEBUG_TRACE("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! RecvMSg_Size = %d", recvMsg_size);
		ret = sync_agent_append_file((char *)(*out_download_path), (char *)recvMsg, recvMsg_size, false);
		if (ret != SYNC_AGENT_NA_SUCCESS) {
			_DEBUG_TRACE("sync_agent_write_file() fail !!");
			ret = SYNC_AGENT_NA_DOWNLOAD_DATA_FAIL;
			goto return_part;
		} else {
			_DEBUG_TRACE("############# sync_agent_write_file() success !!");
		}

		/* set download total size, download current range */
		char *new_download_range = g_strdup_printf("%d-%d", download_current_size, download_current_size + (download_body_size - 1));
		ret = __set_data_info(header_info, new_download_range);
		if (ret != SYNC_AGENT_NA_SUCCESS) {
			_DEBUG_TRACE("Set_Data_Down_Info() fail !!");
			ret = SYNC_AGENT_NA_DOWNLOAD_DATA_FAIL;
			goto return_part;
		}

		_DEBUG_TRACE("############# Set_Data_Down_Info() success !!");
	}

	_INNER_FUNC_EXIT;
	return ret;

 return_part:

	_INNER_FUNC_EXIT;
	return ret;

 free_and_return_part:

	if (backup_send_head_info_list != NULL) {
		for (free_list = backup_send_head_info_list; free_list != NULL;) {
			free_item = (sync_agent_na_common_header_info_s *) (free_list->data);
			free_list = g_list_next(free_list);

			sync_agent_free_na_common_header_info(free_item);
		}
		g_list_free(backup_send_head_info_list);
	}

	_INNER_FUNC_EXIT;
	return ret;
}

static sync_agent_na_result_e __get_data_info(void *http_req_info, void *http_info, int *total_doanload_size, char **current_download_range, char **download_file_name)
{
	_INNER_FUNC_ENTER;

	sync_agent_na_result_e ret = SYNC_AGENT_NA_SUCCESS;

	/* pasing file name */
	char *temp_file_name = 0;	/* strdup("attachment; fileName=\"FW-20110901-13121.bin\""); */
	ret = sync_agent_get_header_info(1, (GList *) http_info, "Content-Disposition", &temp_file_name);
	if (ret == SYNC_AGENT_NA_SUCCESS) {
		_DEBUG_VERBOSE("success in sync_agent_get_header_info = %d", ret);
	} else {
		_DEBUG_VERBOSE("failed in sync_agent_get_header_info = %d", ret);
	}
	_DEBUG_VERBOSE("temp_file_name = %s\n", temp_file_name);
	if (temp_file_name) {
		char *ptr1 = strstr(temp_file_name, "attachment;");
		if (ptr1 != 0) {
			ptr1 = strstr(ptr1, "fileName=");
			if (ptr1 != 0) {

				char *temp = strdup(ptr1);

				char *del = "\"";
				char *temp_ptr = strtok(temp, del);
				_DEBUG_VERBOSE("1st token = %s", temp_ptr);

				temp_ptr = strtok(NULL, del);
				_DEBUG_VERBOSE("2nd token = %s", temp_ptr);

				*download_file_name = strdup(temp_ptr);
				_DEBUG_VERBOSE("*download_file_name = %s", *download_file_name);

				if (temp != 0)
					free(temp);
			}
		}
	} else {
		char *result_name = NULL;

		ret = sync_agent_get_header_info(1, (GList *) http_req_info, "uri", &temp_file_name);
		if (ret == SYNC_AGENT_NA_SUCCESS) {
			_DEBUG_VERBOSE("success in sync_agent_get_header_info = %d", ret);
		} else {
			_DEBUG_VERBOSE("failed in sync_agent_get_header_info = %d", ret);
		}
		_DEBUG_VERBOSE("temp_file_name = %s\n", temp_file_name);
		if (temp_file_name != NULL) {

			result_name = strrchr(temp_file_name, '/');
			_DEBUG_VERBOSE("result_file name : %s", result_name);

			if (result_name != NULL) {
				++result_name;
				_DEBUG_VERBOSE("remove '/' result_file name : %s", result_name);

				*download_file_name = strdup(result_name);
				_DEBUG_VERBOSE("*download_file_name = %s\n", *download_file_name);
			}
		}
	}
	/* parsing data range info */
	char *temp_range_value = 0;	/* strdup("bytes 0-66303/3192624"); */
	ret = sync_agent_get_header_info(1, (GList *) http_info, "Content-Range", &temp_range_value);
	if (ret == SYNC_AGENT_NA_SUCCESS) {
		_DEBUG_VERBOSE("success in sync_agent_get_header_info = %d", ret);
	} else {
		_DEBUG_VERBOSE("failed in sync_agent_get_header_info = %d", ret);
	}
	if (temp_range_value != NULL) {
		char *temp = NULL;
		temp = strdup(temp_range_value + strlen("bytes "));
		_DEBUG_VERBOSE("temp_range = %s\n", temp);
		if (temp != NULL) {
			char *del2 = "/";
			char *ptr2 = strtok(temp, del2);
			if (ptr2 != 0) {
				_DEBUG_VERBOSE("[na_http_plugIn] range = %s\n", ptr2);
				*current_download_range = strdup(ptr2);

				ptr2 = strtok(NULL, del2);
				_DEBUG_VERBOSE("[na_http_plugIn] total = %s\n", ptr2);
				*total_doanload_size = atoi(ptr2);
			}
		}
	}

	_INNER_FUNC_EXIT;
	return ret;
}

static sync_agent_na_result_e __set_data_info(void *http_info, char *current_download_range)
{
	_INNER_FUNC_ENTER;

	sync_agent_na_result_e ret = SYNC_AGENT_NA_SUCCESS;

	GList *iter = NULL;
	for (iter = (GList *) http_info; iter != NULL; iter = g_list_next(iter)) {

		if (((sync_agent_na_common_header_info_s *) (iter->data))->key != 0 && !strcmp(((sync_agent_na_common_header_info_s *) (iter->data))->key, "Range")) {

			char *value = ((sync_agent_na_common_header_info_s *) (iter->data))->value;
			if (value != NULL) {
				free(value);

				((sync_agent_na_common_header_info_s *) (iter->data))->value = g_strdup_printf("bytes=%s", current_download_range);
				_DEBUG_VERBOSE("[After] key : %s, value : %s\n", ((sync_agent_na_common_header_info_s *) (iter->data))->key, ((sync_agent_na_common_header_info_s *) (iter->data))->value);
				break;
			} else {
				_DEBUG_VERBOSE("[na_http_plugIn] value is NULL !! \n");
				ret = SYNC_AGENT_NA_GET_HEADER_INFO_FAIL;
				_INNER_FUNC_EXIT;
				return ret;
			}
		}
	}

	_INNER_FUNC_EXIT;
	return ret;
}
