/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*lib*/
#include <stdio.h>

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "framework/task/oma_dm_task_request.h"
#include "framework/task/oma_dm_task_spec.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG "TASK_SPEC"
#endif

DM_ERROR dm_init_task_request()
{
	_EXTERN_FUNC_ENTER;

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling initt_async_request_task\n");
	sync_agent_request_async_task(EC_MSG_TYPE_DM_INIT_TASK_REQUEST, 0, 0, NULL, NULL, NULL, NULL, NULL, &request_msg_id);

	_DEBUG_INFO("  init_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR dm_reset_task_request()
{
	_EXTERN_FUNC_ENTER;

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling reset_async_request_task\n");
	sync_agent_request_async_task(EC_MSG_TYPE_DM_RESET_TASK_REQUEST, 0, 0, NULL, NULL, NULL, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" reset_async_request_task: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR bootstrap_task_request()
{
	_EXTERN_FUNC_ENTER;

	int bootstrapType = 0;
	void *in_param_value_array[1] = { &bootstrapType };
	int in_param_index_array[1] = { 0 };
	sync_agent_ec_value_type_e in_param_value_type_array[1] = { SYNC_AGENT_EC_VALUE_TYPE_INT };
	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling bootstrap_async_request_task\n");

	sync_agent_request_async_task(EC_MSG_TYPE_BOOTSTRAP_TASK_REQUEST, 0, 1, in_param_index_array, in_param_value_type_array, in_param_value_array, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" bootstrap_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR dm_fumo_common_task_request(Event_Contents * pEvent_data)
{

	_EXTERN_FUNC_ENTER;

	void *in_param_value_array[1] = { &pEvent_data };
	int in_param_index_array[1] = { 0 };
	sync_agent_ec_value_type_e in_param_value_type_array[1] = { SYNC_AGENT_EC_VALUE_TYPE_STRUCT };

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling common_async_request_task\n");

	sync_agent_request_async_task(EC_MSG_TYPE_DM_FUMO_COMMON_TASK_REQUEST, 0, 1, in_param_index_array, in_param_value_type_array, in_param_value_array, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" common_async_request_task done : %d", request_msg_id);

	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR dm_fumo_ui_task_request(Event_Contents * pEvent_data)
{
	_EXTERN_FUNC_ENTER;

	void *in_param_value_array[1] = { &pEvent_data };
	int in_param_index_array[1] = { 0 };
	sync_agent_ec_value_type_e in_param_value_type_array[1] = { SYNC_AGENT_EC_VALUE_TYPE_STRUCT };

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling common_async_request_task\n");

	sync_agent_request_async_task(EC_MSG_TYPE_DM_FUMO_UI_TASK_REQUEST, 0, 1, in_param_index_array, in_param_value_type_array, in_param_value_array, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" common_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR dm_lawmo_common_task_request(Event_Contents * pEvent_data)
{
	_EXTERN_FUNC_ENTER;

	void *in_param_value_array[1] = { &pEvent_data };
	int in_param_index_array[1] = { 0 };
	sync_agent_ec_value_type_e in_param_value_type_array[1] = { SYNC_AGENT_EC_VALUE_TYPE_STRUCT };

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling common_async_request_task\n");

	sync_agent_request_async_task(EC_MSG_TYPE_DM_LAWMO_COMMON_TASK_REQUEST, 0, 1, in_param_index_array, in_param_value_type_array, in_param_value_array, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" common_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;

}

DM_ERROR genericAlert_task_request()
{
	_EXTERN_FUNC_ENTER;

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling generic_async_request_task\n");
	sync_agent_request_async_task(EC_MSG_TYPE_GENERICALERT_TASK_REQUEST, 0, 0, NULL, NULL, NULL, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" generic_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;

}

DM_ERROR fumo_serviceEngine_task_request()
{
	_EXTERN_FUNC_ENTER;

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling service_engine_async_request_task\n");
	sync_agent_request_async_task(EC_MSG_TYPE_FUMO_SERVICE_ENGINE_TASK_REQUEST, 0, 0, NULL, NULL, NULL, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" service_engine_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR lawmo_serviceEngine_task_request()
{
	_EXTERN_FUNC_ENTER;

	int request_msg_id = -1;
	/*sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN; */

	_DEBUG_INFO(" calling service_engine_async_request_task\n");
	sync_agent_request_async_task(EC_MSG_TYPE_LAWMO_SERVICE_ENGINE_TASK_REQUEST, 0, 0, NULL, NULL, NULL, NULL, NULL, &request_msg_id);

	_DEBUG_INFO(" service_engine_async_request_task done: %d", request_msg_id);
	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}
	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

DM_ERROR fumo_schedule_task_request(Reminder_Interval remider_interval_type)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	void *in_param_value_array[1] = { &remider_interval_type };
	int in_param_index_array[1] = { 0 };
	sync_agent_ec_value_type_e in_param_value_type_array[1] = { SYNC_AGENT_EC_VALUE_TYPE_INT };
	int request_msg_id = -1;
	sync_agent_ec_task_error_e task_error = SYNC_AGENT_EC_TASK_ERROR_NOT_YET_RUN;
	sync_agent_ec_uint out_param_cnt = 0;
	sync_agent_ec_param_param_s **out_param_array = NULL;

	_DEBUG_INFO(" calling fumo_schedule_sync_request_task\n");

	sync_agent_request_sync_task(EC_MSG_TYPE_FUMO_SCHEDULE_TASK_REQUEST, 0, 1, in_param_index_array, in_param_value_type_array, in_param_value_array, &request_msg_id, &task_error, &out_param_cnt, &out_param_array);

	_DEBUG_INFO(" fumo_schedule_sync_request_task done: %d", request_msg_id);

	sync_agent_get_param_value(&(out_param_array[0]->param_value), SYNC_AGENT_EC_VALUE_TYPE_INT, &ret);

	if (request_msg_id == -1) {
		_EXTERN_FUNC_EXIT;
		return DM_ERR_ASYNC_TASK;
	}

	_EXTERN_FUNC_EXIT;
	return DM_OK;
}

void cancel_task(int task_id)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_cancel_task(task_id);

	_EXTERN_FUNC_EXIT;
}
