/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <unistd.h>

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_common.h"
#include "common/util/util.h"
#include "framework/ui-event-handler/user-interaction/user_interaction.h"
#include "serviceadapter/sa_util.h"
#include "ipc_agent.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"OMA_DM_UI"
#endif

#define TIMEOUT_MIN_VAL 10
#define TIMEOUT_MAX_VAL 60
#define TIEM_ZERO "0"

void free_uic_value(uic_data ** uic_value)
{
	_EXTERN_FUNC_ENTER;

	if (uic_value != NULL) {
		if (*uic_value != NULL) {
			str_free(&((*uic_value)->display));
			if ((*uic_value)->multi_data != NULL) {
				GList *iter = NULL;
				char *data = NULL;
				for (iter = (*uic_value)->multi_data; iter != NULL; iter = g_list_next(iter)) {
					data = (char *)(iter->data);
					str_free(&(data));

				}
			}
			if ((*uic_value)->pUic_option != NULL) {
				str_free(&((*uic_value)->pUic_option->default_data));
				str_free(&((*uic_value)->pUic_option->echo_type));
				str_free(&((*uic_value)->pUic_option->input_type));
				str_free(&((*uic_value)->pUic_option->max_t));
				str_free(&((*uic_value)->pUic_option->maxlen));
				str_free(&((*uic_value)->pUic_option->min_t));
				free((*uic_value)->pUic_option);
				(*uic_value)->pUic_option = NULL;
			}
			if ((*uic_value)->res_data != NULL) {
				free_res_data((*uic_value)->res_data);
				(*uic_value)->res_data = NULL;
			}
			free((*uic_value));
			(*uic_value) = NULL;
		}
	}

	_EXTERN_FUNC_EXIT;
}

void free_res_data(uic_res_data * res_data)
{
	_EXTERN_FUNC_ENTER;

	retm_if((res_data) == NULL, "res data is NULL!!");

	res_data->status = 0;
	res_data->type = UIC_NONE_TYPE;

	str_free(&(res_data->input_text));

	if (res_data->res_multi_data != NULL) {
		GList *iter = NULL;
		char *data = NULL;
		for (iter = res_data->res_multi_data; iter != NULL; iter = g_list_next(iter)) {
			data = (char *)(iter->data);
			str_free(&(data));
		}
	}
	free(res_data);

	_EXTERN_FUNC_EXIT;
}

DM_ERROR exec_userinteration_option(char *data, uic_option ** uic_value)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	char *findValue = NULL;

	if (data == NULL) {
		_DEBUG_INFO("uic_data null");
		(*uic_value)->min_t = strdup("0");
		(*uic_value)->max_t = g_strdup_printf("%d", TIMEOUT_MAX_VAL);
		(*uic_value)->default_data = strdup("0");
		(*uic_value)->maxlen = strdup("0");
		(*uic_value)->input_type = strdup("0");
		(*uic_value)->echo_type = strdup("0");

	} else {

		ret = findStgringValue(data, UI_OP_MINDT, UI_OP_EQUAL, &findValue);
		if (findValue != NULL) {
			if (!strncmp(findValue, TIEM_ZERO, strlen(TIEM_ZERO))) {
				(*uic_value)->min_t = g_strdup_printf("%d", TIMEOUT_MIN_VAL);
			} else {
				(*uic_value)->min_t = strdup(findValue);
			}
			str_free(&findValue);
		} else {
			(*uic_value)->min_t = g_strdup_printf("%d", TIMEOUT_MIN_VAL);
		}
		_DEBUG_INFO("min time : %s", (*uic_value)->min_t);
		/*UI value setting */
		ret = findStgringValue(data, UI_OP_MAXDT, UI_OP_EQUAL, &findValue);
		if (findValue != NULL) {
			_DEBUG_INFO("max time : %s", findValue);
			if (!strncmp(findValue, TIEM_ZERO, strlen(TIEM_ZERO))) {
				(*uic_value)->max_t = g_strdup_printf("%d", TIMEOUT_MAX_VAL);
			} else {
				(*uic_value)->max_t = strdup(findValue);
			}
			str_free(&findValue);
		} else {
			(*uic_value)->max_t = g_strdup_printf("%d", TIMEOUT_MAX_VAL);
		}
		_DEBUG_INFO("max time : %s", (*uic_value)->max_t);
		/*UI value setting */
		ret = findStgringValue(data, UI_OP_DR, UI_OP_EQUAL, &findValue);
		if (findValue != NULL) {
			_DEBUG_INFO("default data: %s", findValue);
			(*uic_value)->default_data = strdup(findValue);
			str_free(&findValue);
		} else {
			(*uic_value)->default_data = strdup("0");
		}
		/*UI value setting */
		ret = findStgringValue(data, UI_OP_MAXLEN, UI_OP_EQUAL, &findValue);
		if (findValue != NULL) {
			_DEBUG_INFO("max len : %s", findValue);
			(*uic_value)->maxlen = strdup(findValue);
			str_free(&findValue);
		} else {
			(*uic_value)->maxlen = strdup("0");
		}
		/*UI value setting */
		ret = findStgringValue(data, UI_OP_IT, UI_OP_EQUAL, &findValue);
		if (findValue != NULL) {
			_DEBUG_INFO("input type : %s", findValue);
			(*uic_value)->input_type = strdup(findValue);
			str_free(&findValue);
		} else {
			(*uic_value)->input_type = strdup("0");
		}

		/*UI value setting */
		ret = findStgringValue(data, UI_OP_ET, UI_OP_EQUAL, &findValue);
		if (findValue != NULL) {
			_DEBUG_INFO("echo_type : %s", findValue);
			(*uic_value)->echo_type = strdup(findValue);
			str_free(&findValue);
		} else {
			(*uic_value)->echo_type = strdup("0");
		}
	}
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR ui_display(uic_data * uic_value)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((uic_value) == NULL, COMMON_ERR_IS_NULL, "uic_value is NULL!!");

	int ui_ret = 0;

	/* ui_ret = launch_om_dm_fumo_alert_ui();
	   if(ui_ret != 1) {
	   ret = DM_ALERT_UI_LAUNCH_ERROR;
	   goto error;
	   } else {
	   sleep(1);
	 */
	ui_ret = alert_uic_display(uic_value);
	if (ui_ret != 1) {
		uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
		_DEBUG_INFO("display alert fail : %d", DM_ERR_NOT_EXECUTED);
	} else {
		uic_value->res_data->status = DM_OK;
		_DEBUG_INFO("display alert success : %d", uic_value->res_data->status);
	}
	/*} */

	_EXTERN_FUNC_EXIT;
	return ret;
/*error:
	_EXTERN_FUNC_EXIT;
	return ret;*/
}

DM_ERROR ui_confirm_or_reject(uic_data * uic_value)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((uic_value) == NULL, COMMON_ERR_IS_NULL, "uic_value is NULL!!");

	/* cancel -> atomic, sequence */
	int ui_ret = 0;
	/* ui_ret = launch_om_dm_fumo_alert_ui();
	   if(ui_ret != 1) {
	   ret = DM_ALERT_UI_LAUNCH_ERROR;
	   goto error;
	   } else {
	   sleep(1);
	 */
	ui_ret = alert_uic_confirmation(uic_value);

	if (ui_ret == 1) {
		//ok
		uic_value->res_data->status = DM_OK;
		_DEBUG_INFO("display alert success : %d", DM_OK);
	} else {
		/*calcel or app launch fail or timeout */
		uic_value->res_data->status = DM_ERR_NOT_MODIFIED;
		_DEBUG_INFO("confirm alert fail : %d", DM_ERR_NOT_MODIFIED);
	}
	/*} */

	_EXTERN_FUNC_EXIT;
	return ret;
/*error:
	_EXTERN_FUNC_EXIT;
	return ret;*/
}

DM_ERROR ui_text_input(uic_data * uic_value, char **result_data)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((uic_value) == NULL, COMMON_ERR_IS_NULL, "uic_value is NULL!!");

	int ui_ret = 0;
	/* ui_ret = launch_om_dm_fumo_alert_ui();
	   if(ui_ret != 1) {
	   ret = DM_ALERT_UI_LAUNCH_ERROR;
	   goto error;
	   } else {
	   sleep(1);
	 */
	ui_ret = alert_uic_inputText(uic_value);

	if (ui_ret == 1) {

		if (uic_value->res_data->input_text != NULL) {
			(*result_data) = strdup(uic_value->res_data->input_text);
			uic_value->res_data->status = DM_OK;
			_DEBUG_INFO("test input alert success : %d", DM_OK);
		} else {
			(*result_data) = strdup("NULL");
			uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
			_DEBUG_INFO("input alert fail : %d", DM_ERR_NOT_EXECUTED);
		}
	} else {
		/*calcel or app launch fail */
		(*result_data) = strdup("NULL");
		uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
		_DEBUG_INFO("input alert fail : %d", DM_ERR_NOT_EXECUTED);
	}
	/* } */

	_EXTERN_FUNC_EXIT;
	return ret;
/*error:
	_EXTERN_FUNC_EXIT;
	return ret;*/
}

DM_ERROR ui_single_choice(uic_data * uic_value, GList ** responseItems)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((uic_value) == NULL, COMMON_ERR_IS_NULL, "uic_value is NULL!!");

	int ui_ret = 0;
	/* ui_ret = launch_om_dm_fumo_alert_ui();
	   if(ui_ret != 1) {
	   ret = DM_ALERT_UI_LAUNCH_ERROR;
	   goto error;
	   } else {
	   sleep(1);
	 */
	ui_ret = alert_uic_single_choice(uic_value);
	if (ui_ret == 1) {
		if (uic_value->res_data->res_multi_data != NULL) {
			GList *iter = NULL;
			for (iter = uic_value->res_data->res_multi_data; iter != NULL; iter = g_list_next(iter)) {
				char *data = NULL;
				_DEBUG_INFO("choice data : %s", (char *)(iter->data));
				data = strdup((char *)(iter->data));
				(*responseItems) = g_list_append((*responseItems), data);
			}
			uic_value->res_data->status = DM_OK;
			_DEBUG_INFO("single choice alert success : %d", DM_OK);
		} else {
			/*calcel or app launch fail */
			char *data = NULL;
			data = strdup("NULL");
			(*responseItems) = g_list_append((*responseItems), data);
			uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
			_DEBUG_INFO("single choice alert fail : %d", DM_ERR_NOT_EXECUTED);
		}
	} else {
		/*calcel or app launch fail or timeout */
		char *data = NULL;
		data = strdup("NULL");
		(*responseItems) = g_list_append((*responseItems), data);
		uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
		_DEBUG_INFO("single choice alert fail : %d", DM_ERR_NOT_EXECUTED);
	}
	//}

	_EXTERN_FUNC_EXIT;
	return ret;
/*error:
	_EXTERN_FUNC_EXIT;
	return ret;*/
}

DM_ERROR ui_multiple_choice(uic_data * uic_value, GList ** responseItems)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((uic_value) == NULL, COMMON_ERR_IS_NULL, "uic_value is NULL!!");

	int ui_ret = 0;
	/* ui_ret = launch_om_dm_fumo_alert_ui();
	   if(ui_ret != 1) {
	   ret = DM_ALERT_UI_LAUNCH_ERROR;
	   goto error;
	   } else { */
	sleep(1);
	ui_ret = alert_uic_multiple_choice(uic_value);
	if (ui_ret == 1) {
		if (uic_value->res_data->res_multi_data != NULL) {
			GList *iter = NULL;
			for (iter = uic_value->res_data->res_multi_data; iter != NULL; iter = g_list_next(iter)) {
				char *data = NULL;
				_DEBUG_INFO("choice data : %s", (char *)(iter->data));
				data = strdup((char *)(iter->data));
				(*responseItems) = g_list_append((*responseItems), data);
			}
			uic_value->res_data->status = DM_OK;
			_DEBUG_INFO("multiple choice alert success : %d", DM_OK);
		} else {
			/*calcel or app launch fail */
			char *data = NULL;
			data = strdup("NULL");
			(*responseItems) = g_list_append((*responseItems), data);
			uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
			_DEBUG_INFO("multiple choice alert fail : %d", DM_ERR_NOT_EXECUTED);
		}

	} else {
		/*calcel or app launch fail */
		char *data = NULL;
		data = strdup("NULL");
		(*responseItems) = g_list_append((*responseItems), data);
		uic_value->res_data->status = DM_ERR_NOT_EXECUTED;
		_DEBUG_INFO("multiple choice alert fail : %d", DM_ERR_NOT_EXECUTED);
	}
//       }

	_EXTERN_FUNC_EXIT;
	return ret;
/*error:

	_EXTERN_FUNC_EXIT;
	return ret;*/
}
