/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*lib*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <glib.h>
#include <sys/stat.h>

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_common.h"
#include "common/util/util.h"
#include "mo-handler/dm_mo_common.h"
#include "mo-handler/dm_mo_handler.h"
#include "serviceadapter/sa_session.h"
#include "serviceadapter/sa_elements_internal.h"
#include "serviceadapter/sa_elements.h"
#include "serviceadapter/protocolbinder/syncml_def.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG "MO_ENGINE"
#endif

#define DEFAULT_ACL "Get=*&Replace=*&Delete=*"

/*static void __get_mo(sync_agent_dm_mo_type_e type, char *mo_full_path);*/
/*static sync_agent_dm_mo_type_e get_motype(char* full_path);*/
/*static DM_ERROR __map_mottoitem(sync_agent_dm_mo_node_s* mo_node, Item **pItem);*/

static sync_agent_dm_mo_node_type_e _get_mo_node_type(char *format, char *value);
static DM_ERROR _get_child_mo(char *mo_full_path, GList ** mo_node_list);
static DM_ERROR _get_descendant_mo(char *mo_full_path, sync_agent_dm_mo_node_s ** root_node, int *count);
static DM_ERROR _map_mo_listtoitemwithdata(GList * mo_node_list, GList ** pItems);
static DM_ERROR _map_mo_treetoitemwithdata(sync_agent_dm_mo_node_s * mo_root, int count, GList ** pItems);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 /* mo test interface */
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*DM_ERROR pre_define_mo()
{
	_DEBUG_INFO( " start!!\n");

	sync_agent_dm_mo_error_e mo_ret  = sync_agent_begin_transaction_mo();
	_DEBUG_INFO( " transaction begin : %d \n", mo_ret);

	//dmacc_define_fumo();
	//dmacc_define_lawmo();
	//devinfo_define();
	devdetail_define();
	fumo_define();
	lawmo_define();

	mo_ret = sync_agent_end_transaction_mo(SYNC_AGENT_DM_MO_TRANSACTION_COMMIT);
	_DEBUG_INFO( " transaction end : %d \n",  __func__,mo_ret);
	_DEBUG_INFO( " end!!\n");

	return 0;
}*/

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 /* mo handler interface */
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DM_ERROR get_mo_node_format_contenttype(const char *mo_path, char **format, char **contenttype)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((mo_path) == NULL, COMMON_ERR_IS_NULL, "mo_path is NULL!!");

	sync_agent_dm_mo_error_e err;
	sync_agent_dm_mo_node_s *mo_node = NULL;

	err = sync_agent_get_mo_item(mo_path, &mo_node);
	//err = sync_agent_get_mo(mo_type, mo_path, &mo_node, SYNC_AGENT_DM_MO_GET_OPTION_ALL);
	if (err != SYNC_AGENT_DM_MO_SUCCESS) {
		ret = COMMON_ERR_GET_TYPE_NOT_FOUND;
		goto error;
	}

	if (mo_node->runtime_property != NULL) {
		sync_agent_dm_mo_format_e format_enum;
		format_enum = mo_node->runtime_property->format;
		ret = get_format(format_enum, format);
		_DEBUG_INFO("format :%d , : %s", format_enum, *format);
		if (ret != DM_OK) {
			goto error;
		}

		sync_agent_dm_mo_df_type_e contenttype_enum;
		contenttype_enum = mo_node->runtime_property->type;
		_DEBUG_INFO("df type : %d", contenttype_enum);
		if (mo_node->runtime_property->type_value != NULL) {
			(*contenttype) = strdup(mo_node->runtime_property->type_value);
			_DEBUG_INFO("content type : %s", *contenttype);
		}

		/*ret = get_contenttype(contenttype_enum, contenttype);
		   if(ret != DM_OK)
		   goto error; */
	} else {
		/*do nothing */
	}

	err = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO("free mo: %d", err);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	err = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO("free mo : %d", err);

	_EXTERN_FUNC_EXIT;
	return ret;

}

DM_ERROR get_mo_node_property(const char *mo_path, propoerty_type pr_type, char **property)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_path) == NULL, COMMON_ERR_IS_NULL, "mo_path is NULL!!");

	_DEBUG_INFO(" start!!, full_path : %s", mo_path);
	DM_ERROR ret = DM_OK;

	sync_agent_dm_mo_error_e err;
	sync_agent_dm_mo_node_s *mo_node = NULL;
	err = sync_agent_get_mo_items(mo_path, &mo_node);
	//err = sync_agent_get_mo(mo_type, mo_path, &mo_node, SYNC_AGENT_DM_MO_GET_OPTION_ALL);
	if (err != SYNC_AGENT_DM_MO_SUCCESS) {
		ret = COMMON_ERR_GET_TYPE_NOT_FOUND;
		goto error;
	}

	if (mo_node != NULL) {
		if (mo_node->runtime_property != NULL) {
			switch (pr_type) {
			case PROP_TYPE_ACL:
				{
					sync_agent_dm_mo_node_s *iter = NULL;
					char *temp_value = NULL;
					int len = 0;
					temp_value = calloc(1024, sizeof(char));
					if (temp_value == NULL) {
						ret = COMMON_ERR_IS_NULL;
						goto error;
					}
					for (iter = mo_node; iter != NULL; iter = iter->next_node) {
						_DEBUG_INFO("node iter");
						if (iter->runtime_property != NULL && iter->runtime_property->acl != NULL) {
							len = g_strlcat(temp_value, iter->runtime_property->acl, 1024);
							_DEBUG_INFO("temp acl value length : %d", len);
							//(*property) = strdup(mo_node->runtime_property->acl);
							if (iter->next_node != NULL) {
								len = g_strlcat(temp_value, "&", 1024);
								_DEBUG_INFO("temp acl value length : %d", len);
							}
							_DEBUG_INFO("acl : %s", temp_value);

						} else {
							_DEBUG_INFO("not existed acl");
						}
					}
					if (strlen(temp_value) > 0) {
						*property = strdup(temp_value);
						_DEBUG_INFO("acl : %s", *property);
					} else {
						*property = strdup("NULL");
					}

					str_free(&temp_value);
				}
				break;
			case PROP_TYPE_FORMAT:
				{
					sync_agent_dm_mo_format_e format_enum;
					format_enum = mo_node->runtime_property->format;
					ret = get_format(format_enum, property);
					_DEBUG_INFO("format : %s", *property);

					break;
			case PROP_TYPE_SIZE:
					{
						if (mo_node->runtime_property->size != NULL) {
							(*property) = strdup(mo_node->runtime_property->size);
							_DEBUG_INFO("size : %s", *property);
						} else {
							_DEBUG_INFO(" size 0");
							(*property) = strdup("0");
							/*ret = COMMON_ERR_GET_TYPE_NOT_FOUND; */
							/*do nothing */
						}
					}
					break;
			case PROP_TYPE_TYPE:
					{
						sync_agent_dm_mo_df_type_e contenttype_enum;
						contenttype_enum = mo_node->runtime_property->type;
						_DEBUG_INFO("df type : %d", contenttype_enum);
						if (mo_node->runtime_property->type_value != NULL) {
							(*property) = strdup(mo_node->runtime_property->type_value);
							_DEBUG_INFO("content type : %s", *property);
						} else {
							_DEBUG_INFO("content type is not existed");
							(*property) = strdup("MIME");
							_DEBUG_INFO("content type : %s", *property);
						}
					}
					break;
			case PROP_TYPE_NAME:
					{
						if (mo_node->runtime_property->name != NULL) {
							(*property) = strdup(mo_node->runtime_property->name);
							_DEBUG_INFO("name : %s", *property);
						} else {
							_DEBUG_INFO("name is not existed");
							(*property) = strdup(mo_path);
							/*ret = COMMON_ERR_GET_TYPE_NOT_FOUND; */
							/*do nothing */
						}
					}
					break;
			default:
					break;
				}
			}
		} else {
			/*do nothing */
			_DEBUG_INFO("not existed runtime property");
			ret = COMMON_ERR_GET_TYPE_NOT_FOUND;
		}
	} else {
		/*do nothing */
		_DEBUG_INFO("not existed mo node");
		ret = COMMON_ERR_GET_TYPE_NOT_FOUND;
	}

	err = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO("free %d", err);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	err = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO("free %d", err);

	_DEBUG_INFO(" end!! error : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR clean_dm_mo()
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	sync_agent_dm_mo_error_e error;

	error = sync_agent_delete_all_mo_table();
	if (error != SYNC_AGENT_DM_MO_SUCCESS) {
		ret = COMMON_ERR_DELETE;
		goto error;
	}

	_DEBUG_INFO(" end!!\n");
	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR replace_itemtomo(Item * item)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((item) == NULL, COMMON_ERR_IS_NULL, "item is NULL!!");

	char *target_url = NULL;
	char *value = NULL;
	char *contenttype = NULL;
	char *format = NULL;

	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_type_e mo_type;

	GetType prop_type = GET_TYPE_NOMAL;
	char *real_full_path = NULL;

	target_url = (char *)get_location_locuri(item->target);
	if (item->private.data != NULL) {
		value = item->private.data;
		_DEBUG_INFO("replace value : %s", value);
	}
	if (item->contenttype != NULL) {
		contenttype = item->contenttype;
		_DEBUG_INFO("contenttype value : %s", contenttype);
	}
	if (item->format != NULL) {
		format = item->format;
		_DEBUG_INFO("format value : %s", format);
	}

	err_code = sync_agent_get_mo_type(target_url, &mo_type);
	_DEBUG_INFO("get mo type result : %d", err_code);

	if (contenttype != NULL && value != NULL) {
		if (strncmp(contenttype, ELEMENT_DMTNDS_XML, strlen(ELEMENT_DMTNDS_XML)) == 0) {
			err_code = sync_agent_import_tnds_stream(SYNC_AGENT_DM_MO_TNDS_ACTION_REPLACE, SYNC_AGENT_DM_MO_TNDS_TYPE_XML, mo_type, target_url, value, strlen(value));
			goto return_part;
		} else if (strncmp(contenttype, ELEMENT_DMTNDS_WBXML, strlen(ELEMENT_DMTNDS_WBXML)) == 0) {
			err_code = sync_agent_import_tnds_stream(SYNC_AGENT_DM_MO_TNDS_ACTION_REPLACE, SYNC_AGENT_DM_MO_TNDS_TYPE_WBXML, mo_type, target_url, value, strlen(value));
			goto return_part;
		}
	} else {
		_DEBUG_INFO("have not content type");
	}

	ret = get_type_mo_tree(target_url, &prop_type, &real_full_path);
	if (ret != DM_OK)
		goto error;

	_DEBUG_INFO("mo tree type : %d", prop_type);
	switch (prop_type) {
	case GET_TYPE_ACL_PROP:
		{
			_DEBUG_INFO("update acl property type");
			ret = replace_mo_property(real_full_path, NULL, value, NULL, NULL, prop_type);
			if (ret != DM_OK)
				goto error;
		}
		break;
	case GET_TYPE_SIZE_PROP:
	case GET_TYPE_TYPE_PROP:
	case GET_TYPE_FORMAT_PROP:
	case GET_TYPE_NAME_PROP:
		break;
	default:
		{
			_DEBUG_INFO("update value");
			ret = replace_mo(target_url, value, NULL, contenttype, format, prop_type);
			if (ret != DM_OK)
				goto error;
		}
		break;
	}

 return_part:
	str_free(&(real_full_path));
	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	str_free(&(real_full_path));
	_DEBUG_INFO(" error end %d \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;

}

DM_ERROR replace_mo_data(char *mo_full_path, char *value)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

	_DEBUG_INFO("target : %s , value : %s \n", mo_full_path, value);

	DM_ERROR ret = DM_OK;

	ret = replace_mo(mo_full_path, value, NULL, NULL, NULL, GET_TYPE_NOMAL);
	if (ret != DM_OK)
		goto error;

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error end %d \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR replace_mo_property(char *mo_full_path, char *value, char *prop_value, char *contenttype, char *format, GetType prop_type)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

	_DEBUG_INFO(" target : %s , value : %s \n", mo_full_path, prop_value);

	DM_ERROR ret = DM_OK;

	ret = replace_mo(mo_full_path, value, prop_value, contenttype, format, prop_type);
	if (ret != DM_OK)
		goto error;

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error end %d \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR replace_mo(char *mo_full_path, char *value, char *prop_value, char *contenttype, char *format, GetType prop_type)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

	_DEBUG_INFO("target : %s \n", mo_full_path);

	DM_ERROR ret = DM_OK;

	sync_agent_dm_mo_node_s *mo_node = NULL;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_format_e format_enum = SYNC_AGENT_DM_MO_FORMAT_NO_SET;

	err_code = sync_agent_get_mo_item(mo_full_path, &mo_node);
	//err_code = sync_agent_get_mo(type, mo_full_path, &mo_node, SYNC_AGENT_DM_MO_GET_OPTION_ALL);
	if (mo_node == NULL) {
		ret = COMMON_ERR_INTERNAL_NOT_DEFINED;
		goto error;
	}

	if (value != NULL) {
		if (mo_node->value != NULL) {
			free(mo_node->value);
			mo_node->value = NULL;
		} else {
			_DEBUG_INFO(" Not existed mo node value ");
		}
		mo_node->value = strdup(value);
		_DEBUG_INFO(" mo  value  : %s", value);

		if (mo_node->runtime_property != NULL) {
			if (mo_node->runtime_property->size != NULL) {
				free(mo_node->runtime_property->size);
				mo_node->runtime_property->size = NULL;
			} else {
				_DEBUG_INFO(" Not existed mo node runtime property size ");
			}
			int size = 0;
			size = (strlen(value) * sizeof(char));
			mo_node->runtime_property->size = g_strdup_printf("%d", size);
			_DEBUG_INFO(" mo  value size  : %s", mo_node->runtime_property->size);
		}
	} else {
		_DEBUG_INFO(" Not existed value");
	}

	switch (prop_type) {
	case GET_TYPE_ACL_PROP:{
			if (mo_node->runtime_property != NULL) {
				if (mo_node->runtime_property->acl != NULL) {
					free(mo_node->runtime_property->acl);
					mo_node->runtime_property->acl = NULL;
				} else {
					_DEBUG_INFO(" Not existed mo node runtim property acl ");
				}
				mo_node->runtime_property->acl = strdup(prop_value);
				_DEBUG_INFO("  acl value   : %s", prop_value);
			} else {
				_DEBUG_INFO(" Not existed mo node runtim property ");
			}
		}
		break;
	case GET_TYPE_TYPE_PROP:
	case GET_TYPE_NAME_PROP:
	case GET_TYPE_SIZE_PROP:
	case GET_TYPE_FORMAT_PROP:{
			_DEBUG_INFO("do not replace format");
		}
		break;
	default:
		break;
	}

	if (contenttype != NULL) {
		if (mo_node->runtime_property != NULL) {
			if (mo_node->runtime_property->type_value != NULL) {
				free(mo_node->runtime_property->type_value);
				mo_node->runtime_property->type_value = NULL;
			} else {
				_DEBUG_INFO(" Not existed mo node runtim property contenttype ");
			}
			mo_node->runtime_property->type_value = strdup(contenttype);
			_DEBUG_INFO("  contenttype value   : %s", contenttype);
		} else {
			_DEBUG_INFO(" Not existed contenttype ");
		}
	}

	if (format != NULL) {
		if (mo_node->runtime_property != NULL) {
			ret = get_format_enum(format, &format_enum);
			mo_node->runtime_property->format = format_enum;
		}
	}

	err_code = sync_agent_update_mo_item(mo_node);
	//err_code = sync_agent_update_mo(mo_node);
	if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_INFO(" Success to sync_agent_update_mo_item()\n");
	} else {
		_DEBUG_INFO("  Failed to sync_agent_update_mo_item()\n");
		goto error;
	}

	err_code = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO("free : %d", err_code);
	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO("free : %d", err_code);
	_DEBUG_INFO(" error end %d \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR add_itemtomo(Item * item, char *server_id)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((item) == NULL, COMMON_ERR_IS_NULL, "item is NULL!!");

	char *target_url = NULL;
	char *value = NULL;
	char *contenttype = NULL;
	char *format = NULL;

	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	int server_type = SYNC_AGENT_DM_MO_TYPE_NO_TYPE;
	sync_agent_dm_mo_node_type_e node_type = SYNC_AGENT_DM_MO_NODE_INTERIOR;

	target_url = (char *)get_location_locuri(item->target);
	if (item->private.data != NULL) {
		value = item->private.data;
		_DEBUG_INFO("get mo value : %s", value);
	}
	if (item->contenttype != NULL) {
		contenttype = item->contenttype;
		_DEBUG_INFO("get mo contenttype : %s", contenttype);
	}
	if (item->format != NULL) {
		format = item->format;
		_DEBUG_INFO("get mo format : %s", format);
	}

	node_type = _get_mo_node_type(format, value);
	_DEBUG_INFO("get mo node type : %d", node_type);

	ret = get_server_type(server_id, &server_type);
	_DEBUG_INFO("get server type : %d, result : %d", server_type, ret);
	if (ret != DM_OK) {
		_DEBUG_INFO("get server error : %d", ret);
	}

	if (contenttype != NULL && value != NULL) {
		if (strncmp(contenttype, ELEMENT_DMTNDS_XML, strlen(ELEMENT_DMTNDS_XML)) == 0) {
			err_code = sync_agent_import_tnds_stream(SYNC_AGENT_DM_MO_TNDS_ACTION_ADD, SYNC_AGENT_DM_MO_TNDS_TYPE_XML, server_type, target_url, value, strlen(value));
			goto return_part;
		} else if (strncmp(contenttype, ELEMENT_DMTNDS_WBXML, strlen(ELEMENT_DMTNDS_WBXML)) == 0) {
			err_code = sync_agent_import_tnds_stream(SYNC_AGENT_DM_MO_TNDS_ACTION_ADD, SYNC_AGENT_DM_MO_TNDS_TYPE_WBXML, server_type, target_url, value, strlen(value));
			goto return_part;
		}
	}

	ret = add_mo(server_type, target_url, value, node_type, contenttype, format);
	if (ret != DM_OK)
		goto error;

 return_part:
	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error end %d ", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR add_mo(int server_type, char *mo_full_path, char *value, sync_agent_dm_mo_node_type_e node_type, char *contenttype, char *format)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

	DM_ERROR ret = DM_OK;
	sync_agent_dm_mo_error_e mo_error = SYNC_AGENT_DM_MO_FAIL;
	sync_agent_dm_mo_node_s *sync_agent_mo_item = NULL;
	sync_agent_dm_mo_format_e format_enum = SYNC_AGENT_DM_MO_FORMAT_NO_SET;

	mo_error = sync_agent_create_mo_item(&sync_agent_mo_item);
	if (mo_error == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_INFO("Success to sync_agent_create_mo()\n");
	} else {
		_DEBUG_INFO("Failed to sync_agent_create_mo() [%d] \n", mo_error);
		ret = COMMON_ERR_DELETE;
		goto error;
	}

	/*setting mo node value */
	sync_agent_mo_item->type = node_type;
	//sync_agent_mo_item->mo_type = server_type;
	sync_agent_mo_item->server_type = server_type;
	_DEBUG_INFO("node type : %d", node_type);
	_DEBUG_INFO("server type : %d", server_type);

	sync_agent_mo_item->full_path = strdup(mo_full_path);

	if (value != NULL) {
		sync_agent_mo_item->value = strdup(value);
		int size = 0;
		size = strlen(value) * sizeof(char);
		sync_agent_mo_item->runtime_property->size = g_strdup_printf("%d", size);
	}
	/*============================================================================*/

	/*setting mo node runtime property value */
	if (contenttype != NULL) {
		sync_agent_mo_item->runtime_property->type_value = strdup(contenttype);
	}
	if (format != NULL) {
		ret = get_format_enum(format, &format_enum);
		sync_agent_mo_item->runtime_property->format = format_enum;
	}
	sync_agent_mo_item->runtime_property->acl = strdup(DEFAULT_ACL);
	/*============================================================================*/

	/*setting mo node framework property value */
	/* for gcf 2101 test : samsung requested default accessType value */
	sync_agent_mo_item->framework_property->accessType = SYNC_AGENT_DM_MO_ACCESSTYPE_ADD |
	    SYNC_AGENT_DM_MO_ACCESSTYPE_COPY | SYNC_AGENT_DM_MO_ACCESSTYPE_DELETE | SYNC_AGENT_DM_MO_ACCESSTYPE_EXEC | SYNC_AGENT_DM_MO_ACCESSTYPE_GET | SYNC_AGENT_DM_MO_ACCESSTYPE_REPLACE;

	sync_agent_mo_item->framework_property->scope = SYNC_AGENT_DM_MO_SCOPE_DYNAMIC;
	/*============================================================================*/

	mo_error = sync_agent_add_mo_item(sync_agent_mo_item);
	//mo_error = sync_agent_add_mo(mo_type, mo_full_path, &mo_node);
	if (mo_error == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_INFO("Success to sync_agent_add_mo_item()\n");
	} else {
		_DEBUG_INFO("Failed to sync_agent_add_mo_item() [%d] \n", mo_error);
		ret = COMMON_ERR_DELETE;
		goto error;
	}

	mo_error = sync_agent_free_mo_item(sync_agent_mo_item);
	_DEBUG_INFO("free_mo_item : %d", mo_error);

	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	mo_error = sync_agent_free_mo_item(sync_agent_mo_item);
	_DEBUG_INFO("free_mo_item : %d", mo_error);

	_DEBUG_INFO(" end error %d,  db add error  %d\n", ret, mo_error);
	_EXTERN_FUNC_EXIT;
	return ret;

}

DM_ERROR delete_itemtomo(Item * item)
{
	_EXTERN_FUNC_ENTER;
	_DEBUG_INFO(" start ");

	DM_ERROR ret = DM_OK;

	retvm_if((item) == NULL, COMMON_ERR_IS_NULL, "item is NULL!!");

	char *targetUrl = NULL;
	targetUrl = (char *)get_location_locuri(item->target);

	ret = delete_mo(targetUrl);
	if (ret != DM_OK)
		goto error;

	_DEBUG_INFO(" end!! \n");
	return ret;
 error:
	_DEBUG_INFO(" error end %d ", ret);
	return ret;
}

DM_ERROR delete_mo(char *mo_full_path)
{
	_EXTERN_FUNC_ENTER;
	DM_ERROR ret = DM_OK;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

	sync_agent_dm_mo_error_e err_code = sync_agent_delete_mo_item(mo_full_path);

	if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_INFO("Success to sync_agent_delete_mo()\n");
	} else {
		_DEBUG_INFO("Failed to sync_agent_delete_mo() [%d] \n", err_code);
		ret = COMMON_ERR_DELETE;
		goto error;
	}

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error end %d ", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

static DM_ERROR _get_child_mo(char *mo_full_path, GList ** mo_node_list)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

/*	ret = sync_agent_open_mo();
	_DEBUG_TRACE("mo open : %d", ret);*/
	sync_agent_dm_mo_node_s *mo_node = NULL;
	sync_agent_dm_mo_item_s sync_agent_item;

	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_type_e mo_type;
	err_code = sync_agent_get_mo_type(mo_full_path, &mo_type);
	_DEBUG_TRACE("get mo type result : %d", err_code);
	_DEBUG_TRACE("mo full path  : %s", mo_full_path);
	_DEBUG_TRACE("mo type : %d", mo_type);

	sync_agent_item.interface_type = MO_ITEM_CHILD_MO_VALUE_LIST;
	sync_agent_item.mo_path = mo_full_path;
	sync_agent_item.option = SYNC_AGENT_DM_MO_GET_OPTION_ALL;
	sync_agent_item.mo_type = mo_type;

	err_code = sync_agent_query_mo_item(&sync_agent_item, mo_node_list);
	//err_code = sync_agent_get_child_mo_list(type, mo_full_path, mo_node_list,     count, SYNC_AGENT_DM_MO_GET_OPTION_ALL);
	if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_TRACE("Success to sync_agent_get_child_mo_list()\n");
	} else {
		_DEBUG_TRACE("Failed to sync_agent_get_child_mo_list()\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	}

	if ((*mo_node_list) == NULL) {
		_DEBUG_TRACE("Not Existed childe node!!\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	}

	GList *iter = NULL;

	for (iter = *mo_node_list; iter != NULL; iter = g_list_next(iter)) {
		mo_node = (sync_agent_dm_mo_node_s *) (iter->data);

		_DEBUG_TRACE("mo_node->id : %d\n", mo_node->id);
		_DEBUG_TRACE(" mo_node->parent_id : %d\n", mo_node->parent_id);
		_DEBUG_TRACE("mo_node->name : %s\n", mo_node->name);
		_DEBUG_TRACE("mo_node->value : %s\n", mo_node->value);
		_DEBUG_TRACE("mo_node->full_path : %s\n", mo_node->full_path);
		_DEBUG_TRACE("mo_node->type : %d\n\n", mo_node->type);

		_DEBUG_TRACE(" framework_property->accessType : %d\n", mo_node->framework_property->accessType);
		_DEBUG_TRACE(" framework_property->defaultValue : %s\n", mo_node->framework_property->defaultValue);
		_DEBUG_TRACE(" framework_property->description : %s\n", mo_node->framework_property->description);
		_DEBUG_TRACE(" framework_property->dffFormat : %d\n", mo_node->framework_property->dffFormat);
		_DEBUG_TRACE(" framework_property->occurrence : %d\n", mo_node->framework_property->occurrence);
		_DEBUG_TRACE(" framework_property->scope : %d\n", mo_node->framework_property->scope);
		_DEBUG_TRACE(" framework_property->dfTitle : %s\n", mo_node->framework_property->dfTitle);
		_DEBUG_TRACE(" framework_property->dfType : %d\n\n", mo_node->framework_property->dfType);
	}

	_DEBUG_TRACE(" end \n");
	_INNER_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_TRACE(" error end \n");

	_INNER_FUNC_EXIT;
	return ret;

}

int print_Node(sync_agent_dm_mo_node_s * mo_node, int depth, void **data)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_node) == NULL, -1, "mo_node is NULL!!");

	_DEBUG_INFO("mo_node->name : %s\n", mo_node->full_path);
	_EXTERN_FUNC_EXIT;
	return 0;
}

static DM_ERROR _get_descendant_mo(char *mo_full_path, sync_agent_dm_mo_node_s ** root_node, int *count)
{
	_INNER_FUNC_ENTER;

	retvm_if((mo_full_path) == NULL, COMMON_ERR_IS_NULL, "mo_full_path is NULL!!");

	_DEBUG_TRACE(" start full path : %s \n", mo_full_path);

	DM_ERROR ret = DM_OK;
	int e_ret = 0;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_type_e mo_type;
	err_code = sync_agent_get_mo_type(mo_full_path, &mo_type);
	_DEBUG_TRACE("get mo type result : %d", err_code);
	_DEBUG_TRACE("mo type : %d", mo_type);

	err_code = sync_agent_get_descendant_mo_tree(mo_type, mo_full_path, root_node, count, SYNC_AGENT_DM_MO_GET_OPTION_NODE);
	_DEBUG_TRACE(" Descendant nodes count : %d\n", (*count));

	if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_TRACE(" Success to sync_agent_get_descendant_mo_tree()\n");

	} else {
		_DEBUG_TRACE(" Failed to sync_agent_get_descendant_mo_tree()\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	}

	if (*count == 0 || root_node == NULL) {
		_DEBUG_TRACE(" Not Existed Descendant node!!\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	}

	GList **temp = NULL;
	int depth = 0;
	int errStop = 0;

	e_ret = sync_agent_traverse_mo_tree_preorder((*root_node), print_Node, depth, errStop, (void **)temp);
	_DEBUG_TRACE("sync agent traverse mo : %d", e_ret);

	_DEBUG_TRACE(" end \n");
	_INNER_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_INFO(" error end \n");
	_INNER_FUNC_EXIT;
	return ret;
}

DM_ERROR check_mo_scope(char *full_path)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((full_path) == NULL, COMMON_ERR_IS_NULL, "full_path is NULL!!");

	_DEBUG_INFO("  full path : %s \n", full_path);
	DM_ERROR ret = DM_OK;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_node_s *mo_node = NULL;
	err_code = sync_agent_get_mo_item(full_path, &mo_node);
	if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
		_DEBUG_INFO(" Success to sync_agent_get_mo()\n");
	} else {
		_DEBUG_INFO(" Failed to sync_agent_get_mo()\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	}

	if (mo_node == NULL) {
		_DEBUG_INFO(" getted mo_node is null\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	}

	sync_agent_dm_mo_scope_e scope;
	if (mo_node->framework_property != NULL) {
		scope = mo_node->framework_property->scope;
		if (scope == SYNC_AGENT_DM_MO_SCOPE_PERMANENT) {
			_DEBUG_INFO("permanent node");
			ret = DM_ERR_COMMAND_NOT_ALLOWED;
			goto error;
		}
		_DEBUG_INFO("dynamic  node");
	} else {
		_DEBUG_INFO("have not framework property");
	}

	sync_agent_dm_mo_node_type_e node_type;
	node_type = mo_node->type;
	if (node_type == SYNC_AGENT_DM_MO_NODE_ROOT) {
		_DEBUG_INFO("root node");
		ret = DM_ERR_COMMAND_NOT_ALLOWED;
		goto error;
	}

	err_code = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO(" free mo_node : %d", err_code);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	err_code = sync_agent_free_mo_item(mo_node);
	_DEBUG_INFO(" free mo_node : %d", err_code);

	_DEBUG_INFO(" end error");
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR copy_itemtomo(Item * item)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((item) == NULL, COMMON_ERR_IS_NULL, "item is NULL!!");
	retvm_if((item->target) == NULL, COMMON_ERR_IS_NULL, "target_url is NULL!!");
	retvm_if((item->source) == NULL, COMMON_ERR_IS_NULL, "source_url is NULL!!");

	char *source_url = NULL;
	char *target_url = NULL;

	source_url = (char *)get_location_locuri(item->source);
	target_url = (char *)get_location_locuri(item->target);

	ret = copy_mo_data(target_url, source_url);
	if (ret != DM_OK)
		goto error;

 error:
	_EXTERN_FUNC_EXIT;
	_DEBUG_INFO(" error end %d \n", ret);
	return ret;

}

DM_ERROR copy_mo_data(char *target_url, char *source_url)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((target_url) == NULL, COMMON_ERR_IS_NULL, "target_url is NULL!!");

	_DEBUG_INFO("source : %s , target : %s \n", source_url, target_url);
	DM_ERROR ret = DM_OK;

	/*if(type == -1 ) {
	   ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
	   goto error;
	   } */

	ret = copy_mo(target_url, source_url);
	if (ret != DM_OK)
		goto error;

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error end %d \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR copy_mo(char *target_url, char *source_url)
{
	_EXTERN_FUNC_ENTER;
	DM_ERROR ret = DM_OK;

	retvm_if((target_url) == NULL, COMMON_ERR_IS_NULL, "target_url is NULL!!");
	retvm_if((source_url) == NULL, COMMON_ERR_IS_NULL, "source_url is NULL!!");

	_DEBUG_INFO(" start!! source : %s, target_url \n", source_url, target_url);

	sync_agent_dm_mo_node_s *sync_agent_mo_item = NULL;
	sync_agent_dm_mo_node_s *sync_agent_new_mo_item = NULL;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	int is_mo = 0;

	err_code = sync_agent_get_mo_item(source_url, &sync_agent_mo_item);
	//err_code = sync_agent_get_mo(type, source_url, &sync_agent_mo_item, SYNC_AGENT_DM_MO_GET_OPTION_ALL);
	if (sync_agent_mo_item == NULL) {
		ret = COMMON_ERR_INTERNAL_NOT_DEFINED;
		goto error;
	}

	err_code = sync_agent_is_exist_mo(target_url, &is_mo);
	_DEBUG_INFO("is existed mo : %d", err_code);
	if (is_mo == 1) {
		//update
		err_code = sync_agent_get_mo_item(target_url, &sync_agent_new_mo_item);
		//err_code = sync_agent_get_mo(mo_type, target_url, &new_sync_agent_mo_item, SYNC_AGENT_DM_MO_GET_OPTION_ALL);
		if (sync_agent_new_mo_item == NULL) {
			ret = COMMON_ERR_INTERNAL_NOT_DEFINED;
			goto error;
		}

		if (sync_agent_mo_item->value != NULL) {
			str_free(&(sync_agent_new_mo_item->value));
			sync_agent_new_mo_item->value = strdup(sync_agent_mo_item->value);
		} else {
			_DEBUG_INFO(" Not existed mo node value ");
		}

		err_code = sync_agent_update_mo_item(sync_agent_new_mo_item);
		//err_code = sync_agent_update_mo(mo_type, new_sync_agent_mo_item);
		if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
			_DEBUG_INFO(" Success to copy_MO()\n");
		} else {
			_DEBUG_INFO("  Failed to copy_MO()\n");
			goto error;
		}

	} else {
		//add

		err_code = sync_agent_create_mo_item(&sync_agent_new_mo_item);
		if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
			_DEBUG_INFO("Success to sync_agent_create_mo()\n");
		} else {
			_DEBUG_INFO("Failed to sync_agent_create_mo() [%d] \n", err_code);
			ret = COMMON_ERR_DELETE;
			goto error;
		}
		//sync_agent_dm_mo_node_s sync_agent_temp_new_mo_item;
		sync_agent_new_mo_item->full_path = strdup(target_url);

		if (sync_agent_mo_item->value != NULL) {
			sync_agent_new_mo_item->value = strdup(sync_agent_mo_item->value);
		}

		sync_agent_new_mo_item->mo_type = SYNC_AGENT_DM_MO_TYPE_NO_TYPE;
		sync_agent_new_mo_item->type = SYNC_AGENT_DM_MO_NODE_LEAF;
		sync_agent_new_mo_item->server_type = sync_agent_mo_item->server_type;

		err_code = sync_agent_add_mo_item(sync_agent_new_mo_item);
		//err_code = sync_agent_add_mo(new_sync_agent_mo_item->mo_type, target_url, new_mo_node);
		if (err_code == SYNC_AGENT_DM_MO_SUCCESS) {
			_DEBUG_INFO(" Success to copy_MO()\n");
		} else {
			_DEBUG_INFO("  Failed to copy_MO()\n");
			goto error;
		}
	}

	err_code = sync_agent_free_mo_item(sync_agent_mo_item);
	_DEBUG_INFO("free : %d", err_code);

	err_code = sync_agent_free_mo_item(sync_agent_new_mo_item);
	_DEBUG_INFO("free : %d", err_code);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	err_code = sync_agent_free_mo_item(sync_agent_mo_item);
	_DEBUG_INFO("free : %d", err_code);

	err_code = sync_agent_free_mo_item(sync_agent_new_mo_item);
	_DEBUG_INFO("free : %d", err_code);

	_DEBUG_INFO(" error end %d \n", ret);

	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR get_mo_operationtype(char *full_path, char **result_oper_type)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((full_path) == NULL, COMMON_ERR_IS_NULL, "full_path is NULL!!");

	_DEBUG_INFO("full path : %s \n", full_path);
	DM_ERROR ret = DM_OK;

	if (full_path != NULL) {
		int full_size = 0;
		int real_size = 0;
		int root_size = 0;
		char *result_mo_type = NULL;

		ret = get_mo_root_path(full_path, &result_mo_type);
		if (ret != DM_OK) {
			goto error;
		}

		if (result_mo_type != NULL) {
			full_size = strlen(full_path);
			root_size = strlen(result_mo_type);

			real_size = full_size - (root_size);
			(*result_oper_type) = (char *)calloc(1, real_size + 1);
			if ((*result_oper_type) == NULL) {
				_DEBUG_INFO("alloc error");
				ret = COMMON_ERR_ALLOC;
				goto error;
			}
			memcpy((*result_oper_type), full_path + root_size, real_size);
			(*result_oper_type)[real_size] = '\0';
			_DEBUG_INFO(" result_mo_type : %s \n", (*result_oper_type));
		} else {
			_DEBUG_INFO("result mo type is null ");
		}
	} else {
		_DEBUG_INFO("result oper type null ");
	}

	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_INFO(" error end \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;

}

static sync_agent_dm_mo_node_type_e _get_mo_node_type(char *format, char *value)
{
	_INNER_FUNC_ENTER;

	retvm_if((format) == NULL && value == NULL, SYNC_AGENT_DM_MO_NODE_INTERIOR, "format is NULL!!");

	_DEBUG_INFO(" start  , format : %s\n", format);
	sync_agent_dm_mo_node_type_e type = SYNC_AGENT_DM_MO_NODE_LEAF;

	if (value != NULL) {
		type = SYNC_AGENT_DM_MO_NODE_LEAF;
		goto return_part;
	} else {
		if (format != NULL) {
			if (strcmp(format, ELEMENT_NODE) == 0) {
				type = SYNC_AGENT_DM_MO_NODE_INTERIOR;
			}
		}
	}

	return SYNC_AGENT_DM_MO_NODE_INTERIOR;

 return_part:
	_DEBUG_TRACE(" end type : [%d]\n", type);
	_INNER_FUNC_EXIT;
	return type;

}

DM_ERROR get_mo_data(char *full_path, char **data)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((full_path) == NULL, COMMON_ERR_IS_NULL, "full_path is NULL!!");

	/*string tok compare */
	_DEBUG_INFO(" start full_path : %s \n", full_path);

	GList *iter = NULL;
	GList *mo_node_list = NULL;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_node_s *mo_node = NULL;
	sync_agent_dm_mo_node_s *mo_node_iter = NULL;

	int i = 0;
	int size = 0;
	int node_name_size = 0;
	int count = 0;
	char child_buffer[1024] = { 0, };

	err_code = sync_agent_get_mo_item(full_path, &mo_node);
	if (err_code != SYNC_AGENT_DM_MO_SUCCESS || mo_node == NULL) {
		_DEBUG_INFO(" Failed to sync_agent_get_mo()\n");
		ret = COMMON_ERR_MO_NOT_FOUND_IN_DB;
		goto error;
	} else {
		_DEBUG_INFO(" Success to sync_agent_get_mo()\n");
	}

	_DEBUG_INFO(" mo_node->id : %d\n", mo_node->id);
	_DEBUG_INFO(" mo_node->parent_id : %d\n", mo_node->parent_id);
	_DEBUG_INFO(" mo_node->name : %s\n", mo_node->name);
	_DEBUG_INFO(" mo_node->value : %s\n", mo_node->value);
	_DEBUG_INFO(" mo_node->full_path : %s\n", mo_node->full_path);
	_DEBUG_INFO(" mo_node->type : %d\n", mo_node->type);

	if (mo_node->type == SYNC_AGENT_DM_MO_NODE_LEAF) {
		if (mo_node->value != NULL) {
			(*data) = g_strdup(mo_node->value);
		}
	} else {
		//else if( mo_node->type == SYNC_AGENT_DM_MO_NODE_INTERIOR )
		ret = _get_child_mo(full_path, &mo_node_list);
		if (ret != DM_OK)
			goto error;

		count = g_list_length(mo_node_list);
		if (mo_node_list != NULL && count != 0) {

			for (iter = mo_node_list; iter != NULL; iter = g_list_next(iter)) {
				mo_node_iter = (sync_agent_dm_mo_node_s *) iter->data;

				node_name_size = strlen(mo_node_iter->name);
				memcpy(child_buffer + size, mo_node_iter->name, node_name_size);
				size += node_name_size;
				if (i < count - 1) {
					memcpy(child_buffer + size, "/", 1);
					size++;
				}
				_DEBUG_INFO("mo node list : %s", mo_node_iter->name);
				_DEBUG_INFO("child node get : %s", child_buffer);
			}
			(*data) = strdup(child_buffer);
		}

		if (mo_node_list != NULL) {
			err_code = sync_agent_free_mo_item_list(mo_node_list);
			_DEBUG_INFO(" free mo list : %d", err_code);
		}
	}

	sync_agent_free_mo_item(mo_node);
	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	(*data) = NULL;

	sync_agent_free_mo_item(mo_node);

	_DEBUG_INFO(" error end\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

int map_nodeToItemWithData(sync_agent_dm_mo_node_s * mo_node, int depth, void **data)
{
	_EXTERN_FUNC_ENTER;

	_DEBUG_INFO(" start \n");
	DM_ERROR ret = DM_OK;

	retvm_if((mo_node) == NULL, -1, "mo_node is NULL!!");

	Item *pItem = NULL;

	if (mo_node->full_path != NULL) {
		_DEBUG_INFO(" mo_node path :  \n", mo_node->full_path);

		ret = construct_Item(mo_node->full_path, NULL, NULL, mo_node->value, 0, 0, &pItem);
		if (ret != DM_OK)
			_DEBUG_INFO(" ITEM NULL\n");

		(*data) = g_list_append(((GList *) (*data)), pItem);
	}

	_EXTERN_FUNC_EXIT;
	return 0;
}

int map_nodeToItem(sync_agent_dm_mo_node_s * mo_node, int depth, void **data)
{
	_EXTERN_FUNC_ENTER;

	_DEBUG_INFO(" start \n");
	DM_ERROR ret = DM_OK;

	retvm_if((mo_node) == NULL, -1, "mo_node is NULL!!");

	Item *pItem = NULL;

	if (mo_node->full_path != NULL) {
		_DEBUG_INFO(" mo_node path :  \n", mo_node->full_path);

		ret = construct_Item(mo_node->full_path, NULL, NULL, NULL, 0, 0, &pItem);
		if (ret != DM_OK) {
			_DEBUG_INFO(" ITEM NULL\n");
		}

		(*data) = g_list_append(((GList *) (*data)), pItem);
	}

	_EXTERN_FUNC_EXIT;
	return 0;
}

static DM_ERROR _map_mo_treetoitemwithdata(sync_agent_dm_mo_node_s * root_node, int count, GList ** pItems)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((root_node) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "root_node is NULL!!");

	int depth = 0;
	int errStop = 0;
	int e_ret = 0;

	e_ret = sync_agent_traverse_mo_tree_preorder(root_node, map_nodeToItemWithData, depth, errStop, (void **)pItems);
	_DEBUG_TRACE(" sync-agent traverse mo : %d", e_ret);

	_INNER_FUNC_EXIT;
	return ret;
}

static DM_ERROR map_mo_treeToItem(sync_agent_dm_mo_node_s * root_node, int count, GList ** pItems)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	_DEBUG_INFO(" start count : [%d]\n", count);

	retvm_if((root_node) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "root_node is NULL!!");

	int depth = 0;
	int errStop = 0;
	int e_ret = 0;

	e_ret = sync_agent_traverse_mo_tree_preorder(root_node, map_nodeToItem, depth, errStop, (void **)pItems);
	_DEBUG_INFO(" sync-agent traverse mo : %d", e_ret);

	_EXTERN_FUNC_EXIT;
	return ret;
}

static DM_ERROR _map_mo_listtoitemwithdata(GList * mo_node_list, GList ** pItems)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((mo_node_list) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "mo_node_list is NULL!!");

	int i = 0;
	GList *iter = NULL;
	sync_agent_dm_mo_node_s *mo_node = NULL;

	for (iter = mo_node_list; iter != NULL; iter = g_list_next(iter)) {
		Item *pItem = NULL;
		mo_node = (sync_agent_dm_mo_node_s *) (iter->data);

		if (mo_node->value != NULL && strlen(mo_node->value) != 0) {
			ret = construct_Item(mo_node->full_path, NULL, NULL, mo_node->value, 0, 0, &pItem);
			if (ret != DM_OK) {
				free_Item(pItem);
				goto error;
			}

			(*pItems) = g_list_append((*pItems), pItem);

			_DEBUG_TRACE("[%d],  %s", i, pItem->source->locURI);
		}
	}

	_INNER_FUNC_EXIT;
	return ret;
 error:

	_DEBUG_TRACE(" error end\n", ret);
	_INNER_FUNC_EXIT;
	return ret;
}

DM_ERROR get_child_mo_list(char *full_path, GList ** pItems)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	_DEBUG_INFO(" start \n");

	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	//sync_agent_dm_mo_node_s *mo_node_list = NULL ;
	GList *mo_node_list = NULL;
	if (full_path == NULL) {
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}
	_DEBUG_INFO("full_path : %s", full_path);

	ret = _get_child_mo(full_path, &mo_node_list);
	if (ret != DM_OK)
		goto error;

	ret = _map_mo_listtoitemwithdata(mo_node_list, pItems);
	if (ret != DM_OK)
		goto error;

	err_code = sync_agent_free_mo_item_list(mo_node_list);
	_DEBUG_INFO("free mo list L %d ", err_code);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	err_code = sync_agent_free_mo_item_list(mo_node_list);
	_DEBUG_INFO("free mo list L %d ", err_code);
	_DEBUG_INFO(" error end\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR get_type_mo_tree(char *mo_full_path, GetType * get_type, char **real_full_path)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	if (mo_full_path == NULL) {
		ret = COMMON_ERR_INTERNAL_NOT_DEFINED;
		goto error;
	}

	char *result = NULL;
	unsigned int size = 0;

	_DEBUG_INFO(" full_path exist\n");

	result = strstr(mo_full_path, ELEMENT_STRUCTDATA);
	if (result != NULL) {
		_DEBUG_INFO(" structdata\n");
		(*get_type) = GET_TYPE_STRUCTDATA;
		goto returnpart;
	}

	result = strstr(mo_full_path, ELEMENT_STRUCT);
	if (result != NULL) {
		_DEBUG_INFO(" struct\n");
		(*get_type) = GET_TYPE_STRUCT;
		goto returnpart;
	}

	result = strstr(mo_full_path, ELEMENT_TNDS);
	if (result != NULL) {
		_DEBUG_INFO(" tnds\n");
		(*get_type) = GET_TYPE_TNDS;
		goto returnpart;
	}

/*	result = strstr(mo_full_path, ELEMENT_PROPERTY);
	if(result != NULL) {
		_DEBUG_INFO( " property\n");
		(*get_type) = GET_PROPERTY;
		goto returnpart;
	}*/

	result = strstr(mo_full_path, ELEMENT_PROP_ACL);
	if (result != NULL) {
		_DEBUG_INFO(" ACL\n");
		(*get_type) = GET_TYPE_ACL_PROP;
		goto returnpart;
	}

	result = strstr(mo_full_path, ELEMENT_PROP_SIZE);
	if (result != NULL) {
		_DEBUG_INFO(" Size\n");
		(*get_type) = GET_TYPE_SIZE_PROP;
		goto returnpart;
	}

	result = strstr(mo_full_path, ELEMENT_PROP_TYPE);
	if (result != NULL) {
		_DEBUG_INFO(" Type\n");
		(*get_type) = GET_TYPE_TYPE_PROP;
		goto returnpart;
	}

	result = strstr(mo_full_path, ELEMENT_PROP_NAME);
	if (result != NULL) {
		_DEBUG_INFO(" Name\n");
		(*get_type) = GET_TYPE_NAME_PROP;
		goto returnpart;
	}

	result = strstr(mo_full_path, ELEMENT_PROP_FORMAT);
	if (result != NULL) {
		_DEBUG_INFO(" Formt\n");
		(*get_type) = GET_TYPE_FORMAT_PROP;
		goto returnpart;
	}

	_DEBUG_INFO(" end  ret : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;

 returnpart:

	size = strlen(mo_full_path) - strlen(result);
	(*real_full_path) = (char *)calloc(1, sizeof(char) * size + 1);
	if ((*real_full_path) == NULL) {
		_DEBUG_INFO("alloc fail");
		ret = COMMON_ERR_ALLOC;
		goto error;
	}

	memcpy((*real_full_path), mo_full_path, size);

	_DEBUG_INFO(" full_path_len : [%d], result_len : [%d] real_full_path  : %s ", strlen(mo_full_path), size, (*real_full_path));
	_DEBUG_INFO(" returnpart end [%d]\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	_DEBUG_INFO(" error end [%d]\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR get_struct(char *mo_full_path, GList ** pItems)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	sync_agent_dm_mo_error_e err = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_node_s *root_node = NULL;
	int count = 0;

	ret = _get_descendant_mo(mo_full_path, &root_node, &count);
	if (ret != DM_OK)
		goto error;

	ret = map_mo_treeToItem(root_node, count, pItems);
	if (ret != DM_OK)
		goto error;

	err = sync_agent_free_mo_item(root_node);
	if (err != SYNC_AGENT_DM_MO_SUCCESS) {
		goto error;
	}

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	err = sync_agent_free_mo_item(root_node);
	if (err != SYNC_AGENT_DM_MO_SUCCESS) {
		goto error;
	}

	_DEBUG_INFO(" error end : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR get_structdata(char *mo_full_path, GList ** pItems)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	sync_agent_dm_mo_error_e err = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_node_s *root_node = NULL;
	int count = 0;

	ret = _get_descendant_mo(mo_full_path, &root_node, &count);
	if (ret != DM_OK)
		goto error;

	ret = _map_mo_treetoitemwithdata(root_node, count, pItems);
	if (ret != DM_OK)
		goto error;

	err = sync_agent_free_mo_item(root_node);
	if (err != SYNC_AGENT_DM_MO_SUCCESS) {
		goto error;
	}

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	err = sync_agent_free_mo_item(root_node);
	if (err != SYNC_AGENT_DM_MO_SUCCESS) {
		goto error;
	}

	_DEBUG_INFO(" error end : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR get_tnds(char *contenttype, char *mo_full_path, char **output_stream, unsigned int *stream_size)
{
	_EXTERN_FUNC_ENTER;

	_DEBUG_INFO(" start mo full path : %s \n", mo_full_path);
	DM_ERROR ret = DM_OK;

	unsigned int byte_size = 0;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_mo_type_e mo_type;
	char *mo_root = NULL;
	char *result = NULL;
	int size = 0;

	result = strstr(mo_full_path, ELEMENT_TNDS);
	if (result != NULL) {
		size = result - mo_full_path;
		mo_root = (char *)calloc(1, size);
		if (mo_root == NULL) {
			_DEBUG_INFO("alloc fail");
			ret = COMMON_ERR_ALLOC;
			goto error;
		}
		memcpy(mo_root, mo_full_path, size);
		_DEBUG_INFO(" tnds\n");

		err_code = sync_agent_get_mo_type(mo_root, &mo_type);
		_DEBUG_INFO("get mo type result : %d", err_code);
		_DEBUG_INFO("mo type : %d", mo_type);

		if (strcmp(contenttype, ELEMENT_DMTNDS_XML) == 0) {
			err_code = sync_agent_export_tnds_stream(SYNC_AGENT_DM_MO_TNDS_TYPE_XML, mo_type, mo_root, output_stream, &byte_size);
			if (err_code != SYNC_AGENT_DM_MO_SUCCESS) {
				ret = DM_ERR_TNDS;
				goto error;
			}
			*stream_size = byte_size;
		} else if (strcmp(contenttype, ELEMENT_DMTNDS_WBXML) == 0) {
			err_code = sync_agent_export_tnds_stream(SYNC_AGENT_DM_MO_TNDS_TYPE_WBXML, mo_type, mo_root, output_stream, &byte_size);
			if (err_code != SYNC_AGENT_DM_MO_SUCCESS) {
				ret = DM_ERR_TNDS;
				goto error;
			}
			*stream_size = byte_size;
		}
		_DEBUG_INFO(" tnds byte size : %d\n", byte_size);
	} else {
		_DEBUG_INFO(" not tnds");
	}

	if(mo_root != NULL) {
		free(mo_root);
	}
	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error end : %d\n", ret);
	if(mo_root != NULL) {
		free(mo_root);
	}
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR set_client_nextnonce(char *pServer_id, char *next_nonce)
{
	_EXTERN_FUNC_EXIT;

	DM_ERROR ret = DM_OK;

	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_acc_info_s *acc_info = NULL;
	sync_agent_dm_mo_acc_item_info_s sync_agent_mo_acc_info_item;

	if (next_nonce != NULL) {

		sync_agent_mo_acc_info_item.acc_type = DEVICE_MANAGE_ACC_TYPE_CLIENT;
		sync_agent_mo_acc_info_item.serverid = pServer_id;

		err_code = sync_agent_get_mo_acc_item(&sync_agent_mo_acc_info_item, &acc_info);
		//acc_info = sync_agent_get_acc_info(pServer_id, DEVICE_MANAGE_ACC_TYPE_CLIENT);
		if (err_code != SYNC_AGENT_DM_MO_SUCCESS || acc_info == NULL) {
			_DEBUG_INFO(" acc null!!\n");
			ret = COMMON_ERR_IS_NULL;
			goto error;
		}

		str_free(&(acc_info->auth_data));
		acc_info->auth_data = next_nonce;

		//sync_agent_set_acc_info(pServer_id, DEVICE_MANAGE_ACC_TYPE_CLIENT, acc_info);
		err_code = sync_agent_update_mo_acc_item(&sync_agent_mo_acc_info_item, acc_info);
		_DEBUG_INFO("sync agent update result : %d", err_code);
		if (err_code != SYNC_AGENT_DM_MO_SUCCESS) {
			ret = COMMON_ERR_IS_NULL;
			goto error;
		}

	} else {
		//do nothing
	}

	sync_agent_free_mo_acc_item(&acc_info);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	sync_agent_free_mo_acc_item(&acc_info);

	_DEBUG_INFO(" error end : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR set_client_authtype(char *pServer_id, AuthType auth_type)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	char *authname = NULL;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_acc_info_s *acc_info = NULL;
	sync_agent_dm_mo_acc_item_info_s sync_agent_mo_acc_info_item;

	if (pServer_id == NULL) {
		_DEBUG_INFO(" server id is null");
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}

	sync_agent_mo_acc_info_item.acc_type = DEVICE_MANAGE_ACC_TYPE_CLIENT;
	sync_agent_mo_acc_info_item.serverid = pServer_id;

	err_code = sync_agent_get_mo_acc_item(&sync_agent_mo_acc_info_item, &acc_info);
	//acc_info = sync_agent_get_acc_info(pServer_id, DEVICE_MANAGE_ACC_TYPE_CLIENT);
	if (err_code != SYNC_AGENT_DM_MO_SUCCESS || acc_info == NULL) {
		_DEBUG_INFO(" acc null!!\n");
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}

	__set_Session_AuthType_name(auth_type, &authname);

	_DEBUG_INFO("authtype is  %s", authname);
	acc_info->auth_type = authname;

	//err = sync_agent_set_acc_info(pServer_id, DEVICE_MANAGE_ACC_TYPE_CLIENT, acc_info);
	err_code = sync_agent_update_mo_acc_item(&sync_agent_mo_acc_info_item, acc_info);
	_DEBUG_INFO("sync agent update result : %d", err_code);
	if (err_code != SYNC_AGENT_DM_MO_SUCCESS) {
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}

	sync_agent_free_mo_acc_item(&acc_info);

	_EXTERN_FUNC_EXIT;
	return ret;
 error:

	sync_agent_free_mo_acc_item(&acc_info);

	_DEBUG_INFO(" error end : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}
