/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_common_def.h"
#include "common/dm_common.h"
#include "common/util/util.h"
#include "serviceadapter/dm-phase-handler/dm_phase_handler.h"
#include "serviceadapter/protocolbinder/syncml_protocol_binder.h"
#include "serviceadapter/sa_define.h"
#include "serviceadapter/sa_syncml_binders.h"
#include "serviceadapter/sa_elements.h"
#include "serviceadapter/sa_elements_internal.h"
#include "serviceadapter/sa_command.h"
#include "serviceadapter/sa_command_internal.h"
#include "serviceadapter/sa_session.h"
#include "serviceadapter/sa_session_internal.h"
#include "serviceadapter/sa_util.h"
#include "serviceadapter/networkbinder/na_binder.h"
#include "mo-handler/dm_mo_common.h"
#include "mo-handler/dm_mo_handler.h"
#include "dm-engine/dm_common_engine.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"OMA_DM_CONTEXT"
#endif

#define MAX_SESSION_COUNT 5

/*temp size*/
#define LARGE_OBJECT_SIZE 10000

static int file_count = 0;

/*static ErrorType get_AccountInfo(int accountId, char **id, char **pwd, char **serverUrl, int *isBase64, char **nextNonce);*/
static DM_ERROR __get_dmaccInfo(char *pServer_id, char **client_id, char **client_pwd, char **sourceUrl, char **targetUrl, int *isBase64, char **nextNonce);
static DM_ERROR __check_hmac(Session ** session, char *msg, unsigned int msg_size);
static DM_ERROR __create_authentication(Session ** session, char *client_id, char *client_pwd, int isBase64, char *nextNonce);
static DM_ERROR ___generater_deviceInfo_replacecmd(Command ** pReplaceCommand);
static DM_ERROR __create_devinfo_replace_cmd(Session * session, Command ** pCommand);
static DM_ERROR _generate_setupphase_msg(Session ** session, char *pServer_id, char *pSession_id, int session_type, char **msg, unsigned int *msg_size);
static DM_ERROR _generate_management_msg(Session ** session, char **msg, unsigned int *msg_size);
static DM_ERROR _generate_generic_alert(Session ** session, char *pServer_id, char *pSession_id, char **msg, int *msg_size, int service_engine_id, int result);
static DM_ERROR _send_msg(char *accType, int transportType, Session ** session, char *server_url, GList * header_info, char *sendMsg, unsigned int sendMsg_length, GList ** recv_header, char **recvMsg, unsigned int *recvMsg_length);
static DM_ERROR _recv_msg(Session ** session, char *recvMsg, unsigned int recvMsg_length, ENGINE_ID * service_engine_id, int *isFinish, int isgeneticAlert);

static DM_ERROR __get_dmaccInfo(char *pServer_id, char **client_id, char **client_pwd, char **sourceUrl, char **targetUrl, int *isBase64, char **nextNonce)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((pServer_id) == NULL, COMMON_ERR_IS_NULL, "pServer_id is NULL!!");

	char *authType = NULL;
	ret = get_client_dmacc(pServer_id, client_id, client_pwd, sourceUrl, targetUrl, isBase64, nextNonce, &authType);
	if (ret != DM_OK)
		goto error;

	_DEBUG_VERBOSE(" end\n");
	_INNER_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_VERBOSE(" error end [%d]\n", ret);
	_INNER_FUNC_EXIT;
	return ret;
}

static DM_ERROR ___generater_deviceInfo_replacecmd(Command ** pReplaceCommand)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	/*get DevInfo */
	Item *pItem = NULL;
	ret = get_child_mo_list(DM_DEVINFO, &(*pReplaceCommand)->private.change.items);
	if (ret != DM_OK)
		goto error;

	GList *iter = NULL;
	for (iter = (*pReplaceCommand)->private.change.items; iter != NULL; iter = g_list_next(iter)) {
		pItem = iter->data;
		pItem->contenttype = g_strdup(ELEMENT_TEXTPLAIN);
		pItem->format = g_strdup(ELEMENT_CHR);
	}

	_INNER_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_VERBOSE(" error end\n");
	_INNER_FUNC_EXIT;
	return ret;

}

static DM_ERROR __create_devinfo_replace_cmd(Session * session, Command ** pCommand)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	retvm_if((session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");

	ret = create_replace_cmd(session, pCommand);
	if (ret != DM_OK)
		goto error;

	ret = ___generater_deviceInfo_replacecmd(pCommand);
	if (ret != DM_OK)
		goto error;

	_INNER_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_VERBOSE(" error : %d\n", ret);
	_INNER_FUNC_EXIT;
	return ret;

}

static DM_ERROR _generate_setupphase_msg(Session ** session, char *pServer_id, char *pSession_id, int session_type, char **msg, unsigned int *msg_size)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((pServer_id) == NULL, COMMON_ERR_IS_NULL, "pServer_id is NULL!!");

	int isFinal = 1;

	/*Client init */
	if (pSession_id == NULL) {
		//pSession_id = g_strdup_printf("%ld", time(NULL));
		pSession_id = g_strdup_printf("%ld", time(NULL)%10000);
		_DEBUG_INFO("pSession_id [%s]", pSession_id);
	} else {
		_DEBUG_INFO("pSession_id [%s]", pSession_id);
	}

	char *sourceUrl = NULL;
	char *client_id = NULL;
	char *client_pwd = NULL;
	char *targetUrl = NULL;
	int isBase64 = 0;
	char *nextNonce = NULL;

	ret = __get_dmaccInfo(pServer_id, &client_id, &client_pwd, &sourceUrl, &targetUrl, &isBase64, &nextNonce);
	if (ret != DM_OK)
		goto error;

	if (nextNonce == NULL) {
		(nextNonce) = strdup(ELEMENT_DEFAULT_CLIENT_NONCE);
	}
	_DEBUG_INFO("pServer_id [%s] , pSession_id [%s]", pServer_id, pSession_id);
	if ((*session) == NULL) {
		ret = create_syncml_session(VERSION_12, PROTOCOL_TYPE_DM, pServer_id, client_id, pSession_id, sourceUrl, targetUrl, session);
		if (ret != DM_OK)
			goto error;
	} else {
		(*session)->msgID = 0;

#ifdef _DM_BUNDANG_TEST
		/*redirection code */
		if ((*session)->target->locURI != NULL) {
			char *replace = NULL;
			replace = get_new_uri((*session)->target->locURI);
			str_free(&((*session)->target->locURI));
			(*session)->target->locURI = g_strdup(replace);
			str_free(&replace);
			_DEBUG_TRACE(" targer url : %s \n", (*session)->target->locURI);
		}
#endif
		isBase64 = 1;
	}

	_DEBUG_TRACE(" targer url : %s \n", (*session)->target->locURI);
	ret = __create_authentication(session, client_id, client_pwd, isBase64, nextNonce);
	if (ret != DM_OK)
		goto error;

	_DEBUG_TRACE("targer url : %s \n", (*session)->target->locURI);

	GList *commands = NULL;
	GList *commands_last = NULL;

	/*SyncHdr */
	SyncHdr *pSyncHdr = NULL;
	ret = create_syncml_hdr((*session), &pSyncHdr);
	if (ret != DM_OK)
		goto error;

	/*SyncBody */
	/* create alert */
	AlertType syncType;
	_DEBUG_INFO("session_type [%d]", session_type);
	switch (session_type) {
	case 0:		//IP_PUSH_EVENT
	case 1:		//WAP_PUSH_EVENT
		syncType = DM_ALERT_SERVER_INITIATED_MGMT;
		break;
	case 2:		//UI_EVENT
	case 3:		//SERVER_BOOTSTRAP_EVENT
		syncType = DM_ALERT_CLIENT_INITIATED_MGMT;
		break;
	default:
		syncType = DM_ALERT_CLIENT_INITIATED_MGMT;
		break;
	}

	Command *pAlertCommand = NULL;
	ret = create_alert_cmd((*session), syncType, &pAlertCommand);
	if (ret != DM_OK)
		goto error;
	putCmdIntoList(&commands, &commands_last, pAlertCommand);

	/* create replace */
	Command *pReplaceCommand = NULL;
	ret = __create_devinfo_replace_cmd((*session), &pReplaceCommand);
	if (ret != DM_OK)
		goto error;
	putCmdIntoList(&commands, &commands_last, pReplaceCommand);



	/* create generic alert for polling & user init  */
	Command *pAlertDownloadUpdateCommand = NULL;
	Item *pItem = NULL;
	char *rSourceUrl = NULL;
	char *format = NULL;
	char *contenttype = NULL;
	char *data = NULL;
	if(strcmp(pServer_id,"xxxxxxxxxx") == 0 ) { // xxxxxxxxxx is Server_id for Production FOTA server.
		switch (session_type) {
		case 0:		//IP_PUSH_EVENT
		case 1:		//WAP_PUSH_EVENT
			break;
		case 2:		//UI_EVENT
			{
				ret = create_alert_cmd((*session), DM_ALERT_GENERIC_ALERT, &pAlertDownloadUpdateCommand);
				if (ret != DM_OK)
					goto error;
				rSourceUrl = strdup("./FUMO/DownloadAndUpdate");
				format = strdup("chr");
				//contenttype = strdup("org.openmobilealliance.dm.firmwareupdate.devicerequest");
				contenttype = strdup("org.openmobilealliance.dm.firmwareupdate.userrequest");
				data = g_strdup_printf("%d", 0);
				unsigned int size = 0;
				unsigned int moredata = 0;
				//ret = set_correlator(&pAlertDownloadUpdateCommand, pCorrelator);
				ret = construct_Item(rSourceUrl, format, contenttype, data, size, moredata, &pItem);
				pAlertDownloadUpdateCommand->private.alert.items = g_list_append(pAlertDownloadUpdateCommand->private.alert.items, pItem);
				putCmdIntoList(&commands, &commands_last, pAlertDownloadUpdateCommand);
			}
			break;
		case 3:		//SERVER_BOOTSTRAP_EVENT
			break;
		case 4:		//POLL_EVENT
			{
				ret = create_alert_cmd((*session), DM_ALERT_GENERIC_ALERT, &pAlertDownloadUpdateCommand);
				if (ret != DM_OK)
					goto error;
				rSourceUrl = strdup("./FUMO/DownloadAndUpdate");
				format = strdup("chr");
				contenttype = strdup("org.openmobilealliance.dm.firmwareupdate.devicerequest");
				data = g_strdup_printf("%d", 0);
				unsigned int size = 0;
				unsigned int moredata = 0;
				//ret = set_correlator(&pAlertDownloadUpdateCommand, pCorrelator);
				ret = construct_Item(rSourceUrl, format, contenttype, data, size, moredata, &pItem);
				pAlertDownloadUpdateCommand->private.alert.items = g_list_append(pAlertDownloadUpdateCommand->private.alert.items, pItem);
				putCmdIntoList(&commands, &commands_last, pAlertDownloadUpdateCommand);
			}
			break;
		default:
			break;
		}
	}



	/*create SyncML */
	SyncML *pSyncML = NULL;
	ret = create_syncml(pSyncHdr, NULL, commands, isFinal, &pSyncML);
	if (ret != DM_OK)
		goto error;

	/* convert Msg */
	ret = syncml_objectbinder(pSyncML, msg, msg_size);
	if (ret != DM_OK)
		goto error;

	/*xml */
/*	xml_len = remove_xml_header(xml, xml_len);
	_DEBUG_TRACE("msg : %s", msg);
	ret = check_hmac( session,  xml, xml_len);*/

	/*wbxml */
	_DEBUG_TRACE("msg : %s", msg);
	ret = __check_hmac(session, *msg, *msg_size);
	_DEBUG_TRACE(" check hmac:%d\n", ret);

	//free generate structure
	free_syncml(pSyncML);
	pSyncML = NULL;

	//session->status, session->alertCommand are already freed in free_SyncML function
	(*session)->status = NULL;

	if ((*session)->alertCommand != NULL) {
		g_list_free((*session)->alertCommand);
		(*session)->alertCommand = NULL;
	}

	(*session)->isSendingfinal = isFinal;


	str_free(&rSourceUrl);
	str_free(&format);
	str_free(&contenttype);
	str_free(&data);


	_INNER_FUNC_EXIT;
	return ret;

 error:

	_DEBUG_TRACE(" error : %d\n", ret);


	str_free(&rSourceUrl);
	str_free(&format);
	str_free(&contenttype);
	str_free(&data);


	_INNER_FUNC_EXIT;
	return ret;
}

static DM_ERROR _generate_management_msg(Session ** session, char **msg, unsigned int *msg_size)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");

	sync_agent_pb_protocol_binder_function_set_s *pBinder_function_set = NULL;
	sync_agent_pb_error_e err = init_syncml_binder_function_set(&pBinder_function_set);

	SyncML *pSyncML = NULL;
	SyncHdr *pSyncHdr = NULL;

	GList *commands = NULL;
	GList *commands_last = NULL;
	int isFinal = 1;
	bool cancel_flag = false;

	GList *status_iter = NULL;
	Status *pStatus = NULL;

	Command *pAlertCommand = NULL;
	GList *alert_iter = NULL;

	GList *results_iter = NULL;
	Command *pResultsCommand = NULL;
	Command *restResultsCommand = NULL;

	unsigned int estimate_size = 0;
	int data_size = 0;

	GList *results_data_iter = NULL;
	GList *rest_result_data_iter = NULL;

	Item *result_item = NULL;
	Item *rest_result_item = NULL;
	char *result_data = NULL;
	int full_msg = 0;

	sync_agent_pb_protocol_binder_info_s *pBinder = NULL;
	pBinder = syncml_binder_init(pBinder, SYNC_AGENT_PB_ENCODING_WBXML_12, false, true, pBinder_function_set);

#ifdef _DM_BUNDANG_TEST
	/*redirection code */
	if ((*session)->target->locURI != NULL) {
		char *replace = NULL;
		replace = get_new_uri((*session)->target->locURI);
		str_free(&((*session)->target->locURI));
		(*session)->target->locURI = g_strdup(replace);
		str_free(&replace);

		_DEBUG_TRACE(" targer url : %s \n", (*session)->target->locURI);
	}
#endif

	/* create SyncHdr */
	ret = create_syncml_hdr((*session), &pSyncHdr);
	if (ret != DM_OK)
		goto error;
	err = syncml_binder_append(pBinder, PE_SYNCML_START, NULL);
	err = syncml_binder_append(pBinder, PE_HEADER, pSyncHdr);
	err = syncml_binder_append(pBinder, PE_BODY_START, NULL);

/*	ExecuteObject ** responseObj = (ExecuteObject*) pResponseObj;*/
	for (status_iter = (*session)->status; status_iter != NULL; status_iter = g_list_next(status_iter)) {
		_DEBUG_TRACE("======================================status command================================\n");
		pStatus = status_iter->data;
		err = syncml_binder_append(pBinder, PE_STATUS, pStatus);
	}

	//if we have to send alert command(1222,1223 etc)
	cancel_flag = sync_agent_check_cancel_flag();
	_DEBUG_TRACE("======================================alert command================================\n");
	_DEBUG_TRACE("======================================%d================================\n", cancel_flag);

	if (cancel_flag != 0) {
		ret = create_alert_cmd((*session), DM_ALERT_SESSION_ABORT, &pAlertCommand);
		if (ret != DM_OK)
			goto error;
		putCmdIntoList(&commands, &commands_last, pAlertCommand);
		err = syncml_binder_append(pBinder, PE_ALERT, pAlertCommand);
	}

	for (alert_iter = (*session)->alertCommand; alert_iter != NULL; alert_iter = g_list_next(alert_iter)) {
		_DEBUG_TRACE("======================================alert command================================\n");
		pAlertCommand = alert_iter->data;
		putCmdIntoList(&commands, &commands_last, pAlertCommand);
		err = syncml_binder_append(pBinder, PE_ALERT, pAlertCommand);
	}

	_DEBUG_TRACE(" server maxsize : %d ", (*session)->targetMaxMsgSize);
	if ((*session)->resultsCommand != NULL) {
		_DEBUG_TRACE("result command length : %d ", g_list_length((*session)->resultsCommand));

		_DEBUG_TRACE("======================================result command================================\n");
		for (results_iter = (*session)->resultsCommand; results_iter != NULL;) {
			_DEBUG_TRACE("resultcommand iter");
			pResultsCommand = results_iter->data;

			sync_agent_get_stream_size_from_protocol_binder(pBinder, &estimate_size);
			_DEBUG_TRACE("msg extimate size : %d ", estimate_size);

			ret = duplicate_command(pResultsCommand, &restResultsCommand);
			_DEBUG_TRACE("dupicate command : %d", ret);
			if (ret != DM_OK) {
				goto error;
			}

			results_iter = g_list_next(results_iter);

			for (results_data_iter = pResultsCommand->private.results.items, rest_result_data_iter = restResultsCommand->private.results.items; results_data_iter != NULL && rest_result_data_iter != NULL;) {
				_DEBUG_TRACE("resultcommand item iter");

				result_item = (Item *) (results_data_iter->data);
				rest_result_item = (Item *) (rest_result_data_iter->data);

				result_data = result_item->private.data;

				results_data_iter = g_list_next(results_data_iter);
				rest_result_data_iter = g_list_next(rest_result_data_iter);

				if (full_msg == 1) {
					_DEBUG_TRACE("remove item because msg size over");
					pResultsCommand->private.results.items = g_list_remove(pResultsCommand->private.results.items, result_item);
				} else {
					_DEBUG_TRACE("result item add packet");
					if (result_item->moreData == 1) {
						_DEBUG_TRACE("item msg cut by largeObject");
						isFinal = 0;

						char *data = NULL;
						int maxsize = 0;
						data = strdup(result_data);

						str_free(&(result_item->private.data));
						str_free(&(rest_result_item->private.data));

						maxsize = (*session)->targetMaxMsgSize - estimate_size - DEFAULT_REST_SIZE;
						_DEBUG_TRACE("maxsize : %d", maxsize);

						result_item->private.data = (char *)calloc(1, maxsize + 1);
						if (result_item->private.data == NULL) {
							_DEBUG_TRACE("alloc fail");
							ret = COMMON_ERR_ALLOC;
							goto error;
						}
						memcpy(result_item->private.data, data, maxsize);

						_DEBUG_TRACE("rest data size : %d", result_item->size - maxsize);
						rest_result_item->private.data = (char *)calloc(1, result_item->size - maxsize + 1);
						if (rest_result_item->private.data == NULL) {
							_DEBUG_TRACE("alloc fail");
							ret = COMMON_ERR_ALLOC;
							goto error;
						}
						memcpy(rest_result_item->private.data, data + maxsize, result_item->size - maxsize);
						rest_result_item->size = result_item->size - maxsize;

						if (maxsize > result_item->size - maxsize) {
							_DEBUG_TRACE("set msg non more data");
							rest_result_item->moreData = 0;
						}

						data_size += maxsize;
						_DEBUG_TRACE("msg(moredata) data size : %d ", data_size);

						full_msg = 1;
					} else {
						_DEBUG_TRACE("remove item temp list");
						data_size += result_item->size;

						_DEBUG_TRACE("msg data size : %d ", data_size);
						restResultsCommand->private.results.items = g_list_remove(restResultsCommand->private.results.items, rest_result_item);
					}
				}
			}

			/*multiple msg */
			_DEBUG_TRACE(" ESTIMATE SIZE IN RESULTS= %d", estimate_size + data_size + DEFAULT_REST_SIZE);
			if ((*session)->targetMaxMsgSize >= estimate_size + data_size + DEFAULT_REST_SIZE) {
				_DEBUG_TRACE("insert result in msg");
				putCmdIntoList(&commands, &commands_last, pResultsCommand);
				(*session)->resultsCommand = g_list_remove((*session)->resultsCommand, pResultsCommand);

				err = syncml_binder_append(pBinder, PE_RESULTS_START, pResultsCommand);
				err = syncml_binder_append(pBinder, PE_RESULTS_END, NULL);

			} else {
				/*large object */
				isFinal = 0;
				_DEBUG_TRACE("cut result in msg");

				if (full_msg == 1) {
					_DEBUG_TRACE("msg msg over large object");
				} else {
					_DEBUG_TRACE("msg cut by MultiMsg");
				}
				_DEBUG_TRACE("free rest result command");
				/*while break */
				break;
			}

			if (full_msg == 1) {
				/*large object rest part preapend in resultsCommands list for next msg */
				if (g_list_length(restResultsCommand->private.results.items) > 0) {
					(*session)->resultsCommand = g_list_prepend((*session)->resultsCommand, restResultsCommand);
				}
				_DEBUG_TRACE("msg cut by MultiMsg");
				break;
			} else {
				free_command(restResultsCommand);
				restResultsCommand = NULL;
				_DEBUG_TRACE("free rest result command");
			}
		}
		_DEBUG_TRACE("added result command");
	}

	/*syncml */
	ret = create_syncml(pSyncHdr, (*session)->status, commands, isFinal, &pSyncML);
	if (ret != DM_OK)
		goto error;

	_DEBUG_TRACE("final element : %d", isFinal);
	if (isFinal == 1) {
		err = syncml_binder_append(pBinder, PE_FINAL, NULL);
	}

	err = syncml_binder_append(pBinder, PE_BODY_END, NULL);
	err = syncml_binder_append(pBinder, PE_SYNCML_END, NULL);

	syncml_binder_get_stream(pBinder, msg, msg_size);

	char *xml = NULL;
	unsigned int xml_size = 0;
	err = sync_agent_get_xml_from_protocol_binder(pBinder, &xml, &xml_size);
	if (err == SYNC_AGENT_PB_RETURN_OK) {
		fprintf(stderr, " msg from client \n");
		if (fputs(xml, stderr) == EOF) {
			_DEBUG_TRACE("debuging test error");
		}
		set_xml_to_file(xml, OMA_DM_MSG_PATH);
		str_free(&xml);
	}

	syncml_binder_terminate(pBinder);

	/*xml */
/*	xml_len = remove_xml_header(xml, xml_len);
	_DEBUG_TRACE("msg : %s", msg);
	ret = check_hmac( session,  xml, xml_len);*/

	/*wbxml */
	ret = __check_hmac(session, *msg, *msg_size);
	_DEBUG_TRACE(" check hmac:%d\n", ret);

	free_syncml(pSyncML);
	pSyncML = NULL;

	//session->status, session->alertCommand are already freed in free_SyncML function
	(*session)->status = NULL;
	if ((*session)->alertCommand != NULL) {
		g_list_free((*session)->alertCommand);
		(*session)->alertCommand = NULL;
	}
	if (isFinal == 1) {
		g_list_free((*session)->resultsCommand);
		(*session)->resultsCommand = NULL;
	}

	(*session)->isSendingfinal = isFinal;

	_INNER_FUNC_EXIT;
	return ret;

 error:
	free_command(restResultsCommand);
	restResultsCommand = NULL;

	free_synchdr(pSyncHdr);
	pSyncHdr = NULL;

	free_syncml(pSyncML);
	pSyncML = NULL;

	_DEBUG_TRACE(" error :%d\n", ret);
	syncml_binder_terminate(pBinder);

	_INNER_FUNC_EXIT;
	return ret;

	//1500 -> ERROR_INTERNAL_OK : generatePreSyncMsg has been completed without any error
	//1512 -> ERROR_INTERNAL_NO_MEMORY : failed to allocate memory
	//1513 -> ERROR_INTERNAL_NOT_DEFINED : something is not defined that must have
	//1503 -> ERROR_INTERNAL_BINDER_ERROR   : binder error
}

DM_ERROR __check_hmac(Session ** session, char *xml, unsigned int xml_len)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((xml) == NULL, COMMON_ERR_IS_NULL, "xml is NULL!!");
	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");

	AuthType auth_type = AUTH_TYPE_UNKNOWN;
	sync_agent_dm_mo_error_e err_code = SYNC_AGENT_DM_MO_SUCCESS;
	sync_agent_dm_acc_info_s *acc_info = NULL;
	sync_agent_dm_mo_acc_item_info_s sync_agent_mo_acc_info_item;

	sync_agent_mo_acc_info_item.acc_type = DEVICE_MANAGE_ACC_TYPE_CLIENT;
	sync_agent_mo_acc_info_item.serverid = (*session)->pServer_id;

	err_code = sync_agent_get_mo_acc_item(&sync_agent_mo_acc_info_item, &acc_info);
	//acc_info = sync_agent_get_acc_info(pServer_id, DEVICE_MANAGE_ACC_TYPE_CLIENT);
	if (err_code != SYNC_AGENT_DM_MO_SUCCESS || acc_info == NULL) {
		_DEBUG_VERBOSE(" acc null!!\n");
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}

	_DEBUG_VERBOSE(" id = %s\n", acc_info->auth_name);
	_DEBUG_VERBOSE(" pwd = %s\n", acc_info->auth_secret);
	_DEBUG_VERBOSE(" serverUrl = %s\n", acc_info->addr);
	_DEBUG_VERBOSE(" nextNonce = %s\n", acc_info->auth_data);

	if ((acc_info->auth_name) == NULL || (acc_info->auth_secret) == NULL || (acc_info->addr) == NULL) {
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}

	auth_type = __get_Session_AuthType_enum(acc_info->auth_type);
	if (auth_type == AUTH_TYPE_HMAC) {

		_DEBUG_VERBOSE("auth name : %s", acc_info->auth_name);
		_DEBUG_VERBOSE("auth secret : %s", acc_info->auth_secret);
		_DEBUG_VERBOSE("auth data : %s", acc_info->auth_data);
/*		 _DEBUG_VERBOSE("xml : %s", xml);
		 _DEBUG_VERBOSE("xml_len : %d", xml_len);*/

		unsigned char *nonce = NULL;
		unsigned int nonce_size = 0;
		ret = nonce_decode(acc_info->auth_data, &nonce, &nonce_size);
		_DEBUG_VERBOSE("nonce result : %d", ret);
		_DEBUG_VERBOSE("nonce : %s, noncesize : %d", nonce, nonce_size);

		char *mac = NULL;
		ret = gemerate_hmac(acc_info->auth_name, acc_info->auth_secret, nonce, nonce_size, xml, xml_len, &mac);
		if (ret != DM_OK)
			goto error;
		_DEBUG_VERBOSE("mac : %s", mac);

		ret = create_hmac(acc_info->auth_name, "MD5", mac, session);
		str_free(&mac);
		if (ret != DM_OK)
			goto error;
	} else {
		_DEBUG_VERBOSE(" auth type : [%d] \n", auth_type);
	}

	sync_agent_free_mo_acc_item(&acc_info);

	_INNER_FUNC_EXIT;
	return ret;
 error:

	sync_agent_free_mo_acc_item(&acc_info);

	_INNER_FUNC_EXIT;
	_DEBUG_VERBOSE(" end : %d", ret);
	return ret;
}

static DM_ERROR __create_authentication(Session ** session, char *client_id, char *client_pwd, int isBase64, char *nextNonce)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");
	retvm_if((client_id) == NULL, COMMON_ERR_IS_NULL, "client_id is NULL!!");

	Cred *pCred = NULL;
	Chal *chal = (*session)->res_chal;
	FormatType formatType = FORMAT_TYPE_UNKNOWN;
	AuthType pAuthType = AUTH_TYPE_UNKNOWN;
	char *credData = NULL;

	ret = get_dmacc_authType((*session)->pServer_id, &pAuthType);
	if (ret != DM_OK)
		goto error;
	_DEBUG_VERBOSE(" pAuthType : [%d] \n", pAuthType);

	if (chal == NULL) {
		_DEBUG_VERBOSE("CHAL DOES NOT EXIST\n");

		unsigned int out_len = 0;
		unsigned char *nextNonceDecode = NULL;

		switch (pAuthType) {
		case AUTH_TYPE_BASIC:{
				formatType = FORMAT_TYPE_BASE64;
			}
			break;
		case AUTH_TYPE_MD5:
		case AUTH_TYPE_HMAC:
			{
				if (isBase64 == 1)
					formatType = FORMAT_TYPE_BASE64;
				else
					formatType = FORMAT_TYPE_UNKNOWN;

				if (nextNonce != NULL) {
					_DEBUG_VERBOSE("nextnonce : %s", nextNonce);
					ret = nonce_decode(nextNonce, &nextNonceDecode, &out_len);
					_DEBUG_VERBOSE("nonce result : %d", ret);
					_DEBUG_VERBOSE("nextnoncedecode : %s out len : %d", nextNonceDecode, out_len);
				} else {
					pAuthType = AUTH_TYPE_BASIC;
					formatType = FORMAT_TYPE_BASE64;
				}
				if (pAuthType == AUTH_TYPE_HMAC) {
					(*session)->isHMAC = 1;

				}
			}
			break;
		default:
			break;
		}
		_DEBUG_VERBOSE(" client id : %s , client pwd : %s\n", client_id, client_pwd);

		ret = create_auth_credString(pAuthType, client_id, client_pwd, nextNonceDecode, out_len, &credData);
		if (ret != DM_OK)
			goto error;

		ret = create_auth_cred(client_id, client_pwd, pAuthType, formatType, credData, &pCred);
		if (ret != DM_OK)
			goto error;

	} else {
		_DEBUG_VERBOSE("CHAL DOES  EXIST\n");

		unsigned int out_len = 0;
		unsigned char *nextNonceDecode = NULL;

		if ((*session)->res_chal->type == AUTH_TYPE_HMAC) {
			(*session)->isHMAC = 1;

		} else {
			(*session)->isHMAC = 0;
		}

		if (chal->nonce_plain != NULL) {
			_DEBUG_VERBOSE("chal->nonce_plain : %s", chal->nonce_plain);
			ret = nonce_decode(chal->nonce_plain, &nextNonceDecode, &out_len);
			_DEBUG_VERBOSE("nonce result : %d", ret);
			_DEBUG_VERBOSE("nextnonceDecode : %s, out len : %d", nextNonceDecode, out_len);
			if (ret != DM_OK)
				goto error;
		}
		ret = create_auth_credString(chal->type, client_id, client_pwd, chal->type == AUTH_TYPE_BASIC ? (const unsigned char *)(chal->nonce_b64) : (const unsigned char *)(nextNonceDecode), out_len, &credData);
		if (ret != DM_OK)
			goto error;

		ret = create_auth_cred(client_id, client_pwd, chal->type, chal->format, credData, &pCred);
		if (ret != DM_OK)
			goto error;
	}
	str_free(&credData);

	ret = set_session_authcred((*session), pCred);
	if (ret != DM_OK)
		goto error;

	_INNER_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_VERBOSE(" end error %d \n", ret);
	_INNER_FUNC_EXIT;
	return ret;
}

static DM_ERROR _generate_generic_alert(Session ** session, char *pServer_id, char *pSession_id, char **msg, int *msg_size, int service_engine_id, int result)
{
	_INNER_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	engine_status *service_status = NULL;
	int isFinal = 1;

	retvm_if((pServer_id) == NULL, COMMON_ERR_IS_NULL, "pServer_id is NULL!!");

	char *sourceUrl = NULL;
	char *client_id = NULL;
	char *client_pwd = NULL;
	char *targetUrl = NULL;
	int isBase64 = 0;
	char *nextNonce = NULL;
	SyncHdr *pSyncHdr = NULL;
	Command *pAlertCommand = NULL;
	char *format = NULL;
	char *contenttype = NULL;
	Command *pReplaceCommand = NULL;
	char *data = NULL;
	char *pCorrelator = NULL;
	char *rSourceUrl = NULL;

	/*Client init */

	if ((*session) != NULL) {
		_DEBUG_TRACE("session is Exist");
		if ((*session)->sessionID != NULL) {
			_DEBUG_TRACE("session id : %s", (*session)->sessionID);
			pSession_id = (*session)->sessionID;
		}
	} else if (pSession_id == NULL) {
		//pSession_id = g_strdup_printf("%ld", time(NULL));
		pSession_id = g_strdup_printf("%ld", time(NULL)%10000);
		_DEBUG_INFO("generate from time -> pSession_id [%s]", pSession_id);
	}
	_DEBUG_INFO("pSession_id [%s]", pSession_id);
	ret = __get_dmaccInfo(pServer_id, &client_id, &client_pwd, &sourceUrl, &targetUrl, &isBase64, &nextNonce);

	if (ret != DM_OK)
		goto error;

	if (nextNonce == NULL) {
		(nextNonce) = strdup(ELEMENT_DEFAULT_CLIENT_NONCE);
	}

	_DEBUG_TRACE(" id = %s\n", client_id);
	_DEBUG_TRACE(" pwd = %s\n", client_pwd);
	_DEBUG_TRACE(" Session_id = %s\n", pSession_id);
	_DEBUG_TRACE(" sourceUrl = %s\n", sourceUrl);
	_DEBUG_TRACE(" serverUrl = %s\n", targetUrl);
	_DEBUG_TRACE(" nextNonce = %s\n", nextNonce);

	if ((*session) == NULL) {
		ret = create_syncml_session(VERSION_12, PROTOCOL_TYPE_DM, pServer_id, client_id, pSession_id, sourceUrl, targetUrl, session);
		if (ret != DM_OK)
			goto error;
	} else {
		(*session)->msgID = 0;

#ifdef _DM_BUNDANG_TEST
		/*redirection code */
		if ((*session)->target->locURI != NULL) {
			char *replace = NULL;
			replace = get_new_uri((*session)->target->locURI);
			str_free(&((*session)->target->locURI));
			(*session)->target->locURI = g_strdup(replace);
			str_free(&replace);
			_DEBUG_TRACE(" targer url : %s \n", (*session)->target->locURI);
		}
#endif

		//nextNonce = g_nextnonce;
		//nextNonce = strdup( (*session)->res_chal->nonce_plain);
		_DEBUG_TRACE(" NextNonce = %s\n", nextNonce);
		isBase64 = 1;
	}

	ret = __create_authentication(session, client_id, client_pwd, isBase64, nextNonce);
	if (ret != DM_OK)
		goto error;

	/*SyncHdr */

	ret = create_syncml_hdr((*session), &pSyncHdr);
	if (ret != DM_OK)
		goto error;

	GList *commands = NULL;
	GList *commands_last = NULL;

	/* create alert */
	ret = create_alert_cmd((*session), DM_ALERT_CLIENT_INITIATED_MGMT, &pAlertCommand);
	if (ret != DM_OK)
		goto error;
	putCmdIntoList(&commands, &commands_last, pAlertCommand);

	/* create alert */
	pAlertCommand = NULL;
	ret = create_alert_cmd((*session), DM_ALERT_GENERIC_ALERT, &pAlertCommand);
	if (ret != DM_OK)
		goto error;

	_DEBUG_TRACE(" service engine status = [%d]\n", service_engine_id);

	ret = Get_Engine_Status(service_engine_id, &service_status);
	if (ret != DM_OK) {
		goto error;
	}

	_DEBUG_TRACE("STATUS VALUE : %d ", service_status->engine_status);
	_DEBUG_TRACE("generic alert status data  : %d ", result);

	if (service_status->correlator != NULL) {
		pCorrelator = g_strdup(service_status->correlator);
	}
	if (service_status->mo_path != NULL) {
		rSourceUrl = g_strdup(service_status->mo_path);
	}
	if (rSourceUrl == NULL) {
		ret = COMMON_ERR_IS_NULL;
		goto error;
	}

	SERVICE_SERVER_TYPE engine_type = get_service_type(rSourceUrl);
	switch (engine_type) {
	case SAMSUNG_FUMO_TYPE:
	case GCF_TYPE:
		//format = strdup("chr");//NULL;
		//contenttype = strdup("text/plain");//NULL;
		contenttype = strdup("org.openmobilealliance.dm.firmwareupdate.downloadandupdate");
		break;
	case SAMSUNG_FMM_TYPE:
		contenttype = strdup("urn:oma:at:lawmo:1.0:OperationComplete");
		break;
	default:
		break;
	}
	//ret =  get_mo_node_format_contenttype(rSourceUrl ,&format, &contenttype );

	data = g_strdup_printf("%d", result);
	unsigned int size = 0;
	unsigned int moredata = 0;
	_DEBUG_TRACE("generic alert status data  : %s ", data);

	Item *pItem = NULL;
	ret = set_correlator(&pAlertCommand, pCorrelator);

	ret = construct_Item(rSourceUrl, format, contenttype, data, size, moredata, &pItem);
	pAlertCommand->private.alert.items = g_list_append(pAlertCommand->private.alert.items, pItem);
	putCmdIntoList(&commands, &commands_last, pAlertCommand);

	/*       create replace */
	ret = __create_devinfo_replace_cmd((*session), &pReplaceCommand);
	if (ret != DM_OK)
		goto error;
	putCmdIntoList(&commands, &commands_last, pReplaceCommand);

	/*create SyncML */
	SyncML *pSyncML = NULL;
	ret = create_syncml(pSyncHdr, NULL, commands, isFinal, &pSyncML);
	if (ret != DM_OK)
		goto error;

	(*session)->isSendingfinal = isFinal;

	/* convert Msg */
	ret = syncml_objectbinder(pSyncML, msg, (unsigned int *)msg_size);
	if (ret != DM_OK)
		goto error;

	/*xml */
/*	xml_len = remove_xml_header(xml, xml_len);
	_DEBUG_TRACE("msg : %s", msg);
	ret = check_hmac( session,  xml, xml_len);*/

	/*wbxml */
	_DEBUG_TRACE("msg : %s", msg);
	ret = __check_hmac(session, *msg, *msg_size);

	//free generate structure
	free_syncml(pSyncML);
	pSyncML = NULL;

	//session->status, session->alertCommand are already freed in free_SyncML function
	if ((*session)->resultsCommand != NULL) {
		g_list_free((*session)->resultsCommand);
		(*session)->resultsCommand = NULL;
	}

	if ((*session)->alertCommand != NULL) {
		g_list_free((*session)->alertCommand);
		(*session)->alertCommand = NULL;
	}

	(*session)->status = NULL;
	(*session)->isSendingfinal = isFinal;

	Free_Memory_Engine_Status(&service_status, 1);

	str_free(&pCorrelator);
	str_free(&rSourceUrl);
	str_free(&data);

	_INNER_FUNC_EXIT;
	return ret;

 error:
	if (service_status != NULL)
		Free_Memory_Engine_Status(&service_status, 1);

	str_free(&pCorrelator);
	str_free(&rSourceUrl);
	str_free((&data));

	_DEBUG_TRACE(" error : %d\n", ret);
	_INNER_FUNC_EXIT;
	return ret;
}

DM_ERROR send_msg(char *accType, int transportType, Session ** session, GList * header_info, char *sendMsg, unsigned int sendMsg_length, GList ** recv_header, char **recvMsg, unsigned int *recvMsg_length)
{
	_EXTERN_FUNC_ENTER;
	DM_ERROR ret = DM_OK;

	retvm_if((accType) == NULL, COMMON_ERR_IS_NULL, "accType is NULL!!");
	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");
	retvm_if((sendMsg) == NULL, COMMON_ERR_IS_NULL, "sendMsg is NULL!!");

	ret = _send_msg(accType, transportType, session, (*session)->target->locURI, header_info, sendMsg, sendMsg_length, recv_header, recvMsg, recvMsg_length);
	if (ret != DM_OK)
		goto error;

	_EXTERN_FUNC_EXIT;
	return DM_OK;

 error:
	_DEBUG_INFO(" error : %d\n", ret);

	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR recv_msg(Session ** session, char *recvMsg, unsigned int recvMsg_length, ENGINE_ID * service_engine_id, int *isFinish, int isgeneticAlert)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");
	retvm_if((recvMsg) == NULL, COMMON_ERR_IS_NULL, "recvMsg is NULL!!");

	ret = _recv_msg(session, recvMsg, recvMsg_length, service_engine_id, isFinish, isgeneticAlert);
	if (ret != DM_OK)
		goto error;

	_EXTERN_FUNC_EXIT;
	return DM_OK;

 error:
	_DEBUG_INFO(" error : %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR _send_msg(char *accType, int transportType, Session ** session, char *server_url, GList * header_info, char *sendMsg, unsigned int sendMsg_length, GList ** recv_header, char **recvMsg, unsigned int *recvMsg_length)
{
	_INNER_FUNC_ENTER;

	sync_agent_na_result_e res = SYNC_AGENT_NA_SUCCESS;
	DM_ERROR ret = DM_OK;

	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");
	retvm_if((accType) == NULL, COMMON_ERR_IS_NULL, "accType is NULL!!");
	retvm_if((server_url) == NULL, COMMON_ERR_IS_NULL, "server_url is NULL!!");
	retvm_if((sendMsg) == NULL, COMMON_ERR_IS_NULL, "sendMsg is NULL!!");

	int net_session_id = 0;
	bool cancel_flag;

	_DEBUG_TRACE("server url : %s", server_url);
	//timeout : 30 seconds
	res = sync_agent_open_connection(transportType, 30, (unsigned int *)(&(net_session_id)));

	_DEBUG_TRACE("res =%d\n", res);

	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto error;
	}

	sync_agent_register_cancel_callback(network_cancel_callback, &(net_session_id));

	//check cancel flag (note that this can be called every where inside processing logic)
	cancel_flag = sync_agent_check_cancel_flag();
	if (cancel_flag) {
		ret = DM_ERR_USER_CANDELLED;
		sync_agent_unregister_cancel_callback(network_cancel_callback, &(net_session_id));
		goto returnerror;
	}

	int err = 0;
	if ((*session)->reqhmacinfo != NULL) {
		err = NA_http_header_binder(accType, server_url, (*session)->jsessionid, (*session)->isHMAC, (*session)->reqhmacinfo->username, (*session)->reqhmacinfo->mac, &header_info);
	} else {
		err = NA_http_header_binder(accType, server_url, (*session)->jsessionid, (*session)->isHMAC, NULL, NULL, &header_info);
	}

	if (err != 1) {
		_DEBUG_TRACE("header binding error : %d", err);
		ret = DM_ERR_USER_CANDELLED;
		sync_agent_unregister_cancel_callback(network_cancel_callback, &(net_session_id));
		goto returnerror;
	}

	res = sync_agent_send_msg(	/*acc_info (id, pw, uri, ...), */
					 header_info, transportType, sendMsg, sendMsg_length, recv_header, (unsigned char **)recvMsg, recvMsg_length, SYNC_AGENT_NA_SEND_TYPE_SEND_N_RECEIVE, net_session_id);
	_DEBUG_TRACE("  res =%d\n", res);

	sync_agent_unregister_cancel_callback(network_cancel_callback, &(net_session_id));
	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto returnerror;
	} else {
		cancel_flag = sync_agent_check_cancel_flag();
		if (cancel_flag) {
			ret = DM_ERR_USER_CANDELLED;
			goto returnerror;
		}
	}

	//_DEBUG_TRACE("recieve http header : %s ", (*recv_header));

	_DEBUG_TRACE(" CKECK JSESSION\n");
	char *temp_jsessionid = NULL;
	char *real_jsessionid = NULL;
	char *real_other_cookie = NULL;
	res = sync_agent_get_header_info(1, (*recv_header), "Set-Cookie", &temp_jsessionid);
	if (res == SYNC_AGENT_NA_SUCCESS) {
		ret = add_jsession_in_httpheader(*recv_header, temp_jsessionid, &real_jsessionid, &real_other_cookie);
		if (ret == DM_OK) {
			if (real_jsessionid != NULL && (*session)->jsessionid != NULL) {
				if (strcmp((*session)->jsessionid, real_jsessionid) != 0) {
					str_free(&((*session)->jsessionid));
					(*session)->jsessionid = strdup(real_jsessionid);
				} else {
					str_free(&real_jsessionid);
				}
			} else {
				(*session)->jsessionid = strdup(real_jsessionid);
			}

			_DEBUG_TRACE("-------------------------------------------------------------------------------------------------------------------------------------------------");
			_DEBUG_TRACE("jsession id = %s\n", (*session)->jsessionid);
			_DEBUG_TRACE("-------------------------------------------------------------------------------------------------------------------------------------------------");
		}
		ret = DM_OK;
	}

	char *temp_hmac = NULL;
	res = sync_agent_get_header_info(1, (*recv_header), HTTP_X_SYNCML_HMAC, &temp_hmac);
	if (res == SYNC_AGENT_NA_SUCCESS) {
		if (temp_hmac != NULL) {
			ret = add_response_hmacinfo(session, temp_hmac);
			if (ret != DM_OK) {
				res = sync_agent_close_connection(transportType, net_session_id);
				_DEBUG_TRACE(" %d res =%d\n", __LINE__, res);
				goto returnerror;
			}
		}
	}

	res = sync_agent_close_connection(transportType, net_session_id);

	_DEBUG_TRACE("res =%d\n", res);
	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto returnerror;
	}

	if (sendMsg != NULL)
		free(sendMsg);

	_INNER_FUNC_EXIT;
	return ret;

 returnerror:
	_DEBUG_TRACE(" returnerror = %d\n", ret);

	res = sync_agent_close_connection(transportType, net_session_id);
	_DEBUG_TRACE(" %d res =%d\n", __LINE__, res);
	if (res != SYNC_AGENT_NA_SUCCESS) {
		ret = COMMON_ERR_INTERNAL_CONNECTION_ERROR;
		goto error;
	}

	if (sendMsg != NULL)
		free(sendMsg);

	_INNER_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_TRACE(" error = %d\n", ret);

	if (sendMsg != NULL)
		free(sendMsg);

	_INNER_FUNC_EXIT;
	return ret;
}

static DM_ERROR _recv_msg(Session ** session, char *recvMsg, unsigned int recvMsg_length, ENGINE_ID * service_engine_id, int *isFinish, int isgeneticAlert)
{

	_INNER_FUNC_ENTER;
	DM_ERROR ret = DM_OK;

	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");
	retvm_if((recvMsg) == NULL, COMMON_ERR_IS_NULL, "recvMsg is NULL!!");

	reset_cmdid_session((*session));

	SyncML *pSyncML = (SyncML *) calloc(1, sizeof(SyncML));
	if (pSyncML == NULL) {
		_DEBUG_TRACE("alloc fail");
		ret = COMMON_ERR_ALLOC;
		goto error;
	}
	char *xml = NULL;
	unsigned int xml_len = 0;
	ret = reverse_syncml_objectbinder(&pSyncML, recvMsg, recvMsg_length, &xml, &xml_len);
	if (ret != DM_OK)
		goto error;

	/* check receive msg */
	ret = receive_header((*session), pSyncML->hdr);
	if (ret != DM_OK)
		goto error;

	ret = receive_statuses((*session), pSyncML->status);
	if (ret != DM_OK) {
		goto error;
	}

	/* check that pending list is empty */
	GList *o = NULL;
	for (o = (*session)->pendingStatus; o != NULL; o = g_list_next(o)) {
		PendingStatus *pending = o->data;
		_DEBUG_TRACE(" in pendingStatus List msgID : %d, cmdID : %d\n", pending->msgID, pending->cmdID);
	}

	ret = receive_cmd((*session), pSyncML->commands);
	if (ret != DM_OK)
		goto error;

	/*service engine */
	if ((*session)->service_engine_id != NO_SERVICE_ENGINE) {
		*service_engine_id = (*session)->service_engine_id;
	}

	if (pSyncML->final != 0) {
		(*session)->isSendingfinal = 0;
		(*session)->isReceivingFinal = 1;
	} else {
		(*session)->isReceivingFinal = 0;
	}

	_DEBUG_TRACE("receive final : %d , isgeneticAlert : %d ", (*session)->isReceivingFinal, isgeneticAlert);

	if (*session != NULL && (*session)->isReceivingFinal != 0 && ((*session)->status != NULL && g_list_length((*session)->status) == 1 && (*session)->resultsCommand == NULL && isgeneticAlert == 0)) {
		(*isFinish) = 1;
		_DEBUG_TRACE("---------------------------------------------------------------------------recv msg end------------------------------------------------------------------------\n");
		_DEBUG_TRACE("---------------------------------------------------------------------------free session------------------------------------------------------------------------\n");
		free_session((*session));
		*session = NULL;
	} else {
		_DEBUG_TRACE("not final");
	}

	if (recvMsg != NULL)
		free(recvMsg);
	if (pSyncML != NULL)
		free_syncml(pSyncML);

	_INNER_FUNC_ENTER;
	return ret;

 error:
	_DEBUG_TRACE(" error : %d\n", ret);
	if (recvMsg != NULL)
		free(recvMsg);
	if (pSyncML != NULL)
		free_syncml(pSyncML);

	_INNER_FUNC_ENTER;
	return ret;

}

DM_ERROR setup_phase(int transportType, Session ** session, char *pServer_id, char *pSession_id, int session_type, ENGINE_ID * service_engine_id, int *isFinish, int isgenericAlert)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;
	retvm_if((pServer_id) == NULL, COMMON_ERR_IS_NULL, "pServer_id is NULL!!");

	/*generate setupPhase Msg */
	char *msg = NULL;
	unsigned int msg_size = 0;

	/*exchange Msg */
	char *recv_msg = NULL;
	int recv_msg_size = 0;

	GList *recv_header = 0;
	GList *list = NULL;
	int count = 0;

	while (count++ != MAX_SESSION_COUNT) {
		//when errortype is ERROR_AUTH_REQUIRED try request using res_chal just one more time
		//it can be only happend when server required MD5 authentication(we send basic authentication every time by default)
		msg = NULL;
		msg_size = 0;
		if( pSession_id != NULL ) {
			_DEBUG_INFO("pSession_id [%s]", pSession_id);
		} else {
			_DEBUG_INFO("pSession_id is NULL");
		}

		ret = _generate_setupphase_msg(session, pServer_id, pSession_id, session_type, &msg, &msg_size);
		if (ret != DM_OK)
			goto error;

		file_count++;
		char file_name[1000];
		/*debuging */
		snprintf(file_name, (int)sizeof(file_name), "/opt/data/oma-dm-cfg/packet/wbxml_msg_%d_send.wbxml", file_count);
		_DEBUG_INFO("file name : %s", file_name);
		sync_agent_write_whole_file(file_name, msg, msg_size, true);

		list = NULL;
		recv_header = NULL;
		ret = _send_msg("OMA_DM", 1, session, (*session)->target->locURI, list, msg, msg_size, &recv_header, &recv_msg, (unsigned int *)(&recv_msg_size));
		if (ret != DM_OK)
			goto error;
		/*debuging */
		file_count++;
		snprintf(file_name, (int)sizeof(file_name), "/opt/data/oma-dm-cfg/packet/wbxml_msg_%d_receivced.wbxml", file_count);
		_DEBUG_INFO("file name : %s", file_name);
		sync_agent_write_whole_file(file_name, recv_msg, recv_msg_size, true);

		ret = _recv_msg(session, recv_msg, recv_msg_size, service_engine_id, isFinish, isgenericAlert);
		if (ret != DM_ERR_UNAUTHORIZED && ret != DM_ERR_AUTHENTICATION_REQUIRED) {
			_DEBUG_INFO("  authentication : %d\n ", ret);
			break;
		}

		_DEBUG_INFO("revceive end ======================");

	}

	if (ret != DM_OK || (*session != NULL && (*session)->status != NULL && (g_list_length((*session)->status) == 1 && (*session)->resultsCommand == NULL && (*session)->alertCommand == NULL && isgenericAlert == 0))) {
		*isFinish = 1;
		_DEBUG_INFO("---------------------------------------------------------------------------recv msg end------------------------------------------------------------------------\n");
		_DEBUG_INFO("---------------------------------------------------------------------------free session------------------------------------------------------------------------\n");
		free_session((*session));
		*session = NULL;
		if (ret != DM_OK)
			goto error;
	}

	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_INFO(" end error %d\n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR management_phase(int transportType, Session ** session, char *pSession_id, ENGINE_ID * service_engine_id, int *isFinish)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((*session) == NULL, COMMON_ERR_IS_NULL, "session is NULL!!");

	while ((*isFinish) == 0) {

		/*generate management phase Msg */
		char *msg = NULL;
		unsigned int msg_size = 0;

		/*exchange Msg */
		char *recvMsg = NULL;
		unsigned int recvMsg_size = 0;
		GList *recv_header = 0;
		GList *list = NULL;

		ret = _generate_management_msg(session, &msg, &msg_size);
		if (ret != DM_OK)
			goto error;
		/*for debuging */
		file_count++;
		char file_name[1000];
		snprintf(file_name, (int)sizeof(file_name), "/opt/data/oma-dm-cfg/packet/wbxml_msg_%d_management_send.wbxml", file_count);
		_DEBUG_INFO("file name : %s", file_name);
		sync_agent_write_whole_file(file_name, msg, msg_size, true);

		ret = send_msg("OMA_DM", 1, session, list, msg, msg_size, &recv_header, &recvMsg, &recvMsg_size);
		if (ret != DM_OK)
			goto error;

		/*for debuging */
		file_count++;
		snprintf(file_name, (int)sizeof(file_name), "/opt/data/oma-dm-cfg/packet/wbxml_msg_%d_management_receivced.wbxml", file_count);
		_DEBUG_INFO("file name : %s", file_name);
		sync_agent_write_whole_file(file_name, recvMsg, recvMsg_size, true);

		ret = recv_msg(session, recvMsg, recvMsg_size, service_engine_id, isFinish, 0);
		if (ret != DM_OK)
			goto error;

		_DEBUG_INFO(" end  : %d \n", ret);

	}
	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_INFO(" error : %d end \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR generic_alert(int transportType, Session ** session, char *pServer_id, char *pSession_id, int *isFinish, int isgeneticAlert, ENGINE_ID service_engine_id, int result)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	/*generate setupPhase Msg */
	char *msg = NULL;
	unsigned int msg_size = 0;

	/*exchange Msg */
	GList *list = NULL;
	char *recvMsg = NULL;
	unsigned int recvMsg_size = 0;
	GList *recv_header = 0;

	int count = 0;
	while (count++ != MAX_SESSION_COUNT) {

		msg = NULL;
		msg_size = 0;
		ret = _generate_generic_alert(session, pServer_id, pSession_id, &msg, (int *)(&msg_size), service_engine_id, result);
		if (ret != DM_OK)
			goto error;

		file_count++;
		char file_name[1000];

		snprintf(file_name, (int)sizeof(file_name), "/opt/data/oma-dm-cfg/packet/wbxml_generic_send_msg_%d.wbxml", file_count);
		_DEBUG_INFO("file name : %s", file_name);
		sync_agent_write_whole_file(file_name, msg, msg_size, true);

		list = NULL;
		recv_header = NULL;
		ret = send_msg("OMA_DM", 1, session, list, msg, msg_size, &recv_header, &recvMsg, &recvMsg_size);
		if (ret != DM_OK)
			goto error;

		file_count++;
		snprintf(file_name, (int)sizeof(file_name), "/opt/data/oma-dm-cfg/packet/wbxml_generic_receivce_msg_%d.wbxml", file_count);
		_DEBUG_INFO("file name : %s", file_name);
		sync_agent_write_whole_file(file_name, recvMsg, recvMsg_size, true);

		ret = recv_msg(session, recvMsg, recvMsg_size, &service_engine_id, isFinish, isgeneticAlert);
		_DEBUG_INFO(" Recv_Msg : %d count : %d \n ", ret, count);

		if (ret != DM_ERR_UNAUTHORIZED && ret != DM_ERR_AUTHENTICATION_REQUIRED) {
			_DEBUG_INFO(" generic alert authentication : %d\n ", ret);
			break;
		}
	}

	_DEBUG_INFO(" end  : %d \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_INFO(" error : %d end \n", ret);
	_EXTERN_FUNC_EXIT;
	return ret;
}
