/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_common.h"
#include "common/util/util.h"
#include "serviceadapter/networkbinder/na_binder.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"OMA_DM_NA"
#endif

int NA_http_header_binder(char *accType, char *loc_uri, char *sessionid, int isHMAC, char *username, char *mac, GList ** list)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((accType) == NULL, COMMON_ERR_IS_NULL, "accType is NULL!!");
	retvm_if((loc_uri) == NULL, COMMON_ERR_IS_NULL, "loc_uri is NULL!!");

	if (strcmp(accType, "OMA_DM") == 0) {
		_DEBUG_INFO(" header info start\n");
		sync_agent_na_common_header_info_s *method = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (method == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		method->key = "method";
		method->value = "post";
		*list = g_list_append(*list, method);
		_DEBUG_INFO(" post method\n");

		sync_agent_na_common_header_info_s *uri = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (uri == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		uri->key = "uri";
		uri->value = loc_uri;
		*list = g_list_append(*list, uri);
		_DEBUG_INFO(" uri\n");
/*
		sync_agent_na_common_header_info_s *cache_control = (sync_agent_na_common_header_info_s*)malloc(sizeof(sync_agent_na_common_header_info_s));
		memset(cache_control, 0x00, sizeof(sync_agent_na_common_header_info_s));
		cache_control->key = "Cache-Control";
		cache_control->value = "no-store, private";
		*list = g_list_append(*list, cache_control);
		_DEBUG_INFO( " \n");
*/
		sync_agent_na_common_header_info_s *connection = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (connection == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		connection->key = "Connection";
		connection->value = "Keep-Alive";
		*list = g_list_append(*list, connection);
		_DEBUG_INFO(" connection\n");

		sync_agent_na_common_header_info_s *useragent = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (useragent == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		useragent->key = "User-Agent";
		useragent->value = "Samsung electronics GT-I8800 SyncML_DM Client";
		*list = g_list_append(*list, useragent);
		_DEBUG_INFO(" user agent\n");

		sync_agent_na_common_header_info_s *accept = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (accept == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		accept->key = "Accept";
		accept->value = "application/vnd.syncml.dm+wbxml";
		*list = g_list_append(*list, accept);
		_DEBUG_INFO(" accept\n");

/*		sync_agent_na_common_header_info_s *accept_language = (sync_agent_na_common_header_info_s*)malloc(sizeof(sync_agent_na_common_header_info_s));
		memset(accept_language, 0x00, sizeof(sync_agent_na_common_header_info_s));
		accept_language->key = "Accept-Language";
		accept_language->value = "en-US";
		*list = g_list_append(*list, accept_language);
		_DEBUG_INFO( " \n");*/

		sync_agent_na_common_header_info_s *accept_charset = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (accept_charset == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		accept_charset->key = "Accept-Charset";
		accept_charset->value = "UTF-8";
		*list = g_list_append(*list, accept_charset);
		_DEBUG_INFO(" accept charset\n");

		sync_agent_na_common_header_info_s *content_type = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
		if (content_type == NULL) {
			_DEBUG_INFO("alloc error");
			_EXTERN_FUNC_EXIT;
			return 0;
		}
		content_type->key = "Content-Type";
		content_type->value = "application/vnd.syncml.dm+wbxml";
		*list = g_list_append(*list, content_type);
		_DEBUG_INFO(" content type\n");

		if (sessionid != NULL) {
			_DEBUG_INFO(" jsessionid : %s\n", sessionid);
			sync_agent_na_common_header_info_s *cookie = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
			if (cookie == NULL) {
				_DEBUG_INFO("alloc error");
				_EXTERN_FUNC_EXIT;
				return 0;
			}
			cookie->key = HTTP_COOKIE;
			cookie->value = sessionid;
			*list = g_list_append(*list, cookie);
			_DEBUG_INFO(" cookie :  %s\n", cookie->value);
		}

		if (isHMAC == 1) {
			/*      x-syncml-hmac: algorithm=MD5, username="156354238", mac=SF2dMGut4v2Mpmsrs1ydTw== */
			_DEBUG_INFO(" x-syncml-hmac : %d\n", isHMAC);
			sync_agent_na_common_header_info_s *hmac = (sync_agent_na_common_header_info_s *) calloc(1, sizeof(sync_agent_na_common_header_info_s));
			if (hmac == NULL) {
				_DEBUG_INFO("alloc error");
				_EXTERN_FUNC_EXIT;
				return 0;
			}
			hmac->key = HTTP_X_SYNCML_HMAC;
			if (username != NULL) {
				hmac->value = g_strdup_printf("algorithm=MD5, username=\"%s\", mac=%s", username, mac);
				_DEBUG_INFO(" hmac = %s : %s\n", hmac->key, hmac->value);
			}
			*list = g_list_append(*list, hmac);
		}
	}

	GList *iter = NULL;
	for (iter = *list; iter != NULL; iter = g_list_next(iter)) {
		fputs("[NA_Header_Binder] key : ", stdout);
		if (fputs(((sync_agent_na_common_header_info_s *) (iter->data))->key, stdout) == EOF) {
			_DEBUG_INFO("debuging test error");
			break;
		}
		fputs(", value : ", stdout);
		if (fputs(((sync_agent_na_common_header_info_s *) (iter->data))->value, stdout) == EOF) {
			_DEBUG_INFO("debuging test error");
			break;
		}
		fputs("\n", stdout);
	}

	_EXTERN_FUNC_EXIT;
	return 1;
}

int add_jsession_in_httpheader(GList * recv_header, char *temp_jsessionid, char **real_jsessionid, char **real_other_cookie)
{
	_EXTERN_FUNC_ENTER;
	DM_ERROR ret = DM_OK;

	char *start_jessionid_msg = NULL;
	_DEBUG_INFO(" set cookie is : %s \n", temp_jsessionid);

	/*temp_jsessionid = "JSESSIONID=D15C621FD39535094F1B000307D5D520.sdm1; Path=/v1/sdm; Secure" */
	if (temp_jsessionid != NULL) {

		_DEBUG_INFO("-------------------------------------------------------------------------------------------------------------------------------------------------");
		_DEBUG_INFO("cookie is = %s\n", temp_jsessionid);
		_DEBUG_INFO("-------------------------------------------------------------------------------------------------------------------------------------------------");

		start_jessionid_msg = strstr(temp_jsessionid, "JSESSIONID");
		if (start_jessionid_msg != NULL) {
			_DEBUG_INFO(" set cookie is : %s \n", start_jessionid_msg);
			(*real_jsessionid) = strdup(start_jessionid_msg);

			_DEBUG_INFO(" set cookie is : %s \n", (*real_jsessionid));
		} else {
			ret = COMMON_ERR_INTERNAL_NOT_DEFINED;
		}

	} else {
		ret = COMMON_ERR_INTERNAL_NOT_DEFINED;
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}
