/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*sync-agent*/
#include <sync_agent.h>

/*dm-agent*/
#include "common/dm_common.h"
#include "common/util/util.h"
#include "serviceadapter/sa_syncml_binders.h"
#include "serviceadapter/protocolbinder/syncml_protocol_binder.h"
#include "serviceadapter/sa_elements_internal.h"
#include "serviceadapter/sa_command_internal.h"
#include "serviceadapter/sa_util.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"OMA_DM_SA"
#endif

DM_ERROR syncml_objectbinder(SyncML * pSyncML, char **msg, unsigned int *msg_size)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((pSyncML) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "pSyncML is NULL!!");

	sync_agent_pb_protocol_binder_function_set_s *pBinder_function_set = NULL;
	sync_agent_pb_protocol_binder_info_s *pBinder = NULL;

	sync_agent_pb_error_e err = init_syncml_binder_function_set(&pBinder_function_set);
	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}

	pBinder = syncml_binder_init(pBinder, SYNC_AGENT_PB_ENCODING_WBXML_12, false, true, pBinder_function_set);

	err = syncml_binder_append(pBinder, PE_SYNCML_START, NULL);
	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}

	err = syncml_binder_append(pBinder, PE_HEADER, pSyncML->hdr);
	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}
	err = syncml_binder_append(pBinder, PE_BODY_START, NULL);
	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}

	/* TODO : Status handling */
	GList *status_iter = NULL;
	Status *pStatus = NULL;
	for (status_iter = pSyncML->status; status_iter != NULL; status_iter = g_list_next(status_iter)) {
		pStatus = status_iter->data;
		err = syncml_binder_append(pBinder, PE_STATUS, pStatus);
	}

	GList *iter = NULL;
	Command *pCommand = NULL;
	OMA_DM_Protocol_Element protocol_element = PE_UNDEF;
	for (iter = pSyncML->commands; iter != NULL; iter = g_list_next(iter)) {
		pCommand = (Command *) (iter->data);

/*		unsigned int estimate_size =0;
		syncml_binder_get_estimate_doc_size(pBinder, &estimate_size);
		_DEBUG_INFO(" ESTIMATE SIZE = %d\n", estimate_size);*/

		CommandType type = pCommand->type;
		switch (type) {
		case COMMAND_TYPE_UNKNOWN:
			protocol_element = PE_UNDEF;
			break;
		case COMMAND_TYPE_ALERT:
			protocol_element = PE_ALERT;
			break;
		case COMMAND_TYPE_ADD:
			protocol_element = PE_ADD;
			break;
		case COMMAND_TYPE_REPLACE:
			protocol_element = PE_REPLACE;
			break;
		case COMMAND_TYPE_DELETE:
			protocol_element = PE_DELETE;
			break;
		case COMMAND_TYPE_GET:
			protocol_element = PE_GET;
			break;
		case COMMAND_TYPE_RESULTS:
			protocol_element = PE_RESULTS_START;
			break;
		case COMMAND_TYPE_HEADER:
			protocol_element = PE_HEADER;
			break;
		case COMMAND_TYPE_EXEC:
			protocol_element = PE_EXEC;
			break;
		case COMMAND_TYPE_SEQUENCE:
			protocol_element = PE_SEQUENCE;
			break;
		case COMMAND_TYPE_ATOMIC:
			protocol_element = PE_ATOMIC;
			break;
		case COMMAND_TYPE_COPY:
			protocol_element = PE_COPY;
			break;
		default:
			break;
		}

		err = syncml_binder_append(pBinder, protocol_element, pCommand);
		if (err != SYNC_AGENT_PB_RETURN_OK) {
			ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
			goto error;
		}

		if (type == COMMAND_TYPE_RESULTS) {
			err = syncml_binder_append(pBinder, PE_RESULTS_END, NULL);
			if (err != SYNC_AGENT_PB_RETURN_OK) {
				ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
				goto error;
			}
		}
	}

	if (pSyncML->final == 1) {
		err = syncml_binder_append(pBinder, PE_FINAL, NULL);
		if (err != SYNC_AGENT_PB_RETURN_OK) {
			ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
			goto error;
		}
	}

	err = syncml_binder_append(pBinder, PE_BODY_END, NULL);
	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}
	err = syncml_binder_append(pBinder, PE_SYNCML_END, NULL);
	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}
	syncml_binder_get_stream(pBinder, msg, msg_size);

	char *xml = NULL;
	unsigned int xml_len = 0;
	err = sync_agent_get_xml_from_protocol_binder(pBinder, &xml, &xml_len);
	if (err == SYNC_AGENT_PB_RETURN_OK) {
		_DEBUG_INFO(" msg from client \n");
		if (fputs(xml, stderr) == EOF) {
			_DEBUG_INFO("debuging test error");
		}
		set_xml_to_file((xml), OMA_DM_MSG_PATH);
		str_free(&xml);
	}

	syncml_binder_terminate(pBinder);

	if (err != SYNC_AGENT_PB_RETURN_OK) {
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
		goto error;
	}

	_EXTERN_FUNC_EXIT;
	return ret;

 error:
	_DEBUG_INFO("error end  %d \n", ret);

	if(pBinder != NULL) {
		syncml_binder_terminate(pBinder);
	}

	_EXTERN_FUNC_EXIT;
	return ret;
}

DM_ERROR reverse_syncml_objectbinder(SyncML ** pSyncML, char *recv_msg, unsigned int recv_msg_length, char **xml, unsigned int *xml_len)
{
	_EXTERN_FUNC_ENTER;

	DM_ERROR ret = DM_OK;

	retvm_if((recv_msg) == NULL, COMMON_ERR_INTERNAL_NOT_DEFINED, "recv_msg is NULL!!");

	sync_agent_pb_protocol_binder_function_set_s *pBinder_function_set = NULL;

	sync_agent_pb_error_e err = init_syncml_binder_function_set(&pBinder_function_set);
	sync_agent_pb_protocol_binder_reverse_info_s *pBinder = NULL;
	err = reverse_syncml_binder_init(recv_msg, recv_msg_length, SYNC_AGENT_PB_DECODING_WBXML, pBinder_function_set, NULL, &pBinder);

	if (err != SYNC_AGENT_PB_RETURN_OK) {
		_DEBUG_INFO("reverse_oma_dm_1_2_binder_init error\n");
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;
	}

	 /*_DEBUG_INFO( "pBinder = %x\n", pBinder);*/

	/*char* xml = NULL;
	   unsigned int xml_size = 0; */
	err = sync_agent_get_xml_from_reverse_protocol_binder(pBinder, xml, xml_len);
	if (err == SYNC_AGENT_PB_RETURN_OK) {
		fprintf(stderr, " msg from server \n");
		if (fputs((*xml), stderr) == EOF) {
			_DEBUG_INFO("debugging test error");
		}
		set_xml_to_file((*xml), OMA_DM_MSG_PATH);
		//str_free(&xml);
	}

	OMA_DM_Protocol_Element protocol_element = PE_UNDEF;
	char *protocol_element_name = NULL;
	Content_Ptr pContent = NULL;
	int sequence_Atomic = 0;
	Command *tempCommand = NULL;

	while (err == SYNC_AGENT_PB_RETURN_OK) {
		err = reverse_syncml_binder_next(pBinder, &protocol_element, &protocol_element_name, &pContent);

		if (err == SYNC_AGENT_PB_RETURN_HAS_NO_MORE_ELEMENT) {
			break;
		}

		switch (protocol_element) {
		case PE_ADD:
			if (sequence_Atomic != 1)
				(*pSyncML)->commands = g_list_append((*pSyncML)->commands, pContent);
			else
				tempCommand->private.sequence_atomic.commands = g_list_append(tempCommand->private.sequence_atomic.commands, pContent);

			break;
		case PE_COPY:
		case PE_DELETE:
			if (sequence_Atomic != 1)
				(*pSyncML)->commands = g_list_append((*pSyncML)->commands, pContent);
			else
				tempCommand->private.sequence_atomic.commands = g_list_append(tempCommand->private.sequence_atomic.commands, pContent);

			break;
		case PE_EXEC:
		case PE_GET:
			if (sequence_Atomic != 1)
				(*pSyncML)->commands = g_list_append((*pSyncML)->commands, pContent);
			else
				tempCommand->private.sequence_atomic.commands = g_list_append(tempCommand->private.sequence_atomic.commands, pContent);

			break;
		case PE_SEQUENCE_START:
		case PE_ATOMIC_START:
			sequence_Atomic = 1;
			tempCommand = (Command *) pContent;
			break;
		case PE_SEQUENCE_END:
		case PE_ATOMIC_END:
			sequence_Atomic = 0;
			(*pSyncML)->commands = g_list_append((*pSyncML)->commands, tempCommand);
			tempCommand = NULL;
			break;
		case PE_REPLACE:
			if (sequence_Atomic != 1)
				(*pSyncML)->commands = g_list_append((*pSyncML)->commands, pContent);
			else
				tempCommand->private.sequence_atomic.commands = g_list_append(tempCommand->private.sequence_atomic.commands, pContent);

			break;
		case PE_ALERT:
			_DEBUG_INFO("reverse alert command");
			if (sequence_Atomic != 1)
				(*pSyncML)->commands = g_list_append((*pSyncML)->commands, pContent);
			else
				tempCommand->private.sequence_atomic.commands = g_list_append(tempCommand->private.sequence_atomic.commands, pContent);

			break;
		case PE_RESULTS_START:
			(*pSyncML)->commands = g_list_append((*pSyncML)->commands, pContent);

			break;
		case PE_STATUS:
			(*pSyncML)->status = g_list_append((*pSyncML)->status, pContent);
			protocol_element = PE_UNDEF;

			break;
		case PE_HEADER:
			(*pSyncML)->hdr = pContent;
			sync_agent_set_user_data_in_reverse_protocol_binder(pBinder, (*pSyncML)->hdr);

			break;
		case PE_GENERIC:
		case PE_FINAL:
			(*pSyncML)->final = (int)pContent;

			break;
		default:
			break;
		}
	}

	sync_agent_destroy_reverse_protocol_binder(pBinder);

	if (err != SYNC_AGENT_PB_RETURN_OK && err != SYNC_AGENT_PB_RETURN_HAS_NO_MORE_ELEMENT)
		ret = COMMON_ERR_INTERNAL_BINDER_ERROR;

	_EXTERN_FUNC_EXIT;
	return ret;
}
