/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <vconf.h>
#include <glib.h>

#include <wait.h>

/*sync-agent*/
#include <sync_agent.h>

#include "password_generator.h"

#ifndef EXPORT_API
#define EXPORT_API __attribute__ ((visibility("default")))
#endif

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"PLUGIN_DMACC_GCF"
#endif

#define  CSC_VCONF_KEY_SYNCMLDM_NBFIELD					"db/SyncML/oma-dm-service/DevManagement/NbDevManagement"
#define  CSC_VCONF_KEY_SYNCMLDM_ACCNAME				"db/SyncML/oma-dm-service/DevManagement/%d/AccName"
#define  CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE				"db/SyncML/oma-dm-service/DevManagement/%d/AuthType"
#define  CSC_VCONF_KEY_SYNCMLDM_CLIENTNONCE			"db/SyncML/oma-dm-service/DevManagement/%d/ClientNonce"
#define  CSC_VCONF_KEY_SYNCMLDM_SERVADDR				"db/SyncML/oma-dm-service/DevManagement/%d/ServAddr"
#define  CSC_VCONF_KEY_SYNCMLDM_SERVID					"db/SyncML/oma-dm-service/DevManagement/%d/ServID"
#define  CSC_VCONF_KEY_SYNCMLDM_SERVNONCE			"db/SyncML/oma-dm-service/DevManagement/%d/ServNonce"
#define  CSC_VCONF_KEY_SYNCMLDM_SERVPORT				"db/SyncML/oma-dm-service/DevManagement/%d/Port"
#define  CSC_VCONF_KEY_SYNCMLDM_SERVPWD				"db/SyncML/oma-dm-service/DevManagement/%d/ServPwd"
#define  CSC_VCONF_KEY_SYNCMLDM_PASSWORD				"db/SyncML/oma-dm-service/DevManagement/%d/Password"

#define  CSC_VCONF_KEY_SYNCMLDM_USERNAME				"db/SyncML/oma-dm-service/DevManagement/%d/UserName"
#define  CSC_VCONF_KEY_SYNCMLDM_CLIENTPWD				"db/SyncML/oma-dm-service/DevManagement/%d/ClientPwd"


//#define CSC_GCF_SERVER_NUM									3
#define CSC_GCF_SERVER_NUM									1

#define ALTER_SYNCMLDM_NBFIELD 3
#define ALTER_SYNCMLDM_ACCNAME "gcf"
#define ALTER_SYNCMLDM_AUTHTYPE "DIGEST"
#define ALTER_SYNCMLDM_CLIENTNONCE "12345"
#define ALTER_SYNCMLDM_SERVADDR "http://192.168.168.169/dm"
#define ALTER_SYNCMLDM_SERVID "RSate"
#define ALTER_SYNCMLDM_SERVNONCE "12345"
#define ALTER_SYNCMLDM_SERVPORT 80
#define ALTER_SYNCMLDM_SERVPWD "RSatepass"
#define ALTER_SYNCMLDM_PASSWORD ""

#define ALTER_SYNCMLDM_USERNAME "RSuser"
#define ALTER_SYNCMLDM_CLIENTPWD "RSpass"

#define SERVER_NUM 														1

static int _get_index(const char *name);

static char *_get_mo_value_str(const char *vconf_key, int nIndex);
static int _get_mo_value_int(const char *vconf_key, int nIndex);

static void _set_mo_value_str(const char *vconf_key, int nIndex, char *value);
static void _set_mo_value_int(const char *vconf_key, int nIndex, int value);

static sync_agent_get_devinfo_cb func_get_dev_info = 0;

EXPORT_API char **sync_agent_plugin_get_server_id_list(int *server_id_list_cnt)
{
	_EXTERN_FUNC_ENTER;

	*server_id_list_cnt = 0;
	char **temp_id_list = (char **)calloc(SERVER_NUM, sizeof(char *));
	if (temp_id_list == NULL) {
		return 0;
	}

	int server_idx = _get_index(ALTER_SYNCMLDM_ACCNAME);
	char *server_id = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, server_idx);
	if (server_id == NULL) {
		server_id = strdup(ALTER_SYNCMLDM_SERVID);
	}
	*temp_id_list = server_id;
	*server_id_list_cnt = 1;

	_DEBUG_INFO("server_id = %s\n", server_id);

	_DEBUG_INFO("server_id_list_cnt = %d\n", *server_id_list_cnt);

	_EXTERN_FUNC_EXIT;
	return temp_id_list;
}

EXPORT_API int sync_agent_plugin_get_mo_value(const char *mo_full_path, const char *mo_name, char **mo_value)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((mo_full_path) == NULL, -1, "mo_full_path is NULL!!");
	retvm_if((mo_name) == NULL, -1, "mo_name is NULL!!");

	_DEBUG_INFO("dmacc-gcf mo_full_path : %s", mo_full_path);
	_DEBUG_INFO("dmacc-gcf mo_name : %s", mo_name);
	*mo_value = NULL;

	int n_index = _get_index(ALTER_SYNCMLDM_ACCNAME);

	if (!strcmp(mo_name, "AppID")) {
		*mo_value = strdup("w7");

	} else if (!strcmp(mo_name, "ServerID")) {
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_SERVID);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index, ALTER_SYNCMLDM_SERVID);
		}

	} else if (!strcmp(mo_name, "Name")) {
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_ACCNAME, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_ACCNAME);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_ACCNAME, n_index, ALTER_SYNCMLDM_ACCNAME);
		}

	} else if (!strcmp(mo_name, "Addr")) {	/* ./DMAcc/gcf/AppAddr/MSCserver_AppAddr/Addr */
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVADDR, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_SERVADDR);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVADDR, n_index, ALTER_SYNCMLDM_SERVADDR);
		}

	} else if (!strcmp(mo_name, "AddrType")) {	/* ./DMAcc/gcf/AppAddr/MSCserver_AppAddr/AddrType */
		*mo_value = strdup("IPv4");

	} else if (!strcmp(mo_name, "PortNbr")) {	/* ./DMAcc/gcf/AppAddr/MSCserver_AppAddr/Port/MSCserver_Port/PortNbr */
		if (n_index != -1) {
			int port = _get_mo_value_int(CSC_VCONF_KEY_SYNCMLDM_SERVPORT, n_index);
			if (port == -1) {
				port = ALTER_SYNCMLDM_SERVPORT;
				_set_mo_value_int(CSC_VCONF_KEY_SYNCMLDM_SERVPORT, n_index, ALTER_SYNCMLDM_SERVPORT);
			}
			*mo_value = g_strdup_printf("%d", port);
		}

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ServerAppAuth/AAuthLevel")) {
		*mo_value = strdup("SRVCRED");

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ServerAppAuth/AAuthType")) {
		if (n_index != -1) {
			int auth_type = _get_mo_value_int(CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE, n_index);
			if (auth_type == -1) {
				*mo_value = strdup(ALTER_SYNCMLDM_AUTHTYPE);
				_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE, n_index, ALTER_SYNCMLDM_AUTHTYPE);
			} else if (auth_type == 0) {	/* basic */
				*mo_value = strdup("BASIC");
			} else if (auth_type == 1) {	/* md5 */
				*mo_value = strdup("DIGEST");
			} else if (auth_type == 2) {	/* hmac */
				*mo_value = strdup("HMAC");
			} else {
				*mo_value = strdup(ALTER_SYNCMLDM_AUTHTYPE);
				_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE, n_index, ALTER_SYNCMLDM_AUTHTYPE);
			}
		}

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ServerAppAuth/AAuthName")) {
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_SERVID);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index, ALTER_SYNCMLDM_SERVID);
		}

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ServerAppAuth/AAuthSecret")) {
		/* for gcf - start */
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVPWD, n_index);
		if((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_SERVPWD);
		}
		/* for gcf - end */
		/*
		char *device_id = NULL;
		if (func_get_dev_info != 0) {
			func_get_dev_info(2, "DevID", &device_id);
			if (device_id == NULL) {
				_DEBUG_INFO("device id is NULL");
				return -1;
			}
		} else {
			_DEBUG_INFO("pFunc_Get_DevInfo is NULL");
			return -1;
		}
		_DEBUG_INFO("Device Id : %s", device_id);

		char *server_id = 0;
		server_id = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index);
		if (server_id == NULL) {
			server_id = strdup(ALTER_SYNCMLDM_SERVID);
		}
		_DEBUG_INFO("Server Id : %s", server_id);
		*mo_value = generate_device_password((const char *)device_id, (const char *)server_id, SERVER_PW_KEY);

		if (*mo_value == NULL) {
			_DEBUG_INFO("generate device password fail");
			*mo_value = strdup(ALTER_SYNCMLDM_SERVPWD);
		}

		_DEBUG_INFO("server pw : %s", *mo_value);

		free(device_id);
		free(server_id);
		*/
	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ServerAppAuth/AAuthData")) {
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVNONCE, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_SERVNONCE);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVNONCE, n_index, ALTER_SYNCMLDM_SERVNONCE);
		}

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ClientAppAuth/AAuthLevel")) {
		*mo_value = strdup("CLCRED");

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ClientAppAuth/AAuthType")) {
		if (n_index != -1) {
			int auth_type = _get_mo_value_int(CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE, n_index);

			if (auth_type == -1) {
				*mo_value = strdup(ALTER_SYNCMLDM_AUTHTYPE);
				_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE, n_index, ALTER_SYNCMLDM_AUTHTYPE);
			} else if (auth_type == 0) {	/* basic */
				*mo_value = strdup("BASIC");
			} else if (auth_type == 1) {	/* md5 */
				*mo_value = strdup("DIGEST");
			} else if (auth_type == 2) {	/* hmac */
				*mo_value = strdup("HMAC");
			} else {
				*mo_value = strdup(ALTER_SYNCMLDM_AUTHTYPE);
				_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_AUTHTYPE, n_index, ALTER_SYNCMLDM_AUTHTYPE);
			}
		}

	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ClientAppAuth/AAuthName")) {
		/* for gcf - start */
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_USERNAME, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_USERNAME);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_USERNAME, n_index, ALTER_SYNCMLDM_USERNAME);
		}
		/* for gcf - end */
		/*
		char *device_id = NULL;
		if (func_get_dev_info != NULL) {
			func_get_dev_info(2, "DevID", &device_id);
			if (device_id == NULL) {
				_DEBUG_INFO("device id is NULL");
				return -1;
			}
		} else {
			_DEBUG_INFO("pFunc_Get_DevInfo is NULL");
			return -1;
		}
		char *server_id = NULL;
		server_id = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index);
		if (server_id == NULL) {
			server_id = strdup(ALTER_SYNCMLDM_SERVID);
		}
		*mo_value = generate_device_password((const char *)device_id, (const char *)server_id, CLIENT_ID_KEY);

		if (*mo_value == NULL) {
			_DEBUG_INFO("generate device password fail");
			*mo_value = strdup(ALTER_SYNCMLDM_SERVPWD);
		}

		free(device_id);
		free(server_id);
		*/
		/* *mo_value = strdup("IMEI:357584040054001"); */
	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ClientAppAuth/AAuthSecret")) {
		/* for gcf - start */
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_CLIENTPWD, n_index);
		if((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_CLIENTPWD);
		}
		/* for gcf - end */
		/*
		char *device_id = NULL;
		if (func_get_dev_info != NULL) {
			func_get_dev_info(2, "DevID", &device_id);
			if (device_id == NULL) {
				_DEBUG_INFO("device id is NULL");
				return -1;
			}
		} else {
			_DEBUG_INFO("pFunc_Get_DevInfo is NULL");
			return -1;
		}
		char *server_id = NULL;
		server_id = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_SERVID, n_index);
		if (server_id == NULL) {
			server_id = strdup(ALTER_SYNCMLDM_SERVID);
		}
		*mo_value = generate_device_password((const char *)device_id, (const char *)server_id, CLIENT_PW_KEY);

		free(device_id);
		free(server_id);
		*/
	} else if (!strcmp(mo_full_path, "./DMAcc/gcf/AppAuth/ClientAppAuth/AAuthData")) {
		*mo_value = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_CLIENTNONCE, n_index);
		if ((*mo_value) == NULL) {
			*mo_value = strdup(ALTER_SYNCMLDM_CLIENTNONCE);
			_set_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_CLIENTNONCE, n_index, ALTER_SYNCMLDM_CLIENTNONCE);
		}
	} else {
		_DEBUG_INFO("---------------------------------------------------not pased mo element : %s---------------------------------------------------", mo_full_path);
	}

	_DEBUG_INFO("mo_value : %s", *mo_value);

	_EXTERN_FUNC_EXIT;

	return 0;
}

EXPORT_API void sync_agent_set_function_get_devinfo_plugin(sync_agent_get_devinfo_cb func)
{
	_EXTERN_FUNC_ENTER;

	func_get_dev_info = func;

	_EXTERN_FUNC_ENTER;
}
/*
static int _system_command(const char *command)
{
	_EXTERN_FUNC_ENTER;

	int pid = 0, status = 0;
	char *const environ[] = { NULL };

	retvm_if((command) == NULL, -1, "command is NULL!!");

	pid = fork();
	if (pid == -1) {
		_DEBUG_INFO("ERROR : fork failed to create a process\n");
		_EXTERN_FUNC_EXIT;
		return -1;

	} else if (pid == 0) {
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = (char *)command;
		argv[3] = 0;

		execve("/bin/sh", argv, environ);
		abort();
	} else {
		do {
			if (waitpid(pid, &status, 0) == -1) {
				if (errno != EINTR) {
					return -1;
				}

			} else {
				_EXTERN_FUNC_EXIT;
				return status;
			}

		} while (1);
	}
	_EXTERN_FUNC_EXIT;
	return -1;
}
*/

static int _get_index(const char *name)
{
	_INNER_FUNC_ENTER;

	int n_acc_count = 0;
	if (vconf_get_int(CSC_VCONF_KEY_SYNCMLDM_NBFIELD, &n_acc_count)) {
		_DEBUG_TRACE("vconf_get_int FAIL");
		//            return -1;
		n_acc_count =ALTER_SYNCMLDM_NBFIELD;
		_set_mo_value_int(CSC_VCONF_KEY_SYNCMLDM_NBFIELD, CSC_GCF_SERVER_NUM, ALTER_SYNCMLDM_NBFIELD);

		//////// CSC don't include vconf value.
		//int ret = _system_command("/usr/share/oma-dm-cfg/csc-setting/change_csc_value.sh");
		//if (ret != -1) {
		//	_DEBUG_INFO("success");
		//} else {
		//	_DEBUG_INFO("fail");
		//}
		n_acc_count = ALTER_SYNCMLDM_NBFIELD;
	}

	_DEBUG_TRACE("nAccCount : %d", n_acc_count);

	int i = 0;
	char *compare_name = NULL;
	for (; i < n_acc_count; i++) {
		compare_name = _get_mo_value_str(CSC_VCONF_KEY_SYNCMLDM_ACCNAME, i + 1);
		if (compare_name == NULL) {
			continue;
		} else {
			_DEBUG_TRACE("[%d] compare_name : %s [%d], name : %s [%d]", i, compare_name, strlen(compare_name), name, strlen(name));
			if (!strcmp(name, compare_name)) {
				_DEBUG_TRACE("nIndex : %d", i + 1);
				return i + 1;
			}
			if (compare_name != NULL) {
				free(compare_name);
				compare_name = NULL;
			}
		}
	}

	_INNER_FUNC_EXIT;

	return CSC_GCF_SERVER_NUM;
}

static char *_get_mo_value_str(const char *vconf_key, int nIndex)
{
	_INNER_FUNC_ENTER;

	retvm_if((vconf_key) == NULL, NULL, "vconf_key is NULL!!");

	if (nIndex == -1) {
		return 0;
	}

	char input_vconf_path[200] = { 0, };
	char rec_idx[8];

	memset(rec_idx, 0x0, sizeof(rec_idx));
	snprintf(rec_idx, sizeof(rec_idx), "%d", nIndex);
	snprintf(input_vconf_path, strlen(vconf_key) + strlen(rec_idx) + 1, vconf_key, nIndex);

	char *value = vconf_get_str(input_vconf_path);
	if (value == NULL) {
		_DEBUG_TRACE("vconf_get_str FAIL");
	}

	_INNER_FUNC_ENTER;

	return value;
}

static int _get_mo_value_int(const char *vconf_key, int nIndex)
{
	_INNER_FUNC_ENTER;

	retvm_if((vconf_key) == NULL, -1, "vconf_key is NULL!!");

	char input_vconf_path[200] = { 0, };
	char rec_idx[8];

	memset(rec_idx, 0x0, sizeof(rec_idx));
	snprintf(rec_idx, sizeof(rec_idx), "%d", nIndex);
	snprintf(input_vconf_path, strlen(vconf_key) + strlen(rec_idx) + 1, vconf_key, nIndex);

	int value = -1;
	if (vconf_get_int(input_vconf_path, &value) == 0) {
		_DEBUG_TRACE("vconf_get_int : %d", value);
	} else {
		_DEBUG_TRACE("vconf_get_int error");
	}

	_INNER_FUNC_ENTER;

	return value;
}

static void _set_mo_value_str(const char *vconf_key, int nIndex, char *value)
{
	_INNER_FUNC_ENTER;

	retm_if((vconf_key) == NULL, "vconf_key is NULL!!");

	char input_vconf_path[200] = { 0, };
	char rec_idx[8];
	int result = 0;

	memset(rec_idx, 0x0, sizeof(rec_idx));
	snprintf(rec_idx, sizeof(rec_idx), "%d", nIndex);
	snprintf(input_vconf_path, strlen(vconf_key) + strlen(rec_idx) + 1, vconf_key, nIndex);

	result = vconf_set_str(input_vconf_path, value);
	_DEBUG_TRACE("vconf set reult : %d", result);

	_INNER_FUNC_ENTER;

}

static void _set_mo_value_int(const char *vconf_key, int nIndex, int value)
{
	_INNER_FUNC_ENTER;

	retm_if((vconf_key) == NULL, "vconf_key is NULL!!");

	char input_vconf_path[200] = { 0, };
	char rec_idx[8];

	memset(rec_idx, 0x0, sizeof(rec_idx));
	snprintf(rec_idx, sizeof(rec_idx), "%d", nIndex);
	snprintf(input_vconf_path, strlen(vconf_key) + strlen(rec_idx) + 1, vconf_key, nIndex);

	vconf_set_int(input_vconf_path, value);

	_INNER_FUNC_EXIT;

}

EXPORT_API int sync_agent_plugin_initialize()
{
	_EXTERN_FUNC_ENTER;

	int result = 1;

	_EXTERN_FUNC_EXIT;
	return result;
}
