/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>

/*sync-agent*/
#include <sync_agent.h>

#include "plugin_slp_device_dm.h"
#include "plugin_devexe_error.h"
#include "plugin_slp_device_ctrl.h"

#include "pmapi.h"

#ifndef EXPORT_API
#define EXPORT_API __attribute__ ((visibility("default")))
#endif

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"PLUGIN_DEVICE_SLP_DM"
#endif


static sync_agent_execute_dev_cb func_execute_dev_function_common = 0;

static int _execute_dev_fota_flag(int memory_type);
static int _execute_dev_fota_update_result(int *plugin_err);
static int _execute_dev_reboot();
static int _execute_dev_fully_lock();
static int _execute_dev_partially_lock();
static int _execute_dev_unlock();
static int _execute_dev_factory_reset();
static int _execute_dev_wipe();
static sync_agent_dev_return_e __convert_service_err_to_common_error(int service_err);
/*static int __find_file_list(const char *folder_path, GSList **file_list);*/
static int ___find_file(const char *folder_path, const char **file_path);
static int __control_fimware_delta_file(int memory_type);

static int ___get_download_save_folder(int memory_type, char **download_folder, char **save_folder);
int _execute_dev_function_common(char *function_name, int args_cnt, va_list ap);

EXPORT_API int sync_agent_plugin_execute_dev_function(char *function_name, int args_cnt, va_list ap)
{
	_EXTERN_FUNC_ENTER;

	retvm_if((function_name) == NULL, 0, "function_name is NULL!!");

	if (!strncmp(function_name, "pm_lock", strlen("pm_lock")) || !strncmp(function_name, "pm_unlock", strlen("pm_unlock")) || !strncmp(function_name, "pm_change", strlen("pm_change"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_function_common(function_name, args_cnt, ap);

	} else if (!strncmp(function_name, "fota_flag", strlen("fota_flag"))) {

		int memory_type = va_arg(ap, int);

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_fota_flag(memory_type);

	} else if (!strncmp(function_name, "fota_result", strlen("fota_result"))) {

		int *plugin_err = va_arg(ap, int *);

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_fota_update_result(plugin_err);

	} else if (!strncmp(function_name, "reboot", strlen("reboot"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_reboot();

	} else if (!strncmp(function_name, "full_lock", strlen("full_lock"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		return _execute_dev_fully_lock();

	} else if (!strncmp(function_name, "partial_lock", strlen("partial_lock"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_partially_lock();

	} else if (!strncmp(function_name, "unlock", strlen("unlock"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_unlock();

	} else if (!strncmp(function_name, "factory_reset", strlen("factory_reset"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_factory_reset();

	} else if (!strncmp(function_name, "wipe", strlen("wipe"))) {

		_DEBUG_INFO(" Received %s request !!!", function_name);
		_EXTERN_FUNC_EXIT;
		return _execute_dev_wipe();

	} else {
		_DEBUG_INFO("Device function requested not defined in plug-in !!!");
		goto error_part;
	}

 error_part:

	_EXTERN_FUNC_EXIT;
	return 0;
}

EXPORT_API void sync_agent_plugin_set_common_execute_dev_function(sync_agent_execute_dev_cb func)
{
	_EXTERN_FUNC_ENTER;

	func_execute_dev_function_common = func;

	_EXTERN_FUNC_EXIT;
}

static int _execute_dev_fota_flag(int memory_type)
{
	_INNER_FUNC_ENTER;

	/*slp_device_dm_exe_e err = EXE_SUCCESS; */

	int service_ret = 1;
	service_ret = __control_fimware_delta_file(memory_type);
	if (service_ret != 1) {
		_DEBUG_TRACE("__control_fimware_delta_file() Fail");
		goto error;
	}
	_DEBUG_TRACE("control firmware delta file : %d \n", service_ret);

	int try_count = 0;
	for( try_count = 0 ; try_count < 5 ; try_count++) {
		service_ret = slp_device_ctrl_set_fota_flag();
		if (service_ret != 0) {
			_DEBUG_TRACE("slp_device_ctrl_set_FOTA_flag() Fail");
			usleep(400000);
		} else {
			_DEBUG_TRACE("slp_device_ctrl_set_FOTA_flag() Success");
			_INNER_FUNC_EXIT;
			return __convert_service_err_to_common_error(1);
		}
	}

 error:
	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(0);
}

static int _execute_dev_fota_update_result(int *plugin_err)
{
	_INNER_FUNC_ENTER;

	*plugin_err = slp_device_ctrl_get_fota_update_result();

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(1);
}

static int _execute_dev_reboot()
{
	_INNER_FUNC_ENTER;

	int service_ret = slp_device_ctrl_reboot();

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(service_ret);
}

static int _execute_dev_fully_lock()
{
	_INNER_FUNC_ENTER;

	int service_ret = slp_device_ctrl_fully_lock();

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(service_ret);
}

static int _execute_dev_partially_lock()
{
	_INNER_FUNC_ENTER;

	int service_ret = slp_device_ctrl_partially_lock();

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(service_ret);
}

static int _execute_dev_unlock()
{
	_INNER_FUNC_ENTER;

	int service_ret = slp_device_ctrl_unlock();

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(service_ret);
}

static int _execute_dev_factory_reset()
{
	_INNER_FUNC_ENTER;

	int service_ret = slp_device_ctrl_factory_reset();

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(service_ret);
}

static int _execute_dev_wipe()
{
	_INNER_FUNC_ENTER;

	int service_ret = 1;

	_INNER_FUNC_EXIT;
	return __convert_service_err_to_common_error(service_ret);
}

/* static function*/
/*static int __find_file_list(const char *folder_path, GSList **file_list)
{
	_DEBUG_VERBOSE(" start");
	int ret = 1;

	if (folder_path == NULL) {
		ret = -1;
		goto error;
	}

	_DEBUG_VERBOSE(" folder_path = %s", folder_path);

	GSList *fileList = NULL;
	ret = sync_agent_get_file_list(folder_path, &fileList);
	if (ret != 1 || fileList == NULL) {
		ret = -1;
		goto error;
	}

	GSList *iter = NULL;
	char *temp_file_info = NULL;
	char *temp_file_name = NULL;
	char *temp_file_path = NULL;

	for (iter = fileList; iter != NULL; iter = g_slist_next(iter)) {
		temp_file_info = (char *)(iter->data);
		_DEBUG_VERBOSE(" file_name = %s", temp_file_info);

		temp_file_name = strtok(temp_file_info, ",");

		(*file_list) = g_slist_append((*file_list), temp_file_name);
	}

	_DEBUG_VERBOSE(" end");
	return ret;
error:
	_DEBUG_VERBOSE(" end error : %d", ret);
	return ret;
}*/

static int ___find_file(const char *folder_path, const char **file_path)
{
	_INNER_FUNC_ENTER;

	retvm_if((folder_path) == NULL, -1, "folder_path is NULL!!");

	int ret = 1;
	GSList *fileList = NULL;
	ret = sync_agent_get_file_list(folder_path, &fileList);
	if (ret != 1 || fileList == NULL || g_slist_length(fileList) > 1) {
		ret = -1;
		goto error;
	}

	char *temp_file_info = NULL;
	char *temp_file_name = NULL;

	temp_file_info = (char *)(fileList->data);
	_DEBUG_VERBOSE(" file_name = %s", temp_file_info);

	const char *delimiter = ",";
	/*temp_file_name = strtok(temp_file_info, ","); */
	temp_file_name = strtok(temp_file_info, delimiter);
	(*file_path) = g_strdup_printf("%s/%s", folder_path, temp_file_name);

	_INNER_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_VERBOSE(" end error : %d", ret);
	_INNER_FUNC_EXIT;
	return ret;
}


static int __control_fimware_delta_file(int memory_type)
{
	_INNER_FUNC_ENTER;

	int ret = 1;

	char *download_folder = NULL;
	char *save_folder = NULL;

	ret = ___get_download_save_folder(memory_type, &download_folder, &save_folder);
	if (ret != 1)
		goto error;

	const char *file_path = NULL;
	ret = ___find_file(download_folder, &file_path);
	if (ret != 1)
		goto error;

	/*file_path = g_strdup_printf("%s%s%s", download_folder, "/","0715_0720_FW_delta.tar"); */
	sync_agent_util_compress_result_e result = sync_agent_uncompress(SYNC_AGENT_UTIL_COMPRESS_TYPE_TAR, file_path, save_folder);
	if (result != SYNC_AGENT_UTIL_COMPRESS_SUCCESS) {
		_DEBUG_VERBOSE(" Failed to call sync_agent_uncompress()");
		ret = -1;
		goto error;
	} else {
		_DEBUG_VERBOSE(" Success to call sync_agent_uncompress()");
	}
	int fs_error;
	fs_error = sync_agent_empty_directory_contents(download_folder);

	_INNER_FUNC_EXIT;
	return ret;
 error:
	_DEBUG_VERBOSE(" end error : %d", ret);
	_INNER_FUNC_EXIT;
	return ret;
}

static sync_agent_dev_return_e __convert_service_err_to_common_error(int service_err)
{
	_INNER_FUNC_ENTER;

	slp_device_dm_exe_e ret = EXE_SUCCESS;

	switch (service_err) {
	case 1:
		ret = EXE_SUCCESS;
		break;
	default:
		ret = EXE_DEV_OPER_NOT_PERFORMED;
		break;
	}

	_INNER_FUNC_EXIT;
	return ret;
}

static int ___get_download_save_folder(int memory_type, char **download_folder, char **save_folder)
{
	_INNER_FUNC_ENTER;

	int ret = 1;
	switch (memory_type) {
	case 0:
		{
			sync_agent_dev_return_e dci_down_ret = sync_agent_get_devinfo(2, "FOTADownDir", download_folder);
			if (dci_down_ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
				_DEBUG_VERBOSE(" download  = %s\n", *download_folder);
			} else {
				ret = -1;
				goto error;
			}
			sync_agent_dev_return_e dci_save_ret = sync_agent_get_devinfo(2, "FOTASaveDir", save_folder);
			if (dci_save_ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
				_DEBUG_VERBOSE(" save  = %s: \n", *save_folder);
			} else {
				ret = -1;
				goto error;
			}
		}
		break;
	case 1:
		{
			sync_agent_dev_return_e dci_down_ret = sync_agent_get_devinfo(2, "FOTA_SD_DownDir", download_folder);
			if (dci_down_ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
				_DEBUG_VERBOSE(" download  = %s\n", *download_folder);
			} else {
				ret = -1;
				goto error;
			}
			sync_agent_dev_return_e dci_save_ret = sync_agent_get_devinfo(2, "FOTA_SD_SaveDir", save_folder);
			if (dci_save_ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
				_DEBUG_VERBOSE(" save  = %s: \n", *save_folder);
			} else {
				ret = -1;
				goto error;
			}
		}
		break;
	default:
		ret = -1;
		goto error;
		break;
	}
	_INNER_FUNC_EXIT;
	return ret;
 error:
	_INNER_FUNC_EXIT;
	return ret;
}

int _execute_dev_function_common(char *function_name, int args_cnt, va_list ap)
{
	if (function_name == NULL) {
		_DEBUG_TRACE("KEY name is NULL !!!");
		return 0;
	}

	sync_agent_dev_return_e ret = SYNC_AGENT_DEV_RETURN_SUCCESS;

	if (!strncmp(function_name, "pm_lock", strlen("pm_lock"))) {

		pm_state_e state = va_arg(ap, pm_state_e);
		pm_lock_state_flag_e flag = va_arg(ap, pm_lock_state_flag_e);
		unsigned int timeout = va_arg(ap, unsigned int);

		ret = func_execute_dev_function_common(1, function_name, args_cnt, state, flag, timeout);

	} else if (!strncmp(function_name, "pm_unlock", strlen("pm_unlock"))) {

		pm_state_e state = va_arg(ap, pm_state_e);
		pm_unlock_state_flag_e flag = va_arg(ap, pm_unlock_state_flag_e);

		ret = func_execute_dev_function_common(1, function_name, args_cnt, state, flag);

	} else if (!strncmp(function_name, "pm_change", strlen("pm_change"))) {

		pm_state_e state = va_arg(ap, pm_state_e);

		ret = func_execute_dev_function_common(1, function_name, args_cnt, state);

	}

	if (ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
		_DEBUG_TRACE("pFunc_Execute_Dev_Function_Common SUCCESS !!!");
		return 1;
	} else {
		_DEBUG_TRACE("pFunc_Execute_Dev_Function_Common FAIL !!!");
		return 0;
	}
}
