/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>

#include <vconf.h>
#include <vconf-keys.h>

/*sync-agent*/
#include <sync_agent.h>

#include "plugin_slp_device_info.h"

#ifndef EXPORT_API
#define EXPORT_API __attribute__ ((visibility("default")))
#endif

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"PLUGIN_DEVICE_SLP_DM"
#endif

#define IMEI_LEN					5
#define FW_VERSION_LEN		20
#define BUFF_SIZE					1000

/* todo : temporary definition */
#define HW_VERSION				"1.0"

typedef struct {
	char *info_name;
	char *vconf_key;
} dici_name_key_map;

dici_name_key_map dev_key_map[] = {
	{"SwV", VCONFKEY_TELEPHONY_SWVERSION},
	{"HwV", VCONFKEY_TELEPHONY_HWVERSION},
	/*{"Lang", "db/menu_widget/language"}, */
	{"Lang", VCONFKEY_LANGSET},
	{NULL, NULL},
};

static GList *devInfo;
static sync_agent_get_devinfo_cb func_get_dev_info_common = 0;

static char *_get_dev_info_common(char *key);

EXPORT_API int sync_agent_plugin_load_devinfo()
{
	_EXTERN_FUNC_ENTER;

	g_type_init();

	int res = 1;

	devInfo = NULL;

	/* get SwV */
	sync_agent_plugin_device_info_s *swv = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (swv == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(swv, 0x00, sizeof(sync_agent_plugin_device_info_s));
	swv->info_name = strdup("SwV");
	swv->value = _get_dev_info_common(swv->info_name);
	devInfo = g_list_append(devInfo, swv);

	/* get HwV */
	sync_agent_plugin_device_info_s *hwv = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (hwv == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(hwv, 0x00, sizeof(sync_agent_plugin_device_info_s));
	hwv->info_name = strdup("HwV");
	hwv->value = _get_dev_info_common(hwv->info_name);
	devInfo = g_list_append(devInfo, hwv);

	/* get Lang */
	sync_agent_plugin_device_info_s *lang = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (lang == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(lang, 0x00, sizeof(sync_agent_plugin_device_info_s));
	lang->info_name = strdup("Lang");
	lang->value = _get_dev_info_common(lang->info_name);
	devInfo = g_list_append(devInfo, lang);

	/* get DevID */
	sync_agent_plugin_device_info_s *devId = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (devId == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(devId, 0x00, sizeof(sync_agent_plugin_device_info_s));
	devId->info_name = strdup("DevID");
	devId->value = _get_dev_info_common(devId->info_name);
	devInfo = g_list_append(devInfo, devId);
	_DEBUG_INFO("mdi ============= \n");

	/* get Manufacture */
	sync_agent_plugin_device_info_s *man = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (man == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(man, 0x00, sizeof(sync_agent_plugin_device_info_s));
	man->info_name = strdup("Man");
	man->value = _get_dev_info_common(man->info_name);
	devInfo = g_list_append(devInfo, man);

	/* get Model info */
	sync_agent_plugin_device_info_s *mod = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (mod == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(mod, 0x00, sizeof(sync_agent_plugin_device_info_s));
	mod->info_name = strdup("Mod");
	mod->value = _get_dev_info_common(mod->info_name);
	devInfo = g_list_append(devInfo, mod);

	/* get FwV */
	sync_agent_plugin_device_info_s *fw = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (fw == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(fw, 0x00, sizeof(sync_agent_plugin_device_info_s));
	fw->info_name = strdup("FwV");
	fw->value = _get_dev_info_common(fw->info_name);
	devInfo = g_list_append(devInfo, fw);

	/* get OEM */
	sync_agent_plugin_device_info_s *oem = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (oem == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(oem, 0x00, sizeof(sync_agent_plugin_device_info_s));
	oem->info_name = strdup("OEM");
	oem->value = _get_dev_info_common(oem->info_name);
	devInfo = g_list_append(devInfo, oem);

	/* get DevTyp */
	sync_agent_plugin_device_info_s *dev_type = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (dev_type == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(dev_type, 0x00, sizeof(sync_agent_plugin_device_info_s));
	dev_type->info_name = strdup("DevTyp");
	dev_type->value = _get_dev_info_common(dev_type->info_name);
	devInfo = g_list_append(devInfo, dev_type);

	/* get Modem version */
	sync_agent_plugin_device_info_s *modem_version = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (modem_version == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(modem_version, 0x00, sizeof(sync_agent_plugin_device_info_s));
	modem_version->info_name = strdup("ModemV");
	modem_version->value = _get_dev_info_common(modem_version->info_name);
	devInfo = g_list_append(devInfo, modem_version);

	/* get PDA version */
	sync_agent_plugin_device_info_s *pda_version = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (pda_version == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(pda_version, 0x00, sizeof(sync_agent_plugin_device_info_s));
	pda_version->info_name = strdup("PDAV");
	pda_version->value = _get_dev_info_common(pda_version->info_name);
	devInfo = g_list_append(devInfo, pda_version);

	/* get CSC version */
	sync_agent_plugin_device_info_s *csc_version = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (csc_version == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(csc_version, 0x00, sizeof(sync_agent_plugin_device_info_s));
	csc_version->info_name = strdup("CSCV");
	csc_version->value = _get_dev_info_common(csc_version->info_name);
	devInfo = g_list_append(devInfo, csc_version);

	/* get kernel version */
	sync_agent_plugin_device_info_s *kernel_version = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (kernel_version == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(kernel_version, 0x00, sizeof(sync_agent_plugin_device_info_s));
	kernel_version->info_name = strdup("KernelV");
	kernel_version->value = slp_device_info_get_kernel_version();
	devInfo = g_list_append(devInfo, kernel_version);

	/* get FOTA_download dir */
	sync_agent_plugin_device_info_s *fota_download_dir = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (fota_download_dir == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(fota_download_dir, 0x00, sizeof(sync_agent_plugin_device_info_s));
	fota_download_dir->info_name = strdup("FOTADownDir");
	fota_download_dir->value = slp_device_info_get_fota_download_dir();
	devInfo = g_list_append(devInfo, fota_download_dir);

	/* get FOTA save dir */
	sync_agent_plugin_device_info_s *fota_save_dir = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (fota_save_dir == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(fota_save_dir, 0x00, sizeof(sync_agent_plugin_device_info_s));
	fota_save_dir->info_name = strdup("FOTASaveDir");
	fota_save_dir->value = slp_device_info_get_fota_save_dir();
	devInfo = g_list_append(devInfo, fota_save_dir);

	/* get SD_FOTA_download dir */
	sync_agent_plugin_device_info_s *fota_sd_download_dir = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (fota_sd_download_dir == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(fota_sd_download_dir, 0x00, sizeof(sync_agent_plugin_device_info_s));
	fota_sd_download_dir->info_name = strdup("FOTA_SD_DownDir");
	fota_sd_download_dir->value = slp_device_info_get_fota_sd_download_dir();
	devInfo = g_list_append(devInfo, fota_sd_download_dir);

	/* get SD_FOTA save dir */
	sync_agent_plugin_device_info_s *fota_sd_save_dir = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (fota_sd_save_dir == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(fota_sd_save_dir, 0x00, sizeof(sync_agent_plugin_device_info_s));
	fota_sd_save_dir->info_name = strdup("FOTA_SD_SaveDir");
	fota_sd_save_dir->value = slp_device_info_get_fota_sd_save_dir();
	devInfo = g_list_append(devInfo, fota_sd_save_dir);

	/* get DTD version */
	sync_agent_plugin_device_info_s *dtd_version = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (dtd_version == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(dtd_version, 0x00, sizeof(sync_agent_plugin_device_info_s));
	dtd_version->info_name = strdup("DTDV");
	dtd_version->value = slp_device_info_get_dtd_version();
	devInfo = g_list_append(devInfo, dtd_version);

	/* get OPCO info */
	sync_agent_plugin_device_info_s *opco_info = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (opco_info == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(opco_info, 0x00, sizeof(sync_agent_plugin_device_info_s));
	opco_info->info_name = strdup("OPCO");
	opco_info->value = slp_device_info_get_opco_info();
	devInfo = g_list_append(devInfo, opco_info);

	/* get PHONE_NUM info */
	sync_agent_plugin_device_info_s *phone_num = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (phone_num == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(phone_num, 0x00, sizeof(sync_agent_plugin_device_info_s));
	phone_num->info_name = strdup("PHONE_NUM");
	phone_num->value = slp_device_info_get_msisdn_number();
	devInfo = g_list_append(devInfo, phone_num);

	/* get IMSI */
	sync_agent_plugin_device_info_s *imsi_value = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (imsi_value == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(imsi_value, 0x00, sizeof(sync_agent_plugin_device_info_s));
	imsi_value->info_name = strdup("IMSI");
	imsi_value->value = slp_device_info_get_imsi_info();
	devInfo = g_list_append(devInfo, imsi_value);

	/* get CSC */
	sync_agent_plugin_device_info_s *csc_value = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (csc_value == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(csc_value, 0x00, sizeof(sync_agent_plugin_device_info_s));
	csc_value->info_name = strdup("CSC");
	csc_value->value = slp_device_info_get_csc();
	devInfo = g_list_append(devInfo, csc_value);

	/* get MCC */
	sync_agent_plugin_device_info_s *mcc_value = (sync_agent_plugin_device_info_s *) malloc(sizeof(sync_agent_plugin_device_info_s));
	if (mcc_value == NULL) {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	memset(mcc_value, 0x00, sizeof(sync_agent_plugin_device_info_s));
	mcc_value->info_name = strdup("MCC");
	mcc_value->value = slp_device_info_get_mcc();
	devInfo = g_list_append(devInfo, mcc_value);

	/* for test */
	int cnt = 0;
	GList *iter = NULL;
	for (iter = devInfo; iter != NULL; iter = g_list_next(iter)) {
		_DEBUG_INFO("[devInfo_slp_plugIn] cnt : %d,  info_name : %s, value : %s\n", cnt, ((sync_agent_plugin_device_info_s *) (iter->data))->info_name, ((sync_agent_plugin_device_info_s *) (iter->data))->value);
		cnt++;
	}

	_EXTERN_FUNC_EXIT;
	return res;
}

EXPORT_API int sync_agent_plugin_get_devinfo(char *info_name, char **value)
{
	_EXTERN_FUNC_ENTER;;

	retvm_if((info_name) == NULL, 0, "info_name is NULL!!");

	int res = 1;
	int is_exist = 0;
	GList *iter = NULL;
	sync_agent_plugin_device_info_s *iter_data;

	for (iter = devInfo; iter != NULL; iter = g_list_next(iter)) {
		iter_data = NULL;
		iter_data = (sync_agent_plugin_device_info_s *) (iter->data);
		if (iter_data->info_name != NULL) {
			if (!strcmp(iter_data->info_name, info_name)) {
				if (iter_data->value != NULL) {

					*value = strdup(iter_data->value);

					is_exist = 1;
					break;
				} else {
					_DEBUG_INFO("[devInfo_slp_plugIn] %s  value is NULL !!\n", info_name);
					res = 0;
					_EXTERN_FUNC_EXIT;
					return res;
				}
			}
		}
	}

	if (strcmp("Battery", info_name) == 0) {
		*value = _get_dev_info_common(info_name);
		if (*value != NULL)
			is_exist = 1;
	}

	if (is_exist != 1) {
		_DEBUG_INFO("[devInfo_slp_plugIn] not exist  %s  information !!\n", info_name);
		res = -1;
		_EXTERN_FUNC_EXIT;
		return res;
	}

	_EXTERN_FUNC_EXIT;
	return res;
}

EXPORT_API int sync_agent_plugin_clear_devinfo()
{
	_EXTERN_FUNC_ENTER;;

	int res = 1;

	GList *iter = NULL;
	sync_agent_plugin_device_info_s *iter_data;

	if (devInfo != NULL) {
		for (iter = devInfo; iter != NULL;) {
			iter_data = NULL;
			iter_data = ((sync_agent_plugin_device_info_s *) (iter->data));
			iter = g_list_next(iter);
			devInfo = g_list_remove(devInfo, iter_data);

			if (iter_data != NULL) {
				if (((sync_agent_plugin_device_info_s *) (iter_data))->info_name != NULL)
					free(((sync_agent_plugin_device_info_s *) (iter_data))->info_name);
				if (((sync_agent_plugin_device_info_s *) (iter_data))->value != NULL)
					free(((sync_agent_plugin_device_info_s *) (iter_data))->value);
				free((sync_agent_plugin_device_info_s *) (iter_data));
			}
		}

		g_list_free(devInfo);
	}

	_EXTERN_FUNC_EXIT;
	return res;
}

EXPORT_API void sync_agent_plugin_set_common_devinfo_function(sync_agent_get_devinfo_cb func)
{
	_EXTERN_FUNC_ENTER;

	func_get_dev_info_common = func;

	_EXTERN_FUNC_EXIT;
}

static char *_get_dev_info_common(char *key)
{
	_INNER_FUNC_ENTER;

	if (key == NULL) {
		_DEBUG_TRACE("KEY name is NULL !!!");
		_INNER_FUNC_EXIT;
		return NULL;
	}

	char *value = 0;
	sync_agent_dev_return_e ret = SYNC_AGENT_DEV_RETURN_SUCCESS;

	ret = func_get_dev_info_common(1, key, &value);
	if (ret == SYNC_AGENT_DEV_RETURN_SUCCESS) {
		_DEBUG_TRACE(" key = %s, value = %s", key, value);
		_INNER_FUNC_EXIT;
		return value;
	} else {
		_DEBUG_TRACE(" key = %s has NULL value !!!", key, value);
		_INNER_FUNC_EXIT;
		return NULL;
	}
}
