/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <wait.h>
#include <errno.h>
#include <vconf.h>
#include <vconf-keys.h>
/*#include "package-manager.h"*/

/*sync-agent*/
#include <sync_agent.h>

#include "plugin_slp_device_ctrl.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"PLUGIN_DEVICE_SLP_DM"
#endif

#define UA_RESULT_KEY "db/SyncML/oma-dm-service/UAresult"

//#define RB_ERR_CODE_MAX_COUNT 16
#define RB_ERR_CODE_MAX_COUNT 7

typedef struct _type_Redbend_OMADM_Table {
	int result_code;
	char *str_meanning;
	char *result_redbend;
	char *str_redbend;
} type_redbend_omadm_table_s;

type_redbend_omadm_table_s redbend_omadm_table[RB_ERR_CODE_MAX_COUNT] = {
/*
	{200, "Successful", "0", "S_RB_SUCCESS"},
	{410, "Firmware Update Failed", "1", "E_RB_*"},	// juhaki.park
	{402, "Corrupted Firmware Update Package", "8000000D", "E_RB_PKG_CORRUPTED"},
	{403, "Firmware Update Package - Device Mismatch", "80000002", "E_RB_BAD_PARAMS"},
	{403, "Firmware Update Package - Device Mismatch", "80000011", "E_RB_WRONG_UPI_VER"},
	{403, "Firmware Update Package - Device Mismatch", "80000012", "E_RB_WRONG_UPI_UPDATE"},
	{403, "Firmware Update Package - Device Mismatch", "80000013", "E_RB_UPDATE_SECTOR_SIG"},
	{404, "Failed Firmware Update Package Validation", "8001001A", "E_RB_NON_DP_FORMAT_NOT_SUPPORTED"},
	{404, "Failed Firmware Update Package Validation", "80010025", "E_RB_INVALID_DP_HEADER"},
	{404, "Failed Firmware Update Package Validation", "80010026", "E_RB_INVALID_DP_WRONG_SIGNATURE"},
	{404, "Failed Firmware Update Package Validation", "80010027", "E_RB_INVALID_DP"},
	{405, "Firmware Update Package Not Acceptable", "8000000B", "E_RB_PKG_TOO_SHORT"},
	{405, "Firmware Update Package Not Acceptable", "8000000C", "E_RB_PKG_TOO_LONG"},
	{405, "Firmware Update Package Not Acceptable", "8000000E", "E_RB_PKG_NOT_AUTHORIZED"},
	{410, "Firmware Update Failed", "**", "E_RB_*"},
	{502, "Firmware update fails due to device out of memory", "8000001E", "E_RB_NOT_ENOUGH_RAM"},
*/
	{200, "Successful", "0", "S_RB_SUCCESS"},
	{410, "Firmware Update Failed", "FDBC", "E_RB_Bootloader_update_error"},	// juhaki.park
	{410, "Firmware Update Failed", "fdbc", "E_RB_Bootloader_update_error"},	// juhaki.park
	{410, "Firmware Update Failed", "FDBD", "E_RB_Boot_partition_update_error"},	// juhaki.park
	{410, "Firmware Update Failed", "fdbd", "E_RB_Boot_partition_update_error"},	// juhaki.park
	{410, "Firmware Update Failed", "FDBE", "E_RB_Platform_update_error"},	// juhaki.park
	{410, "Firmware Update Failed", "fdbe", "E_RB_Platform_update_error"},	// juhaki.park

/*{0, NULL, NULL, NULL},*/
};

static int _system_command(const char *command);
/*static int __get_mobile_tracker_status();*/
/*static int __return_pkgmgr_cb(pkg_request_id req_id, const char *pkg_type, const char *pkg_name, const char *key, const char *val, const void *pmsg, void *data);*/

int slp_device_ctrl_set_fota_flag()
{ // return value : success[0] faile[1]
	_EXTERN_FUNC_ENTER;

	/*
	   2010.09.08 document base
	   ("echo fota_enable > /sys/devices/platform/s3c_samsung_fota/samsung_fota_config");
	   _DEBUG_INFO("echo fota_enable > /sys/devices/platform/s3c_samsung_fota/samsung_fota_config \n");
	   2011.03.23 document base
	   ("/bin/change-booting-mode.sh --fota /opt/data/fota");
	 */

	int ret = _system_command("/bin/sync");
	_DEBUG_INFO("disk sync - ret : [%d]", ret);

	_DEBUG_INFO("/bin/change-booting-mode.sh --fota /opt/usr/data/fota/save \n");
	ret = system("/bin/change-booting-mode.sh --fota /opt/usr/data/fota/save");
	_DEBUG_INFO("change-booting-mode.sh ret : [%d]", ret);

	_EXTERN_FUNC_EXIT;
	return 0;

/*
	int ret = _system_command("/bin/change-booting-mode.sh --fota /opt/data/fota/save");
	_DEBUG_INFO("change-booting-mode.sh ret : [%d]", ret);

	if (ret != -1) {
		_DEBUG_ERROR("error : %d", ret);

		if(ret == 256) {
			ret = _system_command("reboot fota");
			if(ret != -1) {
				_EXTERN_FUNC_EXIT;
				return 1;
			} else {
				_EXTERN_FUNC_EXIT;
				return 0;
			}
		}

		_EXTERN_FUNC_EXIT;
		return 1;
	} else {
		_EXTERN_FUNC_EXIT;
		return 0;
	}
error:
	return 0;
*/
}

void slp_device_ctrl_get_fota_fail_ua_result(char * str_UA_file_position)
{
	_EXTERN_FUNC_ENTER;
	FILE *file_des;
	long lSize = 0;
	size_t result;
	char file_path[128] = {0,};
	snprintf(file_path,128, "%s/cause", /*FOTA_SAVE_DIR*/ "/opt/usr/data/fota/save");
	file_des = fopen(file_path, "rb");
	if (file_des == NULL) {
		_DEBUG_INFO("fopen fail - cause file \n");
		_EXTERN_FUNC_EXIT;
		return;
	}
	fseek(file_des, 0, SEEK_END);
	lSize = ftell(file_des);
	if (lSize <= 0) {
		_DEBUG_INFO("ftell nagative return ");
		if (file_des != NULL)
			fclose(file_des);
		_EXTERN_FUNC_EXIT;
		return;
	}
	rewind(file_des);
	char *str_result = NULL;
	//str_result = (char *)calloc(1, lSize + 1);
	str_result = (char *)malloc(sizeof(char)*(lSize+1));
	if (str_result == NULL) {
		if (file_des != NULL)
			fclose(file_des);
		_DEBUG_INFO("malloc fail \n");
		_EXTERN_FUNC_EXIT;
		return;
	}
	memset(str_result, 0x0, sizeof(char)*(lSize+1));	// PREVENT

	result = fread(str_result, 1, lSize, file_des);
	if ((result != lSize) || (str_result == NULL)) {
		if (file_des != NULL)
			fclose(file_des);
		_DEBUG_INFO("read fail or str_result == NULL \n");

		if (str_result != NULL)
			free(str_result);

		_EXTERN_FUNC_EXIT;
		return;
	}
	if (str_result[lSize - 1] == '\n')
		str_result[lSize - 1] = 0;

	if(lSize > 0) {
		int vconf_result = 0;
		char str_ua_result[128] = {0,};
		if(str_UA_file_position == NULL) {
			snprintf(str_ua_result,128,"NoData-%s", str_result);
		} else {
			snprintf(str_ua_result,128,"%s-%s", str_UA_file_position, str_result);
		}
		vconf_result = vconf_set_str(UA_RESULT_KEY, str_ua_result);
		if(vconf_result == 0) {
			_DEBUG_VERBOSE("vconf_set_str() success %s [%s]!!", UA_RESULT_KEY, str_ua_result);
		} else {
			_DEBUG_VERBOSE("vconf_set_str() %s failed !!", UA_RESULT_KEY);
		}
	}

	_DEBUG_INFO("str_result [%s] \n", str_result);

	if (str_result != NULL)
		free(str_result);

	if (file_des != NULL)
		fclose(file_des);

	_EXTERN_FUNC_EXIT;
}

int slp_device_ctrl_get_fota_update_result()
{
	_EXTERN_FUNC_ENTER;

	int update_result = 400;
	FILE *file_des;
	long lSize = 0;		/*prevent */
	size_t result;
	//file_des = fopen("/opt/result", "rb");
	char file_path[128] = {0,};
	snprintf(file_path,128, "%s/result", /*FOTA_SAVE_DIR*/ "/opt/usr/data/fota/save");
	file_des = fopen(file_path, "rb");

	int j = 0;
	bool match_found = false;
	if (file_des == NULL) {
		_DEBUG_INFO("fopen fail - result file \n");
		_EXTERN_FUNC_EXIT;
		return 400;
	}
	fseek(file_des, 0, SEEK_END);
	lSize = ftell(file_des);
	if (lSize <= 0) {
		_DEBUG_INFO("ftell nagative return ");
		if (file_des != NULL)
			fclose(file_des);
		_EXTERN_FUNC_EXIT;
		return 400;
	}
	rewind(file_des);
	char *str_result = NULL;
	//str_result = (char *)calloc(1, lSize + 1);
	str_result = (char *)malloc(sizeof(char)*(lSize+1));
	if (str_result == NULL) {
		if (file_des != NULL)
			fclose(file_des);
		_DEBUG_INFO("malloc fail \n");
		_EXTERN_FUNC_EXIT;
		return 400;
	}

	memset(str_result, 0x0, sizeof(char)*(lSize+1));	// PREVENT

	result = fread(str_result, 1, lSize, file_des);
	if ((result != lSize) || (str_result == NULL)) {
		if (file_des != NULL)
			fclose(file_des);
		_DEBUG_INFO("read fail or str_result == NULL \n");

		if (str_result != NULL)
			free(str_result);

		_EXTERN_FUNC_EXIT;
		return 400;
	}
	if (str_result[lSize - 1] == '\n')
		str_result[lSize - 1] = 0;

	for (j = 0; j < RB_ERR_CODE_MAX_COUNT; j++) {
		if (strlen(str_result) == strlen(redbend_omadm_table[j].result_redbend)) {
			if ((strcmp(str_result, redbend_omadm_table[j].result_redbend) == 0)) {
				update_result = redbend_omadm_table[j].result_code;
				match_found = true;
				_DEBUG_INFO("Result info [%s] \n", redbend_omadm_table[j].str_redbend);

				if(update_result != 200) {
					slp_device_ctrl_get_fota_fail_ua_result(redbend_omadm_table[j].str_redbend);
				} else {
					int vconf_result = 0;
					vconf_result = vconf_set_str(UA_RESULT_KEY, redbend_omadm_table[j].str_redbend);
					if(vconf_result == 0) {
						_DEBUG_VERBOSE("vconf_set_str() success %s [%s]!!", UA_RESULT_KEY, redbend_omadm_table[j].str_redbend);
					} else {
						_DEBUG_VERBOSE("vconf_set_str() %s failed !!", UA_RESULT_KEY);
					}
				}

				break;	/* If existing error code, break-out from the for-loop. */
			}
		}
		if (!match_found)
			update_result = 410;
	}

	if (str_result != NULL)
		free(str_result);

	if (file_des != NULL)
		fclose(file_des);

	_DEBUG_INFO("update_result - %d \n", update_result);
	_EXTERN_FUNC_EXIT;
	return update_result;
}

int slp_device_ctrl_reboot()
{
	_EXTERN_FUNC_ENTER;

	int ret = _system_command("/sbin/reboot");
	if (ret != -1) {
		_EXTERN_FUNC_EXIT;
		return 1;
	}

	/*will be insert.. // tel_process_power_command(TAPI_PHONE_POWER_RESET); ?? */
	_EXTERN_FUNC_EXIT;
	return 0;
}

int slp_device_ctrl_fully_lock()
{
	_EXTERN_FUNC_ENTER;

	/*if (__get_mobile_tracker_status() == 0 ) {
	   _DEBUG_INFO("mobile tracker status OFF \n");
	   return 0;
	   }

	   ("/opt/apps/deb.com.samsung.pwlock/bin/pwlock 1 &");
	   ("/bin/sync"); */

	_EXTERN_FUNC_EXIT;
	return 1;
}

int slp_device_ctrl_partially_lock()
{
	_EXTERN_FUNC_ENTER;

	/*if (__get_mobile_tracker_status() == 0 ) {
	   _DEBUG_INFO("mobile tracker status OFF \n");
	   return 0;
	   }

	   ("/opt/apps/deb.com.samsung.pwlock/bin/pwlock 1 &");
	   ("/bin/sync"); */

	_EXTERN_FUNC_EXIT;
	return 1;
}

int slp_device_ctrl_unlock()
{
	_EXTERN_FUNC_ENTER;

	/*if (__get_mobile_tracker_status() == 0 ) {
	   _DEBUG_INFO("mobile tracker status OFF \n");
	   return 0;
	   }

	   ("killall -9 pwlock");
	   ("/bin/sync"); */
	_EXTERN_FUNC_EXIT;
	return 1;
}

int slp_device_ctrl_factory_reset()
{
	_EXTERN_FUNC_ENTER;

	/*if (__get_mobile_tracker_status() == 0 ) {
	   _DEBUG_INFO("mobile tracker status OFF \n");
	   return 0;
	   }
	   ("/usr/bin/factory-reset.sh"); // */

	_EXTERN_FUNC_EXIT;
	return 1;
}

/* static function */
static int _system_command(const char *command)
{
	_INNER_FUNC_ENTER;

	int pid = 0, status = 0;
	char *const environ[] = { NULL };

	retvm_if((command) == NULL, -1, "command is NULL!!");

	pid = fork();
	if (pid == -1) {
		_DEBUG_TRACE("ERROR : fork failed to create a process\n");
		_INNER_FUNC_EXIT;
		return -1;

	} else if (pid == 0) {
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = (char *)command;
		argv[3] = 0;

		execve("/bin/sh", argv, environ);
		abort();
	} else {
		do {
			if (waitpid(pid, &status, 0) == -1) {
				if (errno != EINTR) {
					_INNER_FUNC_EXIT;
					return -1;
				}

			} else {
				_DEBUG_ERROR("error : status [%d]", status);
				_INNER_FUNC_EXIT;
				return status;
			}

		} while (1);
	}
	_INNER_FUNC_EXIT;
	return -1;
}

/* TODO : holding ( wait until the MDM module is completed ) */
/*int Execute_Dev_App_Install(char *file_name, char *application_id)
{
    if( file_name == NULL )
		return 0;

    _DEBUG_INFO("file_name[%s] \n", file_name);
    int result = 0 ;

    int req_id = 0;
    char *descriptor = NULL;
    char abs_setup_file[256] = {0,};
    ecore_init();
      snprintf(abs_setup_file,sizeof(abs_setup_file),"%s", file_name );
      _DEBUG_INFO("[%s] , [%s] \n", abs_setup_file, file_name );
    result = package_manager_install_application(&req_id, descriptor, abs_setup_file, PKG_QUIET PKG_DEFAULT, 0, NULL);
    LOGTRACE("package_manager_install_application result[%d]\n", result);
    if( result < PKG_COMMON_OK) {
		LOGTRACE("Install failed! result[%d] \n", result );
		snprintf(app_ID, PKG_TYPE_STRING_LEN_MAX+PKG_NAME_STRING_LEN_MAX+PKG_VERSION_STRING_LEN_MAX +2 , "%s" , pFile_name );
      //pthread_mutex_unlock(&wait_mutex);
		//return 0;
    }
    else{
		ecore_main_loop_begin();
		snprintf(app_ID, PKG_TYPE_STRING_LEN_MAX+PKG_NAME_STRING_LEN_MAX+PKG_VERSION_STRING_LEN_MAX +2, "%s" , g_pkg_info.pkg_name );
    }

    ("/bin/sync");

    if( result == PKG_COMMON_OK ) {
      _DEBUG_INFO("package_manager_install_application() Success !!\n");
      result = 0;

    } else {
      _DEBUG_INFO("package_manager_install_application() Success !!\n");
      result = 1;
    }

    return result ;  // 1(OK),0(FAIL)
}*/

/*int Execute_Dev_App_UnInstall(char *application_id)
{
    int result = 0 ;

    if( app_ID == NULL )
		return 0 ; // 1(OK),0(FAIL)
    LOGTRACE("app_ID[%s] \n", app_ID);

    if( strlen(app_ID) >= 256 ) // Prevent defect
    {
		LOGTRACE("strlen[app_ID] >= 256 \n");
		return 0 ; // 1(OK),0(FAIL)
    }

#if defined(_D_SLP_PKGMGR_)
    int req_id = 0;
    char *descriptor = NULL;
    char pkg_type[16] = {0,};
    memcpy(pkg_type,app_ID,3);

    ecore_init();
    result = package_manager_uninstall_application(&req_id, pkg_type, app_ID , PKG_QUIET PKG_DEFAULT, return_pkgmgr_cb, NULL);
    if( result < PKG_COMMON_OK)
    {
		LOGTRACE("Uninstall failed! result[%d] \n", result);
		//return 1;
    }
    else {
		ecore_main_loop_begin();
    }
#endif

    ("/bin/sync");
    if( result == PKG_COMMON_OK ) {
		char return_char[512] = {0,};
		snprintf(return_char,510,"%s,success,%s", (char*)_D_APP_UNINSTALL_,app_ID ); // Prevent defect
		QUEUE_sendto ( _D_SYNCML_DM_SERVICE_SOCKET_PATH , (void*) return_char , strlen(return_char) );
    }
    else{
     char return_char[512] = {0,};
     snprintf(return_char,510,"%s,fail,%s", (char*)_D_APP_UNINSTALL_,app_ID ); // Prevent defect
     QUEUE_sendto ( _D_SYNCML_DM_SERVICE_SOCKET_PATH , (void*) return_char , strlen(return_char) );
    }

    return result ;  // 1(OK),0(FAIL)
}*/

/*int Execute_Dev_App_Activate(char *applicaton_id, int activate_status)
{
    int result = 0 ;

    if( app_ID == NULL )
		return 0 ; // 1(OK),0(FAIL)
    LOGTRACE("appID[%s] activate[%d] \n", app_ID, activate);

    if( strlen(app_ID) >= 256 ) // Prevent defect
    {
		LOGTRACE("strlen[app_ID] >= 256 \n");
		return 0 ; // 1(OK),0(FAIL)
    }

#if defined(_D_SLP_PKGMGR_)
    int req_id = 0;
    char *descriptor = NULL;
    char pkg_type[16] = {0,};
    memcpy(pkg_type,app_ID,3);

    ecore_init();
    result = package_manager_activate_application(&req_id, pkg_type, app_ID , activate, return_pkgmgr_cb, NULL);
    if( result < PKG_COMMON_OK)
    {
		LOGTRACE("Uninstall failed! result[%d] \n", result);
		//return 1;
    }
    else {
		ecore_main_loop_begin();
    }
#endif

    ("/bin/sync");
    if( result == PKG_COMMON_OK ) {
		char return_char[512] = {0,};
		if( activate ) {
			snprintf(return_char,510,"%s,success,%s", (char*)_D_APP_ACTIVE_,app_ID ); // prevent defect
		}
		else {
			snprintf(return_char,510,"%s,success,%s", (char*)_D_APP_INACTIVE_,app_ID ); // prevent defect
		}

		QUEUE_sendto ( _D_SYNCML_DM_SERVICE_SOCKET_PATH , (void*) return_char , strlen(return_char) );
	}
    else{
		char return_char[512] = {0,};
		if( activate ){
			snprintf(return_char,510,"%s,fail,%s", (char*)_D_APP_ACTIVE_,app_ID ); // prevent defect
		}
		else {
			snprintf(return_char,510,"%s,fail,%s", (char*)_D_APP_INACTIVE_,app_ID ); // prevent defect
		}
		QUEUE_sendto ( _D_SYNCML_DM_SERVICE_SOCKET_PATH , (void*) return_char , strlen(return_char) );
	}

	return result ;  // 1(OK),0(FAIL)
}*/

/*static int __return_pkgmgr_cb(pkg_request_id req_id, const char *pkg_type, const char *pkg_name, const char *key, const char *val, const void *pmsg, void *data)
{
      LOGTRACE("return_cb req_id[%d] pkg_type[%s] pkg_name[%s] key[%s] val[%s]\n", req_id, pkg_type, pkg_name, key, val);
//    printf("return_cb req_id[%d] pkg_type[%s] pkg_name[%s] key[%s] val[%s]\n", req_id, pkg_type, pkg_name, key, val);

      if( strncmp(key, "end", strlen("end") ) == 0) {
		snprintf(g_pkg_info.pkg_type, PKG_TYPE_STRING_LEN_MAX, "%s", pkg_type);
		snprintf(g_pkg_info.pkg_name, PKG_NAME_STRING_LEN_MAX, "%s", pkg_name);
		//snprintf(g_pkg_info.pkg_version, PKG_VERSION_STRING_LEN_MAX, "%s", val);
		//exit(0); ==> exit ���� �׳� process ��ü�� �׾� ����..
		ecore_main_loop_quit(); // ==> ecore_main_loop_begin() ���� �κ��� ����
		//pthread_cond_signal(&wait_cond); // sending signal
      }
    else if( strncmp(key, "install_percent", strlen("install_percent") ) == 0) {
		LOGTRACE("install_percent[%s] \n", val);
    }
      return 0; // Prevent defect 25315
}*/

/*static int __get_mobile_tracker_status()
{
	char str_key_name[128] = { 0, };
	int int_key = 0;

	memset(str_key_name, 0, sizeof(str_key_name));
	snprintf(str_key_name, sizeof(str_key_name), "%s", VCONFKEY_SETAPPL_STATE_MOBILE_TRACKER_BOOL);	//db/setting/mobile_tracker

	if (!vconf_get_bool(str_key_name, &int_key)) {
		_DEBUG_VERBOSE("%s success , int_key : %d \n", VCONFKEY_SETAPPL_STATE_MOBILE_TRACKER_BOOL, int_key);
	} else {
		_DEBUG_VERBOSE("%s failed \n", VCONFKEY_SETAPPL_STATE_MOBILE_TRACKER_BOOL);
	}

	_DEBUG_VERBOSE(" %s vconf %d value \n", VCONFKEY_SETAPPL_STATE_MOBILE_TRACKER_BOOL, int_key);
	return int_key;
}*/
