/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>
#include <vconf.h>
#include <vconf-keys.h>

#include <tapi_common.h>
#include <ITapiModem.h>
#include <ITapiSim.h>

/*sync-agent*/
#include <sync_agent.h>

#include "plugin_slp_device_info.h"

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"PLUGIN_DEVICE_SLP_DM"
#endif

#define _D_SIZE_64_                     64
#define _D_SIZE_128_                    128

/* todo : temporary definition */
#define DTD_VERSION			"1.2"

#define OPCO						"UK"

//#define FOTA_DOWNLOAD_DIR		"/opt/data/fota/download"
//#define FOTA_SAVE_DIR					"/opt/data/fota/save"
#define FOTA_DOWNLOAD_DIR		"/opt/usr/data/fota/download"
#define FOTA_SAVE_DIR					"/opt/usr/data/fota/save"
#define FOTA_SD_DOWNLOAD_DIR	"/opt/storage/sdcard/data/fota/download"
#define FOTA_SD_SAVE_DIR			"/opt/storage/sdcard/data/fota/save"

/* ========================================*/
/* version information
 *	DTD version
 *	kernel version
 */
/* ========================================*/

char *slp_device_info_get_dtd_version()
{
	_EXTERN_FUNC_ENTER;

	char *dtd_version = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (dtd_version == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	/*snprintf(dtd_version, _D_SIZE_128_ - 1, DTD_VERSION, strlen(DTD_VERSION)); */
	snprintf(dtd_version, _D_SIZE_128_ - 1, "%s", DTD_VERSION);
	_DEBUG_INFO("DTD version = %s\n", dtd_version);

	_EXTERN_FUNC_EXIT;
	return dtd_version;
}

char *slp_device_info_get_kernel_version()
{
	_EXTERN_FUNC_ENTER;

	char *kernel_version = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (kernel_version == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}

	struct utsname buf;
	int uname_result = 0;
	uname_result = uname(&buf);
	if(uname_result != 0 ) {
		snprintf(kernel_version, _D_SIZE_128_ - 1, "%s.%s", "K", "3.4.5");

		_DEBUG_INFO("uname failed - kernel_version [%s]", kernel_version);
		_EXTERN_FUNC_EXIT;
		return kernel_version;
	}

	snprintf(kernel_version, _D_SIZE_128_ - 1, "%s.%s", "K", buf.release);

	_DEBUG_INFO("sysname[%s], nodename[%s], release[%s], version[%s], machine[%s] \n", buf.sysname, buf.nodename, buf.release, buf.version, buf.machine);
	_EXTERN_FUNC_EXIT;
	return kernel_version;
}

char *slp_device_info_get_msisdn_number()
{
	_EXTERN_FUNC_ENTER;

	char *get_str;
	char str_key_name[_D_SIZE_128_ * 2] = { 0, };
	snprintf(str_key_name, _D_SIZE_128_ * 2, "%s", VCONFKEY_TELEPHONY_SUBSCRIBER_NUMBER);
	get_str = vconf_get_str(str_key_name);
	_DEBUG_INFO("MSISDN_number = %s\n", get_str);
	_EXTERN_FUNC_EXIT;
	return get_str;

//      char *out_msisdn = 0;
//      int MSISDN_LEN = 20;
//    int err = SIM_ERROR_NONE;
//    char *msisdn = 0;
//
//    out_msisdn = (char*)calloc(MSISDN_LEN, sizeof(char));
//    if (out_msisdn == NULL)
//      goto return_part;
//
//    err = sim_get_subscriber_number(&msisdn);
//    if (!err && msisdn == NULL) {
//        strncpy(out_msisdn, msisdn, MSISDN_LEN-1);
//       free(msisdn);
//    } else  {
//        strncpy(out_msisdn, "", MSISDN_LEN-1);
//    }
//
// return_part:
//      _DEBUG_INFO("MSISDN = %s", msisdn);
//    return msisdn;
}

char *slp_device_info_get_mcc()
{
	_EXTERN_FUNC_ENTER;

	int mcc_len = 4;
	char *mcc = NULL;
	mcc = (char *)calloc(mcc_len, sizeof(char));
	if (mcc == NULL) {
		goto return_part;
	}

	int b_card_changed = 0;
	TapiHandle *handle;
	handle = tel_init(NULL);
	if (handle == NULL) {
		_DEBUG_INFO("handle is NULL !!");
		free(mcc);
		mcc = NULL;
		_EXTERN_FUNC_EXIT;
		return NULL;
	}
	TelSimImsiInfo_t imsi;
	TapiResult_t err_code = TAPI_API_SUCCESS;
	TelSimCardStatus_t status = TAPI_SIM_STATUS_CARD_ERROR;

	err_code = (TapiResult_t) tel_get_sim_init_info(handle, &status, &b_card_changed);
	if (err_code == TAPI_API_SUCCESS && status == TAPI_SIM_STATUS_SIM_INIT_COMPLETED) {
		err_code = (TapiResult_t) tel_get_sim_imsi(handle, &imsi);
		if (err_code == TAPI_API_SUCCESS) {
			strncpy(mcc, imsi.szMcc, mcc_len);
			_DEBUG_INFO("MCC = %s", mcc);
			_EXTERN_FUNC_EXIT;
			return mcc;
		}
	}

	strncpy(mcc, "0", mcc_len);
	_DEBUG_INFO("MCC = %s", mcc);

	tel_deinit(handle);

 return_part:
	_EXTERN_FUNC_EXIT;

	return mcc;
}

char *slp_device_info_get_csc()
{
	_EXTERN_FUNC_ENTER;

	_EXTERN_FUNC_EXIT;
	return strdup("VD2");
}

/* ====================================
 * device directory information
 *		fota download directory
 *		fota save directory
 * =====================================*/

char *slp_device_info_get_fota_download_dir()
{
	_EXTERN_FUNC_ENTER;

	int ret = sync_agent_create_directory(FOTA_DOWNLOAD_DIR);
	if (ret == 0) {
		_DEBUG_INFO("existed foler or create foler error");
	}

	char *fota_download_dir = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (fota_download_dir == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	/*snprintf(FOTA_Download_Dir, _D_SIZE_128_ - 1, FOTA_DOWNLOAD_DIR, strlen(FOTA_DOWNLOAD_DIR)); */
	snprintf(fota_download_dir, _D_SIZE_128_ - 1, "%s", FOTA_DOWNLOAD_DIR);

	_DEBUG_INFO("FOTA Download dir = %s\n", fota_download_dir);
	_EXTERN_FUNC_EXIT;
	return fota_download_dir;

}

char *slp_device_info_get_fota_save_dir()
{
	_EXTERN_FUNC_ENTER;

	int ret = sync_agent_create_directory(FOTA_SAVE_DIR);
	if (ret == 0) {
		_DEBUG_INFO("existed foler or create foler error");
	}

	char *fota_save_dir = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (fota_save_dir == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	/*snprintf(FOTA_Save_Dir, _D_SIZE_128_ - 1, FOTA_SAVE_DIR, strlen(FOTA_SAVE_DIR)); */
	snprintf(fota_save_dir, _D_SIZE_128_ - 1, "%s", FOTA_SAVE_DIR);

	_DEBUG_INFO("FOTA save dir = %s\n", fota_save_dir);
	_EXTERN_FUNC_EXIT;
	return fota_save_dir;
}

/* ====================================
 * device directory information
 *		fota sd card download directory
 *		fota sd card save directory
 * =====================================*/

char *slp_device_info_get_fota_sd_download_dir()
{
	_EXTERN_FUNC_ENTER;

	int ret = sync_agent_create_directory(FOTA_SD_DOWNLOAD_DIR);
	if (ret == 0) {
		_DEBUG_INFO("existed foler or create foler error");
	}

	char *fota_sd_download_dir = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (fota_sd_download_dir == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	/*snprintf(FOTA_SD_Download_Dir, _D_SIZE_128_ - 1, FOTA_SD_DOWNLOAD_DIR, strlen(FOTA_SD_DOWNLOAD_DIR)); */
	snprintf(fota_sd_download_dir, _D_SIZE_128_ - 1, "%s", FOTA_SD_DOWNLOAD_DIR);

	_DEBUG_INFO("FOTA SD CARD Download dir = %s\n", fota_sd_download_dir);
	_EXTERN_FUNC_EXIT;
	return fota_sd_download_dir;
}

char *slp_device_info_get_fota_sd_save_dir()
{
	_EXTERN_FUNC_ENTER;

	int ret = sync_agent_create_directory(FOTA_SD_SAVE_DIR);
	if (ret == 0) {
		_DEBUG_INFO("existed foler or create foler error");
	}

	char *fota_sd_save_dir = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (fota_sd_save_dir == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	/*snprintf(FOTA_SD_Save_Dir, _D_SIZE_128_ - 1, FOTA_SD_SAVE_DIR, strlen(FOTA_SD_SAVE_DIR)); */
	snprintf(fota_sd_save_dir, _D_SIZE_128_ - 1, "%s", FOTA_SD_SAVE_DIR);

	_DEBUG_INFO("FOTA SD CARD save dir = %s\n", fota_sd_save_dir);
	_EXTERN_FUNC_EXIT;
	return fota_sd_save_dir;
}

/* ====================================
 * device etc information
 * =====================================*/

char *slp_device_info_get_opco_info()
{
	_EXTERN_FUNC_ENTER;

	char *opcoInfo = (char *)calloc(_D_SIZE_128_, sizeof(char));
	if (opcoInfo == NULL) {
		_DEBUG_INFO("CALLOC failed !!!");
		_EXTERN_FUNC_EXIT;
		return 0;
	}
	snprintf(opcoInfo, _D_SIZE_128_ - 1, "%s", OPCO);
	_DEBUG_INFO("Opco Info = %s\n", opcoInfo);
	_EXTERN_FUNC_EXIT;
	return opcoInfo;	/* prevent */
}

char *slp_device_info_get_imsi_info()
{
	_EXTERN_FUNC_ENTER;

	_DEBUG_INFO("Start !!");
/*
	TelSimImsiInfo_t sim_imsi_info;
	int err_code = -1;

	char imsi_value[30] = { 0, };

	TapiHandle *handle;
	handle = tel_init(NULL);
	if (handle == NULL) {
		_DEBUG_INFO("handle is NULL !!");
		_DEBUG_INFO("End !!");
		_EXTERN_FUNC_EXIT;
		return 0;
	} else {
		_DEBUG_INFO("handle : %d", handle);
		err_code = tel_get_sim_imsi(handle, &sim_imsi_info);
	}

	if (err_code != TAPI_API_SUCCESS) {
		_DEBUG_INFO("tel_get_sim_imsi() failed !!, err_code : 0x%02x", err_code);
		_EXTERN_FUNC_EXIT;
		return 0;
	} else {
		_DEBUG_INFO("tel_get_sim_imsi() success !!");

		if (!strcmp(sim_imsi_info.szMcc, " ") && !strcmp(sim_imsi_info.szMnc, " ") && !strcmp(sim_imsi_info.szMsin, " ")) {
			_DEBUG_INFO("mcc : %s", sim_imsi_info.szMcc);
			_DEBUG_INFO("mnc : %s", sim_imsi_info.szMnc);
			_DEBUG_INFO("msin : %s", sim_imsi_info.szMsin);

			snprintf(imsi_value, sizeof(imsi_value), "%s%s%s", sim_imsi_info.szMcc, sim_imsi_info.szMnc, sim_imsi_info.szMsin);
			_DEBUG_INFO("IMSI : %s ( %d )", imsi_value, strlen(imsi_value));

			tel_deinit(handle);

			_DEBUG_INFO("End !!");
			return strdup(imsi_value);
		} else {
			_DEBUG_INFO("invalid IMSI value !!");
			tel_deinit(handle);

			_EXTERN_FUNC_EXIT;
			return 0;
		}
	}
*/
	return strdup("MccMncMsin");
	_EXTERN_FUNC_EXIT;
}
