/*
 * oma-dm-agent
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <glib-object.h>
#include <glib/gprintf.h>
#include <vconf-keys.h>
#include <vconf.h>

#include <tapi_common.h>

/*sync-agent*/
#include <sync_agent.h>

#ifndef EXPORT_API
#define EXPORT_API __attribute__ ((visibility("default")))
#endif

#ifndef OMADM_AGENT_LOG
#undef LOG_TAG
#define LOG_TAG	"PLUGIN_PM_SYSNOTI_TELEPHONY"
#endif

static TapiHandle *tel_handler = NULL;

static int (*registered_telephony_incoming_cb) (int data_id, void *user_data);
static int (*registered_telephony_end_cb) (int data_id, void *user_data);

int _register_telephony_noti_event();
int _unregister_telephony_noti_event();
static void __tel_alert_status_noti_cb(TapiHandle * handle, const char *noti_id, void *data, void *user_data);

EXPORT_API void sync_agent_plugin_init_service_noti(void *data)
{
	_EXTERN_FUNC_ENTER;

	retm_if((data) == NULL, "data is NULL!!");

	sync_agent_pm_register_data_s *reg_data = (sync_agent_pm_register_data_s *) data;
	if (reg_data == NULL) {
		_DEBUG_INFO("data is NULL !!");
		goto free_part;
	}

	g_type_init();

	tel_handler = tel_init(NULL);
	if (tel_handler == NULL) {
		_DEBUG_INFO("tel_init() failed !!");
		goto free_part;
	}

	int ret = _register_telephony_noti_event();
	if (ret != 1) {
		_DEBUG_INFO("register_notiNetworkEvent() failed !!");
	} else {
		_DEBUG_INFO("register_notiNetworkEvent() success !!");
	}

 free_part:
	if (reg_data != NULL) {
		if (reg_data->pkg_name != NULL)
			free(reg_data->pkg_name);

		if (reg_data->additional_data != NULL)
			free(reg_data->additional_data);

		free(reg_data);

		_DEBUG_INFO("free sync_agent_pm_register_data_s !!");
	}

	_EXTERN_FUNC_EXIT;
}

EXPORT_API sync_agent_pm_return_e sync_agent_plugin_unregister_service_noti(void)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_pm_return_e ret = SYNC_AGENT_PM_SUCCESS;

	int err = _unregister_telephony_noti_event();
	if (err != 1) {
		_DEBUG_INFO("__unregister_telephony_noti_event() failed !!");
		return SYNC_AGENT_PM_FAIL;
	} else {
		_DEBUG_INFO("__unregister_telephony_noti_event() success !!");
	}

	_EXTERN_FUNC_EXIT;

	return ret;
}

EXPORT_API sync_agent_pm_return_e sync_agent_plugin_add_service_data(void *add_data, int *added_data_id)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_pm_return_e ret = SYNC_AGENT_PM_SUCCESS;

	_DEBUG_INFO("not support this feature !!");

	_EXTERN_FUNC_EXIT;

	return ret;
}

EXPORT_API sync_agent_pm_return_e sync_agent_plugin_remove_service_data(int remove_data_id)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_pm_return_e ret = SYNC_AGENT_PM_SUCCESS;

	_DEBUG_INFO("not support this feature !!");

	_EXTERN_FUNC_EXIT;

	return ret;
}

EXPORT_API sync_agent_pm_return_e sync_agent_plugin_get_service_data(int get_data_id, void **service_data)
{
	_EXTERN_FUNC_ENTER;

	sync_agent_pm_return_e ret = SYNC_AGENT_PM_SUCCESS;

	_DEBUG_INFO("not support this feature !!");

	_EXTERN_FUNC_EXIT;

	return ret;
}

//void set_user_callback(int callback_counts, ...)
EXPORT_API void sync_agent_plugin_set_user_callback(int callback_counts, va_list list)
{
	_EXTERN_FUNC_ENTER;
	int i = 0;

	//      va_list list;
	//      va_start(list, callback_counts);

	_DEBUG_INFO("callback_counts : %d", callback_counts);
	for (; i < callback_counts; i++) {
		switch (i) {
		case 0:
			registered_telephony_incoming_cb = va_arg(list, sync_agent_user_callback_cb_plugin);
			_DEBUG_INFO("set user callback ( for Telephony Incoming ) !!");
			break;
		case 1:
			registered_telephony_end_cb = va_arg(list, sync_agent_user_callback_cb_plugin);
			_DEBUG_INFO("set user callback ( for Telephony End ) !!");
			break;
		default:
			break;
		}
	}

	//      va_end(list);

	_EXTERN_FUNC_EXIT;
}

int _register_telephony_noti_event()
{
	_EXTERN_FUNC_ENTER;

	int ret_val = 1;
	int api_err = 0;

	api_err = tel_register_noti_event(tel_handler, TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE, __tel_alert_status_noti_cb, NULL);	/* Tapi BS patch */
	if (api_err != 0) {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE ) failed ( err_code : %d ) !!", api_err);
		ret_val = -1;
	} else {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE ) success !!");
	}

	api_err = tel_register_noti_event(tel_handler, TAPI_NOTI_VOICE_CALL_STATUS_IDLE, __tel_alert_status_noti_cb, NULL);	/* Tapi BS patch */
	if (api_err != 0) {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_IDLE ) failed ( err_code : %d ) !!", api_err);
		ret_val = -1;
	} else {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_IDLE ) success !!");
	}

	api_err = tel_register_noti_event(tel_handler, TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE, __tel_alert_status_noti_cb, NULL);	/* Tapi BS patch */
	if (api_err != 0) {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE ) failed ( err_code : %d ) !!", api_err);
		ret_val = -1;
	} else {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE ) success !!");
	}

	api_err = tel_register_noti_event(tel_handler, TAPI_NOTI_VIDEO_CALL_STATUS_IDLE, __tel_alert_status_noti_cb, NULL);	/* Tapi BS patch */
	if (api_err != 0) {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE ) failed ( err_code : %d ) !!", api_err);
		ret_val = -1;
	} else {
		_DEBUG_TRACE("tel_register_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE ) success !!");
	}

	_EXTERN_FUNC_EXIT;

	return ret_val;
}

int _unregister_telephony_noti_event()
{
	_EXTERN_FUNC_ENTER;

	int err = 0;
	int res = 1;

	err = tel_deregister_noti_event(tel_handler, TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE);
	if (err != 0) {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE ) failed ( err_code : %d ) !!", err);
		res = -1;
	} else {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE ) success !!");
	}

	err = tel_deregister_noti_event(tel_handler, TAPI_NOTI_VOICE_CALL_STATUS_IDLE);
	if (err != 0) {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_IDLE ) failed ( err_code : %d ) !!", err);
		res = -1;
	} else {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VOICE_CALL_STATUS_IDLE ) success !!");
	}

	err = tel_deregister_noti_event(tel_handler, TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE);
	if (err != 0) {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE ) failed ( err_code : %d ) !!", err);
		res = -1;
	} else {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE ) success !!");
	}

	err = tel_deregister_noti_event(tel_handler, TAPI_NOTI_VIDEO_CALL_STATUS_IDLE);
	if (err != 0) {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_IDLE ) failed ( err_code : %d ) !!", err);
		res = -1;
	} else {
		_DEBUG_TRACE("tel_deregister_noti_event( TAPI_NOTI_VIDEO_CALL_STATUS_IDLE ) success !!");
	}

	_EXTERN_FUNC_EXIT;

	return res;
}

static void __tel_alert_status_noti_cb(TapiHandle * handle, const char *noti_id, void *data, void *user_data)
{
	_INNER_FUNC_ENTER;

	retm_if((noti_id) == NULL, "noti_id is NULL!!");

	_DEBUG_VERBOSE("NOTI_ID : %s", noti_id);

	if (!strcmp(TAPI_NOTI_VOICE_CALL_STATUS_IDLE, noti_id) || !strcmp(TAPI_NOTI_VIDEO_CALL_STATUS_IDLE, noti_id)) {
		if (registered_telephony_end_cb != NULL) {
			registered_telephony_end_cb(0, (void *)user_data);
		} else {
			_DEBUG_VERBOSE("not registered registered_telephony_end_cb !!");
		}
		_INNER_FUNC_EXIT;
		return;
	}

	if (!strcmp(TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE, noti_id) || !strcmp(TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE, noti_id)) {
		if (registered_telephony_incoming_cb != NULL) {
			registered_telephony_incoming_cb(0, (void *)user_data);
		} else {
			_DEBUG_VERBOSE("not registered registered_telephony_incoming_cb !!");
		}
		_INNER_FUNC_EXIT;
		return;
	}

	_INNER_FUNC_EXIT;
	return;
}
